/*
// This software was originally developed by ETRI and Kyunghee University, in the course
// of development of the ISO/IEC 14496-20 (Lightweight Application Scene Representation 
// -LASeR- and Simple Aggregation Format -SAF-) AMD3: Presentation and Modification of
// Structured Information standard for reference purposes and its performance may not 
// have been optimized.
//
// This software module is an implementation of one or more tools as specified by the
// ISO/IEC 14496-20 AMD3: PMSI standard.
//
// ISO/IEC gives users free license to this software module or modifications thereof
// for use in products claiming conformance to audiovisual and image-coding related
// ITU Recommendations and/or ISO/IEC International Standards.
// 
// ISO/IEC gives users the same free license to this software module or modifications
// thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in products are advised that its use may
// infringe existing patents. 
// 
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
*/
#include "stdafx.h"

#ifdef _WIN32_WCE
#include "..\M3RMobile\MainFrm.h"
#elif _ACTIVEX
#include "..\M3RacX\M3RacXDlg.h"
#else
#include "MainFrm.h"
#endif


UINT	CMainFrame::m_uPlayedWaveHeaders;
WAVEHDR	*CMainFrame::m_apPlayedWaveHeaders[32];

void CALLBACK CMainFrame::WaveProc(HWAVE hWave, UINT uMsg, DWORD dwInstance, DWORD dwParam1, DWORD dwParam2)
{
	if (uMsg == WOM_DONE)
	{
		g_cs.Lock();						// lock critical section
		m_apPlayedWaveHeaders[m_uPlayedWaveHeaders++] = (WAVEHDR*)dwParam1;
		g_cs.Unlock();						// unlock critical section
	}
}

INT8 CMainFrame::DecodeAAC()
{
	BYTE	*pUnit	= NULL;
	UINT	uSize	= 0;

	if (m_bProgressive == 0)
	{
		g_cs.Lock();							// lock critical section

		pUnit	= m_aapDB[m_uAudioTrackIndex][0];
		uSize	= m_aauUnitSizes[m_uAudioTrackIndex][0];

		// remove current unit
		m_aapDB[m_uAudioTrackIndex].pop_front();
		m_aauUnitSizes[m_uAudioTrackIndex].pop_front();

		if (m_aapDB[m_uAudioTrackIndex].size() == 0 || m_aauUnitSizes[m_uAudioTrackIndex].size() == 0)
		{
			// stream is end & buffer is exhausted
			if (m_abDBStatus[m_uAudioTrackIndex] == BUFFER_STREAM_END)
				m_abDBStatus[m_uAudioTrackIndex] = BUFFER_END;

			else
				m_abDBStatus[m_uAudioTrackIndex] = BUFFER_TEMP_EMPTY;
		}

		g_cs.Unlock();							// unlock critical section
	}

	else if (m_bProgressive == 1)
	{
		g_cs.Lock();						// lock critical section

		if (m_aapDB[m_uAudioTrackIndex].size() == m_uAudioDBIndex)
		{
			// stream is end & buffer is exhausted
			if (m_abDBStatus[m_uAudioTrackIndex] == BUFFER_STREAM_END)
				m_abDBStatus[m_uAudioTrackIndex] = BUFFER_END;

			else
				m_abDBStatus[m_uAudioTrackIndex] = BUFFER_TEMP_EMPTY;

			g_cs.Unlock();					// unlock critical section

			return 0;
		}

		else
		{
			pUnit	= m_aapDB[m_uAudioTrackIndex][m_uAudioDBIndex];
			uSize	= m_aauUnitSizes[m_uAudioTrackIndex][m_uAudioDBIndex];

			++m_uAudioDBIndex;
		}

		g_cs.Unlock();						// unlock critical section
	}

	/************************************************************************/
	void *pOutputBuffer = NeAACDecDecode(m_hDecoder, &m_frameInfo, pUnit, uSize);

	/************************************************************************/
	// WIN_Play_Samples
	if ((m_frameInfo.error == 0) && (m_frameInfo.samples > 0))
	{
		HGLOBAL		hG1;
		HGLOBAL		hG2;
		LPWAVEHDR	pWaveHeader;
		void		*pAlloc;

		UINT		uSize = (UINT)(m_frameInfo.channels * m_frameInfo.samples);

		do
		{
			while (m_uPlayedWaveHeaders > 0)	// free used blocks
				FreeWaveMemory();

			// wait for a free block
			if (m_nScheduledBlocks < (sizeof(m_apPlayedWaveHeaders) / sizeof(*m_apPlayedWaveHeaders)))
				break;

			Sleep (26);

		} while (1);

		// allocate some memory for a copy of the buffer
		if ((hG2 = GlobalAlloc(GMEM_MOVEABLE, uSize)) == NULL)
			return PlayerDecoderErr;

		pAlloc = GlobalLock(hG2);

		// here we can call any modification output functions we want
		CopyMemory(pAlloc, pOutputBuffer, uSize);

		// now make a header and WRITE IT!
		if ((hG1 = GlobalAlloc(GMEM_MOVEABLE | GMEM_ZEROINIT, sizeof(WAVEHDR))) == NULL)
			return PlayerDecoderErr;

		pWaveHeader					= (LPWAVEHDR)GlobalLock(hG1);
		pWaveHeader->dwBufferLength	= uSize;
		pWaveHeader->lpData			= (LPSTR)pAlloc;

		if (waveOutPrepareHeader(m_hDevice, pWaveHeader, sizeof(WAVEHDR)) != MMSYSERR_NOERROR)
		{
			GlobalUnlock(hG1);
			GlobalFree(hG1);
			return PlayerDecoderErr;
		}

		if (waveOutWrite(m_hDevice, pWaveHeader, sizeof(WAVEHDR)) != MMSYSERR_NOERROR)
		{
			GlobalUnlock(hG1);
			GlobalFree(hG1);
			return PlayerDecoderErr;
		}

		g_cs.Lock();						// lock critical section
		++m_nScheduledBlocks;
		g_cs.Unlock();						// unlock critical section
	}

	return 0;
}

void CMainFrame::FreeWaveMemory()
{
	WAVEHDR		*pWaveHeader	= NULL;;
	HGLOBAL		hG				= NULL;

	g_cs.Lock();						// lock critical section
	pWaveHeader = m_apPlayedWaveHeaders[--m_uPlayedWaveHeaders];
	--m_nScheduledBlocks;				// decrease the number of USED blocks
	g_cs.Unlock();						// unlock critical section

	waveOutUnprepareHeader(m_hDevice, pWaveHeader, sizeof(WAVEHDR));

	// deallocate the buffer memory
	hG = GlobalHandle(pWaveHeader->lpData);
	GlobalUnlock(hG);
	GlobalFree(hG);

	// deallocate the header memory
	hG = GlobalHandle(pWaveHeader);
	GlobalUnlock(hG);
	GlobalFree(hG);
}

INT8 CMainFrame::InitAACDecoder(BYTE *pBits, const UINT uSize)
{
	// copy from libfaad - decode.c - decodeMP4file
	UINT	uSampleRate	= 0;
	UINT8	uChannels	= 0;

	m_hDecoder = NeAACDecOpen();

	if (NeAACDecInit2(m_hDecoder, pBits, uSize, (unsigned long *)&uSampleRate, &uChannels) < 0)
	{
		NeAACDecClose(m_hDecoder);
		return PlayerDecoderErr;
	}

	return 0;
}

INT8 CMainFrame::InitAACDecoder()
{
	BYTE	*pUnit	= NULL;
	UINT	uSize	= 0;

	g_cs.Lock();								// lock critical section

	pUnit	= m_aapDB[m_uAudioTrackIndex][0];
	uSize	= m_aauUnitSizes[m_uAudioTrackIndex][0];

	g_cs.Unlock();								// unlock critical section

	// decode first encoded bytes
	NeAACDecDecode(m_hDecoder, &m_frameInfo, pUnit, uSize);

	/************************************************************************/
	WAVEFORMATEX outFormat;

	if (waveOutGetNumDevs() == 0)
	{
		NeAACDecClose(m_hDecoder);
		return PlayerDecoderErr;
	}

	outFormat.wFormatTag		= WAVE_FORMAT_PCM;
	outFormat.wBitsPerSample	= 16;
	outFormat.nChannels			= m_frameInfo.channels;
	outFormat.nSamplesPerSec	= m_frameInfo.samplerate;
	outFormat.nBlockAlign		= (WORD)((outFormat.nChannels * outFormat.wBitsPerSample) / 8);
	outFormat.nAvgBytesPerSec	= (WORD)((outFormat.nSamplesPerSec * outFormat.nChannels * outFormat.wBitsPerSample) / 8);

	if (waveOutOpen(&m_hDevice, WAVE_MAPPER, &outFormat, (DWORD)WaveProc, 0, CALLBACK_FUNCTION))
	{
		NeAACDecClose(m_hDecoder);
		return PlayerDecoderErr;
	}

	waveOutReset(m_hDevice);

	return 0;
}

UINT ThreadAACDecoder(LPVOID pParam)
{
	INT8 err = 0;

	CMainFrame *pFrame = (CMainFrame *)pParam;

	if (pFrame == NULL)
		return 0;

	const UINT uAudioTrack = pFrame->GetAudioTrackIndex();

	// if the player is run
	while (pFrame->GetPlayerStatus() == PLAYER_PLAYING)
	{
		// the db's status is temporary empty
		if (pFrame->GetDBStatus(uAudioTrack) == BUFFER_TEMP_EMPTY)
			Sleep(0);

		// the db's status is end & empty
		else if (pFrame->GetDBStatus(uAudioTrack) == BUFFER_END)
		{
			g_cs.Lock();						// lock critical section

			pFrame->m_wndView.SetCBStatus(uAudioTrack, BUFFER_STREAM_END);

			g_cs.Unlock();						// unlock critical section

			break;								// exit thread
		}

		// the db's status is normal
		else
		{
			err = pFrame->DecodeAAC();
			Sleep(20);
		}
	}

	return 0;
}