/*
// This software was originally developed by ETRI and Kyunghee University, in the course
// of development of the ISO/IEC 14496-20 (Lightweight Application Scene Representation 
// -LASeR- and Simple Aggregation Format -SAF-) AMD3: Presentation and Modification of
// Structured Information standard for reference purposes and its performance may not 
// have been optimized.
//
// This software module is an implementation of one or more tools as specified by the
// ISO/IEC 14496-20 AMD3: PMSI standard.
//
// ISO/IEC gives users free license to this software module or modifications thereof
// for use in products claiming conformance to audiovisual and image-coding related
// ITU Recommendations and/or ISO/IEC International Standards.
// 
// ISO/IEC gives users the same free license to this software module or modifications
// thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in products are advised that its use may
// infringe existing patents. 
// 
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
*/
#include "stdafx.h"

#ifdef _WIN32_WCE
#include "..\M3RMobile\MainFrm.h"
#elif _ACTIVEX
#include "..\M3RacX\M3RacXDlg.h"
#else
#include "MainFrm.h"
#endif

#include "AVCDecoder.h"
#include "AVCDecoder2.h"
#include "RComposite.h"


INT8 CMainFrame::DecodeAVC()
{
	BYTE	*pUnit	= NULL;
	UINT	uSize	= 0;

	BYTE	*pRGB	= new BYTE[m_uPicSize];
	BYTE	*pRGB2	= NULL;
	BYTE	*pYUV2	= NULL;

	if (!pRGB || !m_pYUV)
		return PlayerNoMoreMemoryErr;

	memset(pRGB, 0, m_uPicSize);
	memset(m_pYUV, 0, m_uPicSize >> 1);

	vector<UINT>::iterator i;for (i = m_auVideoTrackIndices.begin(); i < m_auVideoTrackIndices.end(); ++i)
	{
		if (m_bProgressive == 0)
		{
			g_cs.Lock();						// lock critical section

			pUnit	= m_aapDB[*i][0];
			uSize	= m_aauUnitSizes[*i][0];

			m_aapDB[*i].pop_front();			// remove current unit
			m_aauUnitSizes[*i].pop_front();

			if (m_aapDB[*i].size() == 0 || m_aauUnitSizes[*i].size() == 0)
			{
				// stream is end & buffer is exhausted
				if (m_abDBStatus[*i] == BUFFER_STREAM_END)
					m_abDBStatus[*i] = BUFFER_END;

				else
					m_abDBStatus[*i] = BUFFER_TEMP_EMPTY;
			}

			g_cs.Unlock();						// unlock critical section
		}

		else if (m_bProgressive == 1)
		{
			g_cs.Lock();						// lock critical section

			if (m_aapDB[*i].size() == m_uVideoDBIndex)
			{
				// stream is end & buffer is exhausted
				if (m_abDBStatus[*i] == BUFFER_STREAM_END)
					m_abDBStatus[*i] = BUFFER_END;

				else
					m_abDBStatus[*i] = BUFFER_TEMP_EMPTY;

				g_cs.Unlock();					// unlock critical section

				return 0;
			}

			else
			{
				pUnit	= m_aapDB[*i][m_uVideoDBIndex];
				uSize	= m_aauUnitSizes[*i][m_uVideoDBIndex];

				if ((i + 1) >= m_auVideoTrackIndices.end())
					++m_uVideoDBIndex;
			}

			g_cs.Unlock();						// unlock critical section
		}

		/************************************************************************/
		// avc file format or 1st video es in svaf
		if (i == m_auVideoTrackIndices.begin())
		{
			if (avc_decode(m_pYUV, pUnit, uSize) == 0)
			{
				memset(m_pYUV, 0, m_uPicSize >> 1);

				return PlayerDecoderErr;
			}

			else
				yuv420_rgb24(m_pYUV, pRGB, m_uVideoWidth, m_uVideoHeight);
		}

		/************************************************************************/
		// 2nd video es in svaf
		if ((i == m_auVideoTrackIndices.begin() + 1) && (m_uCompositionType != 0xFF))
		{
			pRGB2	= new BYTE[m_uPicSize];
			pYUV2	= new BYTE[m_uPicSize >> 1];

			if (!pRGB2 || !pYUV2)
				return PlayerNoMoreMemoryErr;

			memset(pRGB2, 0, m_uPicSize);
			memset(pYUV2, 0, m_uPicSize >> 1);

			if (avc_decode2(pYUV2, pUnit, uSize) == 0)
			{
				delete [] pRGB2;
				pRGB2 = NULL;

				delete [] pYUV2;
				pYUV2 = NULL;

				return PlayerDecoderErr;
			}

			else
				yuv420_rgb24(pYUV2, pRGB2, m_uVideoWidth, m_uVideoHeight);
		}
	}

	// composition
	if ((m_uCompositionType >= 0) && (m_uCompositionType <= 5))
	{
		BYTE *pTemp = new BYTE[m_uPicSize];
		memset(pTemp, 0, m_uPicSize);

		switch (m_uCompositionType)
		{
		case 0:
			rgb24_3d(m_uCompositionType, 1, m_uVideoWidth, m_uVideoHeight, pRGB, pRGB2, pTemp);
			break;

		case 1:
		case 2:
		case 3:
		case 4:
			rgb24_3d(m_uCompositionType, 5, m_uVideoWidth, m_uVideoHeight, pRGB, pRGB2, pTemp);
			break;
		}

		delete [] pRGB;

		if (pRGB2)
		{
			delete [] pRGB2;
			pRGB2 = NULL;
		}

		if (pYUV2)
		{
			delete [] pYUV2;
			pYUV2 = NULL;
		}

		pRGB = pTemp;
	}

	g_cs.Lock();						// lock critical section

	m_wndView.m_aapCB[*m_auVideoTrackIndices.begin()].push_back(pRGB);
	m_wndView.SetCBStatus(*m_auVideoTrackIndices.begin(), BUFFER_NORMAL);

	g_cs.Unlock();						// unlock critical section

	return 0;
}

INT8 CMainFrame::InitAVCDecoder(BYTE *pBits, const UINT uSize)
{
	if ((m_uVideoWidth == 0) && (m_uVideoHeight == 0))
	{
		if (avc_init(pBits, uSize) == 0)
		{
			avc_release();
			return PlayerDecoderErr;
		}

		m_uVideoWidth	= (UINT)get_avc_width();
		m_uVideoHeight	= (UINT)get_avc_height();

		m_uPicSize	= m_uVideoWidth * m_uVideoHeight * 3;
		m_pYUV		= new BYTE[m_uPicSize >> 1];
	}

	else
	{
		if (avc_init2(pBits, uSize) == 0)
		{
			avc_release2();
			return PlayerDecoderErr;
		}
	}

	return 0;
}

UINT ThreadAVCDecoder(LPVOID pParam)
{
	CMainFrame *pFrame = (CMainFrame *)pParam;

	INT8 err = 0;

	if (pFrame == NULL)
		return 0;

	vector<UINT> auVideoTrack = pFrame->GetVideoTrackIndices();

	UINT8 uFlag = 0;

	// if the player is run
	while (pFrame->GetPlayerStatus() == PLAYER_PLAYING)
	{
		// look out the all track
		for (vector<UINT>::iterator i = auVideoTrack.begin(); i < auVideoTrack.end(); ++i)
		{
			// the db's status is temporary empty
			if (pFrame->GetDBStatus(*i) == BUFFER_TEMP_EMPTY)
				uFlag |= BUFFER_TEMP_EMPTY;

			// the db's status is end & empty
			else if (pFrame->GetDBStatus(*i) == BUFFER_END)
			{
				g_cs.Lock();						// lock critical section

				pFrame->m_wndView.SetCBStatus(*i, BUFFER_STREAM_END);

				g_cs.Unlock();						// unlock critical section

				// erase ended track index
				auVideoTrack.erase(i);

				break;
			}

			// the db's status is normal
			else
				uFlag |= BUFFER_NORMAL;
		}

		/************************************************************************/
		if (uFlag & BUFFER_TEMP_EMPTY)
		{
			Sleep(20);

			uFlag = 0;							// reset flag
		}

		else if (uFlag & BUFFER_NORMAL)
		{
			err = pFrame->DecodeAVC();
			Sleep(33);

			uFlag = 0;							// reset flag
		}

		// exit condition
		else if (auVideoTrack.size() == 0)
			break;
	}

	return 0;
}