/*
// This software was originally developed by ETRI and Kyunghee University, in the course
// of development of the ISO/IEC 14496-20 (Lightweight Application Scene Representation 
// -LASeR- and Simple Aggregation Format -SAF-) AMD3: Presentation and Modification of
// Structured Information standard for reference purposes and its performance may not 
// have been optimized.
//
// This software module is an implementation of one or more tools as specified by the
// ISO/IEC 14496-20 AMD3: PMSI standard.
//
// ISO/IEC gives users free license to this software module or modifications thereof
// for use in products claiming conformance to audiovisual and image-coding related
// ITU Recommendations and/or ISO/IEC International Standards.
// 
// ISO/IEC gives users the same free license to this software module or modifications
// thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in products are advised that its use may
// infringe existing patents. 
// 
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
*/

// ChildView.cpp : implementation of the CChildView class
//

#include "stdafx.h"
#include "M3RP.h"
#include "ChildView.h"
#include "MainFrm.h"

#include <algorithm>
#include <fstream>

#include <afxinet.h>
#include <afxsock.h>
#include <direct.h>

#include "Markup.h"

extern UINT ThreadRenderer(LPVOID pParam);


#ifdef _DEBUG
#define new DEBUG_NEW
#endif

BEGIN_MESSAGE_MAP(CChildView, CWnd)
	ON_WM_PAINT()
	ON_WM_CREATE()
	ON_WM_CLOSE()
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEMOVE()
	ON_MESSAGE(WM_PLAY, OnPlay)
	ON_MESSAGE(WM_PAUSE, OnPause)
	ON_MESSAGE(WM_STOP, OnStop)
	ON_MESSAGE(WM_BUFFER, OnBuffer)
	ON_WM_TIMER()
	ON_WM_KEYDOWN()
END_MESSAGE_MAP()


// CChildView

CChildView::CChildView()
{
	m_hMemDC			= NULL;
	m_hBit				= NULL;
	m_hOldBit			= NULL;

	m_pRendererThread	= NULL;

	m_abCBStatus.clear();

	m_uVideoTrackIndex	= 0xFFFFFFFF;
	m_uAudioTrackIndex	= 0xFFFFFFFF;

	m_pSurface			= NULL;
	m_pCR				= NULL;

	m_uViewWidth		= 0;
	m_uViewHeight		= 0;

	m_uWidthUnit		= 0;
	m_uHeightUnit		= 0;

	m_fWidthRatio		= 1;
	m_fHeightRatio		= 1;

	m_uAdaptationType	= 0;

	m_uEventType		= 0;

	m_uTimerIndex		= 0;

	m_uTransformed		= 0;

	m_bcontinouwimage   = 0;
	m_bcontinouwtref    = 0;
	m_bcontinouwvideo   = 0;
	m_bcontinouw        = 0;
}

CChildView::~CChildView()
{
	ClearPlayerStatus();
}

void CChildView::ClearPlayerStatus()
{
	m_wndScrollBar.Reset();						// scroll bar reset

	m_abCBStatus.clear();

	if (m_pRendererThread)
	{
		while (::WaitForSingleObject(m_pRendererThread->m_hThread, 0) == WAIT_TIMEOUT)
			Sleep(10);

		m_pRendererThread = NULL;
	}

	deque< deque<BYTE *> >::iterator i;
	deque<BYTE *>::iterator j;

	for (i = m_aapCB.begin(); i < m_aapCB.end(); ++i)
	{
		for (j = i->begin(); j < i->end(); ++j)
		{
			if (*j)
			{
				delete [] *j;
				*j = NULL;
			}
		}

		i->clear();
	}

	m_aapCB.clear();

	if (m_pSurface)
	{
		cairo_surface_destroy(m_pSurface);
		m_pSurface = NULL;
	}

	if (m_pCR)
	{
		cairo_destroy(m_pCR);
		m_pCR = NULL;
	}

	if (m_hMemDC)
	{
		DeleteDC(m_hMemDC);
		m_hMemDC = NULL;
	}

	if (m_hBit)
	{
		DeleteObject(m_hBit);
		m_hBit = NULL;
	}

	if (m_hOldBit)
	{
		SelectObject(m_hMemDC, m_hOldBit);

		DeleteObject(m_hOldBit);
		m_hOldBit = NULL;
	}

	m_uVideoTrackIndex	= 0xFFFFFFFF;
	m_uAudioTrackIndex	= 0xFFFFFFFF;

	/************************************************************************/
	m_uViewWidth		= 0;
	m_uViewHeight		= 0;

	m_uWidthUnit		= 0;
	m_uHeightUnit		= 0;

	m_fWidthRatio		= 1;
	m_fHeightRatio		= 1;

	m_uAdaptationType	= 0;

	m_apA.clear();
	m_uEventType		= 0;

	for (vector<CLASeR *>::iterator k = m_apTimer.begin(); k < m_apTimer.begin(); ++k)
	{
		if (*k != NULL)
			KillTimer(distance(m_apTimer.begin(), k));
	}

	m_apTimer.clear();
	m_uTimerIndex		= 0;

	m_uTransformed		= 0;

	m_astrImage.clear();

	m_apListener.clear();
	m_apanimate.clear();

	m_bcontinouwimage   = 0;
	m_bcontinouwtref    = 0;
	m_bcontinouwvideo   = 0;
	m_bcontinouw        = 0;
}


// CChildView message handlers

BOOL CChildView::PreCreateWindow(CREATESTRUCT& cs) 
{
	if (!CWnd::PreCreateWindow(cs))
		return FALSE;

	cs.dwExStyle |= WS_EX_CLIENTEDGE;
	cs.style &= ~WS_BORDER;
	cs.lpszClass = AfxRegisterWndClass(CS_HREDRAW|CS_VREDRAW|CS_DBLCLKS, 
		::LoadCursor(NULL, IDC_ARROW), reinterpret_cast<HBRUSH>(COLOR_WINDOW+1), NULL);

	return TRUE;
}

int CChildView::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CWnd::OnCreate(lpCreateStruct) == -1)
		return -1;

	// TODO:  ⿡ Ưȭ ۼ ڵ带 ߰մϴ.
	// attach button
	m_btnPlay.Create(NULL, NULL, WS_CHILD|WS_VISIBLE|GBS_PUSH,
		CRect(0, (INITIAL_HEIGHT - g_uFrameHeight - PLAYBAR_HEIGHT), PLAYBAR_WIDTH, (INITIAL_HEIGHT - g_uFrameHeight)), this, 1);
	m_btnPlay.SetImage(IDB_PLAY, IDB_PAUSE, IDB_PLAY_OVER, IDB_PAUSE_OVER);

	// attach scroll bar
	m_wndScrollBar.Create(NULL, NULL, WS_CHILD|WS_VISIBLE|GBS_PUSH,
		CRect(PLAYBAR_WIDTH, (INITIAL_HEIGHT - g_uFrameHeight - PLAYBAR_HEIGHT),
		(INITIAL_WIDTH - g_uFrameWidth), (INITIAL_HEIGHT - g_uFrameHeight)), this, 1);

	// set scroll bar size
	m_wndScrollBar.SetSize((UINT)(INITIAL_WIDTH - g_uFrameWidth - PLAYBAR_WIDTH));

	m_wndScrollBar.SetImageBar(IDB_SCROLLBAR, IDB_SCROLLHEAD, IDB_SCROLLTAIL);

	return 0;
}

BOOL CChildView::OnCommand(WPARAM wParam, LPARAM lParam)
{
	// TODO: ⿡ Ưȭ ڵ带 ߰ /Ǵ ⺻ Ŭ ȣմϴ.
	CMainFrame *pFrame = (CMainFrame *)(AfxGetApp()->GetMainWnd());

	switch (LOWORD(wParam))
	{
	case 1:
		if (pFrame->GetPlayerStatus() == PLAYER_PLAYING)
		{
			pFrame->PostMessage(WM_PAUSE);
			PostMessage(WM_PAUSE);
		}

		else if (pFrame->GetPlayerStatus() == PLAYER_PAUSE)
		{
			pFrame->PostMessage(WM_PLAY);
			PostMessage(WM_PLAY);
		}

		else if (pFrame->GetPlayerStatus() == PLAYER_STOP)
			pFrame->PostMessage(WM_READY);

		break;

	case 2:
		PostMessage(WM_BUFFER);
		break;
	}

	return CWnd::OnCommand(wParam, lParam);
}

void CChildView::OnPaint()
{
	CPaintDC dc(this); // device context for painting
	
	// TODO: Add your message handler code here
	if (m_hMemDC)
		BitBlt(dc.m_hDC, 0, 0, m_uViewWidth, m_uViewHeight, m_hMemDC, 0, 0, SRCCOPY);
	
	// Do not call CWnd::OnPaint() for painting messages
}

void CChildView::OnClose()
{
	// TODO: ⿡ ޽ ó ڵ带 ߰ /Ǵ ⺻ ȣմϴ.
	if (GetPlayerStatus() == PLAYER_PAUSE)
		SendMessage(WM_PLAY);

	ClearPlayerStatus();

	CWnd::OnClose();
}

/************************************************************************/
void ErrorMessage(INT8 errCode)
{
	if (errCode < 0)
		AfxMessageBox(_T("Error"));
}

void CChildView::PrepareScene()
{
	lsrErr err = LASeRNoErr;

	CSAF *pRoot = GetSceneRoot();

	if (!pRoot)
		return;

	CLASeR	*pLASeR	= NULL;
	CSAF	*pSAF	= pRoot;

	while (1)
	{
		err = GetNextObject(&pLASeR, &pSAF);
		ErrorMessage(err);

		if ((pLASeR != NULL) && (pSAF == NULL))
		{
			err = PrepareEachElement(pLASeR);
			ErrorMessage(err);
		}

		else if ((pLASeR == NULL) && (pSAF != NULL))
		{
			if (SAF_EndOfSAFSession == pSAF->GetElementType())
				break;
		}
	}

	// whether video or audio is used or not
	if (GetMP4Handle())
		m_pRendererThread = ::AfxBeginThread(ThreadRenderer, (LPVOID)this);

	else
	{
		DWORD dwStartTime = timeGetTime();

		DrawScene();

		DWORD dwEndTime = timeGetTime();

		FILE *outFile;

		outFile = fopen("../../test.log", "a");
		fprintf(outFile, "%u\n", (UINT)(dwEndTime - dwStartTime));
		fclose(outFile);
	}
}

lsrErr CChildView::PrepareEachElement(CLASeR *pLASeR)
{
	lsrErr err = LASeRNoErr;

	UINT8 pElementType = pLASeR->GetElementType();

	switch (pElementType)
	{
	case SVG_a:
		PrepareA(pLASeR);
		break;

	case SVG_animate:
		PrepareAnimate(pLASeR);
		break;

	case SVG_animateTransform:
		m_apanimate.push_back(pLASeR);
		break;

	case SVG_audio:
		PrepareAudio(pLASeR);
		break;

	case LSR_conditional:
		PrepareConditional(pLASeR);
		break;

	case SVG_g:
		PrepareG(pLASeR);
		break;

	case SVG_set:
		PrepareSet(pLASeR);
		break;

	case SVG_svg:
		PrepareSVG(pLASeR);
		break;

	case SVG_video:
		PrepareVideo(pLASeR);
		break;

	case LSR_LASeRHeader:
		PrepareLASeRHeader(pLASeR);
		break;

	case LSR_externalReference:
		PrepareExternalReference(pLASeR);
		break;

	default:
		/* nothing */
		break;
	}

	// phased scene adaptation
	if ((m_uAdaptationType == 8) && (pElementType != LSR_LASeRHeader))
		PhasedSceneAdaptation(pLASeR);

	return err;
}

void CChildView::DrawScene()
{
	lsrErr err = LASeRNoErr;

	if (m_uAdaptationType == 8)
	{
		multimap<UINT, CLASeR *>::iterator i;

		float fPreMemory, fPostMemory, fUsingMemory;

		for (i = m_mapElements.begin(); i != m_mapElements.end(); ++i)
		{
 			fPreMemory = GetAvailableMemorySize();

			err = DrawEachElement(i->second);
			ErrorMessage(err);

			fPostMemory = GetAvailableMemorySize();

			fUsingMemory = fPreMemory - fPostMemory;

			if (fUsingMemory > fPostMemory)
				break;
		}
	}

	else
	{
		CSAF *pRoot = GetSceneRoot();

		if (!pRoot)
			return;

		CLASeR	*pLASeR	= NULL;
		CSAF	*pSAF	= pRoot;

		while (1)
		{
			err = GetNextObject(&pLASeR, &pSAF);
			ErrorMessage(err);

			if ((pLASeR != NULL) && (pSAF == NULL))
			{
				err = DrawEachElement(pLASeR);
				ErrorMessage(err);
			}

			else if ((pLASeR == NULL) && (pSAF != NULL))
			{
				if (SAF_EndOfSAFSession == pSAF->GetElementType())
					break;
			}
		}
	}
	Invalidate(FALSE);
}

lsrErr CChildView::DrawEachElement(CLASeR *pLASeR)
{
	lsrErr err = LASeRNoErr;

	UINT8 pElementType = pLASeR->GetElementType();

	switch (pElementType)
	{
	case SVG_a:
		DrawA(pLASeR);
		break;

	case SVG_animate:
		DrawAnimate(pLASeR);
		break;

	case SVG_circle:
		DrawCircle(pLASeR);
		break;

	case SVG_ellipse:
		DrawEllipse(pLASeR);
		break;

	case SVG_g:
		DrawG(pLASeR);
		break;

	case SVG_image:
		DrawImage(pLASeR);
		break;

	case SVG_line:
		DrawLine(pLASeR);
		break;

	case EV_listener:
		PrepareListener(pLASeR);
		break;


	case LSR_parsingSwitch:
		DrawParsingSwitch(pLASeR);
		break;

	case SVG_polygon:
		DrawPolygon(pLASeR);
		break;

	case SVG_polyline:
		DrawPolyline(pLASeR);
		break;

	case SVG_rect:
		DrawRect(pLASeR);
		break;

	case SVG_set:
		DrawSet(pLASeR);
		break;

	case SVG_svg:
		DrawSVG(pLASeR);
		break;

	case SVG_text:
		DrawText(pLASeR);
		break;

	case SVG_textArea:
		DrawTextArea(pLASeR);
		break;

	case SVG_video:
		DrawVideo(pLASeR);
		break;

	default:
		/* nothing */
		break;
	}

	return err;
}

lsrErr CChildView::DeleteEachElement(CLASeR *pLASeR)
{
	lsrErr err = LASeRNoErr;

	UINT8 pElementType = pLASeR->GetElementType();

	switch (pElementType)
	{

	//case SVG_circle:
	//	DrawCircle(pLASeR);
	//	break;

	//case SVG_ellipse:
	//	DrawEllipse(pLASeR);
	//	break;

	//case SVG_g:
	//	DrawG(pLASeR);
	//	break;

	case SVG_image:
		DeleteImage(pLASeR);
		break;

	//case SVG_line:
	//	DrawLine(pLASeR);
	//	break;

	//case EV_listener:
	//	PrepareListener(pLASeR);
	//	break;


	//case LSR_parsingSwitch:
	//	DrawParsingSwitch(pLASeR);
	//	break;

	//case SVG_polygon:
	//	DrawPolygon(pLASeR);
	//	break;

	//case SVG_polyline:
	//	DrawPolyline(pLASeR);
	//	break;

	//case SVG_rect:
	//	DrawRect(pLASeR);
	//	break;

	//case SVG_set:
	//	DrawSet(pLASeR);
	//	break;

	//case SVG_svg:
	//	DrawSVG(pLASeR);
	//	break;

	case SVG_text:
		DrawText(pLASeR);
		break;

	case SVG_textArea:
		DeleteTextArea(pLASeR);
		break;

	//case SVG_video:
	//	DrawVideo(pLASeR);
	//	break;

	default:
		/* nothing */
		break;
	}
	return err;
}
/************************************************************************/
CString CChildView::GetPathName() const
{
	CMainFrame *pFrame = (CMainFrame *)(AfxGetApp()->GetMainWnd());

	return pFrame->GetPathName();
}

CLibMP4Api *CChildView::GetMP4Handle() const
{
	CMainFrame *pFrame = (CMainFrame *)(AfxGetApp()->GetMainWnd());

	return pFrame->GetMP4Handle();
}

UINT8 CChildView::GetLASeRFlag() const
{
	CMainFrame *pFrame = (CMainFrame *)(AfxGetApp()->GetMainWnd());

	return pFrame->GetLASeRFlag();
}

UINT8 CChildView::GetPlayerStatus() const
{
	CMainFrame *pFrame = (CMainFrame *)(AfxGetApp()->GetMainWnd());

	return pFrame->GetPlayerStatus();
}

void CChildView::SetPlayerStatus(const UINT8 bStatus) const
{
	CMainFrame *pFrame = (CMainFrame *)(AfxGetApp()->GetMainWnd());

	pFrame->SetPlayerStatus(bStatus);
}

UINT8 CChildView::GetPictureDelta() const
{
	CMainFrame *pFrame = (CMainFrame *)(AfxGetApp()->GetMainWnd());

	return pFrame->GetPictureDelta();
}

UINT CChildView::GetVideoWidth() const
{
	CMainFrame *pFrame = (CMainFrame *)(AfxGetApp()->GetMainWnd());

	return pFrame->GetVideoWidth();
}

UINT CChildView::GetVideoHeight() const
{
	CMainFrame *pFrame = (CMainFrame *)(AfxGetApp()->GetMainWnd());

	return pFrame->GetVideoHeight();
}

/************************************************************************/
void CChildView::SetCBStatus(const UINT8 bStatus)
{
	for (vector<UINT8>::iterator i = m_abCBStatus.begin(); i < m_abCBStatus.end(); ++i)
		*i = bStatus;
}

void CChildView::SetCBStatus(const UINT uIndex, const UINT8 bStatus)
{
	if (m_abCBStatus.size() > uIndex)
		m_abCBStatus[uIndex] = bStatus;

	else
	{
		m_abCBStatus.resize(uIndex + 1);
		m_abCBStatus[uIndex] = bStatus;
	}
}

/************************************************************************/
LRESULT CChildView::OnPlay(WPARAM wParam, LPARAM lParam)
{
	if (m_pRendererThread)
		m_pRendererThread->ResumeThread();

	return 0;
}

LRESULT CChildView::OnPause(WPARAM wParam, LPARAM lParam)
{
	if (m_pRendererThread)
		m_pRendererThread->SuspendThread();

	return 0;
}

LRESULT CChildView::OnStop(WPARAM wParam, LPARAM lParam)
{
	m_btnPlay.ChangeState(GB_PLAY);

	m_abCBStatus.clear();

	if (m_pRendererThread)
	{
		while (::WaitForSingleObject(m_pRendererThread->m_hThread, 0) == WAIT_TIMEOUT)
			Sleep(10);

		m_pRendererThread = NULL;
	}

	deque< deque<BYTE *> >::iterator i;
	deque<BYTE *>::iterator j;

	for (i = m_aapCB.begin(); i < m_aapCB.end(); ++i)
	{
		for (j = i->begin(); j < i->end(); ++j)
		{
			if (*j)
			{
				delete [] *j;
				*j = NULL;
			}
		}

		i->clear();
	}

	for (vector<CLASeR *>::iterator k = m_apTimer.begin(); k < m_apTimer.begin(); ++k)
	{
		if (*k != NULL)
			KillTimer(distance(m_apTimer.begin(), k));
	}

	m_apTimer.clear();
	m_uTimerIndex		= 0;

	m_astrImage.clear();

	m_apanimate.clear(); //
	m_apListener.clear();

	return 0;
}

LRESULT CChildView::OnBuffer(WPARAM wParam, LPARAM lParam)
{
	CMainFrame *pFrame = (CMainFrame *)(AfxGetApp()->GetMainWnd());

	m_wndScrollBar.SetBuffer(pFrame->GetBufferState());

	return 0;
}

/************************************************************************/
void CChildView::OnTimer(UINT_PTR nIDEvent)
{
	// TODO: ⿡ ޽ ó ڵ带 ߰ /Ǵ ⺻ ȣմϴ.
	CLASeR *pLASeR = m_apTimer[nIDEvent];

	lsrErr err = LASeRNoErr;

	switch (pLASeR->GetElementType())
	{
	case SVG_animateTransform:
		{

			UINT8 uAttrType = pLASeR->GetUINT8Attr(ATTR_ATTRIBUTENAME, &err);
			ErrorMessage(err);

			vector<tstring> astrTo	= pLASeR->GetStrListAttr(ATTR_TO, &err);
			ErrorMessage(err);

			float fdur				 = pLASeR->GetFloatAttr(ATTR_DUR, &err);
			ErrorMessage(err);

			float fcount = fdur;

			TCHAR szTo[300];
			wsprintf(szTo, _T("%s"), astrTo[0].c_str());
			int iTo = _ttoi(szTo);

			float iToresult = iTo/fcount;

			CLASeR *pPLASeR = pLASeR->GetParent();
			UINT childrensize = pPLASeR->GetChildrenSize();

			if (childrensize)
			{
				for (int i=0; i<1; i++)
				{
					CLASeR *pChLASeR = pPLASeR->GetChildren(i);

					if (pChLASeR->GetElementType() == SVG_g)
					{				
						UINT childg = pChLASeR->GetChildrenSize();

						for (int j=0; j<childg; j++)
						{									
							CLASeR *pChildLASeR = pChLASeR->GetChildren(j);
							DeleteEachElement(pChildLASeR);
							Invalidate(TRUE);
						}
						
						vector<float> aftranslation = pPLASeR->GetFloatListAttr(ATTR_LSR_TRANSLATION, &err);
						iToresult += aftranslation[0];			
						aftranslation.clear();
						aftranslation.push_back(iToresult);
						aftranslation.push_back(0);
						pPLASeR->SetAttr(ATTR_LSR_TRANSLATION, aftranslation);
						
						DrawEachElement(pChLASeR);
						DrawEachElement(pChLASeR);

						for (int k=0; k<childg; k++)
						{			
							CLASeR *pChildLASeR = pChLASeR->GetChildren(k);
							DrawEachElement(pChildLASeR);
						}
						
					}			
				}
			}
			KillTimer(nIDEvent);
			break;

		}
	case LSR_conditional:
		{
			DrawConditional(pLASeR);
			m_apTimer[nIDEvent] = NULL;
			KillTimer(nIDEvent);
			break;
		}
		
	case LSR_externalReference:
		{
			CLASeR *pPLASeR = pLASeR->GetParent();

			// reset outside
			while (1)
			{
				if (pPLASeR->GetElementType()==SVG_svg)
				{
					lsrErr err = LASeRNoErr;

					float	fWidth	= pPLASeR->GetFloatAttr(ATTR_WIDTH, &err);
					ErrorMessage(err);
					float	fHeight	= pPLASeR->GetFloatAttr(ATTR_HEIGHT, &err);
					ErrorMessage(err);
					cairo_set_source_rgba(m_pCR, 0, 0, 0, 1);
					cairo_rectangle(m_pCR, 0, 0, fWidth, fHeight);
					cairo_fill_preserve(m_pCR);

					break;
				}
				else
				{
					pPLASeR = pPLASeR->GetParent();
				}
			}
			int childnum = pLASeR->GetChildrenSize();
			for (int i=0; i<childnum; i++)
			{
				CLASeR *pCLASeR = pLASeR->GetChildren(i);
				m_bcontinouwtref = 0;
				DrawEachElement(pCLASeR);
				Invalidate(FALSE);
			}			
			m_apTimer[nIDEvent] = NULL;
			KillTimer(nIDEvent);
			break;
		}
		
	case LSR_externalUpdate:
		{
			tstring strHREF = pLASeR->GetStrAttr(ATTR_XLINK_HREF, &err);
			tstring strFileName = strHREF;
			tstring strPath = strHREF;
			tstring strDownFileName;

			strFileName.erase(strHREF.find('#'));
			strPath.erase(0, strHREF.find('#') + 5);

			strDownFileName = strFileName;
			strDownFileName.erase(0, strDownFileName.rfind('/')+1);
			strDownFileName = _T("C:\\") + strDownFileName;

			/*HRESULT hr = */
			URLDownloadToFile (NULL,strFileName.c_str(), strDownFileName.c_str(), 0, NULL);

			ifstream	file(strDownFileName.c_str(), ios::in|ios::binary);

			UINT		uFileSize		= 0;
			char		*szFileData		= NULL;
			wchar_t		*wszFileData	= NULL;

			// move to file's end
			file.seekg(-1, ios_base::end);

			// get file size
			// 2 : for null
			uFileSize = (UINT)(file.tellg()) + 2;

			szFileData = new char[uFileSize];


			memset(szFileData, 0, uFileSize);

			// move to file's start point to read correctly
			file.seekg(0, ios_base::beg);
			file.read(szFileData, uFileSize);
			file.close();

			/************************************************************************/	
			UINT8 bUnicode = 0;

			if (((szFileData[0] == 0xFF) && (szFileData[1] == 0xFE))
				|| ((szFileData[0] == 0xFE) && (szFileData[1] == 0xFF)))
				bUnicode = 1;

			if (bUnicode)
			{
				/* skip */
			}

			else
			{
				// char -> wchar_t
				wszFileData = new wchar_t[uFileSize];

				memset(wszFileData, 0, sizeof(wszFileData));

				MultiByteToWideChar(CP_ACP, 0, szFileData, uFileSize, wszFileData, uFileSize);
			}

			LPXNode xml;

			xml = new XNode;

			xml->Load(wszFileData);

			LPXNode found = NULL;

			tstring strelementname = strPath;

			strelementname.erase(0, strPath.find(':')+1);

			int a = 1;
			UINT childcount = 0;
			while (a)
			{
				if (strelementname.find(':')>100)
				{
					if (strelementname.find('/') < 100)
					{
						strelementname.erase(strelementname.find('/'));
					}	
					else if (strelementname.find('[') < 100)
					{
						strelementname.erase(strelementname.find('['));
					}
					a = 0;
				}
				else
				{
					strelementname.erase(0, strelementname.find(':')+1);
					childcount++;
				}
			}

			found = xml->Find(strelementname.c_str());

			tstring strAttributeName	= pLASeR->GetStrAttr(ATTR_ATTRIBUTENAME, &err);
			ErrorMessage(err);

			tstring strValue			= pLASeR->GetStrAttr(ATTR_VALUE, &err);
			ErrorMessage(err);

			UINT    uType				= pLASeR->GetUINT8Attr(ATTR_LSR_TYPE, &err);
			ErrorMessage(err);

			if (strAttributeName == _T("textContent"))
			{
				tstring temp = found->value;

				switch (uType)
				{
				case 0:
					temp = strValue;
					break;
				case 1:
					temp += strValue;
					break;
				case 2:
					temp = _T("");
					break;

				}
				found->value = temp;
			}

			while(found->parent)
			{
				found = found->parent;
			}

			tstring strOutput = found->GetXML();

			//wchar_t -> char
			char		*szwriteFileData		= NULL;

			szwriteFileData = new char[strOutput.size()];

			memset(szwriteFileData, 0, sizeof(szwriteFileData));

			WideCharToMultiByte(CP_ACP, 0, strOutput.c_str(), strOutput.size(), szwriteFileData, strOutput.size(), NULL, NULL);


			if(strHREF.find(':') == 1)
			{
				ofstream fileout(strFileName.c_str());
				fileout.write(szwriteFileData, strOutput.size());
				fileout.close();
			}
			else
			{
				ofstream fileout(strDownFileName.c_str());
				fileout.write(szwriteFileData, strOutput.size());
				fileout.close();

				CInternetSession m_Session		= NULL;
				CFtpConnection *m_pConnection	= NULL;
				CFtpFileFind *m_pFileFind		= NULL;
				CString m_strRemoteDirText		= NULL;

				if (!AfxSocketInit())
				{
					AfxMessageBox(_T(" ʱȭ "));
				}

				m_pConnection = m_Session.GetFtpConnection(_T("intmedia.khu.ac.kr"));

				m_pConnection->GetCurrentDirectory(m_strRemoteDirText);

				m_pConnection->GetCurrentDirectory(m_strRemoteDirText);

				BOOL bContinue = TRUE;

				strFileName.erase(0, strFileName.find('/') + 1);
				strFileName.erase(0, strFileName.find('/') + 1);
				strFileName.erase(0, strFileName.find('/'));

				tstring strdirpathname = strFileName.erase(0, strFileName.find('/')+1);			
				tstring strdirname = strdirpathname;

				if(m_pFileFind) 
					delete m_pFileFind;

				// FTP  ϸ 
				m_pFileFind = new CFtpFileFind(m_pConnection);

				m_pFileFind->FindFile(strFileName.c_str());

				while(bContinue)
				{
					// FTP  ϸ  
					bContinue = m_pFileFind->FindNextFile();
					if (m_pFileFind->IsDirectory())
					{
					}
					else
					{
						CString str = m_pFileFind->GetFileName();
						m_pConnection->Remove(str);
					}
				}

				m_pConnection->PutFile(strDownFileName.c_str(),strFileName.c_str());
			}

			m_apTimer[nIDEvent] = NULL;
			KillTimer(nIDEvent);

			break;
		}	

	case EV_listener:
		{
			UINT8 kindofevent = pLASeR->GetUINT8Attr(ATTR_EVENT, &err);

			if (kindofevent == 37)
			{
				tstring strObserver = pLASeR->GetStrAttr(ATTR_OBSERVER, &err);
				ErrorMessage(err);

				strObserver.erase(0, strObserver.find('#')+1);

				CLASeR *pObserver = SearchLASeRElementUsingID(strObserver);
				ErrorMessage(err);

				tstring strHREF = pObserver->GetStrAttr(ATTR_XLINK_HREF, &err);
				ErrorMessage(err);

				// distribute local or remote
				tstring strLocalorRemote = strHREF;
				tstring strFileName = strHREF;
				strLocalorRemote.erase(4);
				if (strLocalorRemote == _T("http"))
				{        //remote
					tstring strDownFileName = strHREF;
					tstring strHREFaddress = strHREF;
					tstring strHREFfolder  = strHREF;

					strDownFileName.erase(strDownFileName.find('#'));

					strHREFaddress.erase(0, strHREFaddress.find('/') + 2);
					strHREFaddress.erase(strHREFaddress.find('/'));

					strHREFfolder.erase(strHREFfolder.find('#'));
					strHREFfolder.erase(0, strHREFfolder.find('/') + 1);
					strHREFfolder.erase(0, strHREFfolder.find('/') + 1);
					strHREFfolder.erase(0, strHREFfolder.find('/') + 1);

					CInternetSession m_Session		= NULL;
					CFtpConnection *m_pConnection	= NULL;
					CFtpFileFind *m_pFileFind		= NULL;
					CString m_strRemoteDirText		= NULL;

					if (!AfxSocketInit())
					{
						AfxMessageBox(_T(" ʱȭ "));
					}

					m_pConnection = m_Session.GetFtpConnection(strHREFaddress.c_str());

					if (strHREFfolder.find('/') > 0 && strHREFfolder.find('/') <100)
					{
						tstring strfolder = strHREFfolder;
						strfolder.erase(strfolder.rfind('/'));
						strHREFfolder.erase(0, strHREFfolder.rfind('/')+1);
						m_pConnection->SetCurrentDirectory(strfolder.c_str());
					}
					m_pConnection->GetCurrentDirectory(m_strRemoteDirText);

					if(m_pFileFind) 
						delete m_pFileFind;

					// FTP  ϸ 
					m_pFileFind = new CFtpFileFind(m_pConnection);

					m_pFileFind->FindFile(strHREFfolder.c_str());
					m_pFileFind->FindNextFile();

					CTime ct;

					m_pFileFind->GetLastWriteTime(ct);

					if (m_pConnection)
					{
						m_pConnection->Close();
						delete m_pConnection;
						m_pConnection = NULL;
					}
					if (m_pFileFind)
					{
						delete m_pFileFind;
					}

					if (m_Time != ct)
					{
						tstring strhandler = pLASeR->GetStrAttr(ATTR_HANDLER, &err);
						ErrorMessage(err);
						strhandler.erase(0, strhandler.find('#') + 1);
						CLASeR *phandler = SearchLASeRElementUsingID(strhandler);
						DrawEachElement(phandler);
						DrawScene();
						KillTimer(nIDEvent);
					}
				}

				else
				{        //local
					tstring strLocalPath = strHREF;
					strLocalPath.erase(strLocalPath.find('#'));

					HANDLE hFile = CreateFile(strLocalPath.c_str(), GENERIC_READ, 0, 0, OPEN_ALWAYS,
						FILE_ATTRIBUTE_NORMAL, 0);

					FILETIME ftLastWriteTime;
					SYSTEMTIME stLastWriteTimeUTC, stLastWriteTimeLocal;

					GetFileTime(hFile, NULL, NULL, &ftLastWriteTime);
					FileTimeToSystemTime(&ftLastWriteTime, &stLastWriteTimeUTC);
					SystemTimeToTzSpecificLocalTime(NULL, &stLastWriteTimeUTC, &stLastWriteTimeLocal);
					CloseHandle(hFile);

					if (m_uslastmilliseconds != stLastWriteTimeLocal.wMilliseconds)
					{
						tstring strhandler = pLASeR->GetStrAttr(ATTR_HANDLER, &err);
						ErrorMessage(err);
						strhandler.erase(0, strhandler.find('#') + 1);
						CLASeR *phandler = SearchLASeRElementUsingID(strhandler);
						DrawEachElement(phandler);
						DrawScene();
						KillTimer(nIDEvent);

					}
				}
//
//				tstring strFileName = strHREF;
//				tstring strPath = strHREF;
//
//				strFileName.erase(strHREF.find('#'));
//				strPath.erase(0, strHREF.find('#') + 5);
//
////				HRESULT hr = URLDownloadToFile (NULL,strFileName.c_str(), _T("c:\\testxmlupdates.xml"), 0, NULL);
//				URLDownloadToFile (NULL,strFileName.c_str(), _T("c:\\testxmlupdates.xml"), 0, NULL);
//
//				ifstream	file("c:\\testxmlupdates.xml", ios::in|ios::binary);
//
//				UINT		uFileSize		= 0;
//				char		*szFileData		= NULL;
//				wchar_t		*wszFileData	= NULL;
//
//				// move to file's end
//				file.seekg(-1, ios_base::end);
//
//				// get file size
//				// 2 : for null
//				uFileSize = (UINT)(file.tellg()) + 2;
//
//				szFileData = new char[uFileSize];
//
//				memset(szFileData, 0, uFileSize);
//
//				// move to file's start point to read correctly
//				file.seekg(0, ios_base::beg);
//				file.read(szFileData, uFileSize);
//				file.close();
//
//				/************************************************************************/
//				UINT8 bUnicode = 0;
//
//				if (((szFileData[0] == 0xFF) && (szFileData[1] == 0xFE))
//					|| ((szFileData[0] == 0xFE) && (szFileData[1] == 0xFF)))
//					bUnicode = 1;
//
//				if (bUnicode)
//				{
//					/* skip */
//				}
//
//				else
//				{
//					// char -> wchar_t
//					wszFileData = new wchar_t[uFileSize];
//
//					memset(wszFileData, 0, sizeof(wszFileData));
//
//					MultiByteToWideChar(CP_ACP, 0, szFileData, uFileSize, wszFileData, uFileSize);
//				}
//
//				XNode xml;
//				xml.Load(wszFileData);
//
//				delete [] szFileData;
//				szFileData = NULL;
//
//				delete [] wszFileData;
//				wszFileData = NULL;
//
//				LPXNode found = NULL;
//
//				tstring strelementname = strPath;
//
//				strelementname.erase(0, strPath.find(':')+1);
//				strelementname.erase(strelementname.find('/'));
//
//				found = xml.Find(strelementname.c_str());
//
//				if (found->value != strxmlupdatevalue)
//				{
//					tstring strhandler = pLASeR->GetStrAttr(ATTR_HANDLER, &err);
//					ErrorMessage(err);
//					strhandler.erase(0, strhandler.find('#')+1);
////					CLASeR *phandler = SearchLASeRElementUsingID(strhandler);
//					SearchLASeRElementUsingID(strhandler);
//					DrawScene();
//					strxmlupdatevalue = found->value;
//					KillTimer(nIDEvent);
//				}
			}
		}
	}

	CWnd::OnTimer(nIDEvent);
}

void CChildView::OnLButtonDown(UINT nFlags, CPoint point)
{
	// TODO: ⿡ ޽ ó ڵ带 ߰ /Ǵ ⺻ ȣմϴ.
	m_uEventType = 3;                           // mousedown

	if (m_apListener.size() > 0)
		TraceEventA(point.x, point.y);

	SetCapture();

	CWnd::OnLButtonDown(nFlags, point);
}

void CChildView::OnLButtonUp(UINT nFlags, CPoint point)
{
	// TODO: ⿡ ޽ ó ڵ带 ߰ /Ǵ ⺻ ȣմϴ.
	lsrErr err = LASeRNoErr;

	if (GetCapture() == this)
	{
		ReleaseCapture();

		CRect wrt;

		GetWindowRect(&wrt);
		ClientToScreen(&point);

		if (wrt.PtInRect(point))
		{
			m_uEventType = 5;					// click
			m_strEventType = _T("click");

			if (m_apA.size() > 0)
				TraceEventA(point.x, point.y);

			if (m_apListener.size() > 0)
			{
				vector<CLASeR *>::iterator i;

				for (i = m_apListener.begin(); i < m_apListener.end(); ++i)
				{
					UINT uEvent_type = (*i)->GetUINT8Attr(ATTR_EVENT, &err);

					if (m_uEventType == uEvent_type)
						TraceEventListener(point.x, point.y);
				}
			}

			if (m_apSet.size() > 0)
			{
				vector<CLASeR *>::iterator i;

				for (i = m_apSet.begin(); i < m_apSet.end(); ++i)
				{
					vector<tstring>	astrBegin = (*i)->GetStrListAttr(ATTR_BEGIN, &err);

					astrBegin[0].erase(0, astrBegin[0].find('.') + 1);

					if (m_strEventType == astrBegin[0])
						TraceEventSet(point.x, point.y);
				}
			}

			if (m_apanimate.size() > 0)
			{
				vector<CLASeR *>::iterator i;

				for (i = m_apanimate.begin(); i < m_apanimate.end(); ++i)
				{
					vector<tstring>	astrBegin = (*i)->GetStrListAttr(ATTR_BEGIN, &err);

					astrBegin[0].erase(0, astrBegin[0].find('.') + 1);

					if (m_strEventType == astrBegin[0])
						TraceEventanmate(point.x, point.y, *i);
				}

			}
		}

		else
		{
			m_uEventType = 4;					// mouseup

			if (m_apListener.size() > 0)
			{
				vector<CLASeR *>::iterator i;

				for (i = m_apListener.begin(); i < m_apListener.end(); ++i)
				{
					UINT uEvent_type = (*i)->GetUINT8Attr(ATTR_EVENT, &err);

					if (m_uEventType == uEvent_type)
						TraceEventListener(point.x, point.y);
				}
			}				
		}
	}

	CWnd::OnLButtonUp(nFlags, point);
}

void CChildView::OnMouseMove(UINT nFlags, CPoint point)
{
	// TODO: ⿡ ޽ ó ڵ带 ߰ /Ǵ ⺻ ȣմϴ.
	m_uEventType = 21;                          // mouseover

	if (m_apA.size() > 0)
		TraceEventA(point.x, point.y);

	CWnd::OnMouseMove(nFlags, point);
}

void CChildView::PhasedSceneAdaptation(CLASeR *pLASeR)
{
	lsrErr err = LASeRNoErr;

	UINT uComplexity = 0;

	multimap<UINT, CLASeR *>::iterator iterElement;

	if ((pLASeR->GetParentElementType() >= SVG_a) && (pLASeR->GetParentElementType() <= LSR_LASeRHeader))
	{
		for (iterElement = m_mapElements.begin(); iterElement != m_mapElements.end(); ++iterElement)
		{
			if (iterElement->second == pLASeR->GetParent())
			{
				uComplexity = iterElement->first;
				break;
			}
		}

		uComplexity += pLASeR->GetUINT8Attr(ATTR_LSR_RENDERINGCOMPLEXITY, &err);
	}

	else
		uComplexity	= pLASeR->GetUINT8Attr(ATTR_LSR_RENDERINGCOMPLEXITY, &err);

	m_mapElements.insert(make_pair(uComplexity, pLASeR));
}

UINT8 CChildView::ConfirmEventRegion(CLASeR *pLASeR, const LONG lPointX, const LONG lPointY)
{
	lsrErr err = LASeRNoErr;

	UINT8 bResult = 0;

	CRect rct;
	GetWindowRect(&rct);

	double	x	= (double)lPointX - rct.left;
	double	y	= (double)lPointY - rct.top;

	ReverseTransform(pLASeR->GetParent(), &x, &y);

	switch (pLASeR->GetElementType())
	{
	case SVG_circle:
		{
			int	nX	= (int)(pLASeR->GetFloatAttr(ATTR_CX, &err) - x);
			ErrorMessage(err);
			int	nY	= (int)(pLASeR->GetFloatAttr(ATTR_CY, &err) - y);
			ErrorMessage(err);
			int	nR	= (int)pLASeR->GetFloatAttr(ATTR_R, &err);
			ErrorMessage(err);

			if ((nR * nR) > ((nX * nX) + (nY * nY)))
				bResult = 1;
		}
		break;

	case SVG_image:
	case SVG_rect:
	case SVG_video:
	case SVG_textArea:
		{
			UINT	uX1	= (UINT)pLASeR->GetFloatAttr(ATTR_X, &err);
			ErrorMessage(err);
			UINT	uY1	= (UINT)pLASeR->GetFloatAttr(ATTR_Y, &err);
			ErrorMessage(err);
			UINT	uX2	= uX1 + (UINT)pLASeR->GetFloatAttr(ATTR_WIDTH, &err);
			ErrorMessage(err);
			UINT	uY2	= uY1 + (UINT)pLASeR->GetFloatAttr(ATTR_HEIGHT, &err);
			ErrorMessage(err);

			if ((x > uX1) && (x < uX2) && (y > uY1) && (y < uY2))
				bResult = 1;
		}
		break;

	default:
		/* nothing */
		break;
	}

	return bResult;
}
void CChildView::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags)
{
	// TODO: ⿡ ޽ ó ڵ带 ߰ /Ǵ ⺻ ȣմϴ.
	m_uEventType = 18;                          // mouseover

	if (m_apListener.size() > 0)
	{
		lsrErr err = LASeRNoErr;

		vector<CLASeR *>::iterator i;

		for (i = m_apListener.begin(); i < m_apListener.end(); ++i)
		{
			UINT uEvent_type = (*i)->GetUINT8Attr(ATTR_EVENT, &err);
			if (m_uEventType == uEvent_type)
			{
				tstring strEventparmeter = (*i)->GetStrAttr(ATTR_EVENT_PARAM, &err);
				char cstrEventparmeter = strEventparmeter[0];

				if (cstrEventparmeter == nChar && m_uEventType == uEvent_type)
				{
					tstring strHREF = (*i)->GetStrAttr(ATTR_HANDLER, &err);

					if (strHREF.find('#') == 0) //inline tref
					{
						strHREF.erase(remove(strHREF.begin(), strHREF.end(), '#'), strHREF.end());
						CLASeR *pElement = SearchLASeRElementUsingID(strHREF);
						PrepareExternalUpdate(pElement);
					}
				}
			}
		}
	}

	CWnd::OnKeyDown(nChar, nRepCnt, nFlags);
}