/*
// This software was originally developed by ETRI and Kyunghee University, in the course
// of development of the ISO/IEC 14496-20 (Lightweight Application Scene Representation 
// -LASeR- and Simple Aggregation Format -SAF-) AMD3: Presentation and Modification of
// Structured Information standard for reference purposes and its performance may not 
// have been optimized.
//
// This software module is an implementation of one or more tools as specified by the
// ISO/IEC 14496-20 AMD3: PMSI standard.
//
// ISO/IEC gives users free license to this software module or modifications thereof
// for use in products claiming conformance to audiovisual and image-coding related
// ITU Recommendations and/or ISO/IEC International Standards.
// 
// ISO/IEC gives users the same free license to this software module or modifications
// thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in products are advised that its use may
// infringe existing patents. 
// 
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
*/

// ChildView.h : interface of the CChildView class
//


#pragma once

#include "libMP4.h"

#include "..\src\SceneTreeParser\SceneTreeManager.h"

#include "cairo-win32.h"

#include "GrpBtn.h"
#include "ScrollTool.h"

#include <map>


#define WM_PLAY			WM_USER + 1
#define WM_PAUSE		WM_USER + 2
#define WM_STOP			WM_USER + 3
#define WM_READY		WM_USER + 4
#define WM_RESIZEVIEW	WM_USER + 5
#define WM_BUFFER		WM_USER + 6

#define PLAYBAR_WIDTH		25
#define PLAYBAR_HEIGHT		25
#define SCROLLBAR_HEIGHT	25

#define PI 3.14159265358979323846

typedef char lsrErr;


// CChildView window

class CChildView : public CWnd
{
// Construction
public:
	CChildView();
	virtual ~CChildView();

// Attributes
public:
	CGrpBtn			m_btnPlay;					// play button
	CScrollTool		m_wndScrollBar;				// scroll Bar

	deque< deque<BYTE *> >	m_aapCB;			// composition memory

protected:
	HDC				m_hMemDC;
	HBITMAP			m_hBit;						// rendered scene
	HBITMAP			m_hOldBit;

	BITMAPINFO		m_bitmapPic;				// decoded picture

	CWinThread		*m_pRendererThread;			// renderer thread

	vector<UINT8>	m_abCBStatus;				// memory status 1 : temporary empty, 2 : normal 4 : buffer end 8 : buffer end & empty

	UINT			m_uVideoTrackIndex;			// one video track or first track among the video tracks
	UINT			m_uAudioTrackIndex;

	/************************************************************************/
	cairo_surface_t	*m_pSurface;
	cairo_t			*m_pCR;

	UINT			m_uViewWidth;				// svg or video size
	UINT			m_uViewHeight;

	UINT8			m_uWidthUnit;
	UINT8			m_uHeightUnit;

	float			m_fWidthRatio;  
    float			m_fHeightRatio;

	// LASeRHeader
	UINT8			m_uAdaptationType;

	// a
	vector<CLASeR *> m_apA;						// 'a' elements
	UINT8			m_uEventType;

	// conditional
	vector<CLASeR *> m_apTimer;					// elements of calling timer
	UINT			m_uTimerIndex;

	// g
	UINT8			m_uTransformed;

	// image
	vector<string>	m_astrImage;				// images for scene

	// listener
	vector<CLASeR *> m_apListener;				// listener elements
	tstring         strxmlupdatevalue;
	unsigned short  m_uslastmilliseconds;        // lastwriteTimelocal
	CTime			m_Time;

	// animate
	vector<CLASeR *> m_apanimate;

	// set
	vector<CLASeR *> m_apSet;
	tstring m_strEventType;

	// phased scene adaptation
	multimap<UINT, CLASeR *> m_mapElements;		// rendering complexity is used

	UINT            m_bcontinouwimage;
	UINT            m_bcontinouwtref;
	UINT            m_bcontinouwvideo;
	UINT            m_bcontinouw;


// Operations
public:
	void			ClearPlayerStatus();

	CString			GetPathName() const;

	CLibMP4Api		*GetMP4Handle() const;
	UINT8			GetLASeRFlag() const;
	UINT8			GetRendererStatus() const			{return (m_pRendererThread != NULL) ? 1 : 0;}

	UINT8			GetPlayerStatus() const;
	void			SetPlayerStatus(const UINT8 bStatus) const;

	// 8 : BUFFER_END
	UINT8			GetCBStatus(const UINT uIndex)		{return (m_abCBStatus.size() > uIndex) ? m_abCBStatus[uIndex] : 8;}
	void			SetCBStatus(const UINT8 bStatus);
	void			SetCBStatus(const UINT uIndex, const UINT8 bStatus);

	UINT			GetVideoTrackIndex() const			{return m_uVideoTrackIndex;}

	UINT			GetViewWidth() const				{return m_uViewWidth;}
	UINT			GetViewHeight() const				{return m_uViewHeight;}

	UINT8			GetPictureDelta() const;

	UINT			GetVideoWidth() const;
	UINT			GetVideoHeight() const;

	/************************************************************************/
	void			PrepareScene();				// prepare the scene
	void			DrawScene();

	// prepare av
	void			PrepareAudio(CLASeR *pLASeR = NULL);
	void			PrepareVideo(CLASeR *pLASeR = NULL);
	// draw av
	void			DrawVideo(CLASeR *pLASeR = NULL);

protected:
	lsrErr			PrepareEachElement(CLASeR *pLASeR);

	lsrErr			DrawEachElement(CLASeR *pLASeR);

	lsrErr          DeleteEachElement(CLASeR *pLASeR);

	// prepare each element
	void			PrepareA(CLASeR *pLASeR);
	void			PrepareAnimate(CLASeR *pLASeR);
	void			PrepareAnimateTransform(CLASeR *pLASeR);
	void			PrepareConditional(CLASeR *pLASeR);
	void			PrepareG(CLASeR *pLASeR);
	void			PrepareListener(CLASeR *pLASeR);
	void			PrepareSet(CLASeR *pLASeR);
	void			PrepareSVG(CLASeR *pLASeR);

	void			PrepareLASeRHeader(CLASeR *pLASeR);

	void            PrepareExternalReference(CLASeR *pLASeR);
	void            PrepareExternalUpdate(CLASeR *pLASeR);

	// draw each element
	void			DrawA(CLASeR *pLASeR);
	void			DrawAnimate(CLASeR *pLASeR);
	void			DrawAnimateTransform(CLASeR *pLASeR);
	void			DrawCircle(CLASeR *pLASeR);
	void			DrawConditional(CLASeR *pLASeR);
	void			DrawEllipse(CLASeR *pLASeR);
	void			DrawG(CLASeR *pLASeR);
	void			DrawImage(CLASeR *pLASeR);
	void			DrawLine(CLASeR *pLAeR);
	void			DrawParsingSwitch(CLASeR *pLASeR);
	void			DrawPolygon(CLASeR *pLASeR);
	void			DrawPolyline(CLASeR *pLASeR);
	void			DrawRect(CLASeR *pLASeR);
	void			DrawSet(CLASeR *pLASeR);
	void			DrawSVG(CLASeR *pLASeR);
	void			DrawText(CLASeR *pLASeR);
	void			DrawTextArea(CLASeR *pLASeR);


	void			DeleteImage(CLASeR *pLASeR);
	void            DeleteTextArea(CLASeR *pLASeR);
	// get media source
	tstring			GetSourceUsingHref(CLASeR *pLASeR);
	string			GetSource(CLASeR *pLASeR);

	// default transform func
	void			Transform(CLASeR *pLASeR);

	// detect event region
	UINT8			ConfirmEventRegion(CLASeR *pLASeR, const LONG lPointX, const LONG lPointY);

	// a
	void			TraceEventA(const LONG x, const LONG y);

	// g
	void			ReverseTransform(CLASeR *pLASeR, double *pX, double *pY);
	void			RestoreSurface(CLASeR *pLASeR);

	// listener
	void			TraceEventListener(const LONG x, const LONG y);

	// listener
	void			TraceEventSet(const LONG x, const LONG y);

	// animate
	void            TraceEventanmate(const LONG x, const LONG y, CLASeR *pLASeR);

	// adaptation funcs
	float			GetDisplaySize();
	UINT			GetHeightResolution();
	float			GetAvailableMemorySize();

	// PMSI funcs
//	tstring			GetTextContent(CLASeR *pLASeR);
	tstring			GettrefContent(CLASeR *pLASeR);

	// comparison func
	void			CompareFunc(CLASeR *pLASeR);

	void			PhasedSceneAdaptation(CLASeR *pLASeR);

// Overrides
protected:
	virtual BOOL PreCreateWindow(CREATESTRUCT& cs);

// Generated message map functions
protected:
	afx_msg void OnPaint();
	DECLARE_MESSAGE_MAP()

public:
	afx_msg int OnCreate(LPCREATESTRUCT lpCreateStruct);
	afx_msg void OnClose();
	afx_msg LRESULT OnPlay(WPARAM wParam, LPARAM lParam);
	afx_msg LRESULT OnPause(WPARAM wParam, LPARAM lParam);
	afx_msg LRESULT OnStop(WPARAM wParam, LPARAM lParam);
	afx_msg LRESULT OnBuffer(WPARAM wParam, LPARAM lParam);

	afx_msg void OnTimer(UINT_PTR nIDEvent);
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnLButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnMouseMove(UINT nFlags, CPoint point);

protected:
	virtual BOOL OnCommand(WPARAM wParam, LPARAM lParam);
public:
	afx_msg void OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags);
};


// additional funcs
void ErrorMessage(INT8 errCode);

LONG ConvertLPtoDP(float fPoint, UINT8 uUnits, int nDPI);