/*
// This software was originally developed by ETRI and Kyunghee University, in the course
// of development of the ISO/IEC 14496-20 (Lightweight Application Scene Representation 
// -LASeR- and Simple Aggregation Format -SAF-) AMD3: Presentation and Modification of
// Structured Information standard for reference purposes and its performance may not 
// have been optimized.
//
// This software module is an implementation of one or more tools as specified by the
// ISO/IEC 14496-20 AMD3: PMSI standard.
//
// ISO/IEC gives users free license to this software module or modifications thereof
// for use in products claiming conformance to audiovisual and image-coding related
// ITU Recommendations and/or ISO/IEC International Standards.
// 
// ISO/IEC gives users the same free license to this software module or modifications
// thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in products are advised that its use may
// infringe existing patents. 
// 
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
*/
#include "stdafx.h"

#ifdef _WIN32_WCE
#include "..\M3RMobile\MainFrm.h"
#elif _ACTIVEX
#include "..\M3RacX\M3RacXDlg.h"
#else
#include "MainFrm.h"
#endif

#include <algorithm>
#include <functional>

#include "libDeSAF.h"

CCriticalSection g_cs;


INT8 CMainFrame::GatherLocalDB(const UINT64 uIndex /* = 0xFFFFFFFFFFFFFFFF*/)
{
	UINT uSize = 0;

	static UINT				uCurTrack = 0;
	static vector<UINT>		auSampleCount;		// sample count
	static vector<UINT>		auCurChunkIndex;	// current chunk index
	static vector<UINT>		auSamplesInCurChunk;// samples in current chunk
	static vector<UINT>		auCurSampleIndex;	// current sample index
	static vector<UINT>		auCurSampleSize;	// current sample size
	static vector<BYTE *>	auCurSampleOffset;	// current sample offset

	/************************************************************************/
	if (m_bFirstTime)
	{
		uCurTrack = 0;
		auSampleCount.clear();
		auCurChunkIndex.clear();
		auSamplesInCurChunk.clear();
		auCurSampleIndex.clear();
		auCurSampleSize.clear();
		auCurSampleOffset.clear();
		m_afPassed.clear();

		for (UINT i = 0; i < m_uTrackCount; ++i)
		{
			auSampleCount.push_back(m_pMP4->GetSampleCount(i));
			auSamplesInCurChunk.push_back(m_pMP4->GetSamplesPerChunk(i, 0));
			auCurSampleSize.push_back(m_pMP4->GetEntrySize(i, 0));
			auCurSampleOffset.push_back(m_pFileStartPoint + m_pMP4->GetChunkOffset(i, 0));
		}

		auCurChunkIndex.assign(m_uTrackCount, 1);
		auCurSampleIndex.assign(m_uTrackCount, 0);
		m_afPassed.assign(m_uTrackCount, 0.0);

		m_bFirstTime = 0;
	}

	if (!MatchDBStatus(BUFFER_STREAM_END))
	{
		/************************************************************************/
		// normal case
		if (uIndex == 0xFFFFFFFFFFFFFFFF)
		{
			uCurTrack = (UINT)distance(m_afPassed.begin(), min_element(m_afPassed.begin(), m_afPassed.end()));

			g_cs.Lock();						// lock critical section

			// target sample offset
			m_aapDB[uCurTrack].push_back(auCurSampleOffset[uCurTrack]);

			// get the payload size
			uSize = auCurSampleSize[uCurTrack];
			m_aauUnitSizes[uCurTrack].push_back(uSize);

			// set buffer normal flag
			m_abDBStatus[uCurTrack] = BUFFER_NORMAL;

			g_cs.Unlock();						// unlock critical section

			// sample index is increased
			++auCurSampleIndex[uCurTrack];

			if (auCurSampleIndex[uCurTrack] < auSampleCount[uCurTrack])
			{
				// sample offset is increased
				auCurSampleOffset[uCurTrack] += uSize;

				// new sample size
				auCurSampleSize[uCurTrack] = m_pMP4->GetEntrySize(uCurTrack, auCurSampleIndex[uCurTrack]);

				// samples in current chunk is decreased
				--auSamplesInCurChunk[uCurTrack];

				// if new chunk is needed
				if (auSamplesInCurChunk[uCurTrack] <= 0)
				{
					// chunk index is increased
					++auCurChunkIndex[uCurTrack];

					// new sample to chunk index
					UINT uSample2ChunkIndex = m_pMP4->GetIndexUsingRealChunk(uCurTrack, auCurChunkIndex[uCurTrack]);

					// new samples in new chunk
					auSamplesInCurChunk[uCurTrack] = m_pMP4->GetSamplesPerChunk(uCurTrack, uSample2ChunkIndex);

					// sample offset is adjusted
					auCurSampleOffset[uCurTrack] = m_pFileStartPoint
						+ m_pMP4->GetChunkOffset(uCurTrack, (auCurChunkIndex[uCurTrack] - 1));
				}
			}

			// passed gauge
			m_afPassed[uCurTrack] = ((float)auCurSampleIndex[uCurTrack] / (float)auSampleCount[uCurTrack]);

			/************************************************************************/
			// check state of stream
			if (m_afPassed[uCurTrack] == 1.0)
			{
				g_cs.Lock();					// lock critical section
				m_abDBStatus[uCurTrack] = BUFFER_STREAM_END;
				g_cs.Unlock();					// unlock critical section
			}
		}

		else									// temporal random access
		{
		}

		g_cs.Lock();							// lock critical section

		if (m_aapDB[uCurTrack].size() != m_aauUnitSizes[uCurTrack].size())
		{
			g_cs.Unlock();						// unlock critical section

			return PlayerBufferErr;
		}

		g_cs.Unlock();							// unlock critical section
	}

	return 0;
}

INT8 CMainFrame::GatherNetworkDB(const UINT64 uIndex /* = 0xFFFFFFFFFFFFFFFF*/)
{
	INT8 err = 0;

	static CLibDeSAFApi	*pDeSAF = new CLibDeSAFApi();

	static vector<UINT>	auTrackIDs;				// track IDs
	static vector<UINT>	auSampleCount;
	static vector<UINT>	auPrevCTS;
	static vector<UINT>	auK;
	static vector<UINT>	auTimeStampRes;

	static UINT			uPrevAUType = 0;

	BYTE	*pTemp	= NULL;
	BYTE	*pData	= NULL;
	UINT	uSize	= 0;
	UINT	uTrack	= 0;

// 	if (m_pController->BufferMonitoring() == 1)
// 	{
// 		pTemp = (BYTE *)m_pController->GetSAFPacket((ULONG *)&uSize);
// 
// 		if (!pTemp)
// 			return 0;
// 
// 		/************************************************************************/
// 		if (m_bFirstTime)
// 		{
// 			ParseMP4(pTemp + 4, uSize - 4);		// parse SII info, 4 : first packet size
// 
// 			auTrackIDs.clear();
// 			auPrevCTS.clear();
// 			auK.clear();
// 			auTimeStampRes.clear();
// 
// 			m_bFirstTime = 0;
// 
// 			return 0;
// 		}
// 
// 		err = pDeSAF->DePackSAF(pTemp);
// 
// 		/************************************************************************/
// 		// 1st, get the access unit type
// 		if ((pDeSAF->GetAUType() == 1) || (pDeSAF->GetAUType() == 2))
// 		{
// 			// get track & track index
// 			if (find(auTrackIDs.begin(), auTrackIDs.end(), pDeSAF->GetStreamID()) == auTrackIDs.end())
// 				auTrackIDs.push_back(pDeSAF->GetStreamID());
// 
// 			uTrack = (UINT)distance(auTrackIDs.begin(), find(auTrackIDs.begin(), auTrackIDs.end(), pDeSAF->GetStreamID()));
// 
// 			// initialize decoder
// 			BYTE *pPayload = pDeSAF->GetPayload();
// 
// 			auSampleCount.push_back(m_pMP4->GetSampleCount(m_pMP4->TrackID2Index(pDeSAF->GetStreamID())));
// 			auPrevCTS.push_back(0);
// 			auK.push_back(0);
// 			auTimeStampRes.push_back(pDeSAF->GetTimeStampResolution(pPayload));
// 			m_afPassed.push_back(0.0);
// 
// 			deque<UINT> a, b;
// 
// 			m_aauRAPoints.push_back(a);
// 			m_aauCompositeTime.push_back(b);
// 
// 			err = InitDecoders(uTrack, pDeSAF->GetOTI(pPayload), pDeSAF->GetStreamType(pPayload), pDeSAF->GetDecSpecificInfo(pPayload));
// 			ErrorMessage(err);
// 		}
// 
// 		else if (pDeSAF->GetAUType() == 3)
// 		{
// 			uTrack = (UINT)distance(auTrackIDs.begin(), find(auTrackIDs.begin(), auTrackIDs.end(), pDeSAF->GetStreamID()));
// 
// 			g_cs.Lock();						// lock critical section
// 			m_abDBStatus[uTrack] = BUFFER_STREAM_END;
// 			g_cs.Unlock();						// unlock critical section
// 		}
// 
// 		else if (pDeSAF->GetAUType() == 4)
// 		{
// 			uTrack = (UINT)distance(auTrackIDs.begin(), find(auTrackIDs.begin(), auTrackIDs.end(), pDeSAF->GetStreamID()));
// 
// 			// copy to permanent memory
// 			uSize = pDeSAF->GetAULength() - 2;
// 			pData = new BYTE[uSize];
// 			memcpy(pData, pDeSAF->GetPayload(), uSize);
// 
// 			g_cs.Lock();						// lock critical section
// 
// 			// push data to decoding buffer
// 			m_aapDB[uTrack].push_back(pData);
// 			m_aauUnitSizes[uTrack].push_back(uSize);
// 			// set buffer normal flag
// 			m_abDBStatus[uTrack] = BUFFER_NORMAL;
// 
// 			g_cs.Unlock();						// unlock critical section
// 
// 			/************************************************************************/
// 			// 2nd, save auxiliary data
// 			// random access point
// 			if (pDeSAF->GetRandomAccessPointFlag())
// 			{
// 				g_cs.Lock();					// lock critical section
// 				m_aauRAPoints[uTrack].push_back((UINT)m_aapDB[uTrack].size());
// 				g_cs.Unlock();					// unlock critical section
// 			}
// 
// 			// composition time
// 			if (pDeSAF->GetCTS() < auPrevCTS[uTrack])
// 				++auK[uTrack];
// 
// 			auPrevCTS[uTrack] = pDeSAF->GetCTS();
// 
// 			// in ms
// 			UINT uCTS = (UINT)((auPrevCTS[uTrack] + (auK[uTrack] * 0x40000000)) * 1000 / auTimeStampRes[uTrack]);
// 
// 			g_cs.Lock();						// lock critical section
// 			m_aauCompositeTime[uTrack].push_back(uCTS);
// 
// 			// passed gauge
// 			m_afPassed[uTrack] = ((float)m_aapDB[uTrack].size() / (float)auSampleCount[uTrack]);
// 			g_cs.Unlock();						// unlock critical section
// 		}
// 
// 		else if (pDeSAF->GetAUType() == 5)
// 		{
// 			g_cs.Lock();						// lock critical section
// 			SetDBStatus(BUFFER_STREAM_END);
// 			g_cs.Unlock();						// unlock critical section
// 		}
// 
// 		/************************************************************************/
// 		// when some packet is lost
// 		if (err == 1)
// 		{
// 			// skip some SAF packet
// 		}
// 
// 		// if player is not playing now, start playing
// 		if ((m_bPlayerStatus != PLAYER_PLAYING) && (uPrevAUType == 4) && (pDeSAF->GetAUType() == 4))
// 			// previous & current au type is 4 - all headers is file was already transmitted
// 		{
// 			if (m_uAudioTrackIndex != 0xFFFFFFFF)
// 			{
// 				// to set aac & wave parameters, get the first unit
// 				if ((m_abDBStatus[m_uAudioTrackIndex] == BUFFER_NORMAL) && (m_aapDB[m_uAudioTrackIndex].size() > 0))
// 					InitAACDecoder();				// 2nd initialization
// 			}
// 
// 			if (m_uLASeRTrackIndex != 0xFFFFFFFF)
// 			{
// 				// already transmitted
// 				if ((m_abDBStatus[m_uLASeRTrackIndex] == BUFFER_STREAM_END) && (m_aapDB[m_uLASeRTrackIndex].size() > 0))
// 				{
// 					g_cs.Lock();					// lock critical section
// 
// 					BYTE	*pUnit	= m_aapDB[m_uLASeRTrackIndex][0];
// 					UINT	uSize	= m_aauUnitSizes[m_uLASeRTrackIndex][0];
// 
// 					g_cs.Unlock();					// unlock critical section
// 
// 					ParseLASeR(pUnit, uSize);
// 				}
// 			}
// 
// 			PostMessage(WM_PLAY);
// 			m_wndView.PostMessage(WM_PLAY);
// 		}
// 
// 		uPrevAUType = pDeSAF->GetAUType();
// 	}
// 
// 	// stream end
// 	else if (m_pController->BufferMonitoring() == 0)
// 	{
// 		g_cs.Lock();							// lock critical section
// 		SetDBStatus(BUFFER_STREAM_END);
// 		g_cs.Unlock();							// unlock critical section
// 	}
// 	
// 	g_cs.Lock();								// lock critical section

	if ((m_aapDB.size() != 0) && (m_aauUnitSizes.size() != 0) && (m_aapDB[uTrack].size() != m_aauUnitSizes[uTrack].size()))
	{
		g_cs.Unlock();							// unlock critical section

		return PlayerBufferErr;
	}

	g_cs.Unlock();								// unlock critical section

	return 0;
}

float CMainFrame::GetBufferState()
{
	return (m_afPassed.size() > 0) ? *min_element(m_afPassed.begin(), m_afPassed.end()) : (float)0.0;
}

UINT ThreadDBGatherer(LPVOID pParam)
{
	INT8 err = 0;

	CMainFrame *pFrame = (CMainFrame *)pParam;

	if (pFrame == NULL)
		return 0;

	if (pFrame->GetMode() == 0)
	{
		// the player isn't stop & stream isn't end
		while ((pFrame->GetPlayerStatus() != PLAYER_STOP)
			&& !(pFrame->MatchDBStatus(BUFFER_STREAM_END)))
		{
			err = pFrame->GatherLocalDB();
			Sleep(0);

			if (err != 0)
			{
				g_cs.Lock();					// lock critical section
				pFrame->SetDBStatus(BUFFER_STREAM_END);
				g_cs.Unlock();					// unlock critical section

				ErrorMessage(err);
				break;
			}
		}
	}

	else if (pFrame->GetMode() == 1)
	{
		// the player isn't stop & stream isn't end
		while (1)
		{
			if (pFrame->GetBufferState() >= 1.0)
				break;

			err = pFrame->GatherNetworkDB();
			Sleep(0);

			if (err != 0)
			{
				g_cs.Lock();					// lock critical section
				pFrame->SetDBStatus(BUFFER_STREAM_END);
				g_cs.Unlock();					// unlock critical section

				ErrorMessage(err);
				break;
			}

			g_cs.Lock();					// lock critical section

			if (pFrame->MatchDBStatus(BUFFER_STREAM_END))
			{
				g_cs.Unlock();				// unlock critical section
				break;
			}

			g_cs.Unlock();					// unlock critical section
		} 
	}

	return 0;
}