/*
// This software was originally developed by ETRI and Kyunghee University, in the course
// of development of the ISO/IEC 14496-20 (Lightweight Application Scene Representation 
// -LASeR- and Simple Aggregation Format -SAF-) AMD3: Presentation and Modification of
// Structured Information standard for reference purposes and its performance may not 
// have been optimized.
//
// This software module is an implementation of one or more tools as specified by the
// ISO/IEC 14496-20 AMD3: PMSI standard.
//
// ISO/IEC gives users free license to this software module or modifications thereof
// for use in products claiming conformance to audiovisual and image-coding related
// ITU Recommendations and/or ISO/IEC International Standards.
// 
// ISO/IEC gives users the same free license to this software module or modifications
// thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in products are advised that its use may
// infringe existing patents. 
// 
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
*/
// GrpBtn3.cpp : implementation file
//

#include "stdafx.h"
#include "GrpBtn.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CGrpBtn

CGrpBtn::CGrpBtn()
{
	State = GB_PAUSE;
}

CGrpBtn::~CGrpBtn()
{
	DeleteAllBitmap();
}


BEGIN_MESSAGE_MAP(CGrpBtn, CWnd)
	//{{AFX_MSG_MAP(CGrpBtn)
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEMOVE()
	ON_WM_TIMER()
	ON_WM_PAINT()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CGrpBtn message handlers

void CGrpBtn::SetImage(WORD Play, WORD Pause, WORD PlayOver, WORD PauseOver)
{
	DeleteAllBitmap();
	Bitmap[0].LoadBitmap(Play);
	Bitmap[1].LoadBitmap(Pause);
	Bitmap[2].LoadBitmap(PlayOver);
	Bitmap[3].LoadBitmap(PauseOver);
	Invalidate(FALSE);
}

void CGrpBtn::DeleteAllBitmap()
{
	int i;

	for (i=0;i<4;i++) {
		if (Bitmap[i].m_hObject) {
			Bitmap[i].DeleteObject();
		}
	}
}

void CGrpBtn::OnLButtonDown(UINT nFlags, CPoint point) 
{
	SetCapture();
	
	CWnd::OnLButtonDown(nFlags, point);
}

void CGrpBtn::OnLButtonUp(UINT nFlags, CPoint point) 
{
	if (GetCapture() == this)
	{
		ReleaseCapture();

		if ((State == GB_PLAY) || (State == GB_PLAY_OVER))
			ChangeState(GB_PAUSE_OVER);

		else if ((State == GB_PAUSE) || (State == GB_PAUSE_OVER))
			ChangeState(GB_PLAY_OVER);
		
		if (IsPtOnMe(point))
			GetParent()->SendMessage(WM_COMMAND, MAKEWPARAM(GetWindowLong(m_hWnd, GWL_ID), GBN_CLICKED), (LPARAM)m_hWnd);
	}
	
	CWnd::OnLButtonUp(nFlags, point);
}

void CGrpBtn::OnMouseMove(UINT nFlags, CPoint point) 
{
	HWND hParent;

	for (hParent = m_hWnd; ::GetParent(hParent); hParent = ::GetParent(hParent)) {;}

	if (::GetForegroundWindow() != hParent) return;

	SetTimer(0, 50, NULL);

	if (State == GB_PLAY)
		ChangeState(GB_PLAY_OVER);

	else if (State == GB_PAUSE)
		ChangeState(GB_PAUSE_OVER);

	CWnd::OnMouseMove(nFlags, point);
}

void CGrpBtn::OnTimer(UINT nIDEvent) 
{
	POINT pt;
	CRect crt;

	GetCursorPos(&pt);
	GetWindowRect(&crt);

	if (crt.PtInRect(pt) == FALSE)
	{
		KillTimer(0);

		if (State == GB_PLAY_OVER)
			ChangeState(GB_PLAY);

		else if (State == GB_PAUSE_OVER)
			ChangeState(GB_PAUSE);
	}

	CWnd::OnTimer(nIDEvent);
}

void CGrpBtn::OnPaint() 
{
	CPaintDC dc(this); // device context for painting
	
	DrawBitmap(&dc,0,0,&Bitmap[State]);
}

void CGrpBtn::DrawBitmap(CDC *pDC, int x, int y, CBitmap *pBit)
{
	BITMAP bitinfo;
	CDC MemDC;
	MemDC.CreateCompatibleDC(pDC);
	CBitmap *pOldBitmap;
	pOldBitmap=MemDC.SelectObject(pBit);
	pBit->GetBitmap(&bitinfo);
	pDC->BitBlt(x,y,bitinfo.bmWidth,bitinfo.bmHeight,&MemDC,0,0,SRCCOPY);
	MemDC.SelectObject(pOldBitmap);
}

BOOL CGrpBtn::IsPtOnMe(CPoint point)
{
	CRect wrt;

	GetWindowRect(&wrt);
	ClientToScreen(&point);

	return wrt.PtInRect(point);
}

void CGrpBtn::ChangeState(eState State)
{
	if (this->State == State) return;

	this->State = State;
	Invalidate(FALSE);
}

eState CGrpBtn::GetState()
{
	return State;
}

void CGrpBtn::SetState(eState State)
{
	if ((GetWindowLong(m_hWnd,GWL_STYLE) & GBS_CHECK)!=0) {
		ChangeState(State);
	}
}