/*
// This software was originally developed by ETRI and Kyunghee University, in the course
// of development of the ISO/IEC 14496-20 (Lightweight Application Scene Representation 
// -LASeR- and Simple Aggregation Format -SAF-) AMD3: Presentation and Modification of
// Structured Information standard for reference purposes and its performance may not 
// have been optimized.
//
// This software module is an implementation of one or more tools as specified by the
// ISO/IEC 14496-20 AMD3: PMSI standard.
//
// ISO/IEC gives users free license to this software module or modifications thereof
// for use in products claiming conformance to audiovisual and image-coding related
// ITU Recommendations and/or ISO/IEC International Standards.
// 
// ISO/IEC gives users the same free license to this software module or modifications
// thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in products are advised that its use may
// infringe existing patents. 
// 
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
*/
#include "stdafx.h"

#ifdef _WIN32_WCE
#include "..\..\M3RMobile\ChildView.h"
#elif _ACTIVEX
#include "..\..\M3RacX\M3RacXView.h"
#else
#include "..\ChildView.h"
#endif

#include <numeric>
#include <algorithm>


void CChildView::DrawA(CLASeR *pLASeR)
{
	lsrErr err = LASeRNoErr;

	/************************************************************************/
	// get attributes
	UINT8	uDisplay	= pLASeR->GetUINT8Attr(ATTR_DISPLAY, &err);
	ErrorMessage(err);
	UINT8	uVisibility	= pLASeR->GetUINT8Attr(ATTR_VISIBILITY, &err);
	ErrorMessage(err);
	/************************************************************************/

	if ((uDisplay != 1) && (uVisibility != 1) && (pLASeR->GetActivation() != 1))
	{
		Transform(pLASeR);

		if (find(m_apA.begin(), m_apA.end(), pLASeR) == m_apA.end())
			m_apA.push_back(pLASeR);

		UINT8 uNumOfChildren = pLASeR->GetChildrenSize();

		for (UINT8 i = 0; i < uNumOfChildren; ++i)
			pLASeR->GetChildren(i)->SetActivate();
	}

	// only if display equal 1
	else if (uDisplay == 1)
	{
		pLASeR->SetDeactivate();

		m_apA.erase(remove(m_apA.begin(), m_apA.end(), pLASeR), m_apA.end());

		UINT8 uNumOfChildren = pLASeR->GetChildrenSize();

		for (UINT8 i = 0; i < uNumOfChildren; ++i)
			pLASeR->GetChildren(i)->SetDeactivate();
	}
}

void CChildView::PrepareA(CLASeR *pLASeR)
{
	CompareFunc(pLASeR);
}

/************************************************************************/
// TraceA
// Desc		: confirm whether 'a' element is selected or not
// Param	: x, y
// History
//	PSH (2009. 8. 13)	- first release
/************************************************************************/
void CChildView::TraceEventA(const LONG x, const LONG y)
{
	lsrErr err = LASeRNoErr;

	vector<CLASeR *>::iterator i;

	UINT8 j, uSelected, uNumOfChildren;
	j = uSelected = uNumOfChildren = 0;

	for (i = m_apA.begin(); i < m_apA.end(); ++i)
	{
		uNumOfChildren = (*i)->GetChildrenSize();

		for (j = 0; j < uNumOfChildren; ++j)
		{
			uSelected = ConfirmEventRegion((*i)->GetChildren(j), x, y);

			if (uSelected) break;
		}

		if (uSelected) break;
	}

	if (uSelected)
	{
		// click
		if (m_uEventType == 5)
		{
			tstring strXlink = ((*i)->GetStrAttr(ATTR_XLINK_HREF, &err));
			ErrorMessage(err);

#ifdef _WIN32_WCE
			SHELLEXECUTEINFO shExIf = {0};

			shExIf.cbSize = sizeof(SHELLEXECUTEINFO);
			shExIf.hwnd = NULL;
			shExIf.lpFile = strXlink.c_str();
			shExIf.nShow = SW_SHOW;
			shExIf.fMask = SEE_MASK_NOCLOSEPROCESS;

			ShellExecuteEx( &shExIf );
#else
			ShellExecute(NULL, NULL, strXlink.c_str(), NULL, _T(""), SW_SHOW);
#endif // _WIN32_WCE
		}

		// mouseover
		else if (m_uEventType == 21)
			SetCursor(LoadCursor(0, IDC_HAND));
	}
}