/*
// This software was originally developed by ETRI and Kyunghee University, in the course
// of development of the ISO/IEC 14496-20 (Lightweight Application Scene Representation 
// -LASeR- and Simple Aggregation Format -SAF-) AMD3: Presentation and Modification of
// Structured Information standard for reference purposes and its performance may not 
// have been optimized.
//
// This software module is an implementation of one or more tools as specified by the
// ISO/IEC 14496-20 AMD3: PMSI standard.
//
// ISO/IEC gives users free license to this software module or modifications thereof
// for use in products claiming conformance to audiovisual and image-coding related
// ITU Recommendations and/or ISO/IEC International Standards.
// 
// ISO/IEC gives users the same free license to this software module or modifications
// thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in products are advised that its use may
// infringe existing patents. 
// 
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
*/
#include "stdafx.h"

#ifdef _WIN32_WCE
#include "..\..\M3RMobile\ChildView.h"
#elif _ACTIVEX
#include "..\..\M3RacX\M3RacXView.h"
#else
#include "..\ChildView.h"
#endif

#include <numeric>
#include <cmath>


void CChildView::DrawG(CLASeR *pLASeR)
{
	lsrErr err = LASeRNoErr;

	/************************************************************************/
	// get attributes
	UINT8	uDisplay		= pLASeR->GetUINT8Attr(ATTR_DISPLAY, &err);
	ErrorMessage(err);
	UINT8	uVisibility		= pLASeR->GetUINT8Attr(ATTR_VISIBILITY, &err);
	ErrorMessage(err);
	/************************************************************************/

	if ((uDisplay != 1) && (uVisibility != 1) && (pLASeR->GetActivation() != 1))
	{
		Transform(pLASeR);

		err = LASeRNoErr;

		UINT8 uNumOfChildren = pLASeR->GetChildrenSize();

		for (UINT8 i = 0; i < uNumOfChildren; ++i)
			pLASeR->GetChildren(i)->SetActivate();
	}

	// only if display equal 1
	else if (uDisplay == 1)
	{
		pLASeR->SetDeactivate();

		UINT8 uNumOfChildren = pLASeR->GetChildrenSize();

		for (UINT8 i = 0; i < uNumOfChildren; ++i)
			pLASeR->GetChildren(i)->SetDeactivate();
	}
}

void CChildView::PrepareG(CLASeR *pLASeR)
{
	CompareFunc(pLASeR);
}

/************************************************************************/
// Transform
// Desc		: transform action
// Param	: element
// History
//	PSH (2009. 8. 13)	- first release
/************************************************************************/
void CChildView::Transform(CLASeR *pLASeR)
{
	lsrErr err = LASeRNoErr;

	vector<UINT8> auTransOrder = pLASeR->GetUINT8ListAttr(ATTR_TRANSFORM_ORDER, &err);
	ErrorMessage(err);

	if (auTransOrder.size() > 0)
	{
		cairo_save(m_pCR);

		/************************************************************************/
		// get attributes
		vector<float>	afRotation		= pLASeR->GetFloatListAttr(ATTR_LSR_ROTATION, &err);
		ErrorMessage(err);
		vector<float>	afScale			= pLASeR->GetFloatListAttr(ATTR_LSR_SCALE, &err);
		ErrorMessage(err);
		vector<float>	afTranslation	= pLASeR->GetFloatListAttr(ATTR_LSR_TRANSLATION, &err);
		ErrorMessage(err);
		vector<float>	afMatrix		= pLASeR->GetFloatListAttr(ATTR_LSR_MATRIX, &err);
		ErrorMessage(err);
		/************************************************************************/

		UINT uRotationCount, uScaleCount, uTranslationCount, uMatrixCount;
		uRotationCount = uScaleCount = uTranslationCount = uMatrixCount = 0;

		// transform according to order
		for (vector<UINT8>::iterator i = auTransOrder.begin(); i < auTransOrder.end(); ++i)
		{
			switch (*i)
			{
			case 1:								// rotate
				if ((afRotation[uRotationCount + 1] != 0.0) || (afRotation[uRotationCount + 2] != 0.0))
				{
					cairo_translate(m_pCR, afRotation[uRotationCount + 1], afRotation[uRotationCount + 2]);
					cairo_rotate(m_pCR, afRotation[uRotationCount]/* * PI / 180.0*/);
					cairo_translate(m_pCR, -afRotation[uRotationCount + 1], -afRotation[uRotationCount + 2]);
				}

				else
					cairo_rotate(m_pCR, afRotation[uRotationCount]/* * PI / 180.0*/);

				uRotationCount += 3;
				break;

			case 2:								// scale
				cairo_scale(m_pCR, afScale[uScaleCount], afScale[uScaleCount + 1]);
				uScaleCount += 2;
				break;

			case 3:								// transition
				cairo_translate(m_pCR, afTranslation[uTranslationCount], afTranslation[uTranslationCount + 1]);
				uTranslationCount += 2;
				break;

			case 4:								// matrix
				{
					cairo_matrix_t matrix;

					cairo_matrix_init(&matrix,
						afMatrix[uMatrixCount],
						afMatrix[uMatrixCount + 1],
						afMatrix[uMatrixCount + 2],
						afMatrix[uMatrixCount + 3],
						afMatrix[uMatrixCount + 4],
						afMatrix[uMatrixCount + 5]);

					cairo_transform(m_pCR, &matrix);
				}

				uMatrixCount += 6;
				break;

			default:
				/* must not be entered */
				break;
			}
		}

		// save children size if this is container element
		switch (pLASeR->GetElementType())
		{
		case SVG_g:
			m_uTransformed = pLASeR->GetChildrenSize();

		default:
			/* nothing */
			break;
		}
	}
}

/************************************************************************/
// ReverseTransform
// Desc		: when event is happen, reverse cursor point
// Param	: element, x, y pointer
// History
//	PSH (2009. 8. 13)	- first release
/************************************************************************/
void CChildView::ReverseTransform(CLASeR *pLASeR, double *pX, double *pY)
{
	lsrErr err = LASeRNoErr;

	if (pLASeR->GetElementType() == SVG_g)
	{
	vector<UINT8> auTransOrder = pLASeR->GetUINT8ListAttr(ATTR_TRANSFORM_ORDER, &err);
	ErrorMessage(err);

	if (auTransOrder.size() > 0)
	{
		/************************************************************************/
		// get attributes
		vector<float>	afRotation		= pLASeR->GetFloatListAttr(ATTR_LSR_ROTATION, &err);
		ErrorMessage(err);
		vector<float>	afScale			= pLASeR->GetFloatListAttr(ATTR_LSR_SCALE, &err);
		ErrorMessage(err);
		vector<float>	afTranslation	= pLASeR->GetFloatListAttr(ATTR_LSR_TRANSLATION, &err);
		ErrorMessage(err);
		vector<float>	afMatrix		= pLASeR->GetFloatListAttr(ATTR_LSR_MATRIX, &err);
		ErrorMessage(err);
		/************************************************************************/

		UINT uRotationCount, uScaleCount, uTranslationCount, uMatrixCount;
		uRotationCount = uScaleCount = uTranslationCount = uMatrixCount = 0;

		cairo_matrix_t result, a;
		cairo_matrix_init_identity(&result);

		// transform according to order
		for (vector<UINT8>::iterator i = auTransOrder.begin(); i < auTransOrder.end(); ++i)
		{
			switch (*i)
			{
			case 1:								// rotate
				if ((afRotation[uRotationCount + 1] != 0.0) || (afRotation[uRotationCount + 2] != 0.0))
				{
					cairo_matrix_init_translate(&a, afRotation[uRotationCount + 1], afRotation[uRotationCount + 2]);
					cairo_matrix_multiply(&result, &a, &result);

					cairo_matrix_init_rotate(&a, afRotation[uRotationCount] * PI / 180.0);
					cairo_matrix_multiply(&result, &a, &result);

					cairo_matrix_init(&a, 1.0, 0.0, 0.0, 1.0, -afRotation[uRotationCount + 1], -afRotation[uRotationCount + 2]);
				}

				else
					cairo_matrix_init_rotate(&a, afRotation[uRotationCount] * PI / 180.0);

				uRotationCount += 3;
				break;

			case 2:								// scale
				cairo_matrix_init_scale(&a, afScale[uScaleCount], afScale[uScaleCount + 1]);
				uScaleCount += 2;
				break;

			case 3:								// transition
				cairo_matrix_init_translate(&a, afTranslation[uTranslationCount], afTranslation[uTranslationCount + 1]);
				uTranslationCount += 2;
				break;

			case 4:								// matrix
				cairo_matrix_init(&a,
					afMatrix[uMatrixCount],
					afMatrix[uMatrixCount + 1],
					afMatrix[uMatrixCount + 2],
					afMatrix[uMatrixCount + 3],
					afMatrix[uMatrixCount + 4],
					afMatrix[uMatrixCount + 5]);

				uMatrixCount += 6;
				break;

			default:
				/* must not be entered */
				break;
			}

			cairo_matrix_multiply(&result, &a, &result);
		}

		err = (lsrErr)cairo_matrix_invert(&result);
		ErrorMessage(err);

		cairo_matrix_transform_point(&result, (double *)pX, (double *)pY);
	}
	}
}

/************************************************************************/
// RestoreSurface
// Desc		: restore surface after container
// Param	: element, x, y pointer
// History
//	PSH (2009. 8. 13)	- first release
/************************************************************************/
void CChildView::RestoreSurface(CLASeR *pLASeR)
{
	lsrErr err = LASeRNoErr;

	vector<UINT8> auTransOrder = pLASeR->GetUINT8ListAttr(ATTR_TRANSFORM_ORDER, &err);
	ErrorMessage(err);

	// restore context from current element
	if (auTransOrder.size() > 0)
		cairo_restore(m_pCR);

	// restore context from parent's container element
	switch (pLASeR->GetParent()->GetElementType())
	{
	case SVG_a:
	case SVG_g:
		if (m_uTransformed > 0)
		{
			--m_uTransformed;

			if (m_uTransformed == 0)
				cairo_restore(m_pCR);
		}

		break;

	default:
		/* nothing */
		break;
	}
}