/*
// This software was originally developed by ETRI and Kyunghee University, in the course
// of development of the ISO/IEC 14496-20 (Lightweight Application Scene Representation 
// -LASeR- and Simple Aggregation Format -SAF-) AMD3: Presentation and Modification of
// Structured Information standard for reference purposes and its performance may not 
// have been optimized.
//
// This software module is an implementation of one or more tools as specified by the
// ISO/IEC 14496-20 AMD3: PMSI standard.
//
// ISO/IEC gives users free license to this software module or modifications thereof
// for use in products claiming conformance to audiovisual and image-coding related
// ITU Recommendations and/or ISO/IEC International Standards.
// 
// ISO/IEC gives users the same free license to this software module or modifications
// thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in products are advised that its use may
// infringe existing patents. 
// 
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
*/
#include "stdafx.h"

#ifdef _WIN32_WCE
#include "..\..\M3RMobile\ChildView.h"
#elif _ACTIVEX
#include "..\..\M3RacX\M3RacXView.h"
#else
#include "..\ChildView.h"
#endif

#include <algorithm>
#include <fstream>
#include "XMLite.h"

void CChildView::DrawImage(CLASeR *pLASeR)
{
	lsrErr err = LASeRNoErr;

	/************************************************************************/
	// get attributes
	float	fWidth		= pLASeR->GetFloatAttr(ATTR_WIDTH, &err);
	ErrorMessage(err);
	float	fHeight		= pLASeR->GetFloatAttr(ATTR_HEIGHT, &err);
	ErrorMessage(err);
	float	fX			= pLASeR->GetFloatAttr(ATTR_X, &err);
	ErrorMessage(err);
	float	fY			= pLASeR->GetFloatAttr(ATTR_Y, &err);
	ErrorMessage(err);
	float	fOpacity	= pLASeR->GetFloatAttr(ATTR_OPACITY, &err);
	ErrorMessage(err);

	UINT8	uDisplay	= pLASeR->GetUINT8Attr(ATTR_DISPLAY, &err);
	ErrorMessage(err);
	UINT8	uVisibility	= pLASeR->GetUINT8Attr(ATTR_VISIBILITY, &err);
	ErrorMessage(err);
	/************************************************************************/

	if ((uDisplay != 1) && (uVisibility != 1) && (pLASeR->GetActivation() != 1))
	{
		Transform(pLASeR);

		cairo_surface_t *pImage;

		int	nImageWidth = 0;
		int	nImageHeight = 0;

		pImage = cairo_image_surface_create_from_png(GetSource(pLASeR).c_str());

		nImageWidth = cairo_image_surface_get_width(pImage);
		nImageHeight = cairo_image_surface_get_height(pImage);

		cairo_save(m_pCR);

		cairo_translate(m_pCR, fX, fY);
		cairo_scale(m_pCR, (float)(fWidth / nImageWidth), (float)(fHeight / nImageHeight));
		cairo_set_source_surface(m_pCR, pImage, 0, 0);
		cairo_paint_with_alpha(m_pCR, fOpacity);

		cairo_restore(m_pCR);

		cairo_surface_destroy(pImage);

		RestoreSurface(pLASeR);
	}
}


void CChildView::DeleteImage(CLASeR *pLASeR)
{
	lsrErr err = LASeRNoErr;

	/************************************************************************/
	// get attributes
	float	fWidth		= pLASeR->GetFloatAttr(ATTR_WIDTH, &err);
	ErrorMessage(err);
	float	fHeight		= pLASeR->GetFloatAttr(ATTR_HEIGHT, &err);
	ErrorMessage(err);
	float	fX			= pLASeR->GetFloatAttr(ATTR_X, &err);
	ErrorMessage(err);
	float	fY			= pLASeR->GetFloatAttr(ATTR_Y, &err);
	ErrorMessage(err);
	float	fOpacity	= pLASeR->GetFloatAttr(ATTR_OPACITY, &err);
	ErrorMessage(err);

	UINT8	uDisplay	= pLASeR->GetUINT8Attr(ATTR_DISPLAY, &err);
	ErrorMessage(err);
	UINT8	uVisibility	= pLASeR->GetUINT8Attr(ATTR_VISIBILITY, &err);
	ErrorMessage(err);
	/************************************************************************/

	if ((uDisplay != 1) && (uVisibility != 1) && (pLASeR->GetActivation() != 1))
	{
		Transform(pLASeR);

		double dRed, dGreen, dBlue;
		dRed = dGreen = dBlue = 255.0;

		cairo_set_source_rgba(m_pCR, dRed, dGreen, dBlue, fOpacity);
		cairo_rectangle(m_pCR, fX, fY, fWidth, fHeight);
		cairo_fill_preserve(m_pCR);

		RestoreSurface(pLASeR);
	}
}
/************************************************************************/
// GetSource
// Desc		: if path is relative path, convert to absolute path in char
// Param	: element, x, y pointer
// History
//	PSH (2009. 8. 13)	- first release
/************************************************************************/
string CChildView::GetSource(CLASeR *pLASeR)
{
	tstring	strTemp		= GetSourceUsingHref(pLASeR).c_str();
	UINT	uLength		= (UINT)strTemp.size();

	string	strSource	= "";
	char	*szSource	= new char[uLength + 1];
	memset(szSource, 0, uLength + 1);

	WideCharToMultiByte(CP_ACP, 0, strTemp.c_str(), uLength, szSource, uLength, NULL, NULL);
	szSource[uLength] = '\0';

	// if relative path is used
	if (find(strTemp.begin(), strTemp.end(), ':') == strTemp.end())
	{
		CString strPathW = GetPathName();

		CT2CA pszConvertedAnsiString(strPathW);

		string strPath(pszConvertedAnsiString);

		strSource = strPath + szSource;
	}

	// if absolute path is used
	else
		strSource = szSource;

	delete [] szSource;
	szSource = NULL;

	return strSource;
}

/************************************************************************/
// GetSourceUsingHref
// Desc		: restore surface after container
// Param	: element, x, y pointer
// History
//	PSH (2009. 8. 13)	- first release
//  PYC (2010. 3. 11)	- with saf:RemoteStreamHeader
/************************************************************************/
tstring CChildView::GetSourceUsingHref(CLASeR *pLASeR)
{
	lsrErr err = LASeRNoErr;

	CSAF	*pSAF	= SearchSAFElementUsingID(pLASeR->GetStrAttr(ATTR_XLINK_HREF, &err));
	ErrorMessage(err);

	if (pSAF->GetElementType() == SAF_RemoteStreamHeader)
	{
		CSAFUnit *pRSH = (CSAFUnit *)pSAF;

		tstring strSource = pRSH->GetStrAttr(ATTR_SOURCE, &err);

		tstring strFileName = strSource;
		tstring strPath = strSource;

		strFileName.erase(strFileName.find('#'));
		strPath.erase(0, strPath.find('#') + 5);

		tstring strDownFileName;
		strDownFileName = strFileName;
		strDownFileName.erase(0, strDownFileName.rfind('/')+1);
		strDownFileName = _T("C:\\") + strDownFileName;

		HRESULT hr = URLDownloadToFile (NULL,strFileName.c_str(), strDownFileName.c_str(), 0, NULL);

		
		ifstream	file(strDownFileName.c_str(), ios::in|ios::binary);

		UINT		uFileSize		= 0;
		char		*szFileData		= NULL;
		wchar_t		*wszFileData	= NULL;

		// move to file's end
		file.seekg(-1, ios_base::end);

		// get file size
		// 2 : for null
		uFileSize = (UINT)(file.tellg()) + 2;

		szFileData = new char[uFileSize];

		memset(szFileData, 0, uFileSize);

		// move to file's start point to read correctly
		file.seekg(0, ios_base::beg);
		file.read(szFileData, uFileSize);
		file.close();

		/************************************************************************/
		UINT8 bUnicode = 0;

		if (((szFileData[0] == 0xFF) && (szFileData[1] == 0xFE))
			|| ((szFileData[0] == 0xFE) && (szFileData[1] == 0xFF)))
			bUnicode = 1;

		if (bUnicode)
		{
			/* skip */
		}

		else
		{
			// char -> wchar_t
			wszFileData = new wchar_t[uFileSize];

			memset(wszFileData, 0, sizeof(wszFileData));

			MultiByteToWideChar(CP_ACP, 0, szFileData, uFileSize, wszFileData, uFileSize);
		}

		XNode xml;
		xml.Load(wszFileData);

		LPXNode found = NULL;

		tstring strelementname = strPath;

		strelementname.erase(0, strPath.find(':') + 1);
		
		while (1)
		{
			if (strelementname.find(':') > 100)
			{
				strelementname.erase(strelementname.find('/'));
				break;
			}

			else
				strelementname.erase(0, strelementname.find(':') + 1);
		}

		found = xml.Find(strelementname.c_str());

		tstring strSourceback = found->value;

		if (strSourceback.find('.')==0)
		{
			strSourceback.erase(0, strSourceback.find('.') + 1);

			if (strSourceback.find('.')==0)
			{
				strSourceback.erase(0, strSourceback.find('.') + 1);
				strFileName.erase(strFileName.rfind('/'));
			}
		}
		strFileName.erase(strFileName.rfind('/'));

		if (strFileName.find(':')==4)
		{
			strSource = strFileName + strSourceback;
		}
		else
		{
			strSource = strSourceback;
		}

		tstring strswitch = strSourceback;
		strswitch.erase(0, strswitch.rfind('.') + 1);

		tstring strdowncontents = strSource;
		strdowncontents.erase(0, strdowncontents.rfind('/')+1);

		strdowncontents = _T("c:\\") + strdowncontents;

		if (strswitch == _T("png"))
		{			
			HRESULT hrimg = URLDownloadToFile (NULL,strSource.c_str(), strdowncontents.c_str(), 0, NULL);
		}

		else if (strswitch == _T("mp4"))
		{
			HRESULT hrimg = URLDownloadToFile (NULL,strSource.c_str(), strdowncontents.c_str(), 0, NULL);
		}

		return strdowncontents;
	}

	else
	{
		CLASeR	*pTempLASeR	= NULL;

		err = GetNextObject(&pTempLASeR, &pSAF);
		ErrorMessage(err);

		CSAFUnit *pES = (CSAFUnit *)pSAF;

		return pES->GetStrAttr(ATTR_SOURCE, &err);
	}
}