/*
// This software was originally developed by ETRI and Kyunghee University, in the course
// of development of the ISO/IEC 14496-20 (Lightweight Application Scene Representation 
// -LASeR- and Simple Aggregation Format -SAF-) AMD3: Presentation and Modification of
// Structured Information standard for reference purposes and its performance may not 
// have been optimized.
//
// This software module is an implementation of one or more tools as specified by the
// ISO/IEC 14496-20 AMD3: PMSI standard.
//
// ISO/IEC gives users free license to this software module or modifications thereof
// for use in products claiming conformance to audiovisual and image-coding related
// ITU Recommendations and/or ISO/IEC International Standards.
// 
// ISO/IEC gives users the same free license to this software module or modifications
// thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in products are advised that its use may
// infringe existing patents. 
// 
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
*/
#include "stdafx.h"

#ifdef _WIN32_WCE
#include "..\..\M3RMobile\ChildView.h"
#elif _ACTIVEX
#include "..\..\M3RacX\M3RacXView.h"
#else
#include "..\ChildView.h"

#endif

#include <numeric>

/************************************************************************/
// DrawParsingSwitch
// Desc		: draw parsingSwitch element
// Param	: parsingSwitch element pointer
// History
//	PYC (2009. 9. 22)	- first release
/************************************************************************/
void CChildView::DrawParsingSwitch(CLASeR *pLASeR)
{
	lsrErr err = LASeRNoErr;

	/************************************************************************/
	// get attributes
	UINT8	uMode	= pLASeR->GetUINT8Attr(ATTR_LSR_MODE, &err);
	ErrorMessage(err);
	/************************************************************************/

	float	fCurMemory = GetAvailableMemorySize();

	UINT8	uNumOfChildren = pLASeR->GetChildrenSize();

	for (UINT8 i = 0; i < uNumOfChildren; ++i)
		pLASeR->GetChildren(i)->SetDeactivate();

	// descending
	if (uMode == 0)
	{
		for (UINT8 j = 0; j < uNumOfChildren; ++j)
		{
			vector<float>	afMinMemory = pLASeR->GetChildren(j)->GetFloatListAttr(ATTR_LSR_MINMEMORY, &err);
			ErrorMessage(err);

			float	fMinMemory = accumulate(afMinMemory.begin(), afMinMemory.end(), (float)0.0);

			if (fCurMemory >= fMinMemory)
			{
				pLASeR->GetChildren(j)->SetActivate();
				break;
			}
		}
	}

	// ascending
	else if (uMode == 1)
	{
		for (UINT8 j = uNumOfChildren - 1; j > 0; --j)
		{
			vector<float>	afMinMemory = pLASeR->GetChildren(j)->GetFloatListAttr(ATTR_LSR_MINMEMORY, &err);
			ErrorMessage(err);

			float	fMinMemory = accumulate(afMinMemory.begin(), afMinMemory.end(), (float)0.0);

			if (fCurMemory >= fMinMemory)
			{
				pLASeR->GetChildren(j)->SetActivate();
				break;
			}
		}
	}

	// incremental
	else if (uMode == 2)
	{
		float	fSumMemory = 0;

		for (UINT8 j = 0; j < uNumOfChildren; ++j)
		{
			vector<float>	afMinMemory = pLASeR->GetChildren(j)->GetFloatListAttr(ATTR_LSR_MINMEMORY, &err);
			ErrorMessage(err);

			float	fMinMemory = accumulate(afMinMemory.begin(), afMinMemory.end(), (float)0.0);

			fSumMemory = fSumMemory + fMinMemory;

			if (fCurMemory >= fSumMemory)
			{
				pLASeR->GetChildren(j)->SetActivate();
			}

			else
				break;
		}
	}
}