/*
// This software was originally developed by ETRI and Kyunghee University, in the course
// of development of the ISO/IEC 14496-20 (Lightweight Application Scene Representation 
// -LASeR- and Simple Aggregation Format -SAF-) AMD3: Presentation and Modification of
// Structured Information standard for reference purposes and its performance may not 
// have been optimized.
//
// This software module is an implementation of one or more tools as specified by the
// ISO/IEC 14496-20 AMD3: PMSI standard.
//
// ISO/IEC gives users free license to this software module or modifications thereof
// for use in products claiming conformance to audiovisual and image-coding related
// ITU Recommendations and/or ISO/IEC International Standards.
// 
// ISO/IEC gives users the same free license to this software module or modifications
// thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in products are advised that its use may
// infringe existing patents. 
// 
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
*/
#include "stdafx.h"

#ifdef _WIN32_WCE
#include "..\..\M3RMobile\ChildView.h"
#elif _ACTIVEX
#include "..\..\M3RacX\M3RacXView.h"
#else
#include "..\ChildView.h"
#endif


void CChildView::DrawPolygon(CLASeR *pLASeR)
{
	lsrErr err = LASeRNoErr;

	/************************************************************************/
	// get attributes
	vector<float>	afPoints		= pLASeR->GetFloatListAttr(ATTR_POINTS, &err);
	ErrorMessage(err);

	UINT8			uDisplay		= pLASeR->GetUINT8Attr(ATTR_DISPLAY, &err);
	ErrorMessage(err);
	vector<UINT8>	auFill			= pLASeR->GetUINT8ListAttr(ATTR_FILL, &err);
	ErrorMessage(err);
	float			fFillOpacity	= pLASeR->GetFloatAttr(ATTR_FILL_OPACITY, &err);
	ErrorMessage(err);
	vector<UINT8>	auStroke		= pLASeR->GetUINT8ListAttr(ATTR_STROKE, &err);
	ErrorMessage(err);
	float			fStrokeOpacity	= pLASeR->GetFloatAttr(ATTR_STROKE_OPACITY, &err);
	ErrorMessage(err);
	float			fStrokeWidth	= pLASeR->GetFloatAttr(ATTR_STROKE_WIDTH, &err);
	ErrorMessage(err);
	UINT8			uVisibility		= pLASeR->GetUINT8Attr(ATTR_VISIBILITY, &err);
	ErrorMessage(err);
	/************************************************************************/

	if ((uDisplay != 1) && (uVisibility != 1) && (pLASeR->GetActivation() != 1))
	{
		Transform(pLASeR);

		double dRed, dGreen, dBlue;
		dRed = dGreen = dBlue = 0.0;

		if (auStroke.size() == 3)
		{
			dRed	= auStroke[0] / 255.0;
			dGreen	= auStroke[1] / 255.0;
			dBlue	= auStroke[2] / 255.0;
		}

		cairo_set_source_rgba(m_pCR, dRed, dGreen, dBlue, fStrokeOpacity);
		cairo_set_line_width(m_pCR, fStrokeWidth);

		for (UINT i = 1; i < afPoints.size(); i += 2) 
		{
			cairo_line_to(m_pCR, afPoints[i - 1], afPoints[i]);
		}

		cairo_close_path(m_pCR);
		cairo_stroke_preserve(m_pCR);

		dRed = dGreen = dBlue = 0.0;

		if (auFill.size() == 3)
		{
			dRed	= auFill[0] / 255.0;
			dGreen	= auFill[1] / 255.0;
			dBlue	= auFill[2] / 255.0;
		}

		cairo_set_source_rgba(m_pCR, dRed, dGreen, dBlue, fFillOpacity);
		cairo_fill(m_pCR);

		RestoreSurface(pLASeR);;
	}
}