/*
// This software was originally developed by ETRI and Kyunghee University, in the course
// of development of the ISO/IEC 14496-20 (Lightweight Application Scene Representation 
// -LASeR- and Simple Aggregation Format -SAF-) AMD3: Presentation and Modification of
// Structured Information standard for reference purposes and its performance may not 
// have been optimized.
//
// This software module is an implementation of one or more tools as specified by the
// ISO/IEC 14496-20 AMD3: PMSI standard.
//
// ISO/IEC gives users free license to this software module or modifications thereof
// for use in products claiming conformance to audiovisual and image-coding related
// ITU Recommendations and/or ISO/IEC International Standards.
// 
// ISO/IEC gives users the same free license to this software module or modifications
// thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in products are advised that its use may
// infringe existing patents. 
// 
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
*/
#include "stdafx.h"

#ifdef _WIN32_WCE
#include "..\..\M3RMobile\ChildView.h"
#include "..\..\M3RMobile\MainFrm.h"
#elif _ACTIVEX
#include "..\..\M3RacX\M3RacXView.h"
#else
#include "..\ChildView.h"
#include "..\MainFrm.h"

#endif

#include <numeric>

/************************************************************************/
// DrawSVG
// Desc		: draw svg element
// Param	: svg element pointer
// History
//	PSH (2009. 6. 30)	- first release
/************************************************************************/
void CChildView::DrawSVG(CLASeR *pLASeR)
{
	lsrErr err = LASeRNoErr;

	/************************************************************************/
	// get attributes
	vector<float>	afViewBox		= pLASeR->GetFloatListAttr(ATTR_VIEWBOX, &err);
	ErrorMessage(err);

	UINT8			uDisplay		= pLASeR->GetUINT8Attr(ATTR_DISPLAY, &err);
	ErrorMessage(err);
	UINT8			uVisibility		= pLASeR->GetUINT8Attr(ATTR_VISIBILITY, &err);
	ErrorMessage(err);

	vector<UINT8>	auViewportFill	= pLASeR->GetUINT8ListAttr(ATTR_VIEWPORT_FILL, &err);
	ErrorMessage(err);
	/************************************************************************/

	if ((uDisplay != 1) && (uVisibility != 1) && (pLASeR->GetActivation() != 1))
	{
		m_uWidthUnit	= pLASeR->GetUINT8Attr(ATTR_WIDTH_UNIT, &err);
		ErrorMessage(err);
		m_uHeightUnit	= pLASeR->GetUINT8Attr(ATTR_HEIGHT_UNIT, &err);
		ErrorMessage(err);

		float	fWidth	= pLASeR->GetFloatAttr(ATTR_WIDTH, &err);
		ErrorMessage(err);
		float	fHeight	= pLASeR->GetFloatAttr(ATTR_HEIGHT, &err);
		ErrorMessage(err);

		// get the client dpi
		CClientDC dc(this);

		int		nDPIX	= GetDeviceCaps(dc, LOGPIXELSX);
		int		nDPIY	= GetDeviceCaps(dc, LOGPIXELSY);

		LONG	nWidth	= ConvertLPtoDP(fWidth, m_uWidthUnit, nDPIX);
		LONG	nHeight	= ConvertLPtoDP(fHeight, m_uHeightUnit, nDPIY);

		if (afViewBox.size() == 4)
		{
			m_fWidthRatio	= (float)(nWidth / (afViewBox[2] - afViewBox[0]));
			m_fHeightRatio	= (float)(nHeight / (afViewBox[3] - afViewBox[1]));
		}

		// when svg size is changed
		if ((m_uViewWidth != (UINT)nWidth) || (m_uViewHeight != (UINT)nHeight))
		{
			m_uViewWidth	= nWidth;
			m_uViewHeight	= nHeight;

			// clear old surface & cairo resource
			if (m_pSurface)
			{
				cairo_surface_destroy(m_pSurface);
				m_pSurface = NULL;
			}

			if (m_pCR)
			{
				cairo_destroy(m_pCR);
				m_pCR = NULL;
			}

#ifndef _ACTIVEX
			CMainFrame *pMainFrame = (CMainFrame*)AfxGetMainWnd();
			pMainFrame->PostMessage(WM_RESIZEVIEW);
#endif // _ACTIVEX

			/************************************************************************/
			// check mem dc
			if (m_hMemDC)
			{
				DeleteObject(m_hMemDC);
				m_hMemDC = NULL;
			}

			CClientDC dc(this);

			// create mem dc
			m_hMemDC	= CreateCompatibleDC(dc.m_hDC);
			m_hBit		= CreateCompatibleBitmap(dc.m_hDC, m_uViewWidth, m_uViewHeight);
			m_hOldBit	= (HBITMAP)SelectObject(m_hMemDC, m_hBit);

			m_pSurface	= cairo_win32_surface_create(m_hMemDC);
			m_pCR		= cairo_create(m_pSurface);
		}

		if (auViewportFill.size() == 3)	cairo_set_source_rgb(m_pCR, auViewportFill[0], auViewportFill[1], auViewportFill[2]);
		else							cairo_set_source_rgb(m_pCR, 0.0, 0.0, 0.0);

		cairo_paint(m_pCR);
	}

	// only if display equal 1
	else if (uDisplay == 1)
	{
		pLASeR->SetDeactivate();

		UINT8 uNumOfChildren = pLASeR->GetChildrenSize();

		for (UINT8 i = 0; i < uNumOfChildren; ++i)
			pLASeR->GetChildren(i)->SetDeactivate();
	}
}

/************************************************************************/
// PrepareSVG
// Desc		: prepare svg element
// Param	: svg element pointer
// History
//	PSH (2009. 8. 4)	- first release
//	PYC (2009. 9. 21)	- add minDisplaySize, minResolution & minMemory adaptation tool
/************************************************************************/
void CChildView::PrepareSVG(CLASeR *pLASeR)
{
	// svg size needs to be modified
	m_uViewWidth	= 0;
	m_uViewHeight	= 0;

	CompareFunc(pLASeR);
}

/************************************************************************/
// ComparisonFunc
// Desc		: comparison func in a prepare step
// Param	: element pointer
// History
//	PYC (2009. 9. 24)	- first release
/************************************************************************/
void CChildView::CompareFunc(CLASeR *pLASeR)
{
	lsrErr err = LASeRNoErr;

	// display size adaptation
	float	fMinDisplaySize	= (float)pLASeR->GetUINTAttr(ATTR_LSR_MINDISPLAYSIZE, &err);
	ErrorMessage(err);

 	float	fCurDisplaySize	= GetDisplaySize();

	if (fCurDisplaySize < fMinDisplaySize)
		pLASeR->SetDeactivate();

	// resolution adaptation
	UINT	uMinResolution	= pLASeR->GetUINTAttr(ATTR_LSR_MINRESOLUTION, &err);
	ErrorMessage(err);

	UINT	uCurResolution	= GetHeightResolution();

	if (uCurResolution < uMinResolution)
		pLASeR->SetDeactivate();

	// memory adaptation
	vector<float> afMinMemory = pLASeR->GetFloatListAttr(ATTR_LSR_MINMEMORY, &err);
	ErrorMessage(err);

	float	fMinMemory = accumulate(afMinMemory.begin(), afMinMemory.end(), (float)0.0);

	float	fCurMemory = GetAvailableMemorySize();

	if (fCurMemory < fMinMemory)
		pLASeR->SetDeactivate();
}

/************************************************************************/
// ConvertLPtoDP
// Desc		: convert logical point to pixel value
// Param	: logical point, unit (inch, cm, ..), dpi
// Return	: pixel value
// History
//	PSH (2009. 6. 30)	- first release
/************************************************************************/
LONG ConvertLPtoDP(float fPoint, UINT8 uUnits, int nDPI)
{
	LONG lConverted = 0;

	switch (uUnits)
	{
	case 0:										// px = value(inch) * dpi
		lConverted = (LONG)(fPoint * nDPI);
		break;

	case 1:										// px = value(cm) * 0.3937 * dpi 
		lConverted = (LONG)(fPoint * nDPI * 0.3937);
		break;

	case 2:										// px = value(mm) * 0.03937 * dpi
		lConverted = (LONG)(fPoint * nDPI * 0.03937);
		break;

	case 3:										// px = value(pt) / 72 * dpi
		lConverted = (LONG)(fPoint * 1.33333);
		break;

	case 4:										// px = value(pc) / 6 * dpi
		lConverted = (LONG)(fPoint * 1.33333 * 12.0);
		break;

	case 5:										// px (nothing to do)
		lConverted = (LONG)fPoint;
		break;
	}

	return lConverted;
}

/************************************************************************/
// GetDisplaySize
// Desc		: get diagonal display size(inch)
// Return	: diagonal display size
// History
//	PYC (2009. 9. 21)	- first release
/************************************************************************/
#include <cmath>

float CChildView::GetDisplaySize()
{
	HMONITOR    hMonitor;
	MONITORINFO mi;
	RECT        rc;

	float		fWidth, fHeight;				// inch
	fWidth = fHeight = 0.0;

	hMonitor = MonitorFromRect(&rc, MONITOR_DEFAULTTONEAREST);
	mi.cbSize = sizeof(mi);
	GetMonitorInfo(hMonitor, &mi);

	CClientDC dc(this);

	// get display size(pixel -> inch)
	fWidth	= (float)((mi.rcMonitor.right - mi.rcMonitor.left) / GetDeviceCaps(dc, LOGPIXELSX));
	fHeight	= (float)((mi.rcMonitor.bottom - mi.rcMonitor.top) / GetDeviceCaps(dc, LOGPIXELSY));

	return (float)sqrt((fWidth * fWidth) + (fHeight * fHeight));
}

/************************************************************************/
// GetResolution
// Desc		: get height resolution(in pixel)
// Return	: height resolution
// History
//	PYC (2009. 9. 21)	- first release
/************************************************************************/
UINT CChildView::GetHeightResolution()
{
	HMONITOR	hMonitor;
	MONITORINFO mi;
	RECT        rc;

	UINT		uResY = 0;

	hMonitor = MonitorFromRect(&rc, MONITOR_DEFAULTTONEAREST);
	mi.cbSize = sizeof(mi);
	GetMonitorInfo(hMonitor, &mi);

	// get height resolution
	uResY = (UINT)(mi.rcMonitor.bottom - mi.rcMonitor.top);

	return uResY;
}

/************************************************************************/
// GetAvailableMemorySize
// Desc		: get available memory size(in kBytes)
// Return	: available memory
// History
//	PYC (2009. 9. 21)	- first release
/************************************************************************/
float CChildView::GetAvailableMemorySize()
{
#ifdef _WIN32_WCE
	MEMORYSTATUS stat;
	GlobalMemoryStatus(&stat);

	float fAvailablePhys = (float)(stat.dwAvailPhys / 1024.0);
#else
	MEMORYSTATUSEX stat;
	GlobalMemoryStatusEx(&stat);

	float fAvailablePhys = (float)(stat.ullAvailPhys / 1024.0);
#endif

	return fAvailablePhys;
}