/*
// This software was originally developed by ETRI and Kyunghee University, in the course
// of development of the ISO/IEC 14496-20 (Lightweight Application Scene Representation 
// -LASeR- and Simple Aggregation Format -SAF-) AMD3: Presentation and Modification of
// Structured Information standard for reference purposes and its performance may not 
// have been optimized.
//
// This software module is an implementation of one or more tools as specified by the
// ISO/IEC 14496-20 AMD3: PMSI standard.
//
// ISO/IEC gives users free license to this software module or modifications thereof
// for use in products claiming conformance to audiovisual and image-coding related
// ITU Recommendations and/or ISO/IEC International Standards.
// 
// ISO/IEC gives users the same free license to this software module or modifications
// thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in products are advised that its use may
// infringe existing patents. 
// 
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
*/
#include "stdafx.h"

#ifdef _WIN32_WCE
#include "..\..\M3RMobile\ChildView.h"
#elif _ACTIVEX
#include "..\..\M3RacX\M3RacXView.h""
#else
#include "..\ChildView.h"

#endif


void CChildView::DrawText(CLASeR *pLASeR)
{
	lsrErr err = LASeRNoErr;

	/************************************************************************/
	// get attributes
	float			fX				= pLASeR->GetFloatAttr(ATTR_X, &err);
	ErrorMessage(err);
	float			fY				= pLASeR->GetFloatAttr(ATTR_Y, &err);
	ErrorMessage(err);

	UINT8			uDisplay		= pLASeR->GetUINT8Attr(ATTR_DISPLAY, &err);
	ErrorMessage(err);
	tstring         strFontFamily	= pLASeR->GetStrAttr(ATTR_FONT_FAMILY, &err);
	ErrorMessage(err);
	UINT            uFontSize		= (UINT)pLASeR->GetFloatAttr(ATTR_FONT_SIZE, &err);
	ErrorMessage(err);
	UINT8			uFontStyle		= pLASeR->GetUINT8Attr(ATTR_FONT_STYLE, &err);
	ErrorMessage(err);
// 	UINT8			uFontWeight		= pLASeR->GetUINT8Attr(ATTR_FONT_WEIGHT, &err);
// 	ErrorMessage(err);
	
	tstring			strTextContent	= pLASeR->GetStrAttr(ATTR_TEXTCONTENT, &err);
	ErrorMessage(err);
	vector<UINT8>	auFill			= pLASeR->GetUINT8ListAttr(ATTR_FILL, &err);
	ErrorMessage(err);
	float			fFillOpacity	= pLASeR->GetFloatAttr(ATTR_FILL_OPACITY, &err);
	ErrorMessage(err);
// 	vector<UINT8>	auStroke		= pLASeR->GetUINT8ListAttr(ATTR_STROKE, &err);
// 	ErrorMessage(err);
// 	float			fStrokeOpacity	= pLASeR->GetFloatAttr(ATTR_STROKE_OPACITY, &err);
// 	ErrorMessage(err);
	float			fStrokeWidth	= pLASeR->GetFloatAttr(ATTR_STROKE_WIDTH, &err);
	ErrorMessage(err);
	UINT8			uVisibility		= pLASeR->GetUINT8Attr(ATTR_VISIBILITY, &err);
	ErrorMessage(err);
	/************************************************************************/

	if ((uDisplay != 1) && (uVisibility != 1) && (pLASeR->GetActivation() != 1))
	{
		Transform(pLASeR);

		double dRed, dGreen, dBlue;
		dRed = dGreen = dBlue = 0.0;

		if (auFill.size() == 3)
		{
			dRed	= auFill[0] / 255.0;
			dGreen	= auFill[1] / 255.0;
			dBlue	= auFill[2] / 255.0;
		}

		cairo_set_source_rgba(m_pCR, dRed, dGreen, dBlue, fFillOpacity);

		char *szFont_Family = NULL;
		int iFontLen = WideCharToMultiByte(CP_ACP, 0, strFontFamily.c_str(), -1, szFont_Family, 0, NULL, NULL);
		szFont_Family = new char[iFontLen + 1];
		WideCharToMultiByte(CP_ACP, 0, strFontFamily.c_str(), -1, szFont_Family, iFontLen, NULL, NULL);
		
		cairo_font_slant_t font_slant;
		switch (uFontStyle)
		{
		case 0:
			font_slant = CAIRO_FONT_SLANT_NORMAL;
			break;

		case 1:
			font_slant = CAIRO_FONT_SLANT_ITALIC;
			break;

		case 2:
			font_slant = CAIRO_FONT_SLANT_OBLIQUE;
			break;

		default:
			font_slant = CAIRO_FONT_SLANT_NORMAL;
			break;
		}

		//PMSI amd3
		if (strTextContent == _T("") && pLASeR->GetChildrenSize() != 0)
			strTextContent = GettrefContent(pLASeR);
		//

		cairo_select_font_face(m_pCR, szFont_Family, font_slant, CAIRO_FONT_WEIGHT_BOLD);
		delete [] szFont_Family;

		cairo_set_font_size(m_pCR, uFontSize);

		cairo_move_to(m_pCR, fX, fY);

		char *szTextContent = NULL;
		int iTextLen = WideCharToMultiByte(CP_ACP, 0, strTextContent.c_str(), -1,szTextContent, 0, NULL, NULL);
		szTextContent = new char[iTextLen + 1];
		WideCharToMultiByte(CP_ACP, 0, strTextContent.c_str(), -1,szTextContent, iTextLen, NULL, NULL);
		cairo_show_text(m_pCR, szTextContent);
		delete [] szTextContent;

		RestoreSurface(pLASeR);
	}
}