/*
// This software was originally developed by ETRI and Kyunghee University, in the course
// of development of the ISO/IEC 14496-20 (Lightweight Application Scene Representation 
// -LASeR- and Simple Aggregation Format -SAF-) AMD3: Presentation and Modification of
// Structured Information standard for reference purposes and its performance may not 
// have been optimized.
//
// This software module is an implementation of one or more tools as specified by the
// ISO/IEC 14496-20 AMD3: PMSI standard.
//
// ISO/IEC gives users free license to this software module or modifications thereof
// for use in products claiming conformance to audiovisual and image-coding related
// ITU Recommendations and/or ISO/IEC International Standards.
// 
// ISO/IEC gives users the same free license to this software module or modifications
// thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in products are advised that its use may
// infringe existing patents. 
// 
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
*/
#include "stdafx.h"

#ifdef _WIN32_WCE
#include "..\..\M3RMobile\ChildView.h"
#elif _ACTIVEX
#include "..\..\M3RacX\M3RacXView.h""
#else
#include "..\ChildView.h"

#endif


void CChildView::DrawTextArea(CLASeR *pLASeR)
{
	lsrErr err = LASeRNoErr;

	/************************************************************************/
	// get attributes
	float			fWidth			= pLASeR->GetFloatAttr(ATTR_WIDTH, &err);
	ErrorMessage(err);
	float			fHeight			= pLASeR->GetFloatAttr(ATTR_HEIGHT, &err);
	ErrorMessage(err);
	float			fX				= pLASeR->GetFloatAttr(ATTR_X, &err);
	ErrorMessage(err);
	float			fY				= pLASeR->GetFloatAttr(ATTR_Y, &err);
	ErrorMessage(err);

	UINT8			uDisplay		= pLASeR->GetUINT8Attr(ATTR_DISPLAY, &err);
	ErrorMessage(err);
	tstring         strFontFamily	= pLASeR->GetStrAttr(ATTR_FONT_FAMILY, &err);
	ErrorMessage(err);
	UINT            uFontSize		= (UINT)pLASeR->GetFloatAttr(ATTR_FONT_SIZE, &err);
	ErrorMessage(err);
	UINT8			uFontStyle		= pLASeR->GetUINT8Attr(ATTR_FONT_STYLE, &err);
	ErrorMessage(err);
	// 	UINT8			uFontWeight		= pLASeR->GetUINT8Attr(ATTR_FONT_WEIGHT, &err);
	// 	ErrorMessage(err);

	tstring			strTextContent	= pLASeR->GetStrAttr(ATTR_TEXTCONTENT, &err);
	ErrorMessage(err);
	vector<UINT8>	auFill			= pLASeR->GetUINT8ListAttr(ATTR_FILL, &err);
	ErrorMessage(err);
	float			fFillOpacity	= pLASeR->GetFloatAttr(ATTR_FILL_OPACITY, &err);
	ErrorMessage(err);
	vector<UINT8>	auStroke		= pLASeR->GetUINT8ListAttr(ATTR_STROKE, &err);
	ErrorMessage(err);
	float			fStrokeOpacity	= pLASeR->GetFloatAttr(ATTR_STROKE_OPACITY, &err);
	ErrorMessage(err);
	float			fStrokeWidth	= pLASeR->GetFloatAttr(ATTR_STROKE_WIDTH, &err);
	ErrorMessage(err);
	UINT8			uVisibility		= pLASeR->GetUINT8Attr(ATTR_VISIBILITY, &err);
	ErrorMessage(err);
	/************************************************************************/

	if ((uDisplay != 1) && (uVisibility != 1) && (pLASeR->GetActivation() != 1))
	{
		Transform(pLASeR);

		double dRed, dGreen, dBlue;
		dRed = dGreen = dBlue = 0.0;

		if (auFill.size() == 3)
		{
			dRed	= auFill[0] / 255.0;
			dGreen	= auFill[1] / 255.0;
			dBlue	= auFill[2] / 255.0;
		}

		cairo_set_source_rgba(m_pCR, dRed, dGreen, dBlue, fFillOpacity);

		UINT uLen = strFontFamily.length() + 1;

		char *szFontFamily = new char[uLen];
		WideCharToMultiByte(CP_ACP, 0, strFontFamily.c_str(), -1, szFontFamily, uLen, NULL, NULL);

		cairo_font_slant_t font_slant;

		switch (uFontStyle)
		{
		case 0:
			font_slant = CAIRO_FONT_SLANT_NORMAL;
			break;

		case 1:
			font_slant = CAIRO_FONT_SLANT_ITALIC;
			break;

		case 2:
			font_slant = CAIRO_FONT_SLANT_OBLIQUE;
			break;

		default:
			font_slant = CAIRO_FONT_SLANT_NORMAL;
			break;
		}

		// PMSI amd3
		if (strTextContent == _T("") && pLASeR->GetChildrenSize() != 0)
			strTextContent = GettrefContent(pLASeR);

		cairo_font_extents_t font_extents;
		cairo_text_extents_t text_extents;

		cairo_select_font_face(m_pCR, szFontFamily, font_slant, CAIRO_FONT_WEIGHT_BOLD);
		delete [] szFontFamily;

		cairo_set_font_size(m_pCR, uFontSize);

		uLen = strTextContent.length() + 1;

		char *szTextContent = new char[uLen];
		WideCharToMultiByte(CP_ACP, 0, strTextContent.c_str(), -1, szTextContent, uLen, NULL, NULL);

		cairo_font_extents(m_pCR, &font_extents);

		char letter[2];				// ӽ 
		float fMX = fX + fWidth;	// area ִ xǥ
		float fMY = fY + fHeight;	// area ִ yǥ

		// textcontent ̸ŭ ݺ
		for (UINT i = 0; i < uLen; i++)
		{ 
			*letter = '\0';

			// ѱھ 
			strncat(letter, szTextContent + i, 1);

			cairo_text_extents(m_pCR, letter, &text_extents);

			// ڰ ϵ ġ xǥ
			text_extents.x_bearing = fX;

			//  ã
			if (letter[0] == 32)
			{
				// fontsize ݸŭ 
				text_extents.width = uFontSize / 2;
			}

			//  ٲٱ
			if (text_extents.x_bearing + text_extents.width > fMX)
			{
				// fX ʱȭ
				fX = fMX - fWidth;
				//  ٲ  ڰ ϵ ġ xǥ 
				text_extents.x_bearing = fX;

				// fontsize ŭ 
				fY += uFontSize;
			}
			// ڰ ϵ ġ yǥ
			text_extents.y_bearing = fY;

			// η ִ  Ѿ 
			if (text_extents.y_bearing + text_extents.height > fMY)
			{
				//  textcontent  break
				break;
			}

			cairo_move_to (m_pCR, text_extents.x_bearing, text_extents.y_bearing + uFontSize * 0.8);

			// ϵ  widthŭ xǥ 
			fX += text_extents.width + uFontSize / 10;

			cairo_show_text(m_pCR, letter);
		}

		delete [] szTextContent;

		RestoreSurface(pLASeR);
	}
}

void CChildView::DeleteTextArea(CLASeR *pLASeR)
{
	lsrErr err = LASeRNoErr;

	/************************************************************************/
	// get attributes
	float			fWidth			= pLASeR->GetFloatAttr(ATTR_WIDTH, &err);
	ErrorMessage(err);
	float			fHeight			= pLASeR->GetFloatAttr(ATTR_HEIGHT, &err);
	ErrorMessage(err);
	float			fX				= pLASeR->GetFloatAttr(ATTR_X, &err);
	ErrorMessage(err);
	float			fY				= pLASeR->GetFloatAttr(ATTR_Y, &err);
	ErrorMessage(err);

	UINT8			uDisplay		= pLASeR->GetUINT8Attr(ATTR_DISPLAY, &err);
	ErrorMessage(err);
	UINT8			uVisibility		= pLASeR->GetUINT8Attr(ATTR_VISIBILITY, &err);
	ErrorMessage(err);
	/************************************************************************/

	if ((uDisplay != 1) && (uVisibility != 1) && (pLASeR->GetActivation() != 1))
	{
		Transform(pLASeR);

		double dRed, dGreen, dBlue;
		dRed = dGreen = dBlue = 255.0;

		cairo_set_source_rgba(m_pCR, dRed, dGreen, dBlue, 1);
		cairo_rectangle(m_pCR, fX, fY, fWidth, fHeight);
		cairo_fill_preserve(m_pCR);

		RestoreSurface(pLASeR);
	}
}