/*
// This software was originally developed by ETRI and Kyunghee University, in the course
// of development of the ISO/IEC 14496-20 (Lightweight Application Scene Representation 
// -LASeR- and Simple Aggregation Format -SAF-) AMD3: Presentation and Modification of
// Structured Information standard for reference purposes and its performance may not 
// have been optimized.
//
// This software module is an implementation of one or more tools as specified by the
// ISO/IEC 14496-20 AMD3: PMSI standard.
//
// ISO/IEC gives users free license to this software module or modifications thereof
// for use in products claiming conformance to audiovisual and image-coding related
// ITU Recommendations and/or ISO/IEC International Standards.
// 
// ISO/IEC gives users the same free license to this software module or modifications
// thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in products are advised that its use may
// infringe existing patents. 
// 
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
*/
#include "stdafx.h"

#ifdef _WIN32_WCE
#include "..\..\M3RMobile\ChildView.h"
#include "..\..\M3RMobile\MainFrm.h"
#elif _ACTIVEX
#include "..\..\M3RacX\M3RacXView.h"
#include "..\..\M3RacX\M3RacXDlg.h"
#else
#include "..\ChildView.h"
#include "..\MainFrm.h"
#endif


UINT ThreadRenderer(LPVOID pParam);

void DrawBitmap(HDC hDC, HBITMAP hBit, int x, int y, int width, int height);


void CChildView::DrawVideo(CLASeR *pLASeR /* = NULL */)
{
	lsrErr err = LASeRNoErr;

	UINT uWidth, uHeight, uX, uY;
	uWidth = uHeight = uX = uY = 0;

	if (GetLASeRFlag())
	{
		/************************************************************************/
		// get attributes
		uWidth	= (UINT)pLASeR->GetFloatAttr(ATTR_WIDTH, &err);
		ErrorMessage(err);
		uHeight	= (UINT)pLASeR->GetFloatAttr(ATTR_HEIGHT, &err);
		ErrorMessage(err);
		uX		= (UINT)pLASeR->GetFloatAttr(ATTR_X, &err);
		ErrorMessage(err);
		uY		= (UINT)pLASeR->GetFloatAttr(ATTR_Y, &err);
		ErrorMessage(err);

		UINT8	uDisplay	= pLASeR->GetUINT8Attr(ATTR_DISPLAY, &err);
		ErrorMessage(err);
		UINT8	uVisibility	= pLASeR->GetUINT8Attr(ATTR_VISIBILITY, &err);
		ErrorMessage(err);
		/************************************************************************/

		if ((uDisplay == 1) || (uVisibility == 1) || (pLASeR->GetActivation() == 1))
			return;
	}

	g_cs.Lock();								// lock critical section

	// for case of composition buffer mismatch
	if (m_aapCB[m_uVideoTrackIndex].size() == 0)
	{
		// stream is end & buffer is exhausted
		if (m_abCBStatus[m_uVideoTrackIndex] == BUFFER_STREAM_END)
			m_abCBStatus[m_uVideoTrackIndex] = BUFFER_END;

		else
			m_abCBStatus[m_uVideoTrackIndex] = BUFFER_TEMP_EMPTY;

		g_cs.Unlock();							// unlock critical section

		return;
	}

	BYTE *pUnit = m_aapCB[m_uVideoTrackIndex][0];

	g_cs.Unlock();								// unlock critical section
	/************************************************************************/

#ifdef _WIN32_WCE
	CClientDC dc(this);

	void *pRaster = NULL;

	g_cs.Lock();								// lock critical section

	HBITMAP hBit = CreateDIBSection(NULL, &m_bitmapPic, DIB_RGB_COLORS, (void **)&pRaster, NULL, 0);

	g_cs.Unlock();								// unlock critical section

	memcpy(pRaster, (void *)pUnit, m_bitmapPic.bmiHeader.biSizeImage);

	DrawBitmap(m_hMemDC, hBit, uX, uY, uWidth, uHeight);

	if (!GetLASeRFlag())
		Invalidate(FALSE);

#else
	CClientDC dc(this);

	g_cs.Lock();								// lock critical section

	HBITMAP hBit = CreateDIBitmap(dc.m_hDC, (BITMAPINFOHEADER *)&m_bitmapPic, CBM_INIT, pUnit, &m_bitmapPic, DIB_RGB_COLORS);

	g_cs.Unlock();								// unlock critical section

	DrawBitmap(m_hMemDC, hBit, uX, uY, uWidth, uHeight);

	if (!GetLASeRFlag())
		Invalidate(FALSE);

#endif

	delete [] pUnit;
	pUnit = NULL;

	/************************************************************************/
	g_cs.Lock();								// lock critical section

	m_aapCB[m_uVideoTrackIndex].pop_front();

	if (m_aapCB[m_uVideoTrackIndex].size() == 0)
	{
		// stream is end & buffer is exhausted
		if (m_abCBStatus[m_uVideoTrackIndex] == BUFFER_STREAM_END)
			m_abCBStatus[m_uVideoTrackIndex] = BUFFER_END;
		
		else
			m_abCBStatus[m_uVideoTrackIndex] = BUFFER_TEMP_EMPTY;
	}

	g_cs.Unlock();								// unlock critical section

	DeleteObject(hBit);

	return;
}

void DrawBitmap(HDC hDC, HBITMAP hBit, int x, int y, int width, int height)
{
	HDC		hMem;
	HBITMAP	hOld;
	BITMAP	bit;

	hMem	= CreateCompatibleDC(hDC);
	hOld	= (HBITMAP)SelectObject(hMem, hBit);

	GetObject(hBit, sizeof(BITMAP), &bit);

	if ((width != 0) || (height != 0))
	{
#ifndef _WIN32_WCE
		SetStretchBltMode(hDC, WHITEONBLACK);
#endif // _WIN32_WCE

		StretchBlt(hDC, x, y, width, height, hMem, 0, 0, bit.bmWidth, bit.bmHeight, SRCCOPY);
	}

	else
		BitBlt(hDC, x, y, bit.bmWidth, bit.bmHeight, hMem, 0, 0, SRCCOPY);

	SelectObject(hMem, hOld);
	DeleteObject(hOld);
	DeleteDC(hMem);
}

void CChildView::PrepareVideo(CLASeR *pLASeR /* = NULL */)
{
#ifdef _ACTIVEX
	CMainFrame *pFrame = (CMainFrame *)GetParent();
#else
	CMainFrame *pFrame = (CMainFrame *)(AfxGetApp()->GetMainWnd());
#endif // _ACTIVEX

	if (GetLASeRFlag())
	{
		CString strVideoSource = GetSourceUsingHref(pLASeR).c_str();

		pFrame->DoFileOpen(strVideoSource);
	}

	vector<UINT> auVideoTrackIndices = pFrame->GetVideoTrackIndices();

	// no video track is existed
	if (auVideoTrackIndices.size() == 0)
		return;

	m_uVideoTrackIndex = *auVideoTrackIndices.begin();

	m_bitmapPic.bmiHeader.biSize			= sizeof(BITMAPINFOHEADER);
	m_bitmapPic.bmiHeader.biWidth			= pFrame->GetVideoWidth();
	m_bitmapPic.bmiHeader.biHeight			= -((int)pFrame->GetVideoHeight());
	m_bitmapPic.bmiHeader.biPlanes			= 1;
	m_bitmapPic.bmiHeader.biBitCount		= 24;
	m_bitmapPic.bmiHeader.biCompression		= BI_RGB;
	m_bitmapPic.bmiHeader.biSizeImage		= m_bitmapPic.bmiHeader.biWidth * (-m_bitmapPic.bmiHeader.biHeight) * 3;
	m_bitmapPic.bmiHeader.biXPelsPerMeter	= 0;
	m_bitmapPic.bmiHeader.biYPelsPerMeter	= 0;
	m_bitmapPic.bmiHeader.biClrUsed			= 0;
	m_bitmapPic.bmiHeader.biClrImportant	= 0;

	// if LASeR is not used, this variable is 0. therefore, video size must be inputed.
	if (!GetLASeRFlag())
	{
		m_uViewWidth	= pFrame->GetVideoWidth();
		m_uViewHeight	= pFrame->GetVideoHeight();
	}

	// resize window frame according to video size (only win32 app)
#ifndef _ACTIVEX
	if (!GetLASeRFlag())	pFrame->PostMessage(WM_RESIZEVIEW);
#endif /* _ACTIVEX */

	// create memory dc
	if (!m_hMemDC)
	{
		HWND hWnd	= this->GetSafeHwnd();
		HDC hDC		= ::GetDC(hWnd);

		m_hMemDC	= CreateCompatibleDC(hDC);
		m_hBit		= CreateCompatibleBitmap(hDC, m_uViewWidth, m_uViewHeight);

		m_hOldBit	= (HBITMAP)SelectObject(m_hMemDC, m_hBit);
	}

	// if LASeR is not used, renderer must run immediately
	if (!GetLASeRFlag())
		m_pRendererThread = ::AfxBeginThread(ThreadRenderer, (LPVOID)this);
}

UINT ThreadRenderer(LPVOID pParam)
{
	CChildView *pView = (CChildView *)pParam;

	if (pView == NULL)
		return 0;

	const UINT8 uPictureDelta = pView->GetPictureDelta();

	// if the player is run
	while (pView->GetPlayerStatus() == PLAYER_PLAYING)
	{
		// the cb's status is temporary empty
		if (pView->GetCBStatus(pView->GetVideoTrackIndex()) == BUFFER_TEMP_EMPTY)
			Sleep(20);

		// the cb's status is end & empty
		else if (pView->GetCBStatus(pView->GetVideoTrackIndex()) == BUFFER_END)
		{
			g_cs.Lock();						// lock critical section
			pView->SetPlayerStatus(PLAYER_STOP);
			g_cs.Unlock();						// unlock critical section

#ifdef _ACTIVEX
			CMainFrame *pFrame = (CMainFrame *)pView->GetParent();
#else
			CMainFrame *pFrame = (CMainFrame *)(AfxGetApp()->GetMainWnd());
#endif // _ACTIVEX

			pFrame->PostMessage(WM_STOP);
			pView->PostMessage(WM_STOP);

			break;
		}

		// the cb's status is normal
		else
		{
			if (pView->GetLASeRFlag())
			{
				pView->DrawScene();
				Sleep(uPictureDelta);
			}

			else
			{
				pView->DrawVideo();
				Sleep(uPictureDelta);
			}
		}
	}

	return 0;
}