/*
// This software was originally developed by ETRI and Kyunghee University, in the course
// of development of the ISO/IEC 14496-20 (Lightweight Application Scene Representation 
// -LASeR- and Simple Aggregation Format -SAF-) AMD3: Presentation and Modification of
// Structured Information standard for reference purposes and its performance may not 
// have been optimized.
//
// This software module is an implementation of one or more tools as specified by the
// ISO/IEC 14496-20 AMD3: PMSI standard.
//
// ISO/IEC gives users free license to this software module or modifications thereof
// for use in products claiming conformance to audiovisual and image-coding related
// ITU Recommendations and/or ISO/IEC International Standards.
// 
// ISO/IEC gives users the same free license to this software module or modifications
// thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in products are advised that its use may
// infringe existing patents. 
// 
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
*/
// MainFrm.cpp : implementation of the CMainFrame class
//
#include "stdafx.h"
#include "M3RP.h"
#include "MainFrm.h"

#include "OptionDlg.h"
#include "ContentsDlg.h"

#include "..\src\XML2SceneTree\XMLFileManager.h"
#include "..\src\libbintoxml\bin2xml.h"

#include "AVCDecoder.h"

#include <algorithm>
#include <functional>


extern UINT ThreadDBGatherer(LPVOID pParam);
extern UINT ThreadAVCDecoder(LPVOID pParam);
extern UINT ThreadAACDecoder(LPVOID pParam);
extern UINT ThreadLASeRDecoder(LPVOID pParam);

UINT	g_uFrameWidth;
UINT	g_uFrameHeight;

#define LISTEN_PORT 6500


#ifdef _DEBUG
#define new DEBUG_NEW
#endif

// CMainFrame

IMPLEMENT_DYNAMIC(CMainFrame, CFrameWnd)

BEGIN_MESSAGE_MAP(CMainFrame, CFrameWnd)
	ON_WM_CREATE()
	ON_WM_SETFOCUS()
	ON_COMMAND(ID_FILE_OPEN, &CMainFrame::OnFileOpen)
// 	ON_COMMAND(ID_FILE_CONNECT, &CMainFrame::OnFileConnect)
// 	ON_COMMAND(ID_TOOLS_OPTION, &CMainFrame::OnToolsOption)
	ON_WM_CLOSE()
	ON_MESSAGE(WM_PLAY, OnPlay)
	ON_MESSAGE(WM_PAUSE, OnPause)
	ON_MESSAGE(WM_STOP, OnStop)
	ON_MESSAGE(WM_READY, OnReady)
	ON_MESSAGE(WM_RESIZEVIEW, OnResizeView)
END_MESSAGE_MAP()


// CMainFrame construction/destruction

CMainFrame::CMainFrame()
{
	// TODO: add member initialization code here
	m_strPath			= _T("C:\\");
	m_strFileName		= _T("");

	m_bProgressive		= 0;
	m_strServerAddr		= "intmedia.khu.ac.kr";
	m_uVideoDBIndex		= 0;
	m_uAudioDBIndex		= 0;
	m_uRenderingCount	= 0;

	m_pMP4				= NULL;
	m_bLASeR			= 0;

	m_bPlayerStatus		= PLAYER_STOP;

	m_bFirstTime		= 1;
	m_uDuration			= 0;
	m_uTrackCount		= 0;
	m_pFileStartPoint	= NULL;

	m_uAudioTrackIndex	= 0xFFFFFFFF;
	m_uLASeRTrackIndex	= 0xFFFFFFFF;

	m_uVideoWidth		= 0;
	m_uVideoHeight		= 0;
	m_uPicSize			= 0;
	m_uCompositionType	= 0xFF;
	m_uPicDelta			= 0;

	m_hDecoder			= NULL;
	m_hDevice			= NULL;
	m_nScheduledBlocks	= 0;
	m_uPlayedWaveHeaders= 0;

	m_pDBGathererThread	= NULL;
	m_pAVCDecoderThread	= NULL;
	m_pAACDecoderThread	= NULL;

	m_pYUV				= NULL;
}

CMainFrame::~CMainFrame()
{
	ClearPlayerStatus();
}

void CMainFrame::ClearPlayerStatus()
{
	m_strPath			= _T("C:\\");

	m_bPlayerStatus		= PLAYER_STOP;
	m_abDBStatus.clear();

	if (m_pDBGathererThread)
	{
		while (::WaitForSingleObject(m_pDBGathererThread->m_hThread, 0) == WAIT_TIMEOUT)
			Sleep(10);

		m_pDBGathererThread = NULL;
	}

	if (m_pAVCDecoderThread)
	{
		while (::WaitForSingleObject(m_pAVCDecoderThread->m_hThread, 0) == WAIT_TIMEOUT)
			Sleep(10);

		m_pAVCDecoderThread = NULL;
	}

	if (m_pAACDecoderThread)
	{
		while (::WaitForSingleObject(m_pAACDecoderThread->m_hThread, 0) == WAIT_TIMEOUT)
			Sleep(10);

		m_pAACDecoderThread = NULL;
	}

	m_bProgressive		= 0;
	m_uVideoDBIndex		= 0;
	m_uAudioDBIndex		= 0;
	m_uRenderingCount	= 0;

	if (m_pMP4)
	{
		m_pMP4->CloseMP4Record();

		delete m_pMP4;
		m_pMP4 = NULL;
	}

	CloseSceneRoot();
	m_bLASeR			= 0;

	m_bFirstTime		= 1;
	m_uDuration			= 0;
	m_uTrackCount		= 0;
	m_pFileStartPoint	= NULL;

	m_auVideoTrackIndices.clear();
	m_uAudioTrackIndex	= 0xFFFFFFFF;
	m_uLASeRTrackIndex	= 0xFFFFFFFF;

	m_afPassed.clear();	

	m_uVideoWidth		= 0;
	m_uVideoHeight		= 0;
	m_uPicSize			= 0;
	m_uCompositionType	= 0xFF;
	m_uPicDelta			= 0;

	m_hDevice			= NULL;
	m_nScheduledBlocks	= 0;
	m_uPlayedWaveHeaders= 0;

	if (m_pYUV)
	{
		delete [] m_pYUV;
		m_pYUV = NULL;
	}

	// buffer clear
	deque< deque<BYTE *> >::iterator i;
	deque<BYTE *>::iterator j;

	deque< deque<UINT> >::iterator k;

	if (m_bProgressive == 1)
	{
		for (i = m_aapDB.begin(); i < m_aapDB.end(); ++i)
		{
			for (j = i->begin(); j < i->end(); ++j)
			{
				if (*j)
				{
					delete [] *j;
					*j = NULL;
				}
			}

			i->clear();
		}

		for (k = m_aauRAPoints.begin(); k < m_aauRAPoints.end(); ++k)
			k->clear();

		for (k = m_aauCompositeTime.begin(); k < m_aauCompositeTime.end(); ++k)
			k->clear();
	}

	else if (m_bProgressive == 0)
	{
		for (i = m_aapDB.begin(); i < m_aapDB.end(); ++i)
			i->clear();
	}

	for (k = m_aauUnitSizes.begin(); k < m_aauUnitSizes.end(); ++k)
		k->clear();

	m_aapDB.clear();
	m_aauUnitSizes.clear();

	m_aauRAPoints.clear();
	m_aauCompositeTime.clear();
}

BOOL CMainFrame::PreCreateWindow(CREATESTRUCT& cs)
{
	if (!CFrameWnd::PreCreateWindow(cs))
		return FALSE;
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	cs.dwExStyle &= ~WS_EX_CLIENTEDGE;
	cs.lpszClass = AfxRegisterWndClass(0);

	cs.cx = INITIAL_WIDTH;
	cs.cy = INITIAL_HEIGHT;

	return TRUE;
}

int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;

	// get frame size
	CRect rct;
	GetClientRect(&rct);

	g_uFrameWidth	= INITIAL_WIDTH - rct.Width();
	g_uFrameHeight	= INITIAL_HEIGHT - rct.Height();

	// create a view to occupy the client area of the frame
	if (!m_wndView.Create(NULL, NULL, AFX_WS_DEFAULT_VIEW,
		CRect(0, 0, 0, 0), this, AFX_IDW_PANE_FIRST, NULL))
	{
		TRACE0("Failed to create view window\n");
		return -1;
	}

	return 0;
}


// CMainFrame diagnostics

#ifdef _DEBUG
void CMainFrame::AssertValid() const
{
	CFrameWnd::AssertValid();
}

void CMainFrame::Dump(CDumpContext& dc) const
{
	CFrameWnd::Dump(dc);
}
#endif //_DEBUG


// CMainFrame message handlers

void CMainFrame::OnSetFocus(CWnd* /*pOldWnd*/)
{
	// forward focus to the view window
	m_wndView.SetFocus();
}

BOOL CMainFrame::OnCmdMsg(UINT nID, int nCode, void* pExtra, AFX_CMDHANDLERINFO* pHandlerInfo)
{
	// let the view have first crack at the command
	if (m_wndView.OnCmdMsg(nID, nCode, pExtra, pHandlerInfo))
		return TRUE;

	// otherwise, do default handling
	return CFrameWnd::OnCmdMsg(nID, nCode, pExtra, pHandlerInfo);
}

void CMainFrame::OnClose()
{
	// TODO: ⿡ ޽ ó ڵ带 ߰ /Ǵ ⺻ ȣմϴ.
	// it don't close well in pause mode
	if (m_bPlayerStatus == PLAYER_PAUSE)
		DoPlay();

	ClearPlayerStatus();

	CFrameWnd::OnClose();
}

/************************************************************************/
// void CMainFrame::OnToolsOption()
// {
// 	// TODO: ⿡  ó ڵ带 ߰մϴ.
// 	INT8 err = 0;
// 
// 	COptionDlg dlg;
// 
// 	dlg.SetServerAddress(m_strServerAddr);
// 
// 	if (dlg.DoModal() == IDOK)
// 		m_strServerAddr = dlg.GetServerAddress();
// 
// 	// if controller is already defined,
// 	if (m_pController)
// 	{
// 		delete m_pController;
// 		m_pController = NULL;
// 	}
// 
// 	// initialize progressive download controller
// 	m_pController = new Controller;
// 	m_pController->SetHWnd(this->GetSafeHwnd());
// 	m_pController->Create();
// 
// 	// connect server
// 	err = (UINT8)m_pController->Connect(m_strServerAddr, LISTEN_PORT);
// 	ErrorMessage(err);
// }

void CMainFrame::OnFileOpen()
{
	const TCHAR szFilter[] = _T("LASeR Stream file (*.lsr)|*.lsr|")
							_T("LASeR XML file (*.xml)|*.xml|")
							_T("ISO base media file format family (*.mp4)|*.mp4|");

	// popup the file open dialog box
	CFileDialog dlg(TRUE, NULL, NULL, OFN_FILEMUSTEXIST, szFilter);

	// call DoFileOpen() if user pick a file
	if (dlg.DoModal() == IDOK)
	{
		// when other file is opened & the player is running
		if (m_bPlayerStatus == PLAYER_PAUSE)
		{
			DoPlay();
			m_wndView.SendMessage(WM_PLAY);
		}

		ClearPlayerStatus();
		m_wndView.ClearPlayerStatus();

		m_strPath		= dlg.GetPathName().Left(dlg.GetPathName().ReverseFind(_T('\\')) + 1);
		m_strFileName	= dlg.GetPathName();

		DoFileOpen();
	}
}

// void CMainFrame::OnFileConnect()
// {
// 	// TODO: ⿡  ó ڵ带 ߰մϴ.
// 	char	*pList		= NULL;
// 	CString	strSelected	= NULL;
// 
// 	CContentsDlg dlg;
// 
// 	// get contents list
// 	pList = m_pController->GetContentsList();
// 
// 	// display contents list
// 	dlg.SetInitialData(pList);
// 
// 	if (dlg.DoModal() == IDOK)
// 	{
// 		// when other file is opened
// 		if (m_bPlayerStatus == PLAYER_PAUSE)
// 		{
// 			DoPlay();
// 			m_wndView.SendMessage(WM_PLAY);
// 		}
// 
// 		ClearPlayerStatus();
// 		m_wndView.ClearPlayerStatus();
// 
// 		strSelected = dlg.GetSelected();
// 
// 		if (strSelected == _T(""))
// 		{
// 			MessageBox(_T(" ϼ."), _T("Caution"), MB_OK);
// 			return;
// 		}
// 
// 		m_pController->StartReceivingContent(strSelected.GetBuffer());
// 
// 		m_bProgressive = 1;						// progressive download mode
// 
// 		// start db gatherer in progressive mode
// 		m_pDBGathererThread	= ::AfxBeginThread(ThreadDBGatherer, (LPVOID)this, THREAD_PRIORITY_LOWEST);
// 	}
// 
// 	return;
// }

void CMainFrame::DoFileOpen(CString strFileName /* = _T("") */)
{
	// get the extension
	CString strFileExt = (strFileName != _T(""))
		? strFileName.Right(strFileName.GetLength() - strFileName.ReverseFind(_T('.')) - 1)
		: m_strFileName.Right(m_strFileName.GetLength() - m_strFileName.ReverseFind(_T('.')) - 1);

	if (strFileExt.CompareNoCase(_T("mp4")) == 0)
	{
		if (m_pMP4)
			return;								// already get the mp4 info

		else
		{
			m_pMP4 = new CLibMP4Api();

			if (m_pMP4 == NULL)
			{
				// no more memory err
				ErrorMessage(PlayerNoMoreMemoryErr);
				return;
			}
		}

		// If the DLL loading is corrupted
		// return value - NULL : corrupted / otherwise : right
		if (!(m_pMP4->TestRightDLL())) {m_pMP4 = NULL; return;}

		ParseMP4(strFileName);
	}

	else if (strFileExt.CompareNoCase(_T("lsr")) == 0)
	{
		ParseLASeRBin(strFileName);
		tstring stFileName = m_strFileName;
		stFileName.replace(stFileName.size() - 4, stFileName.size(), _T("_d.xml"));
		ParseLASeR(stFileName.c_str());
	}

	else if (strFileExt.CompareNoCase(_T("xml")) == 0)
		ParseLASeR(strFileName);
}

// local mp4 player scenario
void CMainFrame::ParseMP4(CString strFileName /* = _T("") */)
{
	INT8 err = 0;

	void *pMP4 = NULL;

	if (strFileName == _T(""))
	{
		// if m_strFileName is absolute path
		if (m_strFileName.Find(':') > 0)
			err = m_pMP4->OpenMP4File(&pMP4, m_strFileName.GetBuffer());

		// if m_strFileName is relative path
		else
		{
			CString strTemp = m_strPath + m_strFileName;

			err = m_pMP4->OpenMP4File(&pMP4, strTemp.GetBuffer());
		}
	}

	else
	{
		// if m_strFileName is absolute path
		if (strFileName.Find(':') > 0)
			err = m_pMP4->OpenMP4File(&pMP4, strFileName.GetBuffer());

		// if m_strFileName is relative path
		else
		{
			CString strTemp = m_strPath + strFileName;

			err = m_pMP4->OpenMP4File(&pMP4, strTemp.GetBuffer());
		}
	}

	ErrorMessage(err);

	// check duration, track count & start point
	m_uDuration			= m_pMP4->GetMovieDuration() * 1000 / m_pMP4->GetMovieTimeScale();
	m_uTrackCount		= m_pMP4->GetTrackCount();
	m_pFileStartPoint	= m_pMP4->GetFileStartPoint();

	// initialize decoder configuration
	for (UINT i = 0; i < m_uTrackCount; ++i)
	{
		err = InitDecoders(i);
		ErrorMessage(err);
	}

	DoReady();
}

void CMainFrame::ParseMP4(BYTE *pBits, const UINT uSize)
{
	INT8 err = 0;

	void *pMP4 = NULL;

	/************************************************************************/
	// set new mp4 file controller
	// if m_pMP4 isn't NULL, the program is corrupted.
	if (m_pMP4)
	{
		ErrorMessage(PlayerUnknownErr);		// unknown err
		return;
	}

	else
	{
		m_pMP4 = new CLibMP4Api();

		if (m_pMP4 == NULL)
		{
			// no more memory err
			ErrorMessage(PlayerNoMoreMemoryErr);
			return;
		}
	}

	// If the DLL loading is corrupted
	// return value - NULL : corrupted / otherwise : right
	if (!(m_pMP4->TestRightDLL())) {m_pMP4 = NULL; return;}

	err = m_pMP4->OpenBitStream(&pMP4, (unsigned char *)pBits, uSize);
	ErrorMessage(err);

	// check duration, track count & start point
	m_uDuration			= m_pMP4->GetMovieDuration() * 1000 / m_pMP4->GetMovieTimeScale();
	m_uTrackCount		= m_pMP4->GetTrackCount();
	m_pFileStartPoint	= m_pMP4->GetFileStartPoint();

	DoReady();
}

// local LASeR xml player scenario
void CMainFrame::ParseLASeR(CString strFileName /* = _T("") */)
{
	lsrErr err = LASeRNoErr;

	void *pRoot = NULL;
	pRoot = NULL;

	if (strFileName == _T(""))
	{
		// if m_strFileName is absolute path
		if (m_strFileName.Find(':') > 0)
			err = OpenXMLFile(m_strFileName, &pRoot);

		// if m_strFileName is relative path
		else
		{
			CString strTemp = m_strPath + m_strFileName;

			err = OpenXMLFile(strTemp, &pRoot);
		}
	}

	else
	{
		// if m_strFileName is absolute path
		if (strFileName.Find(':') > 0)
			err = OpenXMLFile(strFileName, &pRoot);

		// if m_strFileName is relative path
		else
		{
			CString strTemp = m_strPath + strFileName;

			err = OpenXMLFile(strTemp, &pRoot);
		}
	}

	ErrorMessage(err);

	if (pRoot)
	{
		err = SetSceneRoot(pRoot);				// SceneTreeParser
		ErrorMessage(err);

		m_bLASeR = 1;
	}

	DoReady();
}

void CMainFrame::ParseLASeR(BYTE *pBits, const UINT uSize)
{
	lsrErr err = LASeRNoErr;

	void *pRoot = NULL;
	pRoot = NULL;

	// XML2SceneTree
	err = OpenXMLBitStream(pBits, uSize, &pRoot);
	ErrorMessage(err);

	if (pRoot)
	{
		err = SetSceneRoot(pRoot);				// SceneTreeParser
		ErrorMessage(err);

		m_bLASeR = 1;
	}
}

// local LASeR lsr player scenario
void CMainFrame::ParseLASeRBin(CString strFileName /* = _T("") */)
{
	lsrErr err = LASeRNoErr;

	if (strFileName == _T(""))
	{
		// if m_strFileName is absolute path
		if (m_strFileName.Find(':') > 0)
			OpenBinFile(m_strFileName);

		// if m_strFileName is relative path
		else
		{
			CString strTemp = m_strPath + m_strFileName;
			OpenBinFile(strTemp);
		}
	}
	else
	{
		// if m_strFileName is absolute path
		if (strFileName.Find(':') > 0)
			OpenBinFile(strFileName);

		// if m_strFileName is relative path
		else
		{
			CString strTemp = m_strPath + strFileName;

			OpenBinFile(strTemp);
		}
	}
	ErrorMessage(err);
}





/************************************************************************/
#define FOUR_CHAR_CODE(a, b, c, d) (((a) << 24) | ((b) << 16) | ((c) << 8) | (d))

// only mp4
INT8 CMainFrame::InitDecoders(const UINT uTrackIndex, const UINT uSampleDescIndex /* = 0 */)
{
	INT8 err = 0;

	// for avc
	if (m_pMP4->GetSampleDescType(uTrackIndex, uSampleDescIndex) == FOUR_CHAR_CODE('a', 'v', 'c', '1'))
	{
		// insert track index
		if (find(m_auVideoTrackIndices.begin(), m_auVideoTrackIndices.end(), uTrackIndex) == m_auVideoTrackIndices.end())
			m_auVideoTrackIndices.push_back(uTrackIndex);

		// supposed only one parameter set used
		if ((m_pMP4->GetNumOfSPS(uTrackIndex, uSampleDescIndex) == 1) &&
			(m_pMP4->GetNumOfPPS(uTrackIndex, uSampleDescIndex) == 1))
		{
			UINT	uPSSize	= m_pMP4->GetSPSLength(uTrackIndex, uSampleDescIndex, 0)
				+ m_pMP4->GetPPSLength(uTrackIndex, uSampleDescIndex, 0) + 12 + 1;

			BYTE	*pData	= new BYTE[uPSSize];
			UINT    uIndex  = 0;

			if (pData == NULL)
			{
				// no more memory err
				ErrorMessage(PlayerNoMoreMemoryErr);
				return PlayerNoMoreMemoryErr;
			}

			memset(pData, 0, uPSSize);

			pData[0] = pData[1] = pData[2] = 0;
			pData[3] = 1;

			uIndex += 4;

			memcpy((void *)(pData + uIndex),
				(const void *)m_pMP4->GetSPSNALUnit(uTrackIndex, uSampleDescIndex, 0),
				m_pMP4->GetSPSLength(uTrackIndex, uSampleDescIndex, 0));

			uIndex += m_pMP4->GetSPSLength(uTrackIndex, uSampleDescIndex, 0);

			pData[uIndex] = pData[uIndex + 1] = pData[uIndex + 2] = 0;
			pData[uIndex + 3] = 1;

			uIndex += 4;

			memcpy((void *)(pData + uIndex),
				(const void *)m_pMP4->GetPPSNALUnit(uTrackIndex, uSampleDescIndex, 0),
				m_pMP4->GetPPSLength(uTrackIndex, uSampleDescIndex, 0));

			uIndex += m_pMP4->GetPPSLength(uTrackIndex, uSampleDescIndex, 0);

			pData[uIndex] = pData[uIndex + 1] = pData[uIndex + 2] = 0;
			pData[uIndex + 3] = 1;

			uIndex += 4;

			if (InitAVCDecoder(pData, uIndex))
				err = PlayerDecoderErr;
			
			delete [] pData;
			pData = NULL;

			// get composition type
			m_uCompositionType = m_pMP4->GetStereoCompositionType(uTrackIndex);
		}
	}

	// for aac
	else if (m_pMP4->GetSampleDescType(uTrackIndex, uSampleDescIndex) == FOUR_CHAR_CODE('m', 'p', '4', 'a'))
	{
		// match track to proper decoder
		m_uAudioTrackIndex = uTrackIndex;

		BYTE	*pData	= m_pMP4->GetDecSpecificInfoData(uTrackIndex, uSampleDescIndex);
		UINT	uSize	= strlen((const char *)pData);

		if (InitAACDecoder(pData, uSize))
			err = PlayerDecoderErr;
	}

	// for LASeR
	else if (m_pMP4->GetSampleDescType(uTrackIndex, uSampleDescIndex) == FOUR_CHAR_CODE('l', 's', 'r', '1'))
	{
		m_bLASeR = 1;

		m_uLASeRTrackIndex = uTrackIndex;

		BYTE	*pData	= m_pFileStartPoint + m_pMP4->GetChunkOffset(uTrackIndex, uSampleDescIndex);
		UINT	uSize	= m_pMP4->GetSampleSize(uTrackIndex);

		ParseLASeR(pData, uSize);
	}

	return err;
}

INT8 CMainFrame::InitDecoders(const UINT uTrackIndex, const UINT8 uOTI, const UINT8 uStreamType, BYTE *pBits)
{
	INT8 err = 0;

	// for avc
	if (uOTI == 0x21 && uStreamType == 0x04)
	{
		// insert track index
		if (find(m_auVideoTrackIndices.begin(), m_auVideoTrackIndices.end(), uTrackIndex) == m_auVideoTrackIndices.end())
			m_auVideoTrackIndices.push_back(uTrackIndex);

		// supposed only one parameter set used
		UINT	uSPSLength	= (*(pBits + 6) << 8) + *(pBits + 7);
		UINT	uPPSLength	= (*(pBits + uSPSLength + 9) << 8) + *(pBits + uSPSLength + 10);
		UINT	uPSSize		= uSPSLength + uPPSLength + 12 + 1;

		BYTE	*pData		= new BYTE[uPSSize];
		UINT    uIndex		= 0;

		if (pData == NULL)
		{
			// no more memory err
			ErrorMessage(PlayerNoMoreMemoryErr);
			return PlayerNoMoreMemoryErr;
		}

		memset(pData, 0, uPSSize);

		pData[0] = pData[1] = pData[2] = 0;
		pData[3] = 1;

		uIndex += 4;

		memcpy((void *)(pData + uIndex), (const void *)(pBits + 8), uSPSLength);

		uIndex += uSPSLength;

		pData[uIndex] = pData[uIndex + 1] = pData[uIndex + 2] = 0;
		pData[uIndex + 3] = 1;

		uIndex += 4;

		memcpy((void *)(pData + uIndex), (const void *)(pBits + uSPSLength + 11), uPPSLength);

		uIndex += uPPSLength;

		pData[uIndex] = pData[uIndex + 1] = pData[uIndex + 2] = 0;
		pData[uIndex + 3] = 1;

		uIndex += 4;

		if (InitAVCDecoder(pData, uIndex))
			err = PlayerDecoderErr;

		delete [] pData;
		pData = NULL;

		// get composition type
		m_uCompositionType = m_pMP4->GetStereoCompositionType(uTrackIndex);
	}

	// for aac
	if (uOTI == 0x40 && uStreamType == 0x05)
	{
		// match track to proper decoder
		m_uAudioTrackIndex = uTrackIndex;

		UINT	uSize	= 2;
		BYTE	*pData	= new BYTE[uSize + 1];

		memcpy(pData, pBits + 22, uSize);
		pData[uSize] = '\0';

		if (InitAACDecoder(pData, uSize))
			err = PlayerDecoderErr;

		delete [] pData;
		pData = NULL;
	}

	// for LASeR
	else if (uOTI == 0xFF && uStreamType == 0x03)
	{
		m_bLASeR = 1;

		m_uLASeRTrackIndex = uTrackIndex;
	}

	return err;
}

/************************************************************************/
UINT8 CMainFrame::MatchDBStatus(const UINT8 uStatus)
{
	if (m_abDBStatus.size() == 0)
		return 0;

	UINT uCount = count_if(m_abDBStatus.begin(), m_abDBStatus.end(),
		bind2nd(not_equal_to<UINT8>(), uStatus));

	return (uCount >= 1) ? 0 : 1;
}

void CMainFrame::SetDBStatus(const UINT8 bStatus)
{
	for (vector<UINT8>::iterator i = m_abDBStatus.begin(); i < m_abDBStatus.end(); ++i)
		*i = bStatus;
}

void CMainFrame::SetDBStatus(const UINT uIndex, const UINT8 bStatus)
{
	if (m_abDBStatus.size() > uIndex)
		m_abDBStatus[uIndex] = bStatus;

	else
	{
		m_abDBStatus.resize(uIndex + 1);
		m_abDBStatus[uIndex] = bStatus;
	}
}

/************************************************************************/
UINT8 CMainFrame::GetPictureDelta()
{
	if ((m_bProgressive == 1) && (m_aauCompositeTime[*m_auVideoTrackIndices.begin()].size() > 1))
	{
		m_uPicDelta = (UINT8)(m_aauCompositeTime[*m_auVideoTrackIndices.begin()][m_uRenderingCount + 1]
		- m_aauCompositeTime[*m_auVideoTrackIndices.begin()][m_uRenderingCount]);

		++m_uRenderingCount;
	}

	return m_uPicDelta;
}

/************************************************************************/
LRESULT CMainFrame::OnPlay(WPARAM wParam, LPARAM lParam)
{
	DoPlay();

	if (m_pMP4 != NULL)
		m_wndView.m_wndScrollBar.MySetTimer();

	return 0;
}

LRESULT CMainFrame::OnPause(WPARAM wParam, LPARAM lParam)
{
	m_bPlayerStatus = PLAYER_PAUSE;

	if (m_pAVCDecoderThread)
		m_pAVCDecoderThread->SuspendThread();

	if (m_pAACDecoderThread)
		m_pAACDecoderThread->SuspendThread();

	if (m_pMP4 != NULL)
		m_wndView.m_wndScrollBar.MyKillTimer();

	return 0;
}

LRESULT CMainFrame::OnStop(WPARAM wParam, LPARAM lParam)
{
	m_bPlayerStatus = PLAYER_STOP;
	m_abDBStatus.clear();

	if (m_pDBGathererThread)
	{
		while (::WaitForSingleObject(m_pDBGathererThread->m_hThread, 0) == WAIT_TIMEOUT)
			Sleep(10);

		m_pDBGathererThread = NULL;
	}

	if (m_pAVCDecoderThread)
	{
		while (::WaitForSingleObject(m_pAVCDecoderThread->m_hThread, 0) == WAIT_TIMEOUT)
			Sleep(10);

		m_pAVCDecoderThread = NULL;
	}

	if (m_pAACDecoderThread)
	{
		while (::WaitForSingleObject(m_pAACDecoderThread->m_hThread, 0) == WAIT_TIMEOUT)
			Sleep(10);

		m_pAACDecoderThread = NULL;
	}

	m_uVideoDBIndex		= 0;
	m_uAudioDBIndex		= 0;
	m_uRenderingCount	= 0;

	m_bFirstTime		= 1;

	m_afPassed.clear();	

	// buffer clear
	deque< deque<BYTE *> >::iterator i;
	deque<BYTE *>::iterator j;

	deque< deque<UINT> >::iterator k;

	if (m_bProgressive == 1)
	{
		for (i = m_aapDB.begin(); i < m_aapDB.end(); ++i)
		{
			for (j = i->begin(); j < i->end(); ++j)
			{
				if (*j)
				{
					delete [] *j;
					*j = NULL;
				}
			}

			i->clear();
		}

		for (k = m_aauRAPoints.begin(); k < m_aauRAPoints.end(); ++k)
			k->clear();

		for (k = m_aauCompositeTime.begin(); k < m_aauCompositeTime.end(); ++k)
			k->clear();
	}

	else if (m_bProgressive == 0)
	{
		for (i = m_aapDB.begin(); i < m_aapDB.end(); ++i)
			i->clear();
	}

	for (k = m_aauUnitSizes.begin(); k < m_aauUnitSizes.end(); ++k)
		k->clear();

	return 0;
}

LRESULT CMainFrame::OnReady(WPARAM wParam, LPARAM lParam)
{
	m_bPlayerStatus = PLAYER_PAUSE;

	if (m_pMP4 != NULL)
	{
		// set decoding buffer
		m_abDBStatus.assign(m_uTrackCount, BUFFER_TEMP_EMPTY);

		// reset composition buffer
		m_wndView.SetCBStatus(m_uTrackCount - 1, BUFFER_TEMP_EMPTY);
		m_wndView.SetCBStatus(BUFFER_TEMP_EMPTY);
		
		m_pDBGathererThread	= ::AfxBeginThread(ThreadDBGatherer, (LPVOID)this, THREAD_PRIORITY_LOWEST);
	}

	// reset pointer of scroll bar
	m_wndView.m_wndScrollBar.Reset();

	PostMessage(WM_PLAY);

	return 0;
}

LRESULT CMainFrame::OnResizeView(WPARAM wParam, LPARAM lParam)
{
	LONG nWidth, nHeight;

	// using LASeR
	if (GetLASeRFlag())
	{
		// move play button
		m_wndView.m_btnPlay.MoveWindow(0, (int)m_wndView.GetViewHeight(),
			PLAYBAR_WIDTH, (int)(m_wndView.GetViewHeight() + PLAYBAR_HEIGHT));

		// move scroll bar
		// setting the bar size
		m_wndView.m_wndScrollBar.SetSize((UINT)(m_wndView.GetViewWidth() - PLAYBAR_WIDTH));

		m_wndView.m_wndScrollBar.MoveWindow(PLAYBAR_WIDTH, (int)m_wndView.GetViewHeight(),
			(int)m_wndView.GetViewWidth(), (int)(m_wndView.GetViewHeight() + PLAYBAR_HEIGHT));
	
		// plus frame
		nWidth	= m_wndView.GetViewWidth() + g_uFrameWidth;
		nHeight	= m_wndView.GetViewHeight() + g_uFrameHeight + PLAYBAR_HEIGHT;
	}

	else
	{
		// move play button
		m_wndView.m_btnPlay.MoveWindow(0, (int)m_uVideoHeight, PLAYBAR_WIDTH, (int)(m_uVideoHeight + PLAYBAR_HEIGHT));

		// move scroll bar
		// setting the bar size
		m_wndView.m_wndScrollBar.SetSize((UINT)(m_uVideoWidth - PLAYBAR_WIDTH));

		m_wndView.m_wndScrollBar.MoveWindow(PLAYBAR_WIDTH, (int)m_uVideoHeight,
			(int)m_uVideoWidth, (int)(m_uVideoHeight + PLAYBAR_HEIGHT));

		// plus frame
		nWidth	= m_uVideoWidth + g_uFrameWidth;
		nHeight	= m_uVideoHeight + g_uFrameHeight + PLAYBAR_HEIGHT;
	}

	// get frame size
	CRect rct;
	GetWindowRect(&rct);

	MoveWindow(rct.left - g_uFrameWidth + 10, rct.top - g_uFrameHeight + 10, nWidth, nHeight);

	return 0;
}

void CMainFrame::DoReady()
{
	if (m_bPlayerStatus == PLAYER_STOP)
		m_bPlayerStatus = PLAYER_PAUSE;

	SetWindowText(m_strFileName);

	if (m_pMP4 != NULL)
	{
		m_abDBStatus.assign(m_uTrackCount, BUFFER_TEMP_EMPTY);

		// reset composition buffer
		m_wndView.SetCBStatus(m_uTrackCount - 1, BUFFER_TEMP_EMPTY);
		m_wndView.SetCBStatus(BUFFER_TEMP_EMPTY);

		// initialize all buffer
		for (UINT i = 0; i < m_uTrackCount; ++i)
		{
			deque<BYTE *> a, b;
			deque<UINT> c;

			m_aapDB.push_back(a);
			m_wndView.m_aapCB.push_back(b);
			m_aauUnitSizes.push_back(c);
		}

		// set video picture delta
		if (m_auVideoTrackIndices.size() > 0)
			m_uPicDelta = (UINT8)((m_pMP4->GetSampleDelta(m_auVideoTrackIndices[0], 0) * 1000)
			/ m_pMP4->GetMediaTimeScale(m_auVideoTrackIndices[0])) - 1;
		
		// in progressive mode, db gatherer is already started
		if (m_bProgressive == 0)
			m_pDBGathererThread	= ::AfxBeginThread(ThreadDBGatherer, (LPVOID)this, THREAD_PRIORITY_LOWEST);

		// if the audio track is existed
		while (m_uAudioTrackIndex != 0xFFFFFFFF)
		{
			// to set aac & wave parameters, get the first unit
			if ((m_abDBStatus[m_uAudioTrackIndex] == BUFFER_NORMAL) && (m_aapDB[m_uAudioTrackIndex].size() > 0))
			{
				InitAACDecoder();				// 2nd initialization
				break;
			}
		}

		// set the duration of scroll bar
		m_wndView.m_wndScrollBar.SetDuration(m_uDuration);
	}
	
	// if local mode, start instantly
	if (m_bProgressive == 0)
		PostMessage(WM_PLAY);
}

void CMainFrame::DoPlay()
{
	m_bPlayerStatus	= PLAYER_PLAYING;
	m_wndView.m_btnPlay.ChangeState(GB_PAUSE);

	// run avc decoder if video track is used
	if (m_auVideoTrackIndices.size() > 0)
	{
		// avc decoder thread will be pause
		if (m_pAVCDecoderThread)
			m_pAVCDecoderThread->ResumeThread();

		// avc decoder thread is null
		else
			m_pAVCDecoderThread	= ::AfxBeginThread(ThreadAVCDecoder, (LPVOID)this);

		// scroll bar is started
		m_wndView.m_wndScrollBar.MySetTimer();
	}

	// run aac decoder if audio track is used
	if (m_uAudioTrackIndex != 0xFFFFFFFF)		// 0xFFFFFFFF : audio track isn't defined
	{
		// aac decoder thread will be pause
		if (m_pAACDecoderThread)
			m_pAACDecoderThread->ResumeThread();

		// aac decoder thread is null
		else
			m_pAACDecoderThread	= ::AfxBeginThread(ThreadAACDecoder, (LPVOID)this);
	}
	
	// renderer is not initialized = first time
	// MP4 or LASeR data is null
	if (!m_wndView.GetRendererStatus())
	{
		if (m_bLASeR == 1)
			m_wndView.PrepareScene();

		else
			m_wndView.PrepareVideo();
	}
}