/*
// This software was originally developed by ETRI and Kyunghee University, in the course
// of development of the ISO/IEC 14496-20 (Lightweight Application Scene Representation 
// -LASeR- and Simple Aggregation Format -SAF-) AMD3: Presentation and Modification of
// Structured Information standard for reference purposes and its performance may not 
// have been optimized.
//
// This software module is an implementation of one or more tools as specified by the
// ISO/IEC 14496-20 AMD3: PMSI standard.
//
// ISO/IEC gives users free license to this software module or modifications thereof
// for use in products claiming conformance to audiovisual and image-coding related
// ITU Recommendations and/or ISO/IEC International Standards.
// 
// ISO/IEC gives users the same free license to this software module or modifications
// thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in products are advised that its use may
// infringe existing patents. 
// 
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
*/

// MainFrm.h : interface of the CMainFrame class
//
#pragma once

#include "ChildView.h"

#include "neaacdec.h"

#include <deque>

#pragma warning(disable:4201)
#include <afxmt.h>
#include <mmsystem.h>


#define INITIAL_WIDTH	480
#define INITIAL_HEIGHT	360

extern CCriticalSection g_cs;

extern UINT	g_uFrameWidth;
extern UINT	g_uFrameHeight;


enum
{
	PlayerNoMoreMemoryErr	= -3,
	PlayerDecoderErr		= -10,
	PlayerUnknownErr		= -12,
	PlayerBufferErr			= -15,
};

enum
{
	PLAYER_STOP,
	PLAYER_PLAYING,
	PLAYER_PAUSE,
};

enum
{
	BUFFER_TEMP_EMPTY		= 1,
	BUFFER_NORMAL			= 2,
	BUFFER_STREAM_END		= 4,
	BUFFER_END				= 8,				// buffer disabled
};


class CMainFrame : public CFrameWnd
{
public:
	CMainFrame();
	virtual ~CMainFrame();

protected: 
	DECLARE_DYNAMIC(CMainFrame)

// Attributes
protected:
	CString			m_strPath;					// path name
	CString			m_strFileName;				// file name

	UINT8			m_bProgressive;				// whether progressive download mode or not
	CString			m_strServerAddr;			// server address (IP or host name)
	UINT			m_uVideoDBIndex;			// video decoding buffer index in progressive download mode
	UINT			m_uAudioDBIndex;			// audio decoding buffer index in progressive download mode
	UINT			m_uRenderingCount;			// rendering count in progressive download mode

	CLibMP4Api		*m_pMP4;					// mp4 api wrapper
	UINT8			m_bLASeR;					// whether LASeR is used or not

	UINT8			m_bPlayerStatus;			// player status 0 : stop, 1 : play, 2 : pause
	vector<UINT8>	m_abDBStatus;				// buffer status 0 : temporary empty, 1 : normal 2 : stream end 3: stream end & empty

	UINT8			m_bFirstTime;				// first time of call
	UINT64			m_uDuration;				// duration (ms)
	UINT			m_uTrackCount;				// total track
	BYTE			*m_pFileStartPoint;			// file start point

	vector<UINT>	m_auVideoTrackIndices;		// video track index
	UINT			m_uAudioTrackIndex;			// audio track index
	UINT			m_uLASeRTrackIndex;			// LASeR track index

	vector<float>	m_afPassed;					// passed gauge

	UINT			m_uVideoWidth;				// for video decoding
	UINT			m_uVideoHeight;
	UINT			m_uPicSize;
	UINT8			m_uPicDelta;
	UINT8			m_uCompositionType;

	NeAACDecHandle	m_hDecoder;					// for audio decoding
	NeAACDecFrameInfo m_frameInfo;				// for audio playing
	HWAVEOUT		m_hDevice;
	int				m_nScheduledBlocks;
	static UINT		m_uPlayedWaveHeaders;
	static WAVEHDR	*m_apPlayedWaveHeaders[32];	// 32 : maximum wave blocks

	deque< deque<BYTE *> >	m_aapDB;			// decoding buffer
	deque< deque<UINT> >	m_aauUnitSizes;		// each decoding unit size in decoding buffer

	deque< deque<UINT> >	m_aauRAPoints;		// random access points
	deque< deque<UINT> >	m_aauCompositeTime;	// composition time

	CWinThread		*m_pDBGathererThread;		// DB gatherer thread
	CWinThread		*m_pAVCDecoderThread;		// avc decoder thread
	CWinThread		*m_pAACDecoderThread;		// aac decoder thread

	BYTE			*m_pYUV;					// avc decoder needs to this

// Operations
public:
	void			DoFileOpen(CString strFileName = _T(""));

	CString			GetPathName()						{return m_strPath;}

	// get mode that is whether progressive or not
	CLibMP4Api		*GetMP4Handle()						{return m_pMP4;}

	UINT8			GetLASeRFlag() const				{return m_bLASeR;}

	UINT			GetMode() const						{return m_bProgressive;}

	UINT8			GetPlayerStatus() const				{return m_bPlayerStatus;}
	void			SetPlayerStatus(const UINT8 bStatus){m_bPlayerStatus = bStatus;}

	UINT8			GetDBStatus(const UINT uIndex)		{return (m_abDBStatus.size() > uIndex) ? m_abDBStatus[uIndex] : BUFFER_END;}
	UINT8			MatchDBStatus(const UINT8 uStatus);
	void			SetDBStatus(const UINT8 bStatus);
	void			SetDBStatus(const UINT uIndex, const UINT8 bStatus);

	UINT64			GetDuration() const					{return m_uDuration;}
	vector<UINT>	GetVideoTrackIndices() const		{return m_auVideoTrackIndices;}
	UINT			GetAudioTrackIndex() const			{return m_uAudioTrackIndex;}

	UINT8			GetPictureDelta();

	UINT			GetVideoWidth() const				{return m_uVideoWidth;}
	UINT			GetVideoHeight() const				{return m_uVideoHeight;}
	
	// gather db (local)
	INT8			GatherLocalDB(const UINT64 uIndex = 0xFFFFFFFFFFFFFFFF);
	// gather db (network)
	INT8			GatherNetworkDB(const UINT64 uIndex = 0xFFFFFFFFFFFFFFFF);

	float			GetBufferState();

	INT8			DecodeAVC();				// decode avc nal unit
	INT8			DecodeAAC();				// decode aac bitstream

	static void CALLBACK WaveProc(HWAVE hWave, UINT uMsg, DWORD dwInstance, DWORD dwParam1, DWORD dwParam2);
	void			FreeWaveMemory();

protected:
	void			ClearPlayerStatus();

	// for mp4 file in LASeR
	void			ParseMP4(CString strFileName = _T(""));
	void			ParseMP4(BYTE *pBits, const UINT uSize);

	// for LASeR xml file
	void			ParseLASeR(CString strFileName = _T(""));
	void			ParseLASeR(BYTE *pBits, const UINT uSize);

	// for LASeR lsr file
	void            ParseLASeRBin(CString strFileName = _T(""));

	// for local player
	INT8			InitDecoders(const UINT uTrackIndex, const UINT uSampleDescIndex = 0);
	// for progressive download service
	INT8			InitDecoders(const UINT uTrackIndex, const UINT8 uOTI, const UINT8 uStreamType, BYTE *pBits);

	INT8			InitAVCDecoder(BYTE *pBits, const UINT uSize);
	// initialize aac decoder
	INT8			InitAACDecoder(BYTE *pBits, const UINT uSize);
	// set the parameter of aac decoder
	INT8			InitAACDecoder();

	void			DoReady();
	void			DoPlay();

// Overrides
public:
	virtual BOOL PreCreateWindow(CREATESTRUCT& cs);
	virtual BOOL OnCmdMsg(UINT nID, int nCode, void* pExtra, AFX_CMDHANDLERINFO* pHandlerInfo);

// Implementation
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

	CChildView    m_wndView;

// Generated message map functions
protected:
	afx_msg int OnCreate(LPCREATESTRUCT lpCreateStruct);
	afx_msg void OnSetFocus(CWnd *pOldWnd);
	DECLARE_MESSAGE_MAP()

public:
	afx_msg void OnFileOpen();
// 	afx_msg void OnFileConnect();
// 	afx_msg void OnToolsOption();
	afx_msg void OnClose();
	afx_msg LRESULT OnPlay(WPARAM wParam, LPARAM lParam);
	afx_msg LRESULT OnPause(WPARAM wParam, LPARAM lParam);
	afx_msg LRESULT OnStop(WPARAM wParam, LPARAM lParam);
	afx_msg LRESULT OnReady(WPARAM wParam, LPARAM lParam);
	afx_msg LRESULT OnResizeView(WPARAM wParam, LPARAM lParam);
};