/*
// This software was originally developed by ETRI and Kyunghee University, in the course
// of development of the ISO/IEC 14496-20 (Lightweight Application Scene Representation 
// -LASeR- and Simple Aggregation Format -SAF-) AMD3: Presentation and Modification of
// Structured Information standard for reference purposes and its performance may not 
// have been optimized.
//
// This software module is an implementation of one or more tools as specified by the
// ISO/IEC 14496-20 AMD3: PMSI standard.
//
// ISO/IEC gives users free license to this software module or modifications thereof
// for use in products claiming conformance to audiovisual and image-coding related
// ITU Recommendations and/or ISO/IEC International Standards.
// 
// ISO/IEC gives users the same free license to this software module or modifications
// thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in products are advised that its use may
// infringe existing patents. 
// 
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
*/
// CScrollTool.cpp :  Դϴ.
//

#include "stdafx.h"
#include "ScrollTool.h"


// CScrollTool

IMPLEMENT_DYNAMIC(CScrollTool, CWnd)

CScrollTool::CScrollTool()
{
	m_uWidth			= 0;

	m_rctSlide.top		= FRAME_HEIGHT;
	m_rctSlide.bottom	= 25 - FRAME_HEIGHT;
	m_rctSlide.left		= HEAD_TAIL_WIDTH;
	m_rctSlide.right	= 0;

	m_uDuration			= 1;

	m_bOnSlide			= 0;

	m_uBufferFrom		= HEAD_TAIL_WIDTH;
	m_uBufferTo			= HEAD_TAIL_WIDTH;

	Reset();
}

CScrollTool::~CScrollTool()
{
	DeleteScrollBitMap();
}


BEGIN_MESSAGE_MAP(CScrollTool, CWnd)
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEMOVE()
	ON_WM_TIMER()
	ON_WM_PAINT()
END_MESSAGE_MAP()


// CScrollTool ޽ óԴϴ.
void CScrollTool::SetImageBar(WORD bar, WORD head, WORD tail)
{	
	m_bitmapBar.LoadBitmap(bar);
	m_bitmapHead.LoadBitmap(head);
	m_bitmapTail.LoadBitmap(tail);

	Invalidate(FALSE);
}

void CScrollTool::Reset()
{
	m_fPointerX			= HEAD_TAIL_WIDTH;

	m_fPointerSpeed		= 0.0;
}

// deleting the bmp object
void CScrollTool::DeleteScrollBitMap(void)
{
	m_bitmapBar.DeleteObject();
	m_bitmapHead.DeleteObject();
	m_bitmapTail.DeleteObject();
}

void CScrollTool::MyKillTimer(void)
{
	KillTimer(0);
}

void CScrollTool::MySetTimer(void)
{
	SetTimer(0, 1000, NULL);
}

void CScrollTool::OnPaint() 
{
	CPaintDC dc(this);								// device context for painting

	DrawFrame(&dc, 0, 0, &m_bitmapHead);
	DrawFrame(&dc, (int)(HEAD_TAIL_WIDTH + m_uWidth), 0, &m_bitmapTail);

	DrawBar(&dc, HEAD_TAIL_WIDTH, 0, &m_bitmapBar);
}

// draw bar, buffer, moving box 
void CScrollTool::DrawBar(CDC *pDC, int x, int y, CBitmap *pBit)
{
	BITMAP Scrollinfo;
	CDC MemDC;
	MemDC.CreateCompatibleDC(pDC);

	CBitmap *pOldBitmap;
	pOldBitmap = MemDC.SelectObject(pBit);

	// draw bar bitmap
	pBit->GetBitmap(&Scrollinfo);
	pDC->StretchBlt(x, y, m_uWidth, Scrollinfo.bmHeight, &MemDC, 0, 0, Scrollinfo.bmWidth, Scrollinfo.bmHeight, SRCCOPY);
	MemDC.SelectObject(pOldBitmap);

	pDC->SelectStockObject(NULL_PEN);

	CBrush brush, *pOldBrush;

	// draw buffer
	brush.CreateSolidBrush(BUFFER_COLOR);
	pOldBrush = pDC->SelectObject(&brush);
	pDC->Rectangle((int)m_uBufferFrom, 9, (int)m_uBufferTo + 1, 17);
	pDC->SelectObject(pOldBrush);
	brush.DeleteObject();

	// draw pointer
	brush.CreateSolidBrush(POINTER_COLOR);
	pOldBrush = pDC->SelectObject(&brush);
	pDC->Rectangle((int)(m_fPointerX - 3.0), 9, (int)(m_fPointerX + 3.0), 17);
	pDC->SelectObject(pOldBrush);
	brush.DeleteObject();
}

// draw head & tail
void CScrollTool::DrawFrame(CDC *pDC, int x, int y, CBitmap *pBit)
{
	BITMAP Scrollinfo;
	CDC MemDC;
	MemDC.CreateCompatibleDC(pDC);

	CBitmap *pOldBitmap;
	pOldBitmap=MemDC.SelectObject(pBit);

	pBit->GetBitmap(&Scrollinfo);
	pDC->BitBlt(x, y, Scrollinfo.bmHeight, Scrollinfo.bmHeight, &MemDC, 0, 0, SRCCOPY);
	MemDC.SelectObject(pOldBitmap);			
}

// movie the moving box using timer
void CScrollTool::PlayBar(void)
{	
	if (m_fPointerX < (float)m_uBufferTo)
	{
		m_fPointerX += m_fPointerSpeed;

		InvalidateRect(&m_rctSlide, 0);

		if (m_fPointerX > (float)m_uBufferTo)
			m_fPointerX = (float)m_uBufferTo;
	}

	// when buffer is end, don't send the message
	if (m_uBufferTo < (HEAD_TAIL_WIDTH + m_uWidth))
		GetParent()->SendMessage(WM_COMMAND, 2, (LPARAM)m_hWnd);
}

void CScrollTool::SetSize(const UINT uWidth /* = 0 */)
{
	if (uWidth != 0)
		m_uWidth = uWidth - (HEAD_TAIL_WIDTH * 2);

	m_rctSlide.right = HEAD_TAIL_WIDTH + m_uWidth;

	m_fPointerSpeed = (float)(m_uWidth * 1000.0 / (double)m_uDuration);
}

// when the user clicks the bar
void CScrollTool::OnLButtonDown(UINT nFlags, CPoint point)
{
	// TODO: ⿡ ޽ ó ڵ带 ߰ /Ǵ ⺻ ȣմϴ.
	if (IsPtOnMe(point))
	{
		SetCapture();

		m_bOnSlide = 1;
	}

	CWnd::OnLButtonDown(nFlags, point);
}

// when the user make mouse button up
void CScrollTool::OnLButtonUp(UINT nFlags, CPoint point)
{
	// TODO: ⿡ ޽ ó ڵ带 ߰ /Ǵ ⺻ ȣմϴ.
	if (GetCapture() == this)
	{
		ReleaseCapture();

		PointerPositioning(point);

		InvalidateRect(&m_rctSlide, 0);
	}

	m_bOnSlide = 0;

	CWnd::OnLButtonUp(nFlags, point);
}

// When the user moves the moving bar, stopping the SetTimer
void CScrollTool::OnMouseMove(UINT nFlags, CPoint point)
{
	// TODO: ⿡ ޽ ó ڵ带 ߰ /Ǵ ⺻ ȣմϴ.
	if (nFlags && MK_LBUTTON && m_bOnSlide)
	{
		PointerPositioning(point);

		InvalidateRect(&m_rctSlide, 0);
	}
	
	CWnd::OnMouseMove(nFlags, point);
}

// whether cursor is on bar or not
UINT8 CScrollTool::IsPtOnMe(CPoint point)
{
	if ((point.x >= HEAD_TAIL_WIDTH) && (point.x <= (LONG)(HEAD_TAIL_WIDTH + m_uWidth))
		&& (point.y >= FRAME_HEIGHT) && (point.y <= (25 - FRAME_HEIGHT)))
		return 1;

	else
		return 0;
}

// when cursor is out of bar position, relocate indicator
void CScrollTool::PointerPositioning(CPoint point)
{
	if (point.x < HEAD_TAIL_WIDTH)
		m_fPointerX = (float)HEAD_TAIL_WIDTH;

	else if (point.x > (LONG)(HEAD_TAIL_WIDTH + m_uWidth))
		m_fPointerX = (float)(HEAD_TAIL_WIDTH + m_uWidth);

	else
	{
		m_fPointerX		= (float)point.x;
		m_uBufferFrom	= point.x;
	}
}

void CScrollTool::OnTimer(UINT_PTR nIDEvent)
{
	// TODO: ⿡ ޽ ó ڵ带 ߰ /Ǵ ⺻ ȣմϴ.
	if (nIDEvent == 0)
		PlayBar();

	CWnd::OnTimer(nIDEvent);
}