/*
// This software was originally developed by ETRI and Kyunghee University, in the course
// of development of the ISO/IEC 14496-20 (Lightweight Application Scene Representation 
// -LASeR- and Simple Aggregation Format -SAF-) AMD3: Presentation and Modification of
// Structured Information standard for reference purposes and its performance may not 
// have been optimized.
//
// This software module is an implementation of one or more tools as specified by the
// ISO/IEC 14496-20 AMD3: PMSI standard.
//
// ISO/IEC gives users free license to this software module or modifications thereof
// for use in products claiming conformance to audiovisual and image-coding related
// ITU Recommendations and/or ISO/IEC International Standards.
// 
// ISO/IEC gives users the same free license to this software module or modifications
// thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in products are advised that its use may
// infringe existing patents. 
// 
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
*/
#pragma once

#define HEAD_TAIL_WIDTH 10
#define FRAME_HEIGHT 5

#define BUFFER_COLOR RGB(255, 0, 0)
#define POINTER_COLOR RGB(0, 0, 0)


// CScrollTool
class CScrollTool : public CWnd
{
	DECLARE_DYNAMIC(CScrollTool)

public:
	CScrollTool();
	virtual ~CScrollTool();

protected:
	DECLARE_MESSAGE_MAP()


public:
	void SetImageBar(WORD bar, WORD head, WORD tail);
	void Reset();
	void DeleteScrollBitMap();

	void MyKillTimer();							// for stopping the timer 
	void MySetTimer();							// for starting the timer
	
	// drawing the scroll bar
	void DrawBar(CDC *pDC, int x, int y, CBitmap *pBit);
	// drawing the scroll frame
	void DrawFrame(CDC *pDC, int x, int y, CBitmap *pBit);
	void PlayBar();								// moving the progress box using the 

	// setting the size if view : width value
	void SetSize(const UINT uWidth = 0);
	// setting the duration of the movie
	void SetDuration(const UINT64 uDuration)	{m_uDuration = uDuration;}
	// assigning the buffer value
	void SetBuffer(const float fBuffer)			{m_uBufferTo = (UINT)(m_uWidth * fBuffer) + 10;}

	UINT8 IsPtOnMe(CPoint point);
	void PointerPositioning(CPoint point);	// when cursor is out of bar position, relocate pointer
	
	afx_msg void OnPaint();
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnLButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnMouseMove(UINT nFlags, CPoint point);
	afx_msg void OnTimer(UINT_PTR nIDEvent);

protected:
	UINT m_uWidth;

	// bitmap resource
	CBitmap m_bitmapBar, m_bitmapHead, m_bitmapTail;

	float m_fPointerX;							// x point of current pointer

	UINT64 m_uDuration;							// duration (ms)
	float m_fPointerSpeed;						// pointer speed (/s)

	CRect m_rctSlide;							// slide rect
	UINT8 m_bOnSlide;							// when drag is started, check the cursor position whether on slide or not

	UINT m_uBufferFrom, m_uBufferTo;			// point of the buffer status
};