#ifndef _LIBDESAF_H
#define _LIBDESAF_H


#include <basetsd.h>
#include <windows.h>
using namespace std;


#ifdef _WIN32_WCE
#define GET_PROC_ADDR	GetProcAddressA
#else
#define GET_PROC_ADDR	GetProcAddress
#endif // _WIN32_WCE


typedef unsigned char BYTE;
typedef unsigned int UINT;


typedef UINT8	(__cdecl *PROC_DePackSAF)(BYTE *pPacket);
typedef UINT8	(__cdecl *PROC_GetRandomAccessPointFlag)();
typedef UINT	(__cdecl *PROC_GetSequenceNumber)();
typedef UINT	(__cdecl *PROC_GetCTS)();
typedef UINT	(__cdecl *PROC_GetAULength)();
typedef UINT8	(__cdecl *PROC_GetAUType)();
typedef UINT	(__cdecl *PROC_GetStreamID)();
typedef BYTE *	(__cdecl *PROC_GetPayload)();
typedef UINT8	(__cdecl *PROC_GetOTI)(BYTE *pPayload);
typedef UINT8	(__cdecl *PROC_GetStreamType)(BYTE *pPayload);
typedef UINT	(__cdecl *PROC_GetTimeStampResolution)(BYTE *pPayload);
typedef UINT	(__cdecl *PROC_GetBufferSizeDB)(BYTE *pPayload);
typedef UINT	(__cdecl *PROC_GetMimeTypeLength)(BYTE *pPayload);
typedef char *	(__cdecl *PROC_GetMimeType)(BYTE *pPayload);
typedef BYTE *	(__cdecl *PROC_GetDecSpecificInfo)(BYTE *pPayload);


class CLibDeSAFApi
{
protected:
	HMODULE m_hInst;

	PROC_DePackSAF					m_pfnDePackSAF;
	PROC_GetRandomAccessPointFlag	m_pfnGetRandomAccessPointFlag;
	PROC_GetSequenceNumber			m_pfnGetSequenceNumber;
	PROC_GetCTS						m_pfnGetCTS;
	PROC_GetAULength				m_pfnGetAULength;
	PROC_GetAUType					m_pfnGetAUType;
	PROC_GetStreamID				m_pfnGetStreamID;
	PROC_GetPayload					m_pfnGetPayload;

	PROC_GetOTI						m_pfnGetOTI;
	PROC_GetStreamType				m_pfnGetStreamType;
	PROC_GetTimeStampResolution		m_pfnGetTimeStampResolution;
	PROC_GetBufferSizeDB			m_pfnGetBufferSizeDB;
	PROC_GetMimeTypeLength			m_pfnGetMimeTypeLength;
	PROC_GetMimeType				m_pfnGetMimeType;
	PROC_GetDecSpecificInfo			m_pfnGetDecSpecificInfo;

public:
	CLibDeSAFApi()
	{
		// Loading libSAF.dll
		try
		{
#ifdef _WIN32_WCE
			m_hInst = LoadLibrary(_T("libDeSAFCE.dll"));
			if (!m_hInst)						throw(_T("Unable to load libDeSAFCE.dll"));
#else
			m_hInst = LoadLibrary(_T("libDeSAF.dll"));
			if (!m_hInst)						throw(_T("Unable to load libDeSAF.dll"));
#endif

			// Get DLL functions
			m_pfnDePackSAF = (PROC_DePackSAF)GET_PROC_ADDR(m_hInst, "DePackSAF");
			if (!m_pfnDePackSAF)				throw(_T("Don't find 'DePackSAF' function"));

			m_pfnGetRandomAccessPointFlag = (PROC_GetRandomAccessPointFlag)GET_PROC_ADDR(m_hInst, "GetRandomAccessPointFlag");
			if (!m_pfnGetRandomAccessPointFlag)	throw(_T("Don't find 'GetRandomAccessPointFlag' function"));

			m_pfnGetSequenceNumber = (PROC_GetSequenceNumber)GET_PROC_ADDR(m_hInst, "GetSequenceNumber");
			if (!m_pfnGetSequenceNumber)		throw(_T("Don't find 'GetSequenceNumber' function"));

			m_pfnGetCTS = (PROC_GetCTS)GET_PROC_ADDR(m_hInst, "GetCTS");
			if (!m_pfnGetCTS)					throw(_T("Don't find 'GetCTS' function"));

			m_pfnGetAULength = (PROC_GetAULength)GET_PROC_ADDR(m_hInst, "GetAULength");
			if (!m_pfnGetAULength)				throw(_T("Don't find 'GetAULength' function"));

			m_pfnGetAUType = (PROC_GetAUType)GET_PROC_ADDR(m_hInst, "GetAUType");
			if (!m_pfnGetAUType)				throw(_T("Don't find 'GetAUType' function"));

			m_pfnGetStreamID = (PROC_GetStreamID)GET_PROC_ADDR(m_hInst, "GetStreamID");
			if (!m_pfnGetStreamID)				throw(_T("Don't find 'GetStreamID' function"));

			m_pfnGetPayload = (PROC_GetPayload)GET_PROC_ADDR(m_hInst, "GetPayload");
			if (!m_pfnGetPayload)				throw(_T("Don't find 'GetPayload' function"));

			m_pfnGetOTI = (PROC_GetOTI)GET_PROC_ADDR(m_hInst, "GetOTI");
			if (!m_pfnGetOTI)					throw(_T("Don't find 'GetOTI' function"));

			m_pfnGetStreamType = (PROC_GetStreamType)GET_PROC_ADDR(m_hInst, "GetStreamType");
			if (!m_pfnGetStreamType)			throw(_T("Don't find 'GetStreamType' function"));

			m_pfnGetTimeStampResolution = (PROC_GetTimeStampResolution)GET_PROC_ADDR(m_hInst, "GetTimeStampResolution");
			if (!m_pfnGetTimeStampResolution)	throw(_T("Don't find 'GetTimeStampResolution' function"));

			m_pfnGetBufferSizeDB = (PROC_GetBufferSizeDB)GET_PROC_ADDR(m_hInst, "GetBufferSizeDB");
			if (!m_pfnGetBufferSizeDB)			throw(_T("Don't find 'GetBufferSizeDB' function"));

			m_pfnGetMimeTypeLength = (PROC_GetMimeTypeLength)GET_PROC_ADDR(m_hInst, "GetMimeTypeLength");
			if (!m_pfnGetMimeTypeLength)		throw(_T("Don't find 'GetMimeTypeLength' function"));

			m_pfnGetMimeType = (PROC_GetMimeType)GET_PROC_ADDR(m_hInst, "GetMimeType");
			if (!m_pfnGetMimeType)				throw(_T("Don't find 'GetMimeType' function"));

			m_pfnGetDecSpecificInfo = (PROC_GetDecSpecificInfo)GET_PROC_ADDR(m_hInst, "GetDecSpecificInfo");
			if (!m_pfnGetDecSpecificInfo)		throw(_T("Don't find 'GetDecSpecificInfo' function"));
		}

		catch (TCHAR *szErrText)
		{
			m_hInst = NULL;

#ifdef _AFXDLL
			AfxMessageBox(szErrText);
#elif _CONSOLE
			cout << endl << szErrText << endl;
#else
			MessageBox(0, szErrText, _T("error"), 0);
#endif
		}
	}

	~CLibDeSAFApi()
	{
		if (m_hInst)
		{
			FreeLibrary(m_hInst);
			m_hInst = NULL;
		}
	}

	HMODULE	TestRightDLL()	{return m_hInst;}

	// wrapper functions
	UINT8	DePackSAF(BYTE *pPacket)
	{return m_pfnDePackSAF(pPacket);}

	UINT8	GetRandomAccessPointFlag()
	{return m_pfnGetRandomAccessPointFlag();}

	UINT	GetSequenceNumber()
	{return m_pfnGetSequenceNumber();}

	UINT	GetCTS()
	{return m_pfnGetCTS();}

	UINT	GetAULength()
	{return m_pfnGetAULength();}

	UINT8	GetAUType()
	{return m_pfnGetAUType();}

	UINT	GetStreamID()
	{return m_pfnGetStreamID();}

	BYTE	*GetPayload()
	{return m_pfnGetPayload();}

	UINT8	GetOTI(BYTE *pPayload)
	{return m_pfnGetOTI(pPayload);}

	UINT8	GetStreamType(BYTE *pPayload)
	{return m_pfnGetStreamType(pPayload);}

	UINT	GetTimeStampResolution(BYTE *pPayload)
	{return m_pfnGetTimeStampResolution(pPayload);}

	UINT	GetBufferSizeDB(BYTE *pPayload)
	{return m_pfnGetBufferSizeDB(pPayload);}

	UINT	GetMimeTypeLength(BYTE *pPayload)
	{return m_pfnGetMimeTypeLength(pPayload);}

	char	*GetMimeType(BYTE *pPayload)
	{return m_pfnGetMimeType(pPayload);}

	BYTE	*GetDecSpecificInfo(BYTE *pPayload)
	{return m_pfnGetDecSpecificInfo(pPayload);}
};

#endif