#ifndef _LIBSAF_H
#define _LIBSAF_H


#include <vector>
#include <basetsd.h>
#include <windows.h>
using namespace std;


#ifdef _WIN32_WCE
#define GET_PROC_ADDR	GetProcAddressA
#else
#define GET_PROC_ADDR	GetProcAddress
#endif // _WIN32_WCE


typedef unsigned char BYTE;
typedef unsigned int UINT;


typedef void	(__cdecl *PROC_InitMP4Packetizer)(CLibMP4Api *pMP4Api);
typedef void	(__cdecl *PROC_CloseMP4Packetizer)(CLibMP4Api *pMP4Api);
typedef UINT8	(__cdecl *PROC_GetPacket)(CLibMP4Api *pMP4Api, void *pMP4, vector<BYTE> &aPacket, const UINT64 uIndex);


class CLibSAFApi
{
protected:
	HMODULE m_hInst;

	PROC_InitMP4Packetizer	m_pfnInitMP4Packetizer;
	PROC_CloseMP4Packetizer	m_pfnCloseMP4Packetizer;
	PROC_GetPacket			m_pfnGetPacket;

public:
	CLibSAFApi()
	{
		// Loading libSAF.dll
		try
		{
			m_hInst = LoadLibrary(_T("libSAF.dll"));
			if (!m_hInst)				throw(_T("Unable to load libSAF.dll"));

			// Get DLL functions
			m_pfnInitMP4Packetizer = (PROC_InitMP4Packetizer)GET_PROC_ADDR(m_hInst, "InitMP4Packetizer");
			if (!m_pfnInitMP4Packetizer)	throw(_T("Don't find 'InitMP4Packetizer' function"));

			m_pfnCloseMP4Packetizer = (PROC_CloseMP4Packetizer)GET_PROC_ADDR(m_hInst, "CloseMP4Packetizer");
			if (!m_pfnCloseMP4Packetizer)	throw(_T("Don't find 'CloseMP4Packetizer' function"));

			m_pfnGetPacket = (PROC_GetPacket)GET_PROC_ADDR(m_hInst, "GetPacket");
			if (!m_pfnGetPacket)		throw(_T("Don't find 'GetPacket' function"));
		}

		catch (TCHAR *szErrText)
		{
			m_hInst = NULL;

#ifdef _AFXDLL
			AfxMessageBox(szErrText);
#elif _CONSOLE
			cout << endl << szErrText << endl;
#else
			MessageBox(0, szErrText, _T("error"), 0);
#endif
		}
	}

	~CLibSAFApi()
	{
		if (m_hInst)
		{
			FreeLibrary(m_hInst);
			m_hInst = NULL;
		}
	}

	HMODULE	TestRightDLL()	{return m_hInst;}

	// wrapper functions
	void	InitMP4Packetizer(CLibMP4Api *pMP4Api)
	{return m_pfnInitMP4Packetizer(pMP4Api);}

	void	CloseMP4Packetizer(CLibMP4Api *pMP4Api)
	{return m_pfnCloseMP4Packetizer(pMP4Api);}

	UINT8	GetPacket(CLibMP4Api *pMP4Api, void *pMP4, vector<BYTE> &aPacket, const UINT64 uIndex = 0xFFFFFFFFFFFFFFFF)
	{return m_pfnGetPacket(pMP4Api, pMP4, aPacket, uIndex);}
};

#endif