/*
// This software was originally developed by ETRI and Kyunghee University, in the course
// of development of the ISO/IEC 14496-20 (Lightweight Application Scene Representation 
// -LASeR- and Simple Aggregation Format -SAF-) AMD3: Presentation and Modification of
// Structured Information standard for reference purposes and its performance may not 
// have been optimized.
//
// This software module is an implementation of one or more tools as specified by the
// ISO/IEC 14496-20 AMD3: PMSI standard.
//
// ISO/IEC gives users free license to this software module or modifications thereof
// for use in products claiming conformance to audiovisual and image-coding related
// ITU Recommendations and/or ISO/IEC International Standards.
// 
// ISO/IEC gives users the same free license to this software module or modifications
// thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in products are advised that its use may
// infringe existing patents. 
// 
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
*/
#ifndef _SCENE_TREE_MANAGER_H_
#define _SCENE_TREE_MANAGER_H_

#include "..\libLASeR\LASeR.h"
#include "..\libLASeR\SAF.h"


class CSceneTreeManager
{
protected:
	CSAFSession	*m_pRoot;

	CLASeR		*m_pCurLASeRElement;
	CSAF		*m_pCurSAFElement;

	UINT		m_uNextIndex;

public:
	vector<CLASeR *>			m_apNamedLASeRElement;
	vector<CESHeader *>			m_apNamedSAFElement;

	vector<CLSRConditional *>	m_apConditionalElement;

public:
	CSceneTreeManager();
	~CSceneTreeManager();

	void		ResetSceneTree();

	lsrErr		SetRoot(void *pRoot);
	CSAFSession	*GetRoot()				{return m_pRoot;}

	void		SetCurLASeRElement(CLASeR *pCurLASeRElement)
	{m_pCurLASeRElement = pCurLASeRElement;}
	void		SetCurSAFElement(CSAF *pCurSAFElement)
	{m_pCurSAFElement = pCurSAFElement;}

	CLASeR		*GetCurLASeRElement()	{return m_pCurLASeRElement;}
	CSAF		*GetCurSAFElement()		{return m_pCurSAFElement;}

	void		SetIndex(UINT uIndex)	{m_uNextIndex = uIndex;}
	UINT		GetIndex() const		{return m_uNextIndex;}

	lsrErr		GetCousin(CLASeR *pCurLASeRElement, CLASeR **pLASeRElement);
	lsrErr		GetCousin(CSAF *pCurSAFElement, CSAF **pSAFElement);
};

extern "C" _declspec(dllexport)
lsrErr __stdcall SetSceneRoot(const void *pRoot);

extern "C" _declspec(dllexport)
CSAFSession * __stdcall GetSceneRoot();

extern "C" _declspec(dllexport)
void __stdcall CloseSceneRoot();

extern "C" _declspec(dllexport)
CLASeR * __stdcall GetSVG();

extern "C" _declspec(dllexport)
lsrErr __stdcall GetNextObject(CLASeR **pLASeRElement, CSAF **pSAFElement);

extern "C" _declspec(dllexport)
CLASeR * __stdcall SearchLASeRElementUsingID(tstring strID);

extern "C" _declspec(dllexport)
lsrErr __stdcall AddLASeRElementUsingID(CLASeR *pNew);

extern "C" _declspec(dllexport)
lsrErr __stdcall DeleteLASeRElementUsingID(CLASeR *pNew);

extern "C" _declspec(dllexport)
CSAF * __stdcall SearchSAFElementUsingID(tstring strID);

extern "C" _declspec(dllexport)
CLSRConditional * __stdcall SearchConditional(tstring strValue);

extern "C" _declspec(dllexport)
lsrErr __stdcall ProcessCommand(CLASeR *pCommand);

#endif