/*
// This software was originally developed by ETRI and Kyunghee University, in the course
// of development of the ISO/IEC 14496-20 (Lightweight Application Scene Representation 
// -LASeR- and Simple Aggregation Format -SAF-) AMD3: Presentation and Modification of
// Structured Information standard for reference purposes and its performance may not 
// have been optimized.
//
// This software module is an implementation of one or more tools as specified by the
// ISO/IEC 14496-20 AMD3: PMSI standard.
//
// ISO/IEC gives users free license to this software module or modifications thereof
// for use in products claiming conformance to audiovisual and image-coding related
// ITU Recommendations and/or ISO/IEC International Standards.
// 
// ISO/IEC gives users the same free license to this software module or modifications
// thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in products are advised that its use may
// infringe existing patents. 
// 
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
*/
#include "..\ComplexType.h"
#include "..\Datatypes.h"


CSVGLocatable::CSVGLocatable()
{
	m_strTransform = _T("");
}

CSVGLocatable::~CSVGLocatable()
{
	m_afRotation.clear();
	m_afScale.clear();
	m_afTranslation.clear();
	m_afMatrix.clear();

	m_auTransformOrder.clear();
}

/************************************************************************/
// SetAttrs
// Desc		: function of 'locatable' attribute group to set the attributes
// Param	: DOM tree
// History
//	PSH (2008. 8. 16)	- first release
/************************************************************************/
lsrErr CSVGLocatable::SetAttrs(LPXNode pScene, CLASeR *pI)
{
	lsrErr err = LASeRNoErr;

	const TCHAR *szTemp = NULL;

	// temporary variable
	float fTemp = 0;

	/************************************************************************/
	// set attribute
	// transform
	szTemp = pScene->GetAttrValue(_T("transform"));
	if (szTemp)
		ParseTransform(szTemp);

	// rotation
	szTemp = pScene->GetAttrValue(_T("lsr:rotation"));
	if (szTemp)
	{
		fTemp = (float)_tcstod(szTemp, NULL);

		m_afRotation.push_back(fTemp);
		m_afRotation.push_back(0);
		m_afRotation.push_back(0);

		m_auTransformOrder.push_back(1);
	}

	// scale
	szTemp = pScene->GetAttrValue(_T("lsr:scale"));
	if (szTemp)
	{
		m_afScale = ReadFloatListType(szTemp, &err);

		if (m_afScale.size() < 2)
			m_afScale.push_back(*m_afScale.begin());

		else if (m_afScale.size() > 2)
			err = LASeRAttrNotFoundErr;

		m_auTransformOrder.push_back(2);
	}

	// translation
	szTemp = pScene->GetAttrValue(_T("lsr:translation"));
	if (szTemp)
	{
		m_afTranslation = ReadFloatListType(szTemp, &err);

		if (m_afTranslation.size() != 2)
			err = LASeRAttrNotFoundErr;

		m_auTransformOrder.push_back(3);
	}

	return err;
}

/************************************************************************/
// GetAttribute
// Desc		: function of 'locatable' attribute group to get attribute value using it's type
// Param	: attribute type
// History
//	PSH (2008. 8. 28)	- first release
//	PSH (2009. 7. 1)	- add rotation, scale, translate count part
/************************************************************************/
tstring CSVGLocatable::GetStrAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	switch (uAttrType)
	{
	case ATTR_TRANSFORM:
		return m_strTransform;
		break;

	default:
		*pErr = LASeRAttrNotFoundErr;
		break;
	}

	return _T("");
}

vector<UINT8> CSVGLocatable::GetUINT8ListAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	switch (uAttrType)
	{
	case ATTR_TRANSFORM_ORDER:
		return m_auTransformOrder;
		break;

	default:
		*pErr = LASeRAttrNotFoundErr;
		break;
	}

	vector<UINT8> auTemp;
	return auTemp;
}

vector<float> CSVGLocatable::GetFloatListAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	switch (uAttrType)
	{
	case ATTR_LSR_ROTATION:
		return m_afRotation;
		break;

	case ATTR_LSR_SCALE:
		return m_afScale;
		break;

	case ATTR_LSR_TRANSLATION:
		return m_afTranslation;
		break;

	case ATTR_LSR_MATRIX:
		return m_afMatrix;
		break;

	default:
		*pErr = LASeRAttrNotFoundErr;
		break;
	}

	vector<float> afTemp;
	return afTemp;
}

/************************************************************************/
// GetAttrType
// Desc         : function of 'locatable' attribute group to get attribute type using it's name
// Param        : attribute name
// History
//	PSH (2008. 8. 26)       - first release
/************************************************************************/
UINT CSVGLocatable::GetAttrType(tstring strAttrName)
{
	if (strAttrName == _T("lsr:rotation"))			return ATTR_LSR_ROTATION;
	else if (strAttrName == _T("lsr:scale"))		return ATTR_LSR_SCALE;
	else if (strAttrName == _T("lsr:translation"))	return ATTR_LSR_TRANSLATION;
	else if (strAttrName == _T("transform"))		return ATTR_TRANSFORM;
	else											return 0;
}

/************************************************************************/
// GetAttrValType
// Desc         : function of 'locatable' attribute group to get attribute value type using it's type
// Param        : attribute type
// History
//	PSH (2008. 11. 7)       - first release
/************************************************************************/
UINT8 CSVGLocatable::GetAttrValType(UINT uAttrType)
{
	switch (uAttrType)
	{
	case ATTR_LSR_ROTATION:
	case ATTR_LSR_SCALE:
	case ATTR_LSR_TRANSLATION:
		return FLOAT_TYPE;
		break;

	case ATTR_TRANSFORM:
		return STRING_TYPE;
		break;

	default:
		return 0;
		break;
	}
}

/************************************************************************/
// SetAttr
// Desc		: function of 'locatable' attribute group to set attribute value using it's name
// Param	: target attribute name, value
// History
//	PSH (2008. 8. 28)	- first release
/************************************************************************/
lsrErr CSVGLocatable::SetAttr(UINT uAttrType, tstring strValue)
{
	lsrErr err = LASeRNoErr;

	switch (uAttrType)
	{
	case ATTR_TRANSFORM:
		m_strTransform = strValue;

		m_afRotation.clear();
		m_afScale.clear();
		m_afTranslation.clear();
		m_afMatrix.clear();

		m_auTransformOrder.clear();

		ParseTransform(strValue.c_str());

		break;

	default:
		err = LASeRAttrNotFoundErr;
		break;
	}

	return err;
}

lsrErr CSVGLocatable::SetAttr(UINT uAttrType, float fValue)
{
	lsrErr err = LASeRNoErr;

	switch (uAttrType)
	{
	case ATTR_LSR_ROTATION:
		m_afRotation.clear();
		m_afRotation.push_back(fValue);
		break;

	case ATTR_LSR_SCALE:
		m_afScale.clear();
		m_afScale.push_back(fValue);
		m_afScale.push_back(fValue);
		break;

	default:
		err = LASeRAttrNotFoundErr;
		break;
	}

	return err;
}

//pmsi 100715

lsrErr CSVGLocatable::SetAttr(UINT uAttrType, vector<float> afValue)
{
	lsrErr err = LASeRNoErr;

	switch (uAttrType)
	{
	case ATTR_LSR_TRANSLATION:
		m_afTranslation.clear();
		m_afTranslation.push_back(afValue[0]);
		m_afTranslation.push_back(afValue[1]);
		m_auTransformOrder.push_back(3);
		break;

	default:
		err = LASeRAttrNotFoundErr;
		break;
	}

	return err;
}

/************************************************************************/
// ParseTransform
// Desc		: parse the transform attribute
// Param	: transform string value
// History
//	PSH (2009. 7. 6)	- first release
/************************************************************************/
lsrErr CSVGLocatable::ParseTransform(const TCHAR *szValue)
{
	lsrErr err = LASeRNoErr;

	// temporary variable
	vector<float> afTemp;

	m_strTransform = szValue;

	vector<tstring> astrTemp = ReadTransformType(szValue, &err);

	for (vector<tstring>::iterator i = astrTemp.begin(); i < astrTemp.end(); ++i)
	{
		szValue	= (*i).c_str();

		afTemp	= ReadFloatListType(szValue, &err);

		if (_tcsstr(szValue, _T("rotate")))
		{
			if (afTemp.size() < 3)
			{
				afTemp.push_back(0);

				if (afTemp.size() < 3)
					afTemp.push_back(0);
			}

			else if (afTemp.size() > 3)
				err = LASeRAttrNotFoundErr;

			copy(afTemp.begin(), afTemp.end(), back_inserter(m_afRotation));

			m_auTransformOrder.push_back(1);
		}

		else if (_tcsstr(szValue, _T("scale")))
		{
			if (afTemp.size() < 2)
				afTemp.push_back((*afTemp.begin()));

			else if (afTemp.size() > 2)
				err = LASeRAttrNotFoundErr;

			copy(afTemp.begin(), afTemp.end(), back_inserter(m_afScale));

			m_auTransformOrder.push_back(2);
		}

		else if (_tcsstr(szValue, _T("translate")))
		{
			if (afTemp.size() < 2)
				afTemp.push_back(0);

			else if (afTemp.size() > 2)
				err = LASeRAttrNotFoundErr;

			copy(afTemp.begin(), afTemp.end(), back_inserter(m_afTranslation));

			m_auTransformOrder.push_back(3);
		}

		else if (_tcsstr(szValue, _T("matrix")))
		{
			if (afTemp.size() != 6)
				err = LASeRAttrNotFoundErr;

			copy(afTemp.begin(), afTemp.end(), back_inserter(m_afMatrix));

			m_auTransformOrder.push_back(4);
		}

		afTemp.clear();
	}

	return err;
}