/*
// This software was originally developed by ETRI and Kyunghee University, in the course
// of development of the ISO/IEC 14496-20 (Lightweight Application Scene Representation 
// -LASeR- and Simple Aggregation Format -SAF-) AMD3: Presentation and Modification of
// Structured Information standard for reference purposes and its performance may not 
// have been optimized.
//
// This software module is an implementation of one or more tools as specified by the
// ISO/IEC 14496-20 AMD3: PMSI standard.
//
// ISO/IEC gives users free license to this software module or modifications thereof
// for use in products claiming conformance to audiovisual and image-coding related
// ITU Recommendations and/or ISO/IEC International Standards.
// 
// ISO/IEC gives users the same free license to this software module or modifications
// thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in products are advised that its use may
// infringe existing patents. 
// 
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
*/
#include "..\LASeR.h"
#include "..\Datatypes.h"


CLSRExternalReference::CLSRExternalReference()
{
	CSVGObject::CSVGObject();

	m_uElementType		= LSR_externalReference;

	m_pProperties		= new CSVGProperties();

	m_fUpdateInterval	= 0.0;
	m_uSecurity			= 0;
	//entry
}

CLSRExternalReference::~CLSRExternalReference()
{
	DELETEMEMORY(m_pProperties);
}

/************************************************************************/
// SetAttribute
// Desc		: function of 'externalReference' to set the children elements or the attributes
// Param	: DOM tree
// History
//	PYC (2010. 3. 10)	- first release
/************************************************************************/
lsrErr CLSRExternalReference::SetAttrs(LPXNode pScene, CLASeR *pI)
{
	lsrErr err = LASeRNoErr;

	if (!m_pProperties)
	{
		err = LASeRExternalReferenceErr;
		return err;
	}

	const TCHAR *szTemp = NULL;

	/************************************************************************/
	// set attribute
	// svg object
	err = CSVGObject::SetAttrs(pScene, pI);
	CONFIRM(err);

	// svg properties
	err = m_pProperties->SetAttrs(pScene, pI);
	CONFIRM(err);

	// updateInterval
	szTemp = pScene->GetAttrValue(_T("updateInterval"));
	if (szTemp)
	{
		m_fUpdateInterval = ReadIntervalType(szTemp, &err);
		CONFIRM(err);
	}

	// security
	szTemp = pScene->GetAttrValue(_T("security"));
	if (szTemp)
	{
		m_uSecurity = ReadSecurityType(szTemp, &err);
		CONFIRM(err);
	}

	// set children
	err = AddChildren(pScene);
	CONFIRM(err);		

	return err;
}

/************************************************************************/
// CheckRigthChild
// Desc		: function to check the right child element
// Param	: element type
// History
//	PYC (2010. 3. 10)	- first release
/************************************************************************/
lsrErr CLSRExternalReference::CheckRightChild(UINT8 uElementType)
{
	lsrErr err = LASeRNoErr;

	switch (uElementType)
	{
	case SVG_mpath:
	case SVG_stop:
	case SVG_svg:
	case SVG_tspan:
	case SVGT_fontfacesrc:
	case SVGT_fontfaceuri:
	case SVGT_glyph:
	case SVGT_hkern:
	case SVGT_missingglyph:
	case SVG_tBreak:
		err = LASeRExternalReferenceErr;
		return err;
		break;

	default:
		if ((uElementType < SVG_a) || (uElementType > SVG_textArea))
			err = LASeRExternalReferenceErr;

		break;
	}

	return err;
}

/************************************************************************/
// GetAttribute
// Desc		: function of 'externalReference' to get attribute value using it's type
// Param	: attribute type
// History
//	PYC (2010. 3. 10)	- first release
/************************************************************************/
float CLSRExternalReference::GetFloatAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	float fTemp = 0.0;

	switch (uAttrType)
	{
	case ATTR_LSR_UPDATEINTERVAL:
		return m_fUpdateInterval;
		break;

	default:
		fTemp = m_pProperties->GetFloatAttr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return fTemp;

		*pErr = LASeRExternalReferenceErr;

		break;
	}

	return fTemp;
}

UINT8 CLSRExternalReference::GetUINT8Attr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	UINT8 uTemp = 0;

	switch (uAttrType)
	{
	case ATTR_LSR_SECURITY:
		return m_uSecurity;
		break;

	default:
		uTemp = CSVGObject::GetUINT8Attr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return uTemp;
		else *pErr = LASeRNoErr;

		uTemp = m_pProperties->GetUINT8Attr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return uTemp;

		*pErr = LASeRExternalReferenceErr;

		break;
	}

	return uTemp;
}

tstring CLSRExternalReference::GetStrAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	tstring strTemp = _T("");

	strTemp = CSVGObject::GetStrAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return strTemp;

	*pErr = LASeRExternalReferenceErr;


	return strTemp;
}

vector<UINT8> CLSRExternalReference::GetUINT8ListAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	vector<UINT8> auTemp;

	auTemp = m_pProperties->GetUINT8ListAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return auTemp;

	*pErr = LASeRExternalReferenceErr;

	return auTemp;
}

vector<float> CLSRExternalReference::GetFloatListAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	vector<float> afTemp;

	afTemp = m_pProperties->GetFloatListAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return afTemp;

	*pErr = LASeRExternalReferenceErr;

	return afTemp;
}

/************************************************************************/
// GetAttribute
// Desc		: function of 'externalReference' to get attribute value using it's type
// Param	: attribute type
// History
//	PYC (2010. 3. 10)	- first release
/************************************************************************/
UINT CLSRExternalReference::GetAttrType(tstring strAttrName)
{
	UINT uTemp = 0;

	if (strAttrName == _T("updateInterval"))	return ATTR_LSR_UPDATEINTERVAL;
	else if (strAttrName == _T("security"))		return ATTR_LSR_SECURITY;
	else
	{
		uTemp = CSVGObject::GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pProperties->GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;
	}

	return uTemp;
}

/************************************************************************/
// GetAttribute
// Desc		: function of 'externalReference' to get attribute value using it's type
// Param	: attribute type
// History
//	PYC (2010. 3. 10)	- first release
/************************************************************************/
UINT8 CLSRExternalReference::GetAttrValType(UINT uAttrType)
{
	UINT8 uTemp = 0;

	switch (uAttrType)
	{
	case ATTR_LSR_UPDATEINTERVAL:
		return FLOAT_TYPE;
		break;

	case ATTR_LSR_SECURITY:
		return UINT8_TYPE;
		break;

	default:
		uTemp = CSVGObject::GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pProperties->GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		break;
	}

	return uTemp;
}