/*
// This software was originally developed by ETRI and Kyunghee University, in the course
// of development of the ISO/IEC 14496-20 (Lightweight Application Scene Representation 
// -LASeR- and Simple Aggregation Format -SAF-) AMD3: Presentation and Modification of
// Structured Information standard for reference purposes and its performance may not 
// have been optimized.
//
// This software module is an implementation of one or more tools as specified by the
// ISO/IEC 14496-20 AMD3: PMSI standard.
//
// ISO/IEC gives users free license to this software module or modifications thereof
// for use in products claiming conformance to audiovisual and image-coding related
// ITU Recommendations and/or ISO/IEC International Standards.
// 
// ISO/IEC gives users the same free license to this software module or modifications
// thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in products are advised that its use may
// infringe existing patents. 
// 
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
*/
#include "..\svg.h"
#include "..\Datatypes.h"


CSVGTref::CSVGTref()
{
	CSVGObject::CSVGObject();

	m_uElementType				= SVG_tref;

	m_pProperties				= new CSVGProperties();

	m_bExternalResourceRequired	= 0;
	m_uEditable					= 0;
	m_pHref						= new CSVGHref();
}

CSVGTref::~CSVGTref()
{
	DELETEMEMORY(m_pProperties);
	DELETEMEMORY(m_pHref);
}

/************************************************************************/
// SetAttribute
// Desc		: function of 'tref' to set the children elements or the attributes
// Param	: DOM tree
// History
//	PYC (2010. 3. 3)	- first release
/************************************************************************/
lsrErr CSVGTref::SetAttrs(LPXNode pScene, CLASeR *pI)
{
	lsrErr err = LASeRNoErr;

	if (!m_pProperties || !m_pHref)
	{
		err = LASeRTrefErr;
		return err;
	}

	const TCHAR *szTemp = NULL;

	/************************************************************************/
	// set attribute
	// svg object
	err = CSVGObject::SetAttrs(pScene, pI);
	CONFIRM(err);

	// svg properties
	err = m_pProperties->SetAttrs(pScene, pI);
	CONFIRM(err);

	// externalResourcesRequired
	szTemp = pScene->GetAttrValue(_T("externalResourcesRequired"));
	if (szTemp)
	{
		m_bExternalResourceRequired = ReadBooleanType(szTemp, &err);
		CONFIRM(err);
	}

	// editable
	szTemp = pScene->GetAttrValue(_T("editable"));
	if (szTemp)
	{
		m_uEditable = ReadEditableType(szTemp, &err);
		CONFIRM(err);
	}

	// svg href
	err = m_pHref->SetAttrs(pScene, pI);
	CONFIRM(err);

	// set children
	err = AddChildren(pScene);
	CONFIRM(err);		

	return err;
}

/************************************************************************/
// GetAttribute
// Desc		: function of 'tref' to get attribute value using it's type
// Param	: attribute type
// History
//	PYC (2010. 3. 3)	- first release
/************************************************************************/
float CSVGTref::GetFloatAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	float fTemp = 0.0;

	fTemp = m_pProperties->GetFloatAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return fTemp;

	*pErr = LASeRTrefErr;

	return fTemp;
}

UINT8 CSVGTref::GetUINT8Attr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	UINT8 uTemp = 0;

	switch (uAttrType)
	{
	case ATTR_EXTERNALRESOURCESREQUIRED:
		return m_bExternalResourceRequired;
		break;

	case ATTR_EDITABLE:
		return m_uEditable;
		break;

	default:
		uTemp = CSVGObject::GetUINT8Attr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return uTemp;
		else *pErr = LASeRNoErr;

		uTemp = m_pProperties->GetUINT8Attr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return uTemp;
		else *pErr = LASeRNoErr;

		uTemp = m_pHref->GetUINT8Attr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return uTemp;

		*pErr = LASeRTrefErr;

		break;
	}

	return uTemp;
}

tstring CSVGTref::GetStrAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	tstring strTemp = _T("");

	strTemp = CSVGObject::GetStrAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return strTemp;
	else *pErr = LASeRNoErr;

	strTemp = m_pHref->GetStrAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return strTemp;

	*pErr = LASeRTrefErr;


	return strTemp;
}

vector<UINT8> CSVGTref::GetUINT8ListAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	vector<UINT8> auTemp;

	auTemp = m_pProperties->GetUINT8ListAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return auTemp;

	*pErr = LASeRTrefErr;

	return auTemp;
}

vector<float> CSVGTref::GetFloatListAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	vector<float> afTemp;

	afTemp = m_pProperties->GetFloatListAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return afTemp;

	*pErr = LASeRTrefErr;

	return afTemp;
}

/************************************************************************/
// GetAttrType
// Desc         : function of 'tref' attribute group to get attribute type using it's name
// Param        : attribute name
// History
//	PYC (2010. 3. 3)	- first release
/************************************************************************/
UINT CSVGTref::GetAttrType(tstring strAttrName)
{
	UINT uTemp = 0;

	if (strAttrName == _T("externalResourceRequired"))	return ATTR_EXTERNALRESOURCESREQUIRED;
	else if (strAttrName == _T("editable"))				return ATTR_EDITABLE;
	else
	{
		uTemp = CSVGObject::GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pProperties->GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pHref->GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;
	}

	return uTemp;
}

/************************************************************************/
// GetAttrValType
// Desc         : function of 'tref' attribute group to get attribute value type using it's type
// Param        : attribute type
// History
//	PYC (2010. 3. 3)	- first release
/************************************************************************/
UINT8 CSVGTref::GetAttrValType(UINT uAttrType)
{
	UINT8 uTemp = 0;

	switch (uAttrType)
	{
	case ATTR_EXTERNALRESOURCESREQUIRED:
	case ATTR_EDITABLE:
		return UINT8_TYPE;
		break;

	default:
		uTemp = CSVGObject::GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pProperties->GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pHref->GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		break;
	}

	return uTemp;
}