/*
// This software was originally developed by ETRI and Kyunghee University, in the course
// of development of the ISO/IEC 14496-20 (Lightweight Application Scene Representation 
// -LASeR- and Simple Aggregation Format -SAF-) AMD3: Presentation and Modification of
// Structured Information standard for reference purposes and its performance may not 
// have been optimized.
//
// This software module is an implementation of one or more tools as specified by the
// ISO/IEC 14496-20 AMD3: PMSI standard.
//
// ISO/IEC gives users free license to this software module or modifications thereof
// for use in products claiming conformance to audiovisual and image-coding related
// ITU Recommendations and/or ISO/IEC International Standards.
// 
// ISO/IEC gives users the same free license to this software module or modifications
// thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in products are advised that its use may
// infringe existing patents. 
// 
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
*/
#include "laser_error.h"
#include "..\SAF.h"


CRemoteStreamHeader::CRemoteStreamHeader()
{
	CESHeader::CESHeader();

	m_uElementType			= SAF_RemoteStreamHeader;

	m_strStreamID			= _T("");			
	m_uObjectTypeIndication	= 0;
	m_uStreamType			= 0;
	m_strSource				= _T("");
}

/************************************************************************/
// SetAttr
// Desc		: function of 'RemoteStreamHeader' to set the attributes
// Param	: DOM tree
// History
//	PYC (2010. 3. 9)	- first release
/************************************************************************/
lsrErr CRemoteStreamHeader::SetAttrs(LPXNode pScene)
{
	lsrErr err = LASeRNoErr;

	const TCHAR *szTemp = NULL;

	/************************************************************************/
	// set attribute
	// ESHeader
	err = CESHeader::SetAttrs(pScene);
	CONFIRM(err);
	/************************************************************************/

	// streamID
	szTemp = pScene->GetAttrValue(_T("streamID"));
	if (szTemp)
	{
		m_strStreamID	= szTemp;
		m_bHasStreamID	= 1;
	}

	// objectTypeIndication
	szTemp = pScene->GetAttrValue(_T("objectTypeIndication"));
	if (szTemp)
		m_uObjectTypeIndication = (UINT8)XStr2Int(pScene->GetAttrValue(_T("objectTypeIndication")));

	// streamType
	szTemp = pScene->GetAttrValue(_T("streamType"));
	if (szTemp)
		m_uStreamType = (UINT8)XStr2Int(pScene->GetAttrValue(_T("streamType")));

	// source
	szTemp = pScene->GetAttrValue(_T("source"));
	if (szTemp)
	{
		m_strSource	= szTemp;
	}

	return err;
}