/*
// This software was originally developed by ETRI and Kyunghee University, in the course
// of development of the ISO/IEC 14496-20 (Lightweight Application Scene Representation 
// -LASeR- and Simple Aggregation Format -SAF-) AMD3: Presentation and Modification of
// Structured Information standard for reference purposes and its performance may not 
// have been optimized.
//
// This software module is an implementation of one or more tools as specified by the
// ISO/IEC 14496-20 AMD3: PMSI standard.
//
// ISO/IEC gives users free license to this software module or modifications thereof
// for use in products claiming conformance to audiovisual and image-coding related
// ITU Recommendations and/or ISO/IEC International Standards.
// 
// ISO/IEC gives users the same free license to this software module or modifications
// thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in products are advised that its use may
// infringe existing patents. 
// 
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
*/
#include "BitReader.h"


// CBitReader
CBitReader::CBitReader(char *szFileData)
{
	m_szStreambytes	= szFileData;
	m_nCurrentBit	= 0;
}

CBitReader::CBitReader(char *szFileData, int uBitReaderPtr)
{
	m_szStreambytes	= szFileData;
	m_nCurrentBit	= uBitReaderPtr;
}

bool* CBitReader::GetBitsfromByte(char szByte)
{
	unsigned char temp = szByte;
	bool bits[8];

	for (int i = 0; i < 8; i++)
	{
		bits[i] = ((temp % 256) > 127);
		temp = temp << 1;
	}

	return bits;
}

bool CBitReader::GetBit()
{
	bool value = 0;
	bool temp[64];

	m_nStartByte	= m_nCurrentBit / 8;
	m_nCurBit		= m_nCurrentBit % 8;
	m_nCurByte		= m_nStartByte;

	temp[0] = GetBitsfromByte(m_szStreambytes[m_nCurByte])[m_nCurBit];
	value = temp[0];
	m_nCurBit++;

	if (m_nCurBit == 8)
	{
		m_nCurByte++;
		m_nCurBit = 0;
	}

	IncCurrentBit(1);

	return value;
}

int CBitReader::GetBits(int uLength)
{
	unsigned long value = 0;
	bool temp[64];

	m_nStartByte	= m_nCurrentBit / 8;
	m_nCurBit		= m_nCurrentBit % 8;
	m_nCurByte		= m_nStartByte;

	for (int i = 0; i < uLength; i++)
	{
		value = value << 1;
		temp[i] = GetBitsfromByte(m_szStreambytes[m_nCurByte])[m_nCurBit];
		value += temp[i];
		m_nCurBit++;

		if (m_nCurBit == 8)
		{
			m_nCurByte++;
			m_nCurBit = 0;
		}
	}

	IncCurrentBit(uLength);

	return ((int)value);
}

int CBitReader::GetByte()
{
	return GetBits(8);
}

vector<int> CBitReader::GetByteArray(int uLength)
{
	vector<int> temp;

	for (int i = 0; i < uLength; i++)
	{
		temp.push_back(GetByte());
	}

	return temp;
}

int CBitReader::GetVlcInt(int uLength)
{
	int k = 1;
	while (GetBit())	k++;
	int i = GetBits(k * (uLength - 1));

	return i;
}

tstring CBitReader::GetString(int uLength, ofstream &xmlfile)
{
	char *temp;
	temp = new char[uLength];
	tstring temp2 = _T("");

	for (int i = 0; i < uLength; i++)
	{
		temp[i] = GetBits(8);
		xmlfile << temp[i];
		temp2 += temp[i];
	}
	return temp2;
}

tstring CBitReader::GetStringList(CBitReader *pBR)
{
	flush(pBR);

	int uLen = pBR->GetVlcInt(8);

	char *temp;
	temp = new char[uLen];
	tstring temp2 = _T("");

	for (int i = 0; i < uLen; i++)
	{
		temp[i] = GetBits(8);
		temp2 += temp[i];
	}
	return temp2;
}

tstring CBitReader::GetStringList(CBitReader *pBR, ofstream &xmlfile)
{
	flush(pBR);

	int uLen = pBR->GetVlcInt(8);

	char *temp;
	temp = new char[uLen];
	tstring temp2 = _T("");

	for (int i = 0; i < uLen; i++)
	{
		temp[i] = GetBits(8);
		xmlfile << temp[i];
		temp2 += temp[i];
	}
	return temp2;
}

void CBitReader::flush(CBitReader *pBR)
{
	if ((pBR->m_nCurrentBit % 8) != 0)
		pBR->IncCurrentBit(8 - (pBR->m_nCurrentBit % 8));
	return;
}