/*
// This software was originally developed by ETRI and Kyunghee University, in the course
// of development of the ISO/IEC 14496-20 (Lightweight Application Scene Representation 
// -LASeR- and Simple Aggregation Format -SAF-) AMD3: Presentation and Modification of
// Structured Information standard for reference purposes and its performance may not 
// have been optimized.
//
// This software module is an implementation of one or more tools as specified by the
// ISO/IEC 14496-20 AMD3: PMSI standard.
//
// ISO/IEC gives users free license to this software module or modifications thereof
// for use in products claiming conformance to audiovisual and image-coding related
// ITU Recommendations and/or ISO/IEC International Standards.
// 
// ISO/IEC gives users the same free license to this software module or modifications
// thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in products are advised that its use may
// infringe existing patents. 
// 
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
*/
#include "PreDecoder.h"


// CDecoder
CDecoder::CDecoder()
{
	CPreDecoder::CPreDecoder();
}


void CDecoder::elements(CSDLClasses *sdl, CBitReader *pBR, ofstream &xmlfile)
{
	int ch4;
	ch4 = pBR->GetBits(6);
	switch (ch4)
	{
	case 0:
		{
			CA *pA = new CA;
			decode(sdl, pA, pBR, xmlfile);
		}
		break;
	case 1:
		{
			CAnimate *pAnimate = new CAnimate;
			decode(sdl, pAnimate, pBR, xmlfile);
		}
		break;
	case 2:// animateColor
		break;
	case 3:// animateMotion
		break;
	case 4:// animateTransform
		break;
	case 5:
// 		{
// 			CAudio *pAudio = new CAudio;
// 			decode(sdl, pAudio, pBR, xmlfile);
// 		}
		break;
	case 6:
		{
			CCircle *pCircle = new CCircle;
			decode(sdl, pCircle, pBR, xmlfile);
		}
		break;
	case 7:// defs
		break;
	case 8:// desc
		break;
	case 9:// ellipse
		{
			CEllipse *pEllipse = new CEllipse;
			decode(sdl, pEllipse, pBR, xmlfile);
		}
		break;
	case 10:// foreignObject
		break;
	case 11:
		{
			CG *pG = new CG;
			decode(sdl, pG, pBR, xmlfile);
 			//delete pG;
		}
		break;
	case 12:
		{
			CImage *pImage = new CImage;
			decode(sdl, pImage, pBR, xmlfile);
			//delete pImage;
		}
		break;
	case 13:// line();
		{
			CLine *pLine = new CLine;
			decode(sdl, pLine, pBR, xmlfile);
		}
		break;
	case 14:// linearGradient();
		break;
	case 15:// metadata();
		break;
	case 16:// mpath();
		break;
	case 17:// path();
		break;
	case 18:// polygon();
		{
			CPolygon *pPolygon = new CPolygon;
			decode(sdl, pPolygon, pBR, xmlfile);
		}
		break;
	case 19:// polyline();
		{
			CPolyline *pPolyline = new CPolyline;
			decode(sdl, pPolyline, pBR, xmlfile);
		}
		break;
	case 20:// radialGradient();;
		break;
	case 21:
		{
			CDRect *pRect = new CDRect;
			decode(sdl, pRect, pBR, xmlfile);
			//delete pRect;
		}
		break;
	case 22:// sameg();
		break;
	case 23:// sameline();
		break;
	case 24:// samepath();
		break;
	case 25:// samepathfill();
		break;
	case 26:// samepolygon();
		break;
	case 27:// samepolygonfill();
		break;
	case 28:// samepolygonstroke();
		break;
	case 29:// samepolyline();;
		break;
	case 30:// samepolylinefill();
		break;
	case 31:// samepolylinestroke();
		break;
	case 32:// samerect();
		break;
	case 33:// samerectfill();
		break;
	case 34:// sametext();
		break;
	case 35:// sametextfill();
		break;
	case 36:// sameuse();
		break;
	case 37:// script();;
		break;
	case 38:// set();
		{
			CSet *pSet = new CSet;
			decode(sdl, pSet, pBR, xmlfile);
		}
		break;
	case 39:// stop();
		break;
	case 40:// switch();
		break;
	case 41:
		{
			CText *pText = new CText;
			decode(sdl, pText, pBR, xmlfile);
//			delete pText;
		}
		break;
	case 42:// title();
		break;
	case 43:// tspan();
		break;
	case 44:// use();
		break;
	case 45:
 		{
 			CVideo *pVideo = new CVideo;
 			decode(sdl, pVideo, pBR, xmlfile);
// 			delete pVideo;
 		}
		break;
	case 46:
		{
			CListener *pListener = new CListener;
			decode(sdl, pListener, pBR, xmlfile);
// 			delete pListener;
		}
		break;
	case 47:// conditional();
		{
			CConditional *pConditional = new CConditional;
			decode(sdl, pConditional, pBR, xmlfile);
// 			delete pConditional;
		}
		break;
	case 48:// cursorManager();
		break;
	case 49:
		anyElement_elements(sdl, pBR, xmlfile);
		break;
	case 50:// privateElementContainer();
		break;
	case 51:// rectClip();
		break;
	case 52:// selector();
		break;
	case 53:// simpleLayout();
		break;
	case 54:
		{
			CTextContent(sdl->byteAlignedString(pBR, xmlfile));
		}
		break;
	}
}

void CDecoder::updates(CSDLClasses *sdl, CBitReader *pBR, ofstream &xmlfile)
{
	int ch4 = pBR->GetBits(4);
	switch (ch4)
	{
	case 0:// Add
		break;
	case 1:// Clean
		break;
	case 2:
		{
			CDDelete *pDelete = new CDDelete;
			decode(sdl, pDelete, pBR, xmlfile);
		}
		break;
	case 3:
		{
			CDInsert *pInsert = new CDInsert;
			decode(sdl, pInsert, pBR, xmlfile);
		}
		break;
	case 4:
		{
//			CNS *pNewScene = new CDNewScene;
			decode(sdl, pBR, xmlfile);
// 			delete pNewScene;
		}
		break;
	case 5:// RefreshScene
		break;
	case 6:
		{
 			CDReplace *pReplace = new CDReplace;
 			decode(sdl, pReplace, pBR, xmlfile);
// 			delete pReplace;
		}
		break;
	case 7:// Restore
		break;
	case 8:// Save
		break;
	case 9:// SendEvent
		break;
	case 10:// ext
		break;
	case 11:// textContent
		break;
	default:
		break;
	}
}

void CDecoder::decode(CSDLClasses *sdl, CLASeRHeader *self, CBitReader *pBR, ofstream &xmlfile)
{
	xmlfile << "\n<lsr:LASeRHeader";

	self->m_nProfile							= pBR->GetBits(8);
	if (self->m_nProfile)						xmlfile << " profile='" << self->m_nProfile << "'";
	self->m_nLevel								= pBR->GetBits(8);
	if (self->m_nLevel)							xmlfile << " level='" << self->m_nLevel << "'";
	self->m_nReserved							= pBR->GetBits(3);
	self->m_nPointsCodec						= pBR->GetBits(2);
	if (self->m_nPointsCodec)					xmlfile << " pointsCodec='" << self->m_nPointsCodec << "'";
	self->m_nPathComponents						= pBR->GetBits(4);
	if (self->m_nPathComponents)				xmlfile << " pathcomponents='" << self->m_nPathComponents << "'";
	if (self->m_nPathComponents == 15)			self->m_nPathComponents = -1;
	self->m_bUseFullRequestHost					= pBR->GetBit();
	if (self->m_bUseFullRequestHost)			xmlfile << " useFullRequestHost='true'";
	if (pBR->GetBit())
	{
		self->m_nTimeResolution = pBR->GetBits(16);
		xmlfile << " timeResolution='" << self->m_nTimeResolution << "'";
		sdl->m_fTimeResolution = (float)self->m_nTimeResolution;
	}
	self->m_nColorComponentBits_minus_1			= pBR->GetBits(4);
	if (self->m_nColorComponentBits_minus_1)	xmlfile << " colorComponentBits='" << self->m_nColorComponentBits_minus_1 + 1 << "'";
	self->m_nResolution							= pBR->GetBits(4);
	if (self->m_nResolution)					xmlfile << " resolution='" << self->m_nResolution << "'";
	if (self->m_nResolution > 7)				self->m_nResolution -= 16;
	sdl->m_nResolution							= self->m_nResolution;
	self->m_nCoordBits							= pBR->GetBits(5);
	if (self->m_nCoordBits)						xmlfile << " coordBits='" << self->m_nCoordBits << "'";
	sdl->m_nCoordBits							= self->m_nCoordBits;
	self->m_nScaleBits_minus_coordBits			= pBR->GetBits(4);
	if (self->m_nScaleBits_minus_coordBits)		xmlfile << " scaleBits='" << self->m_nScaleBits_minus_coordBits + self->m_nCoordBits << "'";
	sdl->m_nScaleBits							= self->m_nScaleBits_minus_coordBits + self->m_nCoordBits;
	self->m_bNewSceneIndicator					= pBR->GetBit();
	pBR->IncCurrentBit(3);	// reserved
	self->m_nExtensionIDBits					= pBR->GetBits(4);
	bool bHasExtensionDescriptor				= pBR->GetBit();
	if (bHasExtensionDescriptor)
	{
		int len;
		len = pBR->GetVlcInt(5);
		for (int i = 0; i < len; i++)
			self->m_anExtensionDescriptor.push_back(pBR->GetBits(8));
	}
	if (pBR->GetBit())							sdl->extension(pBR);

	xmlfile << "/>";
}

void CDecoder::decode(CSDLClasses *sdl, CDDelete *self, CBitReader *pBR, ofstream &xmlfile)
{
	xmlfile << "\n<lsr:Delete";

	if (pBR->GetBit())												// attributeName
	{
		xmlfile << " attributeName='";
		self->m_pAttributeName = sdl->AttributeName(pBR, xmlfile);
		xmlfile << "'";
	}
	if (pBR->GetBit())	{ self->m_nIndex = pBR->GetVlcInt(5); }		// index
	self->m_nRef = sdl->IDREF(pBR);									// ref
	xmlfile << " ref='" << self->m_nRef << "'";
	bool bFlag = false;
	decodeReplaceContent(sdl, bFlag, pBR, xmlfile);

	if (bFlag)
		xmlfile << "\n</lsr:Delete>";
}

void CDecoder::decode(CSDLClasses *sdl, CDInsert *self, CBitReader *pBR, ofstream &xmlfile)
{
	xmlfile << "\n<lsr:Insert";

	if (pBR->GetBit())												// attributeName
	{
		xmlfile << " attributeName='";
		self->m_pAttributeName = sdl->AttributeName(pBR, xmlfile);
		xmlfile << "'";
	}
	if (pBR->GetBit())	{ self->m_nIndex = pBR->GetVlcInt(5); }		// index
	self->m_nRef = sdl->IDREF(pBR);									// ref
	xmlfile << " ref='" << self->m_nRef << "'";
	if (pBR->GetBit())												// value
	{
		xmlfile << " value='";
		self->m_pValue = sdl->updateValue(pBR, sdl->typeOf(self->m_pAttributeName, self->m_nIndex), xmlfile);
		xmlfile << "'";
	}
	bool bFlag = false;
	decodeReplaceContent(sdl, bFlag, pBR, xmlfile);

	if (bFlag)
		xmlfile << "\n</lsr:Insert>";
}

void CDecoder::decode(CSDLClasses *sdl, CBitReader *pBR, ofstream &xmlfile)
{
	xmlfile << "\n<lsr:NewScene>";

	sdl->anyAttribute(pBR, NULL);
	CSvg *pSvg = new CSvg;
	decode(sdl, pSvg, pBR, xmlfile);
// 	delete pSvg;

	xmlfile << "\n</lsr:NewScene>";
}

void CDecoder::decode(CSDLClasses *sdl, CDReplace *self, CBitReader *pBR, ofstream &xmlfile)
{
	xmlfile << "\n<lsr:Replace";

	if (pBR->GetBit())														// attributeName
	{
		xmlfile << " attributeName='";
		self->m_pAttributeName = sdl->AttributeName(pBR, xmlfile);
		xmlfile << "'";
	}
	if (pBR->GetBit())	{ self->m_nIndex = pBR->GetVlcInt(5); }				// index
	if (pBR->GetBit())	{ self->m_nOperandAttribute = pBR->GetBits(8); }	// operandAttribute
	if (pBR->GetBit())	{ self->m_nOperandElementId = sdl->IDREF(pBR); }	// operandElementId
	self->m_nRef = sdl->IDREF(pBR);											// ref
	xmlfile << " ref='" << self->m_nRef << "'";
	if (pBR->GetBit())														// value
	{
		xmlfile << " value='";
		self->m_pValue = sdl->updateValue(pBR, sdl->typeOf(self->m_pAttributeName, self->m_nIndex), xmlfile);
		xmlfile << "'";
	}
	bool bFlag = false;
	decodeReplaceContent(sdl, bFlag, pBR, xmlfile);

	if (bFlag)
		xmlfile << "\n</lsr:Replace>";
}

void CDecoder::decode(CSDLClasses *sdl, CA *self, CBitReader *pBR, ofstream &xmlfile)
{
	xmlfile << "\n<a";

	if (pBR->GetBit())	{ self->m_nId = sdl->ID(pBR, xmlfile); }						// id
	if (pBR->GetBit())	{ decodeRare(sdl, pBR, xmlfile); }								// rare
	if (pBR->GetBit())																	// fill
	{
		xmlfile << " fill='";
		self->m_nFill = sdl->paint(pBR, xmlfile);
		xmlfile << "'";
	}
	if (pBR->GetBit())																	// stroke
	{
		xmlfile << " stroke='";
		self->m_nStroke = sdl->paint(pBR, xmlfile);
		xmlfile << "'";
	}
	self->m_bExternalResourcesRequired	= pBR->GetBit();								// externalResourcesRequired
	if (pBR->GetBit()) { self->m_strTarget = sdl->byteAlignedString(pBR, xmlfile); }	// target
	if (pBR->GetBit())																	// xlink:href
	{
		xmlfile << " xlink:href='";
		self->setHref(sdl->anyURI(pBR, xmlfile));
	}
	decodeContent(sdl, self, pBR, xmlfile);

	xmlfile << "\n</a>";
}

void CDecoder::decode(CSDLClasses *sdl, CAnimate *self, CBitReader *pBR, ofstream &xmlfile)
{
	xmlfile << "\n<animate";

	if (pBR->GetBit())	{ self->m_nId = sdl->ID(pBR, xmlfile); }	// id
	if (pBR->GetBit())	{ decodeRare(sdl, pBR, xmlfile); }			// rare
	if (pBR->GetBit())												// attributeName
	{
		xmlfile << " attributeName='";
		self->m_pAttributeName = sdl->AttributeName(pBR, xmlfile);
		xmlfile << "'";
	}
	
	//

	bool bFlag = false;
	decodeContent(sdl, self, bFlag, pBR, xmlfile);

	if (bFlag)
		xmlfile << "\n</animate>";
}

// void CDecoder::decode(CSDLClasses *sdl, CAudio *self, CBitReader *pBR, ofstream &xmlfile)
// {
// 	//
// }

void CDecoder::decode(CSDLClasses *sdl, CCircle *self, CBitReader *pBR, ofstream &xmlfile)
{
	xmlfile << "\n<circle";

	if (pBR->GetBit())	{ self->m_nId = sdl->ID(pBR, xmlfile); }	// id
	if (pBR->GetBit())	{ decodeRare(sdl, pBR, xmlfile); }			// rare
	if (pBR->GetBit())												// fill
	{
		xmlfile << " fill='";
		self->m_nFill = sdl->paint(pBR, xmlfile);
		xmlfile << "'";
	}
	if (pBR->GetBit())												// stroke
	{
		xmlfile << " stroke='";
		self->m_nStroke = sdl->paint(pBR, xmlfile);
		xmlfile << "'";
	}
	if (pBR->GetBit())												// cx
	{
		xmlfile << " cx='";
		self->m_nCx = sdl->coordinate(pBR);
		xmlfile << self->m_nCx / 4096 << "'";
	}
	if (pBR->GetBit())												// cy
	{
		xmlfile << " cy='";
		self->m_nCy = sdl->coordinate(pBR);
		xmlfile << self->m_nCy / 4096 << "'";
	}
	xmlfile << " r='";												// r
	self->m_nR = sdl->coordinate(pBR);
	xmlfile << self->m_nR / 4096 << "'";
	bool bFlag = false;
	decodeContent(sdl, self, bFlag, pBR, xmlfile);

	if (bFlag)
		xmlfile << "\n</circle>";
}

void CDecoder::decode(CSDLClasses *sdl, CEllipse *self, CBitReader *pBR, ofstream &xmlfile)
{
	xmlfile << "\n<ellipse";

	if (pBR->GetBit())	{ self->m_nId = sdl->ID(pBR, xmlfile); }	// id
	if (pBR->GetBit())	{ decodeRare(sdl, pBR, xmlfile); }			// rare
	if (pBR->GetBit())												// fill
	{
		xmlfile << " fill='";
		self->m_nFill = sdl->paint(pBR, xmlfile);
		xmlfile << "'";
	}
	if (pBR->GetBit())												// stroke
	{
		xmlfile << " stroke='";
		self->m_nStroke = sdl->paint(pBR, xmlfile);
		xmlfile << "'";
	}
	if (pBR->GetBit())												// cx
	{
		xmlfile << " cx='";
		self->m_nCx = sdl->coordinate(pBR);
		xmlfile << self->m_nCx / 4096 << "'";
	}
	if (pBR->GetBit())												// cy
	{
		xmlfile << " cy='";
		self->m_nCy = sdl->coordinate(pBR);
		xmlfile << self->m_nCy / 4096 << "'";
	}
	xmlfile << " rx='";												// rx
	self->m_nRx = sdl->coordinate(pBR);
	xmlfile << self->m_nRx / 4096 << "'";
	xmlfile << " ry='";												// ry
	self->m_nRy = sdl->coordinate(pBR);
	xmlfile << self->m_nRy / 4096 << "'";
	bool bFlag = false;
	decodeContent(sdl, self, bFlag, pBR, xmlfile);

	if (bFlag)
		xmlfile << "\n</ellipse>";
}

void CDecoder::decode(CSDLClasses *sdl, CG *self, CBitReader *pBR, ofstream &xmlfile)
{
	xmlfile << "\n<g";

	if (pBR->GetBit())	{ self->m_nId = sdl->ID(pBR, xmlfile); }	// id
	if (pBR->GetBit())	{ decodeRare(sdl, pBR, xmlfile); }			// rare
	if (pBR->GetBit())												// fill
	{
		xmlfile << " fill='";
		self->m_nFill = sdl->paint(pBR, xmlfile);
		xmlfile << "'";
	}
	if (pBR->GetBit())												// stroke
	{
		xmlfile << " stroke='";
		self->m_nStroke = sdl->paint(pBR, xmlfile);
		xmlfile << "'";
	}
	self->m_bExternalResourcesRequired	= pBR->GetBit();			// externalResourcesRequired
	decodeContent(sdl, self, pBR, xmlfile);

	xmlfile << "\n</g>";
}

void CDecoder::decode(CSDLClasses *sdl, CImage *self, CBitReader *pBR, ofstream &xmlfile)
{
	xmlfile << "\n<image";

	if (pBR->GetBit())	{ self->m_nId = sdl->ID(pBR, xmlfile); }							// id
	if (pBR->GetBit())	{ decodeRare(sdl, pBR, xmlfile); }									// rare
	self->m_bExternalResourcesRequired	= pBR->GetBit();									// externalResourcesRequired
	if (pBR->GetBit())																		// height
	{
		xmlfile << " height='";
		self->m_nHeight = sdl->coordinate(pBR);
		xmlfile << self->m_nHeight / 4096 << "'";
	}
	if (pBR->GetBit())																		// opacity
	{
		xmlfile << " opacity='";
		self->m_nOpacity = sdl->float0to1(pBR, xmlfile);
	}
	if (pBR->GetBit())	{ self->m_nPreserveAspectRatio = sdl->preserveAspectRatio(pBR); }	// preserveAspectRatio
	if (pBR->GetBit())																		// type
	{
		xmlfile << " type='";
		self->m_strType = sdl->byteAlignedString(pBR, xmlfile);
		xmlfile << "'";
	}
	if (pBR->GetBit())																		// width
	{
		xmlfile << " width='";
		self->m_nWidth = sdl->coordinate(pBR);
		xmlfile << self->m_nWidth / 4096 << "'";
	}
	if (pBR->GetBit())																		// x
	{
		xmlfile << " x='";
		self->m_nX = sdl->coordinate(pBR);
		xmlfile << self->m_nX / 4096 << "'";
	}
	if (pBR->GetBit())																		// y
	{
		xmlfile << " y='";
		self->m_nY = sdl->coordinate(pBR);
		xmlfile << self->m_nY / 4096 << "'";
	}
	if (pBR->GetBit())																		// xlink:href
	{
		xmlfile << " xlink:href='";
		self->setHref(sdl->anyURI(pBR, xmlfile));
	}
	if (pBR->GetBit())	{ self->m_nTransformBehavior = pBR->GetBits(4); }					// transformBehavior
	bool bFlag = false;
	decodeContent(sdl, self, bFlag, pBR, xmlfile);

	if (bFlag)
		xmlfile << "\n</image>";
}

void CDecoder::decode(CSDLClasses *sdl, CLine *self, CBitReader *pBR, ofstream &xmlfile)
{
	xmlfile << "\n<line";

	if (pBR->GetBit())	{ self->m_nId = sdl->ID(pBR, xmlfile); }	// id
	if (pBR->GetBit())	{ decodeRare(sdl, pBR, xmlfile); }			// rare
	if (pBR->GetBit())												// fill
	{
		xmlfile << " fill='";
		self->m_nFill = sdl->paint(pBR, xmlfile);
		xmlfile << "'";
	}
	if (pBR->GetBit())												// stroke
	{
		xmlfile << " stroke='";
		self->m_nStroke = sdl->paint(pBR, xmlfile);
		xmlfile << "'";
	}
	if (pBR->GetBit())												// x1
	{
		xmlfile << " x1='";
		self->m_nX1 = sdl->coordinate(pBR);
		xmlfile << self->m_nX1 / 4096 << "'";
	}
	xmlfile << " x2='";												// x2
	self->m_nX2 = sdl->coordinate(pBR);
	xmlfile << self->m_nX2 / 4096 << "'";
	if (pBR->GetBit())												// y1
	{
		xmlfile << " y1='";
		self->m_nY1 = sdl->coordinate(pBR);
		xmlfile << self->m_nY1 / 4096 << "'";
	}
	xmlfile << " y2='";												// y2
	self->m_nY2 = sdl->coordinate(pBR);
	xmlfile << self->m_nY2 / 4096 << "'";
	bool bFlag = false;
	decodeContent(sdl, self, bFlag, pBR, xmlfile);

	if (bFlag)
		xmlfile << "\n</line>";
}

void CDecoder::decode(CSDLClasses *sdl, CPolygon *self, CBitReader *pBR, ofstream &xmlfile)
{
	xmlfile << "\n<polygon";

	if (pBR->GetBit())	{ self->m_nId = sdl->ID(pBR, xmlfile); }	// id
	if (pBR->GetBit())	{ decodeRare(sdl, pBR, xmlfile); }			// rare
	if (pBR->GetBit())												// fill
	{
		xmlfile << " fill='";
		self->m_nFill = sdl->paint(pBR, xmlfile);
		xmlfile << "'";
	}
	if (pBR->GetBit())												// stroke
	{
		xmlfile << " stroke='";
		self->m_nStroke = sdl->paint(pBR, xmlfile);
		xmlfile << "'";
	}
	xmlfile << " points='";											// points
	self->m_anPoints = sdl->pointSequence(pBR, xmlfile);
	xmlfile << "'";
	bool bFlag = false;
	decodeContent(sdl, self, bFlag, pBR, xmlfile);

	if (bFlag)
		xmlfile << "\n</polygon>";
}

void CDecoder::decode(CSDLClasses *sdl, CPolyline *self, CBitReader *pBR, ofstream &xmlfile)
{
	xmlfile << "\n<polyline";

	if (pBR->GetBit())	{ self->m_nId = sdl->ID(pBR, xmlfile); }	// id
	if (pBR->GetBit())	{ decodeRare(sdl, pBR, xmlfile); }			// rare
	if (pBR->GetBit())												// fill
	{
		xmlfile << " fill='";
		self->m_nFill = sdl->paint(pBR, xmlfile);
		xmlfile << "'";
	}
	if (pBR->GetBit())												// stroke
	{
		xmlfile << " stroke='";
		self->m_nStroke = sdl->paint(pBR, xmlfile);
		xmlfile << "'";
	}
	xmlfile << " points='";											// points
	self->m_anPoints = sdl->pointSequence(pBR, xmlfile);
	xmlfile << "'";
	bool bFlag = false;
	decodeContent(sdl, self, bFlag, pBR, xmlfile);

	if (bFlag)
		xmlfile << "\n</polyline>";
}

void CDecoder::decode(CSDLClasses *sdl, CDRect *self, CBitReader *pBR, ofstream &xmlfile)
{
	xmlfile << "\n<rect";

	if (pBR->GetBit())	{ self->m_nId = sdl->ID(pBR, xmlfile); }	// id
	if (pBR->GetBit())	{ decodeRare(sdl, pBR, xmlfile); }			// rare
	if (pBR->GetBit())												// fill
	{
		xmlfile << " fill='";
		self->m_nFill = sdl->paint(pBR, xmlfile);
		xmlfile << "'";
	}
	if (pBR->GetBit())												// stroke
	{
		xmlfile << " stroke='";
		self->m_nStroke = sdl->paint(pBR, xmlfile);
		xmlfile << "'";
	}
	xmlfile << " height='";											// height
	self->m_nHeight = sdl->coordinate(pBR);
	xmlfile << self->m_nHeight / 4096 << "'";
	if (pBR->GetBit())												// rx
	{
		xmlfile << " rx='";
		self->m_nX = sdl->coordinate(pBR);
		xmlfile << self->m_nRx / 4096 << "'";
	}
	if (pBR->GetBit())												// ry
	{
		xmlfile << " ry='";
		self->m_nY = sdl->coordinate(pBR);
		xmlfile << self->m_nRy / 4096 << "'";
	}
	xmlfile << " width='";											// width
	self->m_nWidth = sdl->coordinate(pBR);
	xmlfile << self->m_nWidth / 4096 << "'";
	if (pBR->GetBit())												// x
	{
		xmlfile << " x='";
		self->m_nX = sdl->coordinate(pBR);
		xmlfile << self->m_nX / 4096 << "'";
	}
	if (pBR->GetBit())												// y
	{
		xmlfile << " y='";
		self->m_nY = sdl->coordinate(pBR);
		xmlfile << self->m_nY / 4096 << "'";
	}
	bool bFlag = false;
	decodeContent(sdl, self, bFlag, pBR, xmlfile);

	if (bFlag)
		xmlfile << "\n</rect>";
}

void CDecoder::decode(CSDLClasses *sdl, CSet *self, CBitReader *pBR, ofstream &xmlfile)
{
	xmlfile << "\n<set";

	if (pBR->GetBit())	{ self->m_nId = sdl->ID(pBR, xmlfile); }		// id
	if (pBR->GetBit())	{ decodeRare(sdl, pBR, xmlfile); }				// rare
	if (pBR->GetBit())													// attributeName
	{
		xmlfile << " attributeName='";
		self->m_pAttributeName = sdl->AttributeName(pBR, xmlfile);
		xmlfile << "'";
	}
	if (pBR->GetBit())	{self->m_nAttributeType = pBR->GetBits(2); }	// attributeType
	if (pBR->GetBit())													// begin
	{
		xmlfile << " begin='";
		self->m_apBegin = sdl->timesType(pBR, xmlfile);
		xmlfile << "'";
	}
	if (pBR->GetBit())													// dur
	{
		xmlfile << " dur='";
		self->m_nDur = sdl->time(pBR, xmlfile);
		xmlfile << "'";
	}
	if (pBR->GetBit())													// fill
	{
		self->m_nFill = pBR->GetBit();
		if (self->m_nFill)	{ xmlfile << " fill='remove'"; }
		else { xmlfile << " fill='freeze'"; }
	}
	if (pBR->GetBit())													// repeatCount
	{
	//	self.repeatCount = SDLClasses.repeatCountType(br, "repeatCount");
	}
	if (pBR->GetBit())													// repeatDur
	{
	//	self.repeatDur = SDLClasses.repeatDurType(br, "repeatDur");
	}
	if (pBR->GetBit())													// restart
	{ 
	//	self.restart = br.readBits(2, "restart");
	}
	if (pBR->GetBit())													// to
	{
		xmlfile << " to='";
		self->m_pTo = sdl->AnimatedValue(pBR, self->m_pAttributeName, xmlfile);
		xmlfile << "'";
	}
	if (pBR->GetBit())													// xlink:href
	{
		xmlfile << " xlink:href='";
		self->setHref(sdl->anyURI(pBR, xmlfile));
	}
	self->m_bEnabled	= pBR->GetBit();							// enabled
	if (self->m_bEnabled)	{ xmlfile << " enabled='true'"; }
	bool bFlag = false;
	decodeContent(sdl, self, bFlag, pBR, xmlfile);

	if (bFlag)
		xmlfile << "\n</set>";
}

void CDecoder::decode(CSDLClasses *sdl, CSvg *self, CBitReader *pBR, ofstream &xmlfile)
{
	xmlfile << "\n<svg";

	if (pBR->GetBit())	{ self->m_nId = sdl->ID(pBR, xmlfile); }									// id
	if (pBR->GetBit())	{ decodeRare(sdl, pBR, xmlfile); }											// rare
	if (pBR->GetBit())																				// fill
	{
		xmlfile << " fill='";
		self->m_nFill = sdl->paint(pBR, xmlfile);
		xmlfile << "'";
	}
	if (pBR->GetBit())																				// stroke
	{
		xmlfile << " stroke='";
		self->m_nStroke = sdl->paint(pBR, xmlfile);
		xmlfile << "'";
	}
	if (pBR->GetBit())	{ self->m_strBaseProfile = sdl->byteAlignedString(pBR, xmlfile); }			// baseProfile
	if (pBR->GetBit())	{ self->m_strContentScriptType = sdl->byteAlignedString(pBR, xmlfile); }	// contentScriptType
	self->m_bExternalResourcesRequired	= pBR->GetBit();											// externalResourcesRequired
	xmlfile << " height='";
	self->m_anHeight					= sdl->valueWithUnits(pBR, xmlfile);						// height
	if (pBR->GetBit())	{ self->m_nPlaybackOrder = pBR->GetBit(); }									// playbackOrder
	if (pBR->GetBit())	{ self->m_nPreserveAspectRatio = sdl->preserveAspectRatio(pBR); }			// preserveAspectRatio
	if (pBR->GetBit())	{ self->m_nSnapshotTime = sdl->time(pBR, true); }							// snapshotTime
	if (pBR->GetBit())	{ self->m_nSyncBehaviorDefault = pBR->GetBits(2); }							// syncBehaviorDefault
	if (pBR->GetBit())	{ self->m_nSyncToleranceDefault = sdl->syncTolerance(pBR); }				// syncToleranceDefault
	if (pBR->GetBit())	{ self->m_nTimelineBegin = pBR->GetBit(); }									// timelineBegin
	if (pBR->GetBit())	{ self->m_strVersion = sdl->byteAlignedString(pBR, xmlfile); }				// version
	if (pBR->GetBit())																				// viewBox
	{
		xmlfile << " viewBox='";
		self->m_anViewBox = sdl->floatList(4, pBR, xmlfile);
	}
	xmlfile << " width='";
	self->m_anWidth						= sdl->valueWithUnits(pBR, xmlfile);						// width
	if (pBR->GetBit())	{ self->m_nZoomAndPan = pBR->GetBit(); }									// zoomAndPan
	decodeContent(sdl, self, pBR, xmlfile);

	xmlfile << "\n</svg>";
}

void CDecoder::decode(CSDLClasses *sdl, CText *self, CBitReader *pBR, ofstream &xmlfile)
{
	xmlfile << "\n<text";

	if (pBR->GetBit())	{ self->m_nId = sdl->ID(pBR, xmlfile); }				// id
	if (pBR->GetBit())	{ decodeRare(sdl, pBR, xmlfile); }						// rare
	if (pBR->GetBit())															// fill
	{
		xmlfile << " fill='";
		self->m_nFill = sdl->paint(pBR, xmlfile);
		xmlfile << "'";
	}
	if (pBR->GetBit())															// stroke
	{
		xmlfile << " stroke='";
		self->m_nStroke = sdl->paint(pBR, xmlfile);
		xmlfile << "'";
	}
	self->m_bEditable	= pBR->GetBit();										// editable
	if (pBR->GetBit())	{ self->m_anRotate = sdl->floatList(pBR, xmlfile); }	// rotate
	if (pBR->GetBit())															// x
	{
		xmlfile << " x='";
		self->m_anX = sdl->coordinateListType(pBR, xmlfile);
	}
	if (pBR->GetBit())															// y
	{
		xmlfile << " y='";
		self->m_anY = sdl->coordinateListType(pBR, xmlfile);
	}
	decodeContent(sdl, self, pBR, xmlfile);

	xmlfile << "</text>";
}

void CDecoder::decode(CSDLClasses *sdl, CVideo *self, CBitReader *pBR, ofstream &xmlfile)
{
	xmlfile << "\n<video";

	if (pBR->GetBit())	{ self->m_nId = sdl->ID(pBR, xmlfile); }							// id
	if (pBR->GetBit())	{ decodeRare(sdl, pBR, xmlfile); }									// rare
	if (pBR->GetBit())																		// begin
	{
		//self.begin = SDLClasses.timesType(br, "begin");
	}
	if (pBR->GetBit())																		// dur
	{
		//self.dur = SDLClasses.class_time(br, "dur", true);
	}
	self->m_bExternalResourcesRequired	= pBR->GetBit();									// externalResourcesRequired
	if (pBR->GetBit())																		// height
	{
		xmlfile << " height='";
		self->m_nHeight = sdl->coordinate(pBR);
		xmlfile << self->m_nHeight / 4096 << "'";
	}
	if (pBR->GetBit())																		// overlay
	{
		//self.overlay = SDLClasses.class_overlay(br, "overlay");
	}
	if (pBR->GetBit())	{ self->m_nPreserveAspectRatio = sdl->preserveAspectRatio(pBR); }	// preserveAspectRatio
	if (pBR->GetBit())																		// repeatCount
	{
		//self.repeatCount = SDLClasses.repeatCountType(br, "repeatCount");
	}
	if (pBR->GetBit())																		// repeatDur
	{
		//self.repeatDur = SDLClasses.repeatDurType(br, "repeatDur");
	}
	if (pBR->GetBit())																		// restart
	{
		//self.restart = br.readBits(2, "restart");
	}
	if (pBR->GetBit())																		// syncBehavior
	{
		//self.syncBehavior = br.readBits(2, "syncBehavior")
	}
	if (pBR->GetBit())																		// syncTolerance
	{
		//self.syncTolerance = SDLClasses.syncTolerance(br, "syncTolerance");
	}
	if (pBR->GetBit())	{ self->m_nTransformBehavior = pBR->GetBits(4); }					// transformBehavior
	if (pBR->GetBit())																		// type
	{
		xmlfile << " type='";
		self->m_strType = sdl->byteAlignedString(pBR, xmlfile);
		xmlfile << "'";
	}
	if (pBR->GetBit())																		// width
	{
		xmlfile << " width='";
		self->m_nWidth = sdl->coordinate(pBR);
		xmlfile << self->m_nWidth / 4096 << "'";
	}
	if (pBR->GetBit())																		// x
	{
		xmlfile << " x='";
		self->m_nX = sdl->coordinate(pBR);
		xmlfile << self->m_nX / 4096 << "'";
	}
	if (pBR->GetBit())																		// y
	{
		xmlfile << " y='";
		self->m_nY = sdl->coordinate(pBR);
		xmlfile << self->m_nY / 4096 << "'";
	}
	if (pBR->GetBit())																		// xlink:href
	{
		xmlfile << " xlink:href='";
		self->setHref(sdl->anyURI(pBR, xmlfile));
	}
	if (pBR->GetBit())																		// clipBegin
	{
		//self.clipBegin = SDLClasses.class_time(br, "clipBegin", true);
	}
	if (pBR->GetBit())																		// clipEnd
	{
		//self.clipEnd = SDLClasses.class_time(br, "clipEnd", true);
	}
	if (pBR->GetBit())																		// syncReference
	{
		//self.syncReference = SDLClasses.class_anyURI(br, "syncReference");
	}
	bool bFlag = false;
	decodeContent(sdl, self, bFlag, pBR, xmlfile);

	if (bFlag)
		xmlfile << "\n</video>";
}

void CDecoder::decode(CSDLClasses *sdl, CListener *self, CBitReader *pBR, ofstream &xmlfile)
{
	xmlfile << "\n<ev:listener";

	if (pBR->GetBit())	{ self->m_nId = sdl->ID(pBR, xmlfile); }	// id
	if (pBR->GetBit())	{ decodeRare(sdl, pBR, xmlfile); }			// rare
	if (pBR->GetBit())	{ self->m_nDefaultAction = pBR->GetBit(); }	// defaultAction
	if (pBR->GetBit())												// event
	{
		xmlfile << " event='";
		self->m_pEvent = sdl->ev(pBR, xmlfile);
		xmlfile << "'";
	}
	if (pBR->GetBit())												// handler
	{
		xmlfile << " handler='";
		self->m_pHandler = sdl->anyURI(pBR, xmlfile);
	}
	if (pBR->GetBit())												// observer
	{
		xmlfile << " observer='";
		self->m_nObserver = sdl->IDREF(pBR);
		xmlfile << self->m_nObserver << "'";
	}
	self->m_nPhase		= pBR->GetBit();							// phase
	if (pBR->GetBit())	{ self->m_nPropagate = pBR->GetBit(); }		// propagate
	if (pBR->GetBit())	{ self->m_nTarget = sdl->IDREF(pBR); }		// target
//	if (pBR->GetBit())	{ self->m_nDelay = sdl->time(pBR, true); }	// delay
//	if (pBR->GetBit())	{ self->m_nTimeAttribute = pBR->GetBit(); }	// timeAttribute
	self->m_bEnabled	= pBR->GetBit();							// enabled
	if (self->m_bEnabled)	{ xmlfile << " enabled='true'"; }
	bool bFlag = false;
	decodeContent(sdl, self, bFlag, pBR, xmlfile);

	if (bFlag)
		xmlfile << "\n</ev:listener>";
}

void CDecoder::decode(CSDLClasses *sdl, CConditional *self, CBitReader *pBR, ofstream &xmlfile)
{
	xmlfile << "\n<lsr:conditional";

	if (pBR->GetBit())	{ self->m_nId = sdl->ID(pBR, xmlfile); }	// id
	if (pBR->GetBit())	{ decodeRare(sdl, pBR, xmlfile); }			// rare
	if (pBR->GetBit())												// begin
	{
		xmlfile << " begin='";
		self->m_apBegin = sdl->timesType(pBR, xmlfile);
		xmlfile << "'";
	}
	self->m_bExternalResourcesRequired	= pBR->GetBit();			// externalResourcesRequired
	self->m_bEnabled					= pBR->GetBit();			// enabled
	sdl->anyAttribute(pBR, self);
	xmlfile << ">";
	pBR->GetVlcInt(5);	// len
	pBR->flush(pBR);	// align
	int occ0 = pBR->GetBits(5);	// occ0
	for (int i = 0; i < occ0; i++)
	{
		CDecoder *pDecoder = new CDecoder();
		pDecoder->updates(sdl, pBR, xmlfile);
	}
	pBR->flush(pBR);	// align
	self->m_apPrivateAttributes = sdl->privateAttributeContainer(pBR);
	
	xmlfile << "\n</lsr:conditional>";
}

void CDecoder::anyElement_elements(CSDLClasses *sdl, CBitReader *pBR, ofstream &xmlfile)
{
	int extensionID = pBR->GetBits(sdl->m_nExtensionIDBits);	// extensionID
	int bitlen = pBR->GetVlcInt(5);	// bitSkipLen
	if (extensionID != 2 && extensionID != 4)
	{
		if (bitlen % 8 != 0)
			pBR->GetBits(bitlen % 8);
		if (bitlen >> 3 != 0)
			pBR->GetByteArray(bitlen >> 3);
	}
	else if (extensionID == 2)
	{
		pBR->GetVlcInt(5);	// must be 87
		int len = pBR->GetVlcInt(5);
//		vector<tstring> v;
		for (int i = 0; i < len; i++)
		{
			// v2 elements
			int ch2 = pBR->GetBits(2);
			switch (ch2) 
			{
			case 0:
				{
					int ch5 = pBR->GetBits(4);
					switch (ch5)
					{
					case 0:// animation();
						break;
					case 1:// discard();
						break;
					case 2:// font();;
						break;
					case 3:// fontFace();
						break;
					case 4:// fontFaceSrc();;
						break;
					case 5:// fontFaceUri();
						break;
					case 6:// glyph();
						break;
					case 7:// handler();;
						break;
					case 8:// hkern();
						break;
					case 9:// missingGlyph();
						break;
					case 10:// prefetch();;
						break;
					case 11:// solidColor();
						break;
					case 12:// tBreak();
						break;
					case 13:
						{
							CTextArea *pTextArea = new CTextArea;
							decode(sdl, pTextArea, pBR, xmlfile);
							//delete pTextArea;
						}
						break;
					}
				}
				break;
			case 2:
				{
					int ch2b = pBR->GetBits(2);
					switch (ch2b)
					{
					case 0:// animateScroll();
						break;
					case 1:// setScroll();
						break;
					case 2:// streamSource();
						break;
					case 3:// updateSource();
						break;
					}
				}
				break;
			}
//			v.push_back();
		}
//		return v;
	}
	else if (extensionID == 4)
	{
		pBR->GetVlcInt(5);	// must be 87
		int len = pBR->GetVlcInt(5);
//		vector<tstring> v;
		for (int i = 0; i < len; i++)
		{
			// v4 elements
			int ch2 = pBR->GetBits(2);
			switch (ch2)
			{
			case 0:
				{
					// svg
					int ch5 = pBR->GetBit();
					switch (ch5)
					{
					case 0:
						{
							CTref *pTref = new CTref;
							decode(sdl, pTref, pBR, xmlfile);
						}
						break;
					}
					break;
				}
			case 2:
				{
					// laser
					int ch2b = pBR->GetBit();
					switch (ch2b)
					{
					case 0:
						{
 							CExternalReference *pExternalReference = new CExternalReference;
 							decode(sdl, pExternalReference, pBR, xmlfile);
						}
						break;
					case 1:
						{
							CExternalUpdate *pExternalUpdate = new CExternalUpdate;
							decode(sdl, pExternalUpdate, pBR, xmlfile);
						}
						break;
					}
					break;
				}
			}
//			v.push_back();
		}
//		return v;
	}
}

void CDecoder::decode(CSDLClasses *sdl, CTextArea *self, CBitReader *pBR, ofstream &xmlfile)
{
	xmlfile << "\n<textArea";

	if (pBR->GetBit())	{ self->m_nId = sdl->ID(pBR, xmlfile); }	// id
	if (pBR->GetBit())	{ decodeRare(sdl, pBR, xmlfile); }			// rare
	if (pBR->GetBit())												// fill
	{
		xmlfile << " fill='";
		self->m_nFill = sdl->paint(pBR, xmlfile);
		xmlfile << "'";
	}
	if (pBR->GetBit())												// stroke
	{
		xmlfile << " stroke='";
		self->m_nStroke = sdl->paint(pBR, xmlfile);
		xmlfile << "'";
	}
	if (pBR->GetBit())	{ self->m_bEditable = pBR->GetBit();	}	// editable
	if (pBR->GetBit())												// height
	{
		xmlfile << " height='";
		self->m_nHeight = sdl->coordinate(pBR);
		xmlfile << self->m_nHeight / 4096 << "'";
	}
	if (pBR->GetBit())												// width
	{
		xmlfile << " width='";
		self->m_nWidth = sdl->coordinate(pBR);
		xmlfile << self->m_nWidth / 4096 << "'";
	}
	if (pBR->GetBit())												// x
	{
		xmlfile << " x='";
		self->m_nX = sdl->coordinate(pBR);
		xmlfile << self->m_nX / 4096 << "'";
	}
	if (pBR->GetBit())												// y
	{
		xmlfile << " y='";
		self->m_nY = sdl->coordinate(pBR);
		xmlfile << self->m_nY / 4096 << "'";
	}
	decodeContent(sdl, self, pBR, xmlfile);

	xmlfile << "</textArea>";
}

void CDecoder::decode(CSDLClasses *sdl, CTref *self, CBitReader *pBR, ofstream &xmlfile)
{
	xmlfile << "<tref";

	if (pBR->GetBit())	{ self->m_nId = sdl->ID(pBR, xmlfile); }	// id
	if (pBR->GetBit())	{ decodeRare(sdl, pBR, xmlfile); }			// rare
	if (pBR->GetBit())												// fill
	{
		xmlfile << " fill='";
		self->m_nFill = sdl->paint(pBR, xmlfile);
		xmlfile << "'";
	}
	if (pBR->GetBit())												// stroke
	{
		xmlfile << " stroke='";
		self->m_nStroke = sdl->paint(pBR, xmlfile);
		xmlfile << "'";
	}
	self->m_bExternalResourcesRequired	= pBR->GetBit();			// externalResourcesRequired
	self->m_bEditable					= pBR->GetBit();			// editable
	if (pBR->GetBit())												// xlink:href
	{
		xmlfile << " xlink:href='";
		self->setHref(sdl->anyURI(pBR, xmlfile));
	}
	decodeContent(sdl, self, pBR, xmlfile);
}

void CDecoder::decode(CSDLClasses *sdl, CExternalReference *self, CBitReader *pBR, ofstream &xmlfile)
{
	xmlfile << "\n<lsr:externalReference";

	if (pBR->GetBit())	{ self->m_nId = sdl->ID(pBR, xmlfile); }							// id
	if (pBR->GetBit())	{ decodeRare(sdl, pBR, xmlfile); }									// rare
	self->m_bExternalResourcesRequired	= pBR->GetBit();									// externalResourcesRequired
	if (pBR->GetBit())	{ self->m_nUpdateInterval = sdl->updateInterval(pBR, xmlfile); }	// updateInterval
	if (pBR->GetBit())	{ self->m_nSecurity = pBR->GetBits(2); }							// security
	if (pBR->GetBit())	{}																	// entry
	decodeContent(sdl, self, pBR, xmlfile);

	xmlfile << "\n</lsr:externalReference>";
}

void CDecoder::decode(CSDLClasses *sdl, CExternalUpdate *self, CBitReader *pBR, ofstream &xmlfile)
{
	xmlfile << "\n<lsr:externalUpdate";

	if (pBR->GetBit())	{ self->m_nId = sdl->ID(pBR, xmlfile); }							// id
	if (pBR->GetBit())	{ decodeRare(sdl, pBR, xmlfile); }									// rare
	self->m_bExternalResourcesRequired	= pBR->GetBit();									// externalResourcesRequired
	if (pBR->GetBit())	{ self->m_nUpdateInterval = sdl->updateInterval(pBR, xmlfile); }	// updateInterval
	if (pBR->GetBit())	{ self->m_nType = sdl->externalUpdateType(self, pBR, xmlfile); }	// type
	bool bFlag = false;
	decodeContent(sdl, self, bFlag, pBR, xmlfile);

	if (bFlag)
		xmlfile << "\n</lsr:externalUpdate>";
}