/*
// This software was originally developed by ETRI and Kyunghee University, in the course
// of development of the ISO/IEC 14496-20 (Lightweight Application Scene Representation 
// -LASeR- and Simple Aggregation Format -SAF-) AMD3: Presentation and Modification of
// Structured Information standard for reference purposes and its performance may not 
// have been optimized.
//
// This software module is an implementation of one or more tools as specified by the
// ISO/IEC 14496-20 AMD3: PMSI standard.
//
// ISO/IEC gives users free license to this software module or modifications thereof
// for use in products claiming conformance to audiovisual and image-coding related
// ITU Recommendations and/or ISO/IEC International Standards.
// 
// ISO/IEC gives users the same free license to this software module or modifications
// thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in products are advised that its use may
// infringe existing patents. 
// 
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
*/
#include "Manager.h"


// CManager
CManager::CManager()
{
	m_nNbColorsAlreadySaved		= 0;
	m_nNbFontsAlreadySaved		= 0;
	m_nNbNameSpacesAlreadySaved	= 0;
	m_nNbTagsAlreadySaved		= 0;
//	m_bDebug					= false;
}

CManager::~CManager()
{
	m_anColors.clear();
	m_astrFonts.clear();
	m_anNameSpaces.clear();
	m_anPrivateTags.clear();
}

int CManager::addColor(int c)
{
	m_anColors.push_back(c);
	return m_anColors.size() - 1;
}

int CManager::getColor(int i, ofstream &xmlfile)
{
	xmlfile << "rgb(" << m_anColors[i] / 65536 << "," << (m_anColors[i] % 65536) / 256 << "," << m_anColors[i] % 256 << ")";
	return m_anColors[i];
}

int CManager::registerFont(tstring s)
{
	if (!s.c_str())
	{

	}
	for (int i = 0; i < m_astrFonts.size(); i++)
	{
		tstring ss = m_astrFonts[i];
		if (s.c_str() == ss.c_str())
			return i;
	}
	m_astrFonts.push_back(s);
	return m_astrFonts.size() - 1;
}

tstring CManager::getFont(int i, ofstream &xmlfile)
{
	char *temp;
	temp = new char[m_astrFonts[i].size()];

	WideCharToMultiByte(CP_ACP, 0, m_astrFonts[i].c_str(),m_astrFonts[i].size(), temp, m_astrFonts[i].size(), NULL, NULL);

	for (int k = 0; k < m_astrFonts[i].size(); k++)
	{
		xmlfile << temp[k];
	}
	xmlfile << "'";
	return m_astrFonts[i];
}

// CIDManager
CIDManager::CIDManager()
{
	m_bElementIDHole	= false;
	m_bStreamHole		= false;
}

CIDManager::~CIDManager()
{
	m_anElementIDs.clear();
	m_astrStreamDefs.clear();
	m_apPaintURI.clear();
}

void CIDManager::setStreamID(tstring s, int j)
{
	if (m_astrStreamDefs.size() <= j)
	{
		int max = j - m_astrStreamDefs.size();
		for (int i = 0; i < max; i++)
		{
			m_astrStreamDefs.push_back(NULL);
			m_bStreamHole = true;
		}
		m_astrStreamDefs.push_back(s);
	} 
	else
	{
// 		tstring ss = m_astrStreamDefs[j].c_str();
// 		if (ss != _T("") && ss != s)
// 		{
// 			// "DuplicateStreamID : "+ ss + " <> " + s + " j:" + j
// 		}
		m_astrStreamDefs.push_back(s);
	}
}

int CIDManager::registerPaintURI(CURI *u)
{
	for (int i = 0; i < m_apPaintURI.size(); i++)
	{
		CURI *v = (CURI*)m_apPaintURI[i];
		if (v->equals(u))
			return i;
	}
	m_apPaintURI.push_back(u);
	return m_apPaintURI.size() - 1;
}

// CTag
CTag::CTag()
{
	m_pName	= NULL;
}

CTag::~CTag()
{
	m_astrAttrNames.clear();
}