/*
// This software was originally developed by ETRI and Kyunghee University, in the course
// of development of the ISO/IEC 14496-20 (Lightweight Application Scene Representation 
// -LASeR- and Simple Aggregation Format -SAF-) AMD3: Presentation and Modification of
// Structured Information standard for reference purposes and its performance may not 
// have been optimized.
//
// This software module is an implementation of one or more tools as specified by the
// ISO/IEC 14496-20 AMD3: PMSI standard.
//
// ISO/IEC gives users free license to this software module or modifications thereof
// for use in products claiming conformance to audiovisual and image-coding related
// ITU Recommendations and/or ISO/IEC International Standards.
// 
// ISO/IEC gives users the same free license to this software module or modifications
// thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in products are advised that its use may
// infringe existing patents. 
// 
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
*/
#include "PreDecoder.h"


// CPreDecoder
CPreDecoder::CPreDecoder()
{
	m_bDebug	= false;
}

// CSAF_SL_PacketHeader
CSAF_SL_PacketHeader::CSAF_SL_PacketHeader()
{
	m_nPacketSequenceNumber	= 0;
	m_bRandomAccessFlag		= false;
	m_nTime					= 0;
}

void CSAF_SL_PacketHeader::read(CSDLClasses *sdl, CBitReader *pBR)
{
	m_bRandomAccessFlag = pBR->GetBit();
	m_nPacketSequenceNumber = pBR->GetBits(15);
	if (pBR->GetBits(2) != 1)
	{
		// wrong flags in SAF header
	}
	m_nTime = pBR->GetBits(30);
	sdl->m_nCurrentTimeInTics = m_nTime;
	m_nAccessUnitLength = pBR->GetBits(16);
}

// CSAF_AU_Header
CSAF_AU_Header::CSAF_AU_Header()
{
	m_nAccessUnitType	= -1;
	m_nStreamID			= -1;
	m_nPayloadLength	= -1;
	m_bIsFragment		= false;
}

void CSAF_AU_Header::read(CSAF_SL_PacketHeader *ssph, CBitReader *pBR)
{
	m_nAccessUnitType = pBR->GetBits(4);
	m_nStreamID = pBR->GetBits(12);
	if (m_nStreamID == 0xFFF)
	{
		m_nStreamID = -1;
	}
	if (ssph->m_nAccessUnitLength == 0)
	{
		m_bIsFragment = true;
		m_nPayloadLength = pBR->GetBits(16);
	} 
	else if (ssph->m_nAccessUnitLength == 1)
	{
		// length=1 is forbidden in the SAF SL Header
	}
}

// CSimpleDecoderConfigDescriptor
CSimpleDecoderConfigDescriptor::CSimpleDecoderConfigDescriptor()
{
	m_nObjectTypeIndication	= -1;
	m_nStreamType			= -1;
	m_nTimeStampResolution	= -1;
	m_nBufferSize			= -1;
}

void CSimpleDecoderConfigDescriptor::read(CBitReader *pBR)
{
	m_nObjectTypeIndication	= pBR->GetBits(8);
	m_nStreamType			= pBR->GetBits(8);
	m_nTimeStampResolution	= pBR->GetBits(24);
	m_nBufferSize			= pBR->GetBits(16);
}

// CSAFPacket
CSAFPacket::CSAFPacket()
{
	m_nTime					= -1;
	// 	resource = null;
	tstring	m_strStreamID	= _T("");
}

// CSAFHeader
CSAFHeader::CSAFHeader()
{
	CSAFPacket::CSAFPacket();

	m_nStreamType			= -1;
	m_nObjectTypeIndication = -1;
	m_nStartOffset			= 0;
	m_nLength				= 0;
	m_bIsPermanent			= false;
	m_nTimeStampResolution	= 0;
	m_nBufferSize			= 0;
	m_strMimeType			= _T("");
}

// CSAFHeader* CSAFHeader::lastHeaderWithThisStreamID(tstring sid)
// {
// 	CSAFHeader *res = NULL;
// 	//
// 
// 	return res;
// }

// CLASeRHeader
CLASeRHeader::CLASeRHeader()
{
	m_strISO_15938_1_DecoderInit				= _T("");
	m_nISO_15938_1_DecoderInit_Configuration	= -1;
	m_bNewSceneIndicator						= false;
	m_nColorComponentBits_minus_1				= 7;
	m_nLevel									= 0;
	m_nPathComponents							= -1;
	m_nPointsCodec								= 0;
	m_nProfile									= 0;
	m_nTimeResolution							= -1;
	m_nResolution								= 0;
	m_nCoordBits								= 12;
	m_nScaleBits_minus_coordBits				= 0;
	m_bUseFullRequestHost						= false;
	m_nExtensionIDBits							= 4;
}

CLASeRHeader::~CLASeRHeader()
{
	m_anExtensionDescriptor.clear();
}

// CSceneHeader
CSceneHeader::CSceneHeader()
{
	CSAFHeader::CSAFHeader();

	m_pLh					= NULL;

	m_nObjectTypeIndication = 9;	// LASeROTI
	m_nStreamType			= 3;	// SceneDescriptionStreamType
	m_nTimeStampResolution	= 1000;
	m_nBufferSize			= 0;
}

// CSceneUnit
CSceneUnit::CSceneUnit()
{
	CSAFPacket::CSAFPacket();

	m_bIsRAP				= false;
	m_nSize					= -1;
	m_bResetEncodingContext	= false;

	m_pStringIDTable		= NULL;

	m_nTime					= 0;
}

// CEndOfSAFSession
CEndOfSAFSession::CEndOfSAFSession()
{
	CSAFPacket::CSAFPacket();
}

// CRemoteStreamHeader
CRemoteStreamHeader::CRemoteStreamHeader()
{
	CSAFHeader::CSAFHeader();

	m_strSource		= _T("");
}

////////////////////////////////////////////////////////////////////////
void CPreDecoder::decodeRare(CSDLClasses *sdl, /*CX *b,*/ CBitReader *pBR, ofstream &xmlfile)
{
	int nb = pBR->GetBits(6);	// nbOfRares
	for (int i = 0; i < nb; i++)
	{
		int rareID = pBR->GetBits(6);	// rareId
		switch (rareID)
		{
		case /* RARE_CLASS */ 0:
			break;
		case /* RARE_AUDIO_LEVEL */ 1:
			break;
		case /* RARE_COLOR */ 2:
			break;
		case /* RARE_COLOR_RENDERING */ 3:
			break;
		case /* RARE_DISPLAY */ 4:
			{
				CXpropElem *b = new CXpropElem;
				xmlfile << " display='";
				b->setDisplay(pBR->GetBits(5), xmlfile);
			}
			break;
		case /* RARE_DISPLAY_ALIGN */ 5:
			break;
		case /* RARE_FILL_OPACITY */ 6:
			{
				CXpropElem *b = new CXpropElem;
				xmlfile << " fill-opacity='";
				b->m_nFill_opacity = sdl->float0to1(pBR, xmlfile);
			}
			break;
		case /* RARE_FILL_RULE */ 7:
			break;
		case /* RARE_IMAGE_RENDERING */ 8:
			break;
		case /* RARE_LINE_INCREMENT */ 9:
			break;
		case /* RARE_POINTER_EVENTS */ 10:
			break;
		case /* RARE_SHAPE_RENDERING */ 11:
			break;
		case /* RARE_SOLID_COLOR */ 12:
			break;
		case /* RARE_SOLID_OPACITY */ 13:
			break;
		case /* RARE_STOP_COLOR */ 14:
			break;
		case /* RARE_STOP_OPACITY */ 15:
			break;
		case /* RARE_STROKE_DASHARRAY */ 16:
			break;
		case /* RARE_STROKE_DASHOFFSET */ 17:
			break;
		case /* RARE_STROKE_LINECAP */ 18:
			break;
		case /* RARE_STROKE_LINEJOIN */ 19:
			break;
		case /* RARE_STROKE_MITERLIMIT */ 20:
			break;
		case /* RARE_STROKE_OPACITY */ 21:
			{
				CXpropElem *b = new CXpropElem;
				xmlfile << " stroke-opacity='";
				b->m_nFont_size = sdl->float0to1(pBR, xmlfile);
			}
			break;
		case /* RARE_STROKE_WIDTH */ 22:
			{
				CXpropElem *b = new CXpropElem;
				xmlfile << " stroke-width='";
				b->m_nViewport_fill = sdl->fixed_16_8i(pBR, xmlfile);
			}
			break;
		case /* RARE_TEXT_ANCHOR */ 23:
			break;
		case /* RARE_TEXT_RENDERING */ 24:
			break;
		case /* RARE_VIEWPORT_FILL */ 25:
			{
				CXpropElem *b = new CXpropElem;
				xmlfile << " viewport-fill='";
				b->m_nFont_size = sdl->paint(pBR, xmlfile);
			}
			break;
		case /* RARE_VIEWPORT_FILL_OPACITY */ 26:
			break;
		case /* RARE_VECTOR_EFFECT */ 27:
			{
				CXpropElem *b = new CXpropElem;
				xmlfile << " vector-effect='";
				b->setVector_effectType(pBR->GetBits(4), xmlfile);
			}
			break;
		case /* RARE_VISIBILITY */ 28:
			{
				CXpropElem *b = new CXpropElem;
				xmlfile << " visibility='";
				b->setVisibility(pBR->GetBits(2), xmlfile);
			}
			break;
		case /* RARE_REQUIREDEXTENSIONS */ 29:
			break;
		case /* RARE_REQUIREDFEATURES */ 30:
			break;
		case /* RARE_REQUIREDFORMATS */ 31:
			break;
		case /* RARE_SYSTEMLANGUAGE */ 32:
			break;
		case /* RARE_XML_BASE */ 33:
			break;
		case /* RARE_XML_LANG */ 34:
			break;
		case /* RARE_XML_SPACE */ 35:
			break;
		case /* RARE_FOCUSNEXT */ 36:
			break;
		case /* RARE_FOCUSNORTH */ 37:
			break;
		case /* RARE_FOCUSNORTHEAST */ 38:
			break;
		case /* RARE_FOCUSNORTHWEST */ 39:
			break;
		case /* RARE_FOCUSPREV */ 40:
			break;
		case /* RARE_FOCUSSOUTH */ 41:
			break;
		case /* RARE_FOCUSSOUTHEAST */ 42:
			break;
		case /* RARE_FOCUSSOUTHWEST */ 43:
			break;
		case /* RARE_FOCUSWEST */ 44:
			break;
		case /* RARE_FOCUSABLE */ 45:
			break;
		case /* RARE_FOCUSEAST */ 46:
			break;
		case /* RARE_TRANSFORM */ 47:
			{
				CIlocatable *b = new CIlocatable;
				b->setTransform(sdl->transform(pBR, xmlfile));
			}
			break;
			/*case 48:
			case 49:*/
		case /* RARE_FONT_VARIANT */ 50:
			break;
		case /* RARE_FONT_FAMILY */ 51:
			{
				CXpropElem *b = new CXpropElem;
				xmlfile << " font-family='";
				b->m_nstrFont_family = sdl->fontFamily(pBR, xmlfile);
			}
			break;
		case /* RARE_FONT_SIZE */ 52:
			{
				CXpropElem *b = new CXpropElem;
				xmlfile << " font-size='";
				b->m_nFont_size = sdl->fixed_16_8i(pBR, xmlfile);
			}
			break;
		case /* RARE_FONT_STYLE */ 53:
			{
				CXpropElem *b = new CXpropElem;
				xmlfile << " font-style='";
				b->m_nFont_style = sdl->fontStyle(pBR, xmlfile);
			}
			break;
		case /* RARE_TEXT_DECORATION */ 48:
			break;
		case /* RARE_FONT_WEIGHT */ 54:
			{
				CXpropElem *b = new CXpropElem;
				xmlfile << " font-weight='";
				b->m_nFont_weight = sdl->fontWeight(pBR, xmlfile);
			}
			break;
		case /* RARE_HREF_TITLE */ 55:
			break;
		case /* RARE_HREF_TYPE */ 56:
			break;
		case /* RARE_HREF_ROLE */ 57:
			break;
		case /* RARE_HREF_ARCROLE */ 58:
			break;
		case /* RARE_HREF_ACTUATE */ 59:
			break;
		case /* RARE_HREF_SHOW */ 60:
			break;
		case /* RARE_END */ 61:
			break;
		case /* RARE_MAX */ 62:
			break;
		case /* RARE_MIN */ 63:
			break;
		case 49: // extension (rare2)
			break;
		}
	}
}

void CPreDecoder::decodeContent(CSDLClasses *sdl, CX *self, CBitReader *pBR, ofstream &xmlfile)
{
	sdl->anyAttribute(pBR, self);
	self->m_apPrivateAttributes = sdl->privateAttributeContainer(pBR);
	if (pBR->GetBit())	// loop
	{
		xmlfile << ">";

		int occ0 = pBR->GetVlcInt(5);
		for (int i = 0; i < occ0; i++)
		{
			CDecoder *pDecoder = new CDecoder();
			pDecoder->elements(sdl, pBR, xmlfile);
		}
	}
	else
		xmlfile << "/>";
}

void CPreDecoder::decodeContent(CSDLClasses *sdl, CX *self, bool bFlag, CBitReader *pBR, ofstream &xmlfile)
{
	sdl->anyAttribute(pBR, self);
	self->m_apPrivateAttributes = sdl->privateAttributeContainer(pBR);
	if (pBR->GetBit())	// loop
	{
		xmlfile << ">";

		int occ0 = pBR->GetVlcInt(5);
		for (int i = 0; i < occ0; i++)
		{
			CDecoder *pDecoder = new CDecoder();
			pDecoder->elements(sdl, pBR, xmlfile);
		}
		bFlag = true;
	}
	else
		xmlfile << "/>";
}

void CPreDecoder::decodeReplaceContent(CSDLClasses *sdl, /*CDReplace *self,*/ bool bFlag, CBitReader *pBR, ofstream &xmlfile)
{
	sdl->anyAttribute(pBR, NULL);
	if (pBR->GetBit())	// loop
	{
		xmlfile << ">";

		pBR->GetVlcInt(5);	// occ0
// 		for (int i = 0; i < occ0; i++)
// 		{
// 			
// 		}
		bFlag = true;
	}
	else
		xmlfile << "/>";
}

void CPreDecoder::PreDecode(CScene *pScene, CSDLClasses *sdl, CBitReader *pBR, char *szFileData, ofstream &xmlfile)
{
	sdl->m_pCurrentScene = pScene;
	sdl->m_pMan = pScene->m_pMan;
	bool bGoOn = true;
	CSAF_SL_PacketHeader			*ssph		= new CSAF_SL_PacketHeader;
	CSAF_AU_Header					*sah		= new CSAF_AU_Header;
	CSimpleDecoderConfigDescriptor	*sdcd		= new CSimpleDecoderConfigDescriptor;
	CSimpleDecoderConfigDescriptor	*sdcd1		= new CSimpleDecoderConfigDescriptor;
	while (bGoOn)
	{
		ssph->read(sdl, pBR);
		sah->read(ssph, pBR);
		if (sah->m_bIsFragment)
		{
			// here, deal with fragments
			// in this version, skip
			pBR->GetByteArray(sah->m_nPayloadLength);
		} 
		else
		{
			bool bPermanent = false;
			switch (sah->m_nAccessUnitType)
			{
			case 1:
				// header, not permanent
				{
					sdcd->read(pBR);
					if (sdcd->m_nStreamType == 3)	// SceneDescriptionStreamType
					{
						if (sdcd->m_nObjectTypeIndication == 6)	// FontDataOTI
						{
							// this is a fontStreamHeader
							//parseFontHeader
// 							if (m_bDebug)
// 							{
// 								// font header
// 							}
						} 
						else if(sdcd->m_nObjectTypeIndication == 9)	// LASeROTI
						{
							// this is a sceneHeader
							CSceneHeader *sh = new CSceneHeader;
							parseSceneHeader(pScene, sdl, sh, ssph, sah, sdcd, pBR, bPermanent, xmlfile);
// 							if (m_bDebug)
// 							{
// 								// scene header
// 							}
						}
					}
					//
				}
				break;
			case 2:
				// header, permanent
				break;
			case 3:
				break;
			case 4:
				// access unit
				if (sdcd == NULL)
				{
					// Missing header for stream
				}
				else if (sdcd->m_nStreamType == 3)	// SceneDescriptionStreamType
				{
					if (sdcd->m_nObjectTypeIndication == 6)	// FontDataOTI
					{
						// this is a fontStreamHeader
						//parseFontUnit
// 						if (m_bDebug)
// 						{
// 							// font unit
// 						}
					}
					else if(sdcd->m_nObjectTypeIndication == 9)	// LASeROTI
					{
						// this is a sceneHeader
						CSceneUnit *su = new CSceneUnit;
						parseSceneUnit(pScene, sdl, su, ssph, sah, sdcd, pBR, xmlfile);
// 						if (m_bDebug)
// 						{
// 							// scene unit
// 						}
					}
					//
				}
				break;
			case 5:
				// end of saf session
				{
					CEndOfSAFSession *au1 = new CEndOfSAFSession;
					parseEndOfSAFSession(pScene, sdl, au1, ssph, sah, pBR);
					bGoOn = false;
// 					if (m_bDebug)
// 					{
// 						// end of saf session
// 					}
				}
				break;
			case 6:
				break;
			case 7:
				// remote header
				{
					sdcd1->read(pBR);
					CRemoteStreamHeader *rsh = new CRemoteStreamHeader;
					parseRemoteStreamHeader(pScene, sdl, rsh, ssph, sah, sdcd1, pBR, false, xmlfile);
// 					if (m_bDebug)
// 					{
// 						// remote header
// 					}
				}
				break;
			}
			pBR->flush(pBR);
		}
	}
	xmlfile << "\n<saf:endOfSAFSession/>";
	xmlfile << "\n</saf:SAFSession>";
}

void CPreDecoder::parseSceneHeader(CScene *pScene, CSDLClasses *sdl, CSceneHeader *sh, CSAF_SL_PacketHeader *ssph, CSAF_AU_Header *sah, CSimpleDecoderConfigDescriptor *sdcd, CBitReader *pBR, bool bPermanent, ofstream &xmlfile)
{
	xmlfile << "\n<saf:sceneHeader";

	sh->m_nBufferSize					= sdcd->m_nBufferSize;
	sh->m_nObjectTypeIndication			= sdcd->m_nObjectTypeIndication;
	if (sh->m_nObjectTypeIndication)	xmlfile << " objectTypeIndication='" << sh->m_nObjectTypeIndication << "'";
	sh->m_nStreamType					= sdcd->m_nStreamType;
	if (sh->m_nStreamType)				xmlfile << " streamType='" << sh->m_nStreamType << "'";
	sh->m_nTimeStampResolution			= sdcd->m_nTimeStampResolution;
	tstring strStreamName				= _T("Stream");
	strStreamName						= strAppend(strStreamName, sah->m_nStreamID);
	pScene->m_pIdMan->setStreamID(strStreamName, sah->m_nStreamID);
	sh->setStreamID(strStreamName);

	xmlfile << ">";

	sh->m_pLh							= new CLASeRHeader;
	CDecoder *pDecoder					= new CDecoder();
	pDecoder->decode(sdl, sh->m_pLh, pBR, xmlfile);
	sdl->m_nColorComponentBits			= sh->m_pLh->m_nColorComponentBits_minus_1 + 1;
	sdl->m_nScaleBits					= sh->m_pLh->m_nScaleBits_minus_coordBits;
	sdl->m_nResolution					= sh->m_pLh->m_nResolution;
	sdl->m_nExtensionIDBits				= sh->m_pLh->m_nExtensionIDBits;

	xmlfile << "\n</saf:sceneHeader>";
}

void CPreDecoder::parseSceneUnit(CScene *pScene, CSDLClasses *sdl, CSceneUnit *su, CSAF_SL_PacketHeader *ssph, CSAF_AU_Header *sah, CSimpleDecoderConfigDescriptor *sdcd, CBitReader *pBR, ofstream &xmlfile)
{
	xmlfile << "\n<saf:sceneUnit>";

	su->m_bIsRAP				= ssph->m_bRandomAccessFlag;
	su->m_strStreamID			= _T("Stream");
	su->m_strStreamID			= strAppend(su->m_strStreamID, sah->m_nStreamID);
	su->m_nTime					= ssph->m_nTime;
	su->m_bResetEncodingContext = pBR->GetBit();
	if (pBR->GetBit())			sdl->extension(pBR);
	initialisation(sdl, su, pBR, xmlfile);
	int occ1					= pBR->GetVlcInt(5);
	for (int t = 0; t < occ1 + 1; t++)
	{
		CDecoder *pDecoder = new CDecoder();
		pDecoder->updates(sdl, pBR, xmlfile);
	}
	if (pBR->GetBit())			sdl->extension(pBR);

	xmlfile << "\n</saf:sceneUnit>";
}

void CPreDecoder::parseEndOfSAFSession(CScene *pScene, CSDLClasses *sdl, CEndOfSAFSession *au1, CSAF_SL_PacketHeader *ssph, CSAF_AU_Header *sah, CBitReader *pBR)
{
	//
}

void CPreDecoder::parseRemoteStreamHeader(CScene *pScene, CSDLClasses *sdl, CRemoteStreamHeader *rsh, CSAF_SL_PacketHeader *ssph, CSAF_AU_Header *sah, CSimpleDecoderConfigDescriptor *sdcd1, CBitReader *pBR, bool bPermanent, ofstream &xmlfile)
{
	xmlfile << "\n<saf:RemoteStreamHeader";

	int nStreamID = sah->m_nStreamID;
	tstring strStreamName				= _T("Stream");
	strStreamName						= strAppend(strStreamName, nStreamID);
	xmlfile << " streamID='Stream" << nStreamID << "'";
	pScene->m_pIdMan->setStreamID(strStreamName, nStreamID);
	rsh->setStreamID(strStreamName);
	rsh->m_nBufferSize					= sdcd1->m_nBufferSize;
	rsh->m_nObjectTypeIndication		= sdcd1->m_nObjectTypeIndication;
	if (rsh->m_nObjectTypeIndication)	xmlfile << " objectTypeIndication='" << rsh->m_nObjectTypeIndication << "'";
	rsh->m_nStreamType					= sdcd1->m_nStreamType;
	if (rsh->m_nStreamType)				xmlfile << " streamType='" << rsh->m_nStreamType << "'";
	rsh->m_nTimeStampResolution			= sdcd1->m_nTimeStampResolution;
	int len = 0;
	if (rsh->m_nObjectTypeIndication == 0xFF && rsh->m_nStreamType == 0xFF)
	{
		len = pBR->GetBits(16);	// mimeTypeLength
		rsh->m_strMimeType = pBR->GetString(len, xmlfile);	// mimeType
		len += 2;	// to get the number of bytes read in this branch
	}
	int len2 = pBR->GetBits(16);	// urlLength
	xmlfile << " source='";
	rsh->m_strSource = pBR->GetString(len2, xmlfile);	// url
	xmlfile << "'";
	len2 += 2;	// to get the number of bytes read in this branch
	if (ssph->m_nAccessUnitLength > 9 + len + len2)
	{
		//
	}
	
	xmlfile << "/>";
}

int CPreDecoder::reframeComponent(CSDLClasses *sdl, int comp)
{
	if (sdl->m_nColorComponentBits > 8)
		return 0xFF & (comp >> (sdl->m_nColorComponentBits - 8));
	else
		return 0xFF & (comp << (8 - sdl->m_nColorComponentBits));
}

void CPreDecoder::initialisation(CSDLClasses *sdl, CSceneUnit *su, CBitReader *pBR, ofstream &xmlfile)	// sdl
{
	CUtils *pUtils = new CUtils();

	bool b = pBR->GetBit();	// hasColors
	if (b)
	{
		int len = pBR->GetVlcInt(5);	// nb of colors
		for (int i = 0; i < len; i++)
		{
			int col = 0;
			col += reframeComponent(sdl, pBR->GetBits(sdl->m_nColorComponentBits)) << 16;	// red
			col += reframeComponent(sdl, pBR->GetBits(sdl->m_nColorComponentBits)) << 8;	// green
			col += reframeComponent(sdl, pBR->GetBits(sdl->m_nColorComponentBits));			// blue
			sdl->m_pMan->addColor(col);
		}
		sdl->m_nColorIndexBits = pUtils->nbBitsForPositiveInteger(sdl->m_pMan->nbOfColors());
	}
	b = pBR->GetBit();	// hasFonts
	if (b)
	{
		int len = pBR->GetVlcInt(5);	// nb of fonts
		for (int i = 0; i < len; i++)
			sdl->m_pMan->registerFont(sdl->byteAlignedString(pBR));
		sdl->m_nFontIndexBits = pUtils->nbBitsForPositiveInteger(sdl->m_pMan->nbOfFonts());
	}
	b = pBR->GetBit();	// hasPrivateDataIdentifiers
	if (b)
	{
// 		// a privateDataIdentifiertable in front of each AU
// 		m_uNbPrivateDataIdentifiers	= pDec->GetValue(5);
// 		for (UINT i = 0; i < m_uNbPrivateDataIdentifiers; i++)
// 		{
// 			m_nPrivateDataIdentifierIndex++;
// 			//
// 		}
// 		//
	} 
	b = pBR->GetBit();	// hasTags
	if (b)
	{
// 		// a tag table in front of each AU
// 		m_uNbTags	= pDec->GetValue(5);
// 		for (UINT i = 0; i < m_uNbTags; i++)
// 		{
// 			//
// 		}
// 		//
	} 

	// extendedInitialisation
	// stringIDTable: external string ID references
	int len = pBR->GetVlcInt(5);	// countG
	if (len != 0)
	{
		//
	}
	for (int i = 0; i < len; i++)
	{
		//
	}
	if (pBR->GetBit())	// ext
	{
		//
	}
}

//////////////////////////////////////////////////////////////////////////
tstring CPreDecoder::strAppend(tstring str, int i)
{
	char		*szFileData		= NULL;
	wchar_t		*wszFileData	= NULL;
	wszFileData = new wchar_t[100];
	memset(wszFileData, 0, sizeof(wszFileData));
	itoa(i, (LPSTR)wszFileData, 10);
	MultiByteToWideChar(CP_ACP, 0, szFileData, 100, wszFileData, 100);
	str.append(wszFileData);

	return str;
}