/*
// This software was originally developed by ETRI and Kyunghee University, in the course
// of development of the ISO/IEC 14496-20 (Lightweight Application Scene Representation 
// -LASeR- and Simple Aggregation Format -SAF-) AMD3: Presentation and Modification of
// Structured Information standard for reference purposes and its performance may not 
// have been optimized.
//
// This software module is an implementation of one or more tools as specified by the
// ISO/IEC 14496-20 AMD3: PMSI standard.
//
// ISO/IEC gives users free license to this software module or modifications thereof
// for use in products claiming conformance to audiovisual and image-coding related
// ITU Recommendations and/or ISO/IEC International Standards.
// 
// ISO/IEC gives users the same free license to this software module or modifications
// thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in products are advised that its use may
// infringe existing patents. 
// 
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
*/
#include "DecTools.h"

#include "StringIDTable.h"

#include "bin2xml.h"


class CSAF_SL_PacketHeader
{
public:
	CSAF_SL_PacketHeader();

public:
	int		m_nPacketSequenceNumber;
	bool	m_bRandomAccessFlag;
	int		m_nTime;	// compositionTimeStamp or validity
	int		m_nAccessUnitLength;

public:
	void	read(CSDLClasses *sdl, CBitReader *pBR);	
};


class CSAF_AU_Header
{
public:
	CSAF_AU_Header();

public:
	int		m_nAccessUnitType;
	int		m_nStreamID;
	int		m_nPayloadLength ;
	bool	m_bIsFragment;

public:
	void	read(CSAF_SL_PacketHeader *ssph, CBitReader *pBR);
};


class CSimpleDecoderConfigDescriptor
{
public:
	CSimpleDecoderConfigDescriptor();

public:
	int		m_nObjectTypeIndication;
	int		m_nStreamType;
	int		m_nTimeStampResolution;
	int		m_nBufferSize;

public:
	void	read(CBitReader *pBR);
};


class CSAFPacket
{
public:
	CSAFPacket();

public:
	int		m_nTime;
	tstring	m_strStreamID;

public:
	void	setStreamID(tstring sid){m_strStreamID = sid;}
};


class CSAFHeader : public CSAFPacket
{
public:
	CSAFHeader();

public:
	int		m_nStreamType;
	int		m_nObjectTypeIndication;
	int		m_nStartOffset;
	int		m_nLength;
	bool	m_bIsPermanent;
	int		m_nTimeStampResolution;
	int		m_nBufferSize;
	tstring	m_strMimeType;
};


class CLASeRHeader
{
public:
	CLASeRHeader();
	~CLASeRHeader();

public:
	tstring		m_strISO_15938_1_DecoderInit;
	int			m_nISO_15938_1_DecoderInit_Configuration;
	bool		m_bNewSceneIndicator;
	int			m_nColorComponentBits_minus_1;
	int			m_nLevel;
	int			m_nPathComponents;
	int			m_nPointsCodec;
	int			m_nProfile;
	int			m_nTimeResolution;
	int			m_nResolution;
	int			m_nCoordBits;
	int			m_nScaleBits_minus_coordBits;
	bool		m_bUseFullRequestHost;
	int			m_nExtensionIDBits;
	vector<int>	m_anExtensionDescriptor;
	int			m_nReserved;
};


class CSceneHeader : public CSAFHeader
{
public:
	CSceneHeader();

public:
	CLASeRHeader *m_pLh;
};


class CSceneUnit : public CSAFPacket
{
public:
	CSceneUnit();

public:
	bool			m_bIsRAP;
	int				m_nSize;
	bool			m_bResetEncodingContext;

	CStringIDTable	*m_pStringIDTable;	
};


class CEndOfSAFSession : public CSAFPacket
{
public:
	CEndOfSAFSession();
};


class CRemoteStreamHeader : public CSAFHeader
{
public:
	CRemoteStreamHeader();

public:
	tstring		m_strSource;
};


class CPreDecoder
{
public:
	CPreDecoder();

public:
	bool	m_bDebug;

public:
	void	decodeRare(CSDLClasses *sdl, /*CX *b,*/ CBitReader *pBR, ofstream &xmlfile);
	void	decodeContent(CSDLClasses *sdl, CX *self, CBitReader *pBR, ofstream &xmlfile);
	void	decodeContent(CSDLClasses *sdl, CX *self, bool bFlag, CBitReader *pBR, ofstream &xmlfile);
	void	decodeReplaceContent(CSDLClasses *sdl, /*CDReplace *self,*/ bool bFlag, CBitReader *pBR, ofstream &xmlfile);

	void	PreDecode(CScene *pScene, CSDLClasses *sdl, CBitReader *pBR, char *szFileData, ofstream &xmlfile);
	void	parseSceneHeader(CScene *pScene, CSDLClasses *sdl, CSceneHeader *sh, CSAF_SL_PacketHeader *ssph, CSAF_AU_Header *sah, CSimpleDecoderConfigDescriptor *sdcd, CBitReader *pBR, bool bPermanent, ofstream &xmlfile);
	void	parseSceneUnit(CScene *pScene, CSDLClasses *sdl, CSceneUnit *su, CSAF_SL_PacketHeader *ssph, CSAF_AU_Header *sah, CSimpleDecoderConfigDescriptor *sdcd, CBitReader *pBR, ofstream &xmlfile);
	void	parseEndOfSAFSession(CScene *pScene, CSDLClasses *sdl, CEndOfSAFSession *au1, CSAF_SL_PacketHeader *ssph, CSAF_AU_Header *sah, CBitReader *pBR);
	void	parseRemoteStreamHeader(CScene *pScene, CSDLClasses *sdl, CRemoteStreamHeader *rsh, CSAF_SL_PacketHeader *ssph, CSAF_AU_Header *sah, CSimpleDecoderConfigDescriptor *sdcd1, CBitReader *pBR, bool bPermanent, ofstream &xmlfile);

	int		reframeComponent(CSDLClasses *sdl, int comp);
	void	initialisation(CSDLClasses *sdl, CSceneUnit *su, CBitReader *pBR, ofstream &xmlfile);

	//////////////////////////////////////////////////////////////////////////
	tstring strAppend(tstring str, int i);
};


class CDecoder : public CPreDecoder
{
public:
	CDecoder();

public:
	void	elements(CSDLClasses *sdl, CBitReader *pBR, ofstream &xmlfile);

	void	updates(CSDLClasses *sdl, CBitReader *pBR, ofstream &xmlfile);
	void	decode(CSDLClasses *sdl, CLASeRHeader *self, CBitReader *pBR, ofstream &xmlfile);
	
	void	decode(CSDLClasses *sdl, CDDelete *self, CBitReader *pBR, ofstream &xmlfile);
	void	decode(CSDLClasses *sdl, CDInsert *self, CBitReader *pBR, ofstream &xmlfile);
	void	decode(CSDLClasses *sdl, CBitReader *pBR, ofstream &xmlfile);
	void	decode(CSDLClasses *sdl, CDReplace *self, CBitReader *pBR, ofstream &xmlfile);
	void	decode(CSDLClasses *sdl, CA *pA, CBitReader *pBR, ofstream &xmlfile);
	void	decode(CSDLClasses *sdl, CAnimate *pAnimate, CBitReader *pBR, ofstream &xmlfile);
//	void	decode(CSDLClasses *sdl, CAudio *pAudio, CBitReader *pBR, ofstream &xmlfile);
	void	decode(CSDLClasses *sdl, CCircle *pCircle, CBitReader *pBR, ofstream &xmlfile);
	void	decode(CSDLClasses *sdl, CEllipse *pEllipse, CBitReader *pBR, ofstream &xmlfile);
	void	decode(CSDLClasses *sdl, CG *pG, CBitReader *pBR, ofstream &xmlfile);
	void	decode(CSDLClasses *sdl, CImage *pImage, CBitReader *pBR, ofstream &xmlfile);
	void	decode(CSDLClasses *sdl, CLine *pLine, CBitReader *pBR, ofstream &xmlfile);
	void	decode(CSDLClasses *sdl, CPolygon *pPolygon, CBitReader *pBR, ofstream &xmlfile);
	void	decode(CSDLClasses *sdl, CPolyline *pPolyline, CBitReader *pBR, ofstream &xmlfile);
	void	decode(CSDLClasses *sdl, CDRect *pRect, CBitReader *pBR, ofstream &xmlfile);
	void	decode(CSDLClasses *sdl, CSet *pSet, CBitReader *pBR, ofstream &xmlfile);
	void	decode(CSDLClasses *sdl, CSvg *pSvg, CBitReader *pBR, ofstream &xmlfile);
	void	decode(CSDLClasses *sdl, CText *pText, CBitReader *pBR, ofstream &xmlfile);
	void	decode(CSDLClasses *sdl, CVideo *pVideo, CBitReader *pBR, ofstream &xmlfile);
	void	decode(CSDLClasses *sdl, CListener *pListener, CBitReader *pBR, ofstream &xmlfile);
	void	decode(CSDLClasses *sdl, CConditional *pConditional, CBitReader *pBR, ofstream &xmlfile);

	void	anyElement_elements(CSDLClasses *sdl, CBitReader *pBR, ofstream &xmlfile);

	void	decode(CSDLClasses *sdl, CTextArea *pTextArea, CBitReader *pBR, ofstream &xmlfile);
	void	decode(CSDLClasses *sdl, CTref *pTref, CBitReader *pBR, ofstream &xmlfile);
	void	decode(CSDLClasses *sdl, CExternalReference *pExternalReference, CBitReader *pBR, ofstream &xmlfile);
	void	decode(CSDLClasses *sdl, CExternalUpdate *pExternalUpdate, CBitReader *pBR, ofstream &xmlfile);
};