/*
// This software was originally developed by ETRI and Kyunghee University, in the course
// of development of the ISO/IEC 14496-20 (Lightweight Application Scene Representation 
// -LASeR- and Simple Aggregation Format -SAF-) AMD3: Presentation and Modification of
// Structured Information standard for reference purposes and its performance may not 
// have been optimized.
//
// This software module is an implementation of one or more tools as specified by the
// ISO/IEC 14496-20 AMD3: PMSI standard.
//
// ISO/IEC gives users free license to this software module or modifications thereof
// for use in products claiming conformance to audiovisual and image-coding related
// ITU Recommendations and/or ISO/IEC International Standards.
// 
// ISO/IEC gives users the same free license to this software module or modifications
// thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in products are advised that its use may
// infringe existing patents. 
// 
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
*/
#include "SDLClasses.h"

#include <fstream>


// CSDLClasses
CSDLClasses::CSDLClasses()
{
	m_nScaleBits			= 0;
	m_nCoordBits			= 0;
	m_nResolution			= 0;
	m_nColorComponentBits	= 0;
	m_nColorIndexBits		= 0;
	m_nFontIndexBits		= 0;
	m_nNameSpaceIndexBits	= 0;
	m_nTagIndexBits			= 0;
	m_nCodecBits			= 0;
	m_nExtensionIDBits		= 4;
	m_nPrivateDataIdBits	= 0;
	m_pCurrentScene			= new CScene;
	m_pMan					= new CManager;
	m_nPointBits			= 0;
	m_nFixed_16_8			= 0;
	m_fTimeResolution		= 1000;
	// matrixBits = 0;
	m_nCurrentTimeInTics	= 0;
}

CSDLClasses::~CSDLClasses()
{
	m_pCurrentScene	= new CScene;
	m_pMan			= new CManager;
}

CAttribute* CSDLClasses::AttributeName(CBitReader *pBR, ofstream &xmlfile)
{
	CAttribute *a = new CAttribute;
	if (pBR->GetBit())
	{
		a->m_bIsLaser = false;
 		pBR->GetBits(5);	// tagID
// 		if (tagID != 0)
// 			a.tag = man.getTag(tagID);
//		a.attr = man.getAttr(tagID, br.readVlcInt(5, msg));
	}
	else
	{
		a->m_bIsLaser = true;
		a->m_nAttributeIndex = pBR->GetBits(8);
		switch (a->m_nAttributeIndex)
		{
		case 0:		xmlfile << "a.target";					break;
		case 1:		xmlfile << "accumulate";				break;
		case 2:		xmlfile << "additive";					break;
		case 3:		xmlfile << "audio-level";				break;
		case 4:		xmlfile << "bandwidth";					break;
		case 5:		xmlfile << "begin";						break;
		case 6:		xmlfile << "calcMode";					break;
		case 7:		xmlfile << "children";					break;
		case 8:		xmlfile << "choice";					break;
		case 9:		xmlfile << "clipBegin";					break;
		case 10:	xmlfile << "clipEnd";					break;
		case 11:	xmlfile << "color";						break;
		case 12:	xmlfile << "color-rendering";			break;
		case 13:	xmlfile << "cx";						break;
		case 14:	xmlfile << "cy";						break;
		case 15:	xmlfile << "d";							break;
		case 16:	xmlfile << "delta";						break;
		case 17:	xmlfile << "display";					break;
		case 18:	xmlfile << "display-align";				break;
		case 19:	xmlfile << "dur";						break;
		case 20:	xmlfile << "editable";					break;
		case 21:	xmlfile << "enabled";					break;
		case 22:	xmlfile << "end";						break;
		case 23:	xmlfile << "event";						break;
		case 24:	xmlfile << "externalResourcesRequired";	break;
		case 25:	xmlfile << "fill";						break;
		case 26:	xmlfile << "fill-opacity";				break;
		case 27:	xmlfile << "fill-rule";					break;
		case 28:	xmlfile << "focusable";					break;
		case 29:	xmlfile << "font-family";				break;
		case 30:	xmlfile << "font-size";					break;
		case 31:	xmlfile << "font-style";				break;
		case 32:	xmlfile << "font-variant";				break;
		case 33:	xmlfile << "font-weight";				break;
		case 34:	xmlfile << "fullscreen";				break;
		case 35:	xmlfile << "gradientUnits";				break;
		case 37:	xmlfile << "height";					break;
		case 38:	xmlfile << "image-rendering";			break;
		case 39:	xmlfile << "keyPoints";					break;
		case 40:	xmlfile << "keySplines";				break;
		case 41:	xmlfile << "keyTimes";					break;
		case 42:	xmlfile << "line-increment";			break;
		case 43:	xmlfile << "listener.target";			break;
		case 44:	xmlfile << "mediaCharacterEncoding";	break;
		case 45:	xmlfile << "mediaContentEncodings";		break;
		case 46:	xmlfile << "mediaSize";					break;
		case 47:	xmlfile << "mediaTime";					break;
		case 48:	xmlfile << "nav-down";					break;
		case 49:	xmlfile << "nav-down-left";				break;
		case 50:	xmlfile << "nav-down-right";			break;
		case 51:	xmlfile << "nav-left";					break;
		case 52:	xmlfile << "nav-next";					break;
		case 53:	xmlfile << "nav-prev";					break;
		case 54:	xmlfile << "nav-right";					break;
		case 55:	xmlfile << "nav-up";					break;
		case 56:	xmlfile << "nav-up-left";				break;
		case 57:	xmlfile << "nav-up-right";				break;
		case 58:	xmlfile << "observer";					break;
		case 59:	xmlfile << "offset";					break;
		case 60:	xmlfile << "opacity";					break;
		case 61:	xmlfile << "overflow";					break;
		case 62:	xmlfile << "overlay";					break;
		case 63:	xmlfile << "path";						break;
		case 64:	xmlfile << "pathLength";				break;
		case 65:	xmlfile << "pointer-events";			break;
		case 66:	xmlfile << "points";					break;
		case 67:	xmlfile << "preserveAspectRatio";		break;
		case 68:	xmlfile << "r";							break;
		case 69:	xmlfile << "repeatCount";				break;
		case 70:	xmlfile << "repeatDur";					break;
		case 71:	xmlfile << "requiredExtensions";		break;
		case 72:	xmlfile << "requiredFeatures";			break;
		case 73:	xmlfile << "requiredFormats";			break;
		case 74:	xmlfile << "restart";					break;
		case 75:	xmlfile << "rotate";					break;
		case 76:	xmlfile << "rotation";					break;
		case 77:	xmlfile << "rx";						break;
		case 78:	xmlfile << "ry";						break;
		case 79:	xmlfile << "scale";						break;
		case 80:	xmlfile << "shape-rendering";			break;
		case 81:	xmlfile << "size";						break;
		case 82:	xmlfile << "solid-color";				break;
		case 83:	xmlfile << "solid-opacity";				break;
		case 84:	xmlfile << "stop-color";				break;
		case 85:	xmlfile << "stop-opacity";				break;
		case 86:	xmlfile << "stroke";					break;
		case 87:	xmlfile << "stroke-dasharray";			break;
		case 88:	xmlfile << "stroke-dashoffset";			break;
		case 89:	xmlfile << "stroke-linecap";			break;
		case 90:	xmlfile << "stroke-linejoin";			break;
		case 91:	xmlfile << "stroke-miterlimit";			break;
		case 92:	xmlfile << "stroke-opacity";			break;
		case 93:	xmlfile << "stroke-width";				break;
		case 94:	xmlfile << "svg.height";				break;
		case 95:	xmlfile << "svg.width";					break;
		case 96:	xmlfile << "syncBehavior";				break;
		case 97:	xmlfile << "syncBehaviorDefault";		break;
		case 98:	xmlfile << "syncReference";				break;
		case 99:	xmlfile << "syncTolerance";				break;
		case 100:	xmlfile << "syncToleranceDefault";		break;
		case 101:	xmlfile << "systemLanguage";			break;
		case 102:	xmlfile << "text-align";				break;
		case 103:	xmlfile << "text-anchor";				break;
		case 104:	xmlfile << "text-decoration";			break;
		case 105:	xmlfile << "text-display";				break;
		case 106:	xmlfile << "text-rendering";			break;
		case 107:	xmlfile << "textContent";				break;
		case 108:	xmlfile << "transform";					break;
		case 109:	xmlfile << "transformBehavior";			break;
		case 110:	xmlfile << "translation";				break;
		case 111:	xmlfile << "vector-effect";				break;
		case 112:	xmlfile << "viewBox";					break;
		case 113:	xmlfile << "viewport-fill";				break;
		case 114:	xmlfile << "viewport-fill-opacity";		break;
		case 115:	xmlfile << "visibility";				break;
		case 116:	xmlfile << "width";						break;
		case 117:	xmlfile << "x";							break;
		case 118:	xmlfile << "x1";						break;
		case 119:	xmlfile << "x2";						break;
		case 120:	xmlfile << "xlink:actuate";				break;
		case 121:	xmlfile << "xlink:arcrole";				break;
		case 122:	xmlfile << "xlink:href";				break;
		case 123:	xmlfile << "xlink:role";				break;
		case 124:	xmlfile << "xlink:show";				break;
		case 125:	xmlfile << "xlink:title";				break;
		case 126:	xmlfile << "xlink:type";				break;
		case 127:	xmlfile << "xml:base";					break;
		case 128:	xmlfile << "xml:lang";					break;
		case 129:	xmlfile << "y";							break;
		case 130:	xmlfile << "y1";						break;
		case 131:	xmlfile << "y2";						break;
		case 132:	xmlfile << "zoomAndPan";				break;
		}
	}
	return a;
}

void CSDLClasses::anyAttribute(CBitReader *pBR, CX *s)
{
	if (pBR->GetBit())
	{
		do 
		{
			pBR->GetBits(m_nExtensionIDBits);	// extID
			pBR->GetVlcInt(5);					// bitlen
// 			if (extID == 2)
// 			{
// 				if (s == NULL)
// 					// decoding AMD1 extension attribute for wrong class
// 				if (pBR->GetBit())	// hasRequiredFonts
// 					s->m_strRequiredFonts = pBR->GetStringList(pBR);
// 			}
// 			else
// 			{
// 				//
// 			}
		} while (pBR->GetBit());
	}
}

vector<CPrivAttr>* CSDLClasses::privateAttributeContainer(CBitReader *pBR)
{
	if (pBR->GetBit())	// hasPrivAttr
	{
		int ch4 = pBR->GetBits(4);
		switch (ch4)
		{
		case 0:
			return anyXMLAttribute(pBR);
		case 1:
			{
				int skipLen = pBR->GetVlcInt(5);
				pBR->flush(pBR);
				pBR->GetBits(m_nPrivateDataIdBits);
				pBR->flush(pBR);
				pBR->GetByteArray(skipLen - ((m_nPrivateDataIdBits + 7) >> 3));
			}
			break;
		case 2:
			anyAttribute(pBR, NULL);
			break;
		default:
			extension(pBR);
			break;
		}
	}
	return NULL;
}

vector<CPrivAttr>* CSDLClasses::anyXMLAttribute(CBitReader *pBR)
{
	pBR->GetBits(5);	// nb(of attrs)
	vector<CPrivAttr> *pa = NULL;
//	CUtils *pUtils = new CUtils();
//	int bits = pUtils->nbBitsForPositiveInteger(m_pMan->nbOfAttrs(0));
// 	for (int i = 0; i < nb; i++)
// 	{
// 
// 	}
	return pa;
}

vector<int> CSDLClasses::coordinateListType(CBitReader *pBR, ofstream &xmlfile)
{
	int len = pBR->GetVlcInt(5);
	vector<int> res;
	for (int i = 0; i < len; i++)
		res.push_back(coordinate(pBR));

	xmlfile << res[0] / 4096;
	if (len != 1)
		xmlfile << "." << res[1];
	xmlfile << "'";
	return res;
}

vector<CSmilTime>* CSDLClasses::timesType(CBitReader *pBR, ofstream &xmlfile)
{
	if (pBR->GetBit())	// msg
		return NULL;
		//return new SmilTime[] {new SmilTime()};
	else
	{
		int len = pBR->GetBits(5);	// nbTimes
		vector<CSmilTime> *res = new vector<CSmilTime>;
		for (int i = 0; i < len; i++)
			res->push_back(*smilTime(pBR, xmlfile));
		return res;
	}
}

CSmilTime* CSDLClasses::smilTime(CBitReader *pBR, ofstream &xmlfile)
{
	CSmilTime *res = new CSmilTime();
	if (pBR->GetBit())	// hasEvent
	{
		if (pBR->GetBit())	// hasId
		{
			res->m_nId = IDREF(pBR);
			xmlfile << res->m_nId << ".";
		}

		res->m_pEvent = ev(pBR, xmlfile);
	}
	if (pBR->GetBit())	// hasClock
	{
		bool neg = pBR->GetBit();	// sign
		res->m_nDelay = translateTimeUponReading(pBR->GetBits(5), false);
		if (neg)	res->m_nDelay = -res->m_nDelay;
	}
	return res;
}

void CSDLClasses::extension(CBitReader *pBR)
{
	int i = pBR->GetVlcInt(5);
	pBR->GetByteArray(i);
}

CURI* CSDLClasses::anyURI(CBitReader *pBR, ofstream &xmlfile)
{
	CURI *uri = new CURI;
	bool b = pBR->GetBit();	// hasURL
	if (b)
	{
		uri->m_strUri = byteAlignedString(pBR, xmlfile);
		// optional binary data part of the URL
		if (pBR->GetBit())	// hasData
		{
			//data part of the URL
			int len = pBR->GetVlcInt(5);	// data len
			uri->m_anData = pBR->GetByteArray(len);	// uri data
		}
		xmlfile << "'";
		// end of optional binary data part of the URL
	}
	b = pBR->GetBit();	// hasID
	if (b)
	{
		uri->m_nId = IDREF(pBR);	// ID
		xmlfile << "#" << uri->m_nId <<"'";
	}
	b = pBR->GetBit();	// hasStreamID
	if (b)
	{
		uri->m_nStreamID = IDREF(pBR);	// streamID
		xmlfile << "stream:Stream" << uri->m_nStreamID <<"'";
	}
	return uri;
}

CUpdateValue* CSDLClasses::updateValue(CBitReader *pBR, int type, ofstream &xmlfile)
{
	CUpdateValue *uv = new CUpdateValue(type);
//	int max = 0;
	switch (type)
	{
	case 0:		// TYPE_boolean
		{
			uv->m_nX = pBR->GetBit();
			switch (type)
			{
			case 0:
				xmlfile << "false";
				break;
			case 1:
				xmlfile << "true";
				break;
			}
		}
		break;
	case 1:		// TYPE_enum
		{
			uv->m_bIsDefault = pBR->GetBit();	// isDefault
			if (!uv->m_bIsDefault)
				uv->m_nX = pBR->GetVlcInt(5);
		}
		break;
	case 2:		// TYPE_color
		uv->m_nX = paint(pBR, xmlfile);
		break;
	case 3:		// TYPE_fraction
		break;
	case 4:		// TYPE_float
		{
			uv->m_bIsDefault = pBR->GetBit();	// isDefault
			if (!uv->m_bIsDefault)
			{
				if (pBR->GetBit())	// enum
					uv->m_nEscapeEnum = pBR->GetBits(2);
				else
					uv->m_nX = fixed_16_8(pBR, xmlfile);
			}
		}
		break;
	case 5:		// TYPE_time
		break;
	case 6:		// TYPE_point
		break;
	case 7:		// TYPE_matrix
		break;
	case 8:		// TYPE_string
		break;
	case 9:		// TYPE_points
		break;
	case 10:	// TYPE_path
		break;
	case 11:	// TYPE_ints
		break;
	case 12:	// TYPE_floats
		break;
	case 13:	// TYPE_smil_times
		break;
	case 14:	// TYPE_unit
		break;
	case 15:	// TYPE_index
		break;
	case 16:	// TYPE_URI
		break;
	case 17:	// TYPE_ID
		break;
	case 18:	// TYPE_event
		break;
	case 19:	// TYPE_scale
		break;
	case 20:	// TYPE_coordinate
		break;
	case 21:	// TYPE_keyTimes
		break;
	case -1:	// error
		break;
		//
	}
	return uv;	
}

int CSDLClasses::typeOf(CAttribute *a, int index)
{
	if (a->m_bIsLaser)
	{
		if (index >= 0)
			//indexed
			return updateIndexedType(a->m_nAttributeIndex);
		else
			// not indexed
			return updateType(a->m_nAttributeIndex);
	}
	else
		return 8;	// TYPE_string
}

int CSDLClasses::updateType(int nAttributeIndex)
{
	switch (nAttributeIndex)
	{
	case 1:
	case 2:
	case 20:
	case 21:
	case 24:
	case 28:
	case 34:
	case 132:
		return 0;	// TYPE_boolean
		break;
	case 6:
	case 12:
	case 17:
	case 18:
	case 27:
	case 31:
	case 32:
	case 33:
	case 35:
	case 38:
	case 61:
	case 62:
	case 65:
	case 74:
	case 80:
	case 89:
	case 90:
	case 96:
	case 97:
	case 102:
	case 103:
	case 104:
	case 105:
	case 106:
	case 109:
	case 111:
	case 115:
	case 120:
	case 124:
	case 126:
		return 1;	// TYPE_enum
		break;
	case 11:
	case 25:
	case 82:
	case 84:
	case 86:
	case 113:
		return 2;	// TYPE_color
		break;
	case 3:
	case 26:
	case 59:
	case 60:
	case 83:
	case 85:
	case 92:
	case 114:
		return 3;	// TYPE_fraction
		break;
	case 4:
	case 30:
	case 42:
	case 46:
	case 64:
	case 69:
	case 88:
	case 91:
	case 93:
	case 99:
	case 100:
		return 4;	// TYPE_float
		break;
	case 9:
	case 10:
	case 19:
	case 47:
	case 70:
	case 76:
		return 5;	// TYPE_time
		break;
	case 81:
	case 110:
		return 6;	// TYPE_point
		break;
	case 108:
		return 7;	// TYPE_matrix
		break;
	case 0:
	case 44:
	case 45:
	case 71:
	case 73:
	case 101:
	case 107:
	case 121:
	case 123:
	case 125:
	case 127:
	case 128:
		return 8;	// TYPE_string
		break;
	case 66:
		return 9;	// TYPE_points
		break;
	case 15:
	case 63:
		return 10;	// TYPE_path
		break;
	case 72:
		return 11;	// TYPE_ints
		break;
	case 16:
	case 39:
	case 40:
	case 75:
	case 87:
	case 112:
		return 12;	// TYPE_floats
		break;
	case 5:
	case 22:
		return 13;	// TYPE_smil_times
		break;
	case 94:
	case 95:
	case 116:
	case 117:
	case 118:
	case 119:
	case 129:
	case 130:
	case 131:
		return 14;	// TYPE_unit
		break;
	case 8:
	case 29:
		return 15;	// TYPE_index
		break;
	case 98:
		return 16;	// TYPE_URI
		break;
	case 36:
	case 43:
	case 48:
	case 49:
	case 50:
	case 51:
	case 52:
	case 53:
	case 54:
	case 55:
	case 56:
	case 57:
	case 58:
	case 122:
		return 17;	// TYPE_ID
		break;
	case 23:
		return 18;	// TYPE_event
		break;
	case 79:
		return 19;	// TYPE_scale
		break;
	case 13:
	case 14:
	case 37:
	case 68:
	case 77:
	case 78:
		return 20;	// TYPE_coordinate
		break;
	case 41:
		return 21;	// TYPE_keyTimes
		break;
	case 7:
	case 67:
		return -1;
		break;
	}
}

int CSDLClasses::updateIndexedType(int nAttributeIndex)
{
	switch (nAttributeIndex)
	{
	case 0:
	case 1:
	case 2:
	case 3:
	case 4:
	case 6:
	case 7:
	case 8:
	case 9:
	case 10:
	case 11:
	case 12:
	case 13:
	case 14:
	case 15:
	case 16:
	case 17:
	case 18:
	case 19:
	case 20:
	case 21:
	case 23:
	case 24:
	case 25:
	case 26:
	case 27:
	case 28:
	case 29:
	case 30:
	case 31:
	case 32:
	case 33:
	case 34:
	case 35:
	case 36:
	case 37:
	case 38:
	case 42:
	case 43:
	case 44:
	case 45:
	case 46:
	case 47:
	case 48:
	case 49:
	case 50:
	case 51:
	case 52:
	case 53:
	case 54:
	case 55:
	case 56:
	case 57:
	case 58:
	case 59:
	case 60:
	case 61:
	case 62:
	case 63:
	case 64:
	case 65:
	case 67:
	case 68:
	case 69:
	case 70:
	case 71:
	case 73:
	case 74:
	case 75:
	case 76:
	case 77:
	case 78:
	case 79:
	case 80:
	case 81:
	case 82:
	case 83:
	case 84:
	case 85:
	case 86:
	case 88:
	case 89:
	case 90:
	case 91:
	case 92:
	case 93:
	case 94:
	case 95:
	case 96:
	case 97:
	case 98:
	case 99:
	case 100:
	case 101:
	case 102:
	case 103:
	case 104:
	case 105:
	case 106:
	case 107:
	case 108:
	case 109:
	case 110:
	case 111:
	case 113:
	case 114:
	case 115:
	case 116:
	case 117:
	case 118:
	case 119:
	case 120:
	case 121:
	case 122:
	case 123:
	case 124:
	case 125:
	case 126:
	case 127:
	case 128:
	case 129:
	case 130:
	case 131:
	case 132:
		return -1;
		break;
	case 66:
		return 100;	// ITYPE_point
		break;
	case 72:
		return 101;	// ITYPE_int
		break;
	case 40:
	case 87:
	case 112:
		return 102;	// ITYPE_float
		break;
	case 41:
		return 103;	// ITYPE_keytime
		break;
	case 39:
		return 104;	// ITYPE_0to1
		break;
	case 5:
	case 22:
		return 105;	// ITYPE_smil_time
		break;
	}
}

vector<int>	CSDLClasses::valueWithUnits(CBitReader *pBR, ofstream &xmlfile)
{
	vector<int> u;
	u.push_back(pBR->GetBits(32));
	xmlfile << u[0] / 256;
	u.push_back(pBR->GetBits(3));
	if (u[1] != 0)
		xmlfile << "." << u[1];
	xmlfile << "'";
	return u;
}

tstring CSDLClasses::fontFamily(CBitReader *pBR, ofstream &xmlfile)
{
	if (pBR->GetBit())
	{
		xmlfile << "inherit'";
		return _T("inherit");
	}
	return m_pMan->getFont(pBR->GetBits(m_nFontIndexBits), xmlfile);
}

int CSDLClasses::fontStyle(CBitReader *pBR, ofstream &xmlfile)
{
	int fs = pBR->GetBits(3);
	switch (fs)
	{
	case 0:		xmlfile << "inherit";	break;
	case 1:		xmlfile << "italic";	break;
	case 2:		xmlfile << "normal";	break;
	case 3:		xmlfile << "oblique";	break;
	}
	xmlfile << "'";
	return fs;
}

int CSDLClasses::fontWeight(CBitReader *pBR, ofstream &xmlfile)
{
	int fw = pBR->GetBits(4);
	switch (fw)
	{
	case 0:		xmlfile << "100";		break;
	case 1:		xmlfile << "200";		break;
	case 2:		xmlfile << "300";		break;
	case 3:		xmlfile << "400";		break;
	case 4:		xmlfile << "500";		break;
	case 5:		xmlfile << "600";		break;
	case 6:		xmlfile << "700";		break;
	case 7:		xmlfile << "800";		break;
	case 8:		xmlfile << "900";		break;
	case 9:		xmlfile << "bold";		break;
	case 10:	xmlfile << "bolder";	break;
	case 11:	xmlfile << "inherit";	break;
	case 12:	xmlfile << "lighter";	break;				
	case 13:	xmlfile << "normal";	break;
	}
	xmlfile << "'";
	return fw;
}

CPath* CSDLClasses::path(CBitReader *pBR, ofstream &xmlfile)
{
	CPath* path = new CPath;
	path->m_anPointSeq = pointSequence(pBR, xmlfile);
	path->m_anTypes = pBR->GetByteArray(pBR->GetVlcInt(5));
	for (int i = 0; i < path->m_anTypes.size()  ; i++)
		path->m_anTypes[i] = pBR->GetBits(5);
	if ( (0 == path->m_anPointSeq.size()) || (0 == path->m_anTypes.size()) )
		path = NULL;            
	return path;
}

vector<int>	CSDLClasses::pointSequence(CBitReader *pBR, ofstream &xmlfile)
{
	int len = pBR->GetVlcInt(5);	// nbOfPoints
	vector<int> res;
	if (pBR->GetBit())	// newCodec
	{
		int k = pBR->GetBits(4);	// k
		int bits = pBR->GetBits(5);	// nbBitsFirst
		int firstX = translateUponReading(pBR->GetBits(bits), bits);	// firstX
		res.push_back(firstX);
		xmlfile << firstX / 4096 << ",";
		int firstY = translateUponReading(pBR->GetBits(bits), bits);	// firstY
		res.push_back(firstY);
		xmlfile << firstY / 4096 ;
		for (int i = 2; i < len * 2; i += 2)
		{
			int mvalue = 0;
			while (!pBR->GetBit())	// mvalue bits x
				mvalue++;
			int info = getInfo(pBR->GetBits(mvalue + k), k, mvalue);	// dx
			int dx = extract(info);
			firstX = firstX + translateUponReading(dx, mvalue + k + 1);
			res.push_back(firstX);
			xmlfile << " " << firstX / 4096 << ",";
			mvalue = 0;
			while (!pBR->GetBit())	// mvalue bits x
				mvalue++;
			info = getInfo(pBR->GetBits(mvalue + k), k, mvalue);	// dy
			int dy = extract(info);
			firstY = firstY + translateUponReading(dy, mvalue + k + 1);
			res.push_back(firstY);
			xmlfile << firstY / 4096;
		}
	}
	else
		read1(pBR, res, len, xmlfile);
	return res;
}

int CSDLClasses::extract(int i)
{
	if (i == 0) return i;
	if ((i & 1) == 0)
		return -(i >> 1);
	return (i + 1) >> 1;
}

int CSDLClasses::getInfo(int codeNum, int kvalue, int Mvalue)
{
	int k = 1;
	for (int i = 0; i < kvalue; i++)
		k = k * 2;
	int m = k;
	for (int j = kvalue - 1; j < Mvalue; j++)
		m = m * 2;
	return codeNum - k + m;
}

void CSDLClasses::read1(CBitReader *pBR, vector<int> fv, int len, ofstream &xmlfile)
{
	/*
	If the number of points is less than 3, a simple encoding is used.
	A number of 5 bits give the number of bits on which the coordinates are encoded.
	*/
	if (len < 3)
	{
		int bits = pBR->GetBits(5);	// nbBits
		for (int i = 0; i < len * 2; i++)
			fv.push_back(translateUponReading(pBR->GetBits(bits), bits));	// coord
	}
	else
	{
		/*
		If the number of points LEN in the polygon are more than 3, then:
		- the number N1 is read on 5 bits
		- the first two coordinates are read from the stream using N1 bits
		- the number NX and NY are read on 5 bits
		- then LEN-1 points are read, using NX points for the X value and NY bits for the Y value, both X and Y
		being differences from the previous point
		*/
		int bits	= pBR->GetBits(5);	// nbBitsFirst
		int firstX	= translateUponReading(pBR->GetBits(bits), bits);	// firstX
		fv.push_back(firstX);
		xmlfile << firstX / 4096 << ",";
		int firstY	= translateUponReading(pBR->GetBits(bits), bits);	// firstY
		fv.push_back(firstY);
		xmlfile << firstY / 4096;
		int bitsX	= pBR->GetBits(5);	// nbBitsX
		int bitsY	= pBR->GetBits(5);	// nbBitsY
		for (int i = 2; i < len * 2; i += 2)
		{
			int cur		= translateUponReading(pBR->GetBits(bitsX), bitsX);	// nextX
			fv.push_back(firstX + cur);
			firstX		= firstX + cur;
			xmlfile << " " << firstX / 4096 << ",";
			cur			= translateUponReading(pBR->GetBits(bitsY), bitsY);	// nextY
			fv.push_back(firstY + cur);
			firstY		= firstY + cur;
			xmlfile << firstY / 4096;
		}
	}
}

int CSDLClasses::float0to1(CBitReader *pBR, ofstream &xmlfile)
{
	int i = (pBR->GetBits(8) << 12) / 255;
	xmlfile << (float)i / 4096 << "'";
	return i;
}

int CSDLClasses::fixed_16_8(CBitReader *pBR, ofstream &xmlfile)
{
	int i = pBR->GetBits(24) << 4;
	if ((i & 0x08000000) != 0)
		i |= 0xF0000000;
	xmlfile << i / 4096;
	return i;
}

int CSDLClasses::fixed_16_8i(CBitReader *pBR, ofstream &xmlfile)
{
	bool b = pBR->GetBit();
	if (b)
		return -1;	// inherit
	int i = pBR->GetBits(24) << 4;
	if ((i & 0x08000000) != 0)
		i |= 0xF0000000;
	xmlfile << i / 4096 << "'";
	return i;
}


CMatrix* CSDLClasses::transform(CBitReader *pBR, ofstream &xmlfile)
{
	if (!pBR->GetBit())	// isNotMatrix
	{
		CMatrix *m = new CMatrix;
		if (pBR->GetBit())
		{
			m->m_nXx = translateScale(pBR->GetBits(m_nCoordBits + m_nScaleBits));
			m->m_nYy = translateScale(pBR->GetBits(m_nCoordBits + m_nScaleBits));
			m->m_bXxValid = true;
			m->m_bYyValid = true;
		}
		if (pBR->GetBit())
		{
			m->m_nXy = translateScale(pBR->GetBits(m_nCoordBits + m_nScaleBits));
			m->m_nYx = translateScale(pBR->GetBits(m_nCoordBits + m_nScaleBits));
			m->m_bXyValid = true;
			m->m_bYxValid = true;
		}
		if (m->m_bXxValid && m->m_bYyValid)
		{
			xmlfile << " lsr:scale='";
			if (m->m_nYy == 0)
				xmlfile << m->m_nXx / cos(atan((float)m->m_nYx / (float)m->m_nXx)) / 4096 << "'";
			else
				xmlfile << m->m_nXx / cos(atan((float)m->m_nYx / (float)m->m_nXx)) / 4096 << "," << m->m_nYy / cos(atan((float)m->m_nYx / (float)m->m_nXx)) / 4096 << "'";
		}
		if (m->m_bXyValid && m->m_bYxValid)
		{
			xmlfile << " lsr:rotation='";
			xmlfile << atan((float)m->m_nYx / (float)m->m_nXx) << "'";
		}
		if (pBR->GetBit())
		{
			xmlfile << " lsr:translation='";
			m->m_nXz = translateUponReading(pBR->GetBits(m_nCoordBits + m_nScaleBits), m_nCoordBits + m_nScaleBits);
			m->m_nYz = translateUponReading(pBR->GetBits(m_nCoordBits + m_nScaleBits), m_nCoordBits + m_nScaleBits);
			m->m_bXzValid = true;
			m->m_bYzValid = true;
			xmlfile << m->m_nXz / 4096 << "," << m->m_nYz / 4096 << "'";
		}
		return m;
	} 
	else
	{
		// to take into account ref(svg[,x,y]), another case is inserted here
		if (pBR->GetBit())	// isRef
		{
			CMatrix *m = new CMatrix;
			m->m_bIsRefSVG = true;
			if (pBR->GetBit())	// hasXY
			{
				m->m_nXz = fixed_16_8(pBR, xmlfile);
				m->m_bXzValid = true;
				m->m_nYz = fixed_16_8(pBR, xmlfile);
				m->m_bYzValid = true;
			}
			return m;
		} 
		else
			// end insertion of ref(svg)
			extension(pBR);
		return NULL;
	}
}


int CSDLClasses::ID(CBitReader *pBR, ofstream &xmlfile)
{
	xmlfile << " id='";

	int i = pBR->GetVlcInt(5);
	if (pBR->GetBit())	// reserved
	{
		int len = pBR->GetVlcInt(5);
		if (len % 8 > 0)
			pBR->GetBits(len % 8);
		if (len >> 3 > 0)
			pBR->GetByteArray(len >> 3);
	}
	xmlfile << i << "'";
	return i;
}

int CSDLClasses::IDREF(CBitReader *pBR)
{
	int i = pBR->GetVlcInt(5);
	if (pBR->GetBit())	// reserved
	{
		int len = pBR->GetVlcInt(5);
		if (len % 8 > 0)
			pBR->GetBits(len % 8);
		if (len >> 3 > 0)
			pBR->GetByteArray(len >> 3);
	}
	return i;
}

int CSDLClasses::paint(CBitReader *pBR, ofstream &xmlfile)
{
	if (pBR->GetBit())	// isColorIndex
		return m_pMan->getColor(pBR->GetBits(m_nColorIndexBits), xmlfile);
	else
	{
		switch (pBR->GetBits(2))	// ch2
		{
		case 0:
			// enum
			switch (pBR->GetBits(2))
			{
			case 0:
				return -3;
			case 1:
				return -1;
			case 2:
				return -2;
			}
			break;
		case 1:
			// URI
			return -4 - m_pCurrentScene->m_pIdMan->registerPaintURI(anyURI(pBR, xmlfile));
		case 2:
			// System Paint Server
			byteAlignedString(pBR, xmlfile);
			break;
		case 3:
			// extension
			extension(pBR);
			break;
		}
	}
	return -1;
}

int CSDLClasses::syncTolerance(CBitReader *pBR)
{
	int sw = pBR->GetBit();
	switch (sw)
	{
	case 0:
		return translateTimeUponReading(pBR->GetVlcInt(5), true);
	case 1:
		return -1;
	}
	return -2;
}

tstring CSDLClasses::byteAlignedString(CBitReader *pBR)
{
	return pBR->GetStringList(pBR);
}

tstring CSDLClasses::byteAlignedString(CBitReader *pBR, ofstream &xmlfile)
{
	return pBR->GetStringList(pBR, xmlfile);
}

CDEvent* CSDLClasses::ev(CBitReader *pBR, ofstream &xmlfile)
{
	CDEvent *e = new CDEvent;
	if (pBR->GetBit())	// isEnum
	{
		e->m_nEvent = pBR->GetBits(6);	// event
		switch (e->m_nEvent)
		{
		case 1:		// accessKey
//		case 15:	// longAccessKey
//		case 25:	// repeatKey
//		case 29:	// shortAccessKey
			e->m_nKeyCode = pBR->GetVlcInt(5);	// keyCode
			switch (e->m_nKeyCode) 
			{
			case 0:		xmlfile << "accessKey(*)";			break;
			case 1:		xmlfile << "accessKey(0)";			break;
			case 2:		xmlfile << "accessKey(1)";			break;
			case 3:		xmlfile << "accessKey(2)";			break;
			case 4:		xmlfile << "accessKey(3)";			break;
			case 5:		xmlfile << "accessKey(4)";			break;
			case 6:		xmlfile << "accessKey(5)";			break;
			case 7:		xmlfile << "accessKey(6)";			break;
			case 8:		xmlfile << "accessKey(7)";			break;
			case 9:		xmlfile << "accessKey(8)";			break;
			case 10:	xmlfile << "accessKey(9)";			break;
			case 11:	xmlfile << "accessKey(ANY_KEY)";	break;
			case 12:	xmlfile << "accessKey(DOWN)";		break;
			case 13:	xmlfile << "accessKey(FIRE)";		break;
			case 14:	xmlfile << "accessKey(LEFT)";		break;
			case 15:	xmlfile << "accessKey(NO_KEY)";		break;
			case 16:	xmlfile << "accessKey(RIGHT)";		break;
			case 17:	xmlfile << "accessKey(SHARP)'";		break;
			case 18:	xmlfile << "accessKey(SOFT_KEY_1)";	break;
			case 19:	xmlfile << "accessKey(SOFT_KEY_2)";	break;
			case 20:	xmlfile << "accessKey(UP)";			break;
			}
			break;
		case 5:		// click
			xmlfile << "click";
			break;
		case 30:	// xmlUpdate
			xmlfile << "xmlUpdate";
			break;
		}
	}
	else
	{
		e->m_strEventString = byteAlignedString(pBR, xmlfile);
	}
	return e;	
}

CAnimValue* CSDLClasses::AnimatedValue(CBitReader *pBR, CAttribute *attributeName, ofstream &xmlfile)
{
	CAnimValue *av = new CAnimValue;
	av->m_nType = pBR->GetBits(4);
	return animValue1(av, pBR, attributeName, xmlfile);
}

CAnimValue* CSDLClasses::animValue1(CAnimValue *av, CBitReader *pBR, CAttribute *attributeName, ofstream &xmlfile)
{
	if (pBR->GetBit())	// escapeFlag
	{
		av->m_nEscapeEnum = pBR->GetBits(2);	// escapeEnum
		return av;
	}
	switch (av->m_nType)
	{
	case 0:		// ANIMTYPE_string
		av->m_strString = byteAlignedString(pBR, xmlfile);
		break;
	case 1:		// ANIMTYPE_float
		av->m_nFractionOrFloatOrColor = fixed_16_8(pBR, xmlfile);
		break;
	case 12:	// ANIMTYPE_uri
		av->m_pUri = anyURI(pBR, xmlfile);
		break;
	case 2:		// ANIMTYPE_path
		av->m_pPath = path(pBR, xmlfile);
		break;
	case 3:		// ANIMTYPE_pointSeq
		av->m_anPointSeq = pointSequence(pBR, xmlfile);
		break;
	case 4:		// ANIMTYPE_fraction
		av->m_nFractionOrFloatOrColor = float0to1(pBR, xmlfile);
		break;
	case 5:		// ANIMTYPE_color
		av->m_nFractionOrFloatOrColor = paint(pBR, xmlfile);
		break;
	case 6:		// ANIMTYPE_enum
		{
			av->m_nFractionOrFloatOrColor = pBR->GetVlcInt(5);
			switch (attributeName->m_nAttributeIndex)
			{
			case 115:
				{
					switch (av->m_nFractionOrFloatOrColor)
					{
					case 1:	xmlfile << "inherit";	break;
					case 0:	xmlfile << "hidden";	break;
					case 2:	xmlfile << "visible";	break;
					}
				}
				break;
			}
		}
		break;
	case 10:	// ANIMTYPE_id
		av->m_nFractionOrFloatOrColor = IDREF(pBR);
		break;
	case 11:	// ANIMTYPE_font
		av->m_strString = m_pMan->getFont(pBR->GetVlcInt(5), xmlfile);
		break;
	case 8:		// ANIMTYPE_floats
		{
			int max = pBR->GetVlcInt(5);
			for (int i = 0; i < max; i++)
				av->m_anTab[i] = fixed_16_8(pBR, xmlfile);
		}
		break;
// 	case 7:		// ANIMTYPE_ints
// 		int ma = br.readVlcInt(5, msg);
// 		av.tab = new int[ma];
// 		for (int i = 0; i < ma; i++) {
// 		av.tab[i] = br.readVlcInt(5, msg);
// 		}
// 		break;
	case 9:		// ANIMTYPE_point
		{
			av->m_anTab[0] = coordinate(pBR);
			av->m_anTab[1] = coordinate(pBR);
		}
		break;
	case 13:
		extension(pBR);
		break;
	}
	return av;
}

int CSDLClasses::time(CBitReader *pBR, bool dur)
{
	if (pBR->GetBit())
	{
		// time values need to be negative sometimes, thus the addition of a sign bit
		bool b = pBR->GetBit();	// sign
		int a = pBR->GetVlcInt(5);
		return translateTimeUponReading((b ? -a : a), dur);
	}
	else
	{
		switch (pBR->GetBits(2))	// time
		{
		case 1:
			return -1;	// indefinite
		case 2:
			return -2;	// media
		case 0:
			return -3;	// auto
		}
	}
	return -4;	// illegal
}

vector<int>	CSDLClasses::floatList(int max, CBitReader *pBR, ofstream &xmlfile)
{
	vector<int> u;
	for (int i = 0; i < max; i++)
	{
		u.push_back(fixed_16_8(pBR, xmlfile));
		if (i != max - 1)
			xmlfile << " ";
		else
			xmlfile << "'";
	}
	return u;
}

int CSDLClasses::negp(int i, int nbBits)
{
	int j = 1;
	for (int k = 0; k < nbBits - 1; k++)
		j = j * 2;
	if (i > (j - 1))
	{
		if (nbBits == 0)
			i -= j;
		else
			i -= j * 2;
	}
	return i;
}

int CSDLClasses::translateUponReading(int n, int bits)
{
	int a = negp(n, bits);
	return a << (12 - m_nResolution);
}

int CSDLClasses::translateTimeUponReading(int i, bool dur)
{
	// read time as ticks
	// return time as 20.12 float
	return ((int)(((float)i) * 4096 / m_fTimeResolution)) + (dur ? 0 : m_nCurrentTimeInTics);
}

int CSDLClasses::translateScale(int n)
{
	int j = 1;
	for (int k = 0; k < m_nScaleBits + m_nCoordBits; k++)
		j = j * 2;
	if (n >= j / 2)
		n -= j;
	return (m_nScaleBits > 4 ? n >> (m_nScaleBits - 4) : n << (4 - m_nScaleBits));
}

int CSDLClasses::updateInterval(CBitReader *pBR, ofstream &xmlfile)
{
	xmlfile << " updateInterval='";

	if (!pBR->GetBit())
	{
		// time values need to be negative sometimes, thus the addition of a sign bit
		bool b = pBR->GetBit();	// sign
		int a = pBR->GetVlcInt(5);	// <Clock-value>
		xmlfile << translateTimeUponReading((b ? -a : a), true) / 4096 << "'";
		return translateTimeUponReading((b ? -a : a), true);
	}
	else
	{
		switch (pBR->GetBit())	// update
		{
		case 0:
			xmlfile << "free'";
			return -1;	// free
		case 1:
			xmlfile << "none'";
			return -2;	// none
		}
	}
	xmlfile << "free'";
	return -1;	// free
}

int CSDLClasses::externalUpdateType(CExternalUpdate *pEU, CBitReader *pBR, ofstream &xmlfile)
{
	xmlfile << " type='";

	switch (pBR->GetBits(2))	// type
	{
	case 0:
		xmlfile << "replace'";
		Replace(pEU, pBR, xmlfile);
		return 0;
	case 1:
		xmlfile << "insert'";
		Insert(pEU, pBR, xmlfile);
		return 1;
	case 2:
		xmlfile << "delete'";
		Delete(pEU, pBR, xmlfile);
		return 2;
	default:
		break;
	}
	return 0;
}

void CSDLClasses::Replace(CExternalUpdate *pEU, CBitReader *pBR, ofstream &xmlfile)
{
	if (pBR->GetBit())																	// attributeName
	{
		xmlfile << " attributeName='";
		pEU->m_strAttributeName = byteAlignedString(pBR, xmlfile);
		xmlfile << "'";
	}
	if (pBR->GetBit())	{}																// index
	if (pBR->GetBit())	{ pEU->m_nOperandElementId = IDREF(pBR); }						// operandElementId
	if (pBR->GetBit())	{ pEU->m_pOperandAttributeName = AttributeName(pBR, xmlfile); }	// operandAttributeName
	if (pBR->GetBit())																	// value
	{
		xmlfile << " value='";
		pEU->m_strValue = byteAlignedString(pBR, xmlfile);
		xmlfile << "'";
	}
	if (pBR->GetBit())																	// xlink:href
	{
		xmlfile << " xlink:href='";
		pEU->setHref(anyURI(pBR, xmlfile));
	}
	pEU->m_apPrivateAttributes = privateAttributeContainer(pBR);
}

void CSDLClasses::Insert(CExternalUpdate *pEU, CBitReader *pBR, ofstream &xmlfile)
{
	if (pBR->GetBit())												// attributeName
	{
		xmlfile << " attributeName='";
		pEU->m_strAttributeName = byteAlignedString(pBR, xmlfile);
		xmlfile << "'";
	}
	if (pBR->GetBit())	{}											// index
	if (pBR->GetBit())												// value
	{
		xmlfile << " value='";
		pEU->m_strValue = byteAlignedString(pBR, xmlfile);
		xmlfile << "'";
	}
	if (pBR->GetBit())												// xlink:href
	{
		xmlfile << " xlink:href='";
		pEU->setHref(anyURI(pBR, xmlfile));
	}
	pEU->m_apPrivateAttributes = privateAttributeContainer(pBR);
}

void CSDLClasses::Delete(CExternalUpdate *pEU, CBitReader *pBR, ofstream &xmlfile)
{
	if (pBR->GetBit())												// attributeName
	{
		xmlfile << " attributeName='";
		pEU->m_strAttributeName = byteAlignedString(pBR, xmlfile);
		xmlfile << "'";
	}
	if (pBR->GetBit())	{}											// index
	if (pBR->GetBit())												// xlink:href
	{
		xmlfile << " xlink:href='";
		pEU->setHref(anyURI(pBR, xmlfile));
	}
}

