/*
// This software was originally developed by ETRI and Kyunghee University, in the course
// of development of the ISO/IEC 14496-20 (Lightweight Application Scene Representation 
// -LASeR- and Simple Aggregation Format -SAF-) AMD3: Presentation and Modification of
// Structured Information standard for reference purposes and its performance may not 
// have been optimized.
//
// This software module is an implementation of one or more tools as specified by the
// ISO/IEC 14496-20 AMD3: PMSI standard.
//
// ISO/IEC gives users free license to this software module or modifications thereof
// for use in products claiming conformance to audiovisual and image-coding related
// ITU Recommendations and/or ISO/IEC International Standards.
// 
// ISO/IEC gives users the same free license to this software module or modifications
// thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in products are advised that its use may
// infringe existing patents. 
// 
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
*/
#ifndef _DEC_SDLCLASSES_H_
#define _DEC_SDLCLASSES_H_

#include "DecTools.h"

#include "Utils.h"

#include "Scene.h"
#include "BitReader.h"


class CSDLClasses
{
public:
	CSDLClasses();
	~CSDLClasses();

public:
	int			m_nScaleBits;
	int			m_nCoordBits;
	int			m_nResolution;
	int			m_nColorComponentBits;
	int			m_nColorIndexBits;
	int			m_nFontIndexBits;
	int			m_nNameSpaceIndexBits;
	int			m_nTagIndexBits;
	int			m_nCodecBits;
	int			m_nExtensionIDBits;
	int			m_nPrivateDataIdBits;
	CScene		*m_pCurrentScene;
	CManager	*m_pMan;
	int			m_nPointBits;
	int			m_nFixed_16_8;
	float		m_fTimeResolution;
//	int			matrixBits;
	int			m_nCurrentTimeInTics;

public:
	CAttribute*			AttributeName(CBitReader *pBR, ofstream &xmlfile);
	void				anyAttribute(CBitReader *pBR, CX *s);
	vector<CPrivAttr>*	privateAttributeContainer(CBitReader *pBR);
	vector<CPrivAttr>*	anyXMLAttribute(CBitReader *pBR);
	int					preserveAspectRatio(CBitReader *pBR)	{ return pBR->GetBits(6); }
	vector<int>			coordinateListType(CBitReader *pBR, ofstream &xmlfile);
	vector<CSmilTime>*	timesType(CBitReader *pBR, ofstream &xmlfile);
	CSmilTime*			smilTime(CBitReader *pBR, ofstream &xmlfile);
	void				extension(CBitReader *pBR);
	CURI*				anyURI(CBitReader *pBR, ofstream &xmlfile);
	CUpdateValue*		updateValue(CBitReader *pBR,int type, ofstream &xmlfile);
	int					typeOf(CAttribute *a, int index);
	int					updateType(int nAttributeIndex);
	int					updateIndexedType(int nAttributeIndex);
	vector<int>			valueWithUnits(CBitReader *pBR, ofstream &xmlfile);
	tstring				fontFamily(CBitReader *pBR, ofstream &xmlfile);
	int					fontStyle(CBitReader *pBR, ofstream &xmlfile);
	int					fontWeight(CBitReader *pBR, ofstream &xmlfile);
	CPath*				path(CBitReader *pBR, ofstream &xmlfile);
	vector<int>			pointSequence(CBitReader *pBR, ofstream &xmlfile);
	int					extract(int i);
	int					getInfo(int codeNum, int kvalue, int Mvalue);
	void				read1(CBitReader *pBR, vector<int> fv, int len, ofstream &xmlfile);
	int					float0to1(CBitReader *pBR, ofstream &xmlfile);
	int					fixed_16_8(CBitReader *pBR, ofstream &xmlfile);
	int					fixed_16_8i(CBitReader *pBR, ofstream &xmlfile);
	int					coordinate(CBitReader *pBR)	{ return translateUponReading(pBR->GetBits(m_nCoordBits), m_nCoordBits); }
	CMatrix*			transform(CBitReader *pBR, ofstream &xmlfile);

	int			ID(CBitReader *pBR, ofstream &xmlfile);
	int			IDREF(CBitReader *pBR);
	int			paint(CBitReader *pBR, ofstream &xmlfile);
	int			syncTolerance(CBitReader *pBR);
	tstring		byteAlignedString(CBitReader *pBR);
	tstring		byteAlignedString(CBitReader *pBR, ofstream &xmlfile);
	CDEvent*	ev(CBitReader *pBR, ofstream &xmlfile);
	CAnimValue*	AnimatedValue(CBitReader *pBR, CAttribute *attributeName, ofstream &xmlfile);
	CAnimValue* animValue1(CAnimValue *av, CBitReader *pBR, CAttribute *attributeName, ofstream &xmlfile);
	int			time(CBitReader *pBR, bool dur);
	vector<int>	floatList(CBitReader *pBR, ofstream &xmlfile)	{ return floatList(pBR->GetVlcInt(5), pBR, xmlfile); }
	vector<int>	floatList(int max, CBitReader *pBR, ofstream &xmlfile);
	int			negp(int i, int nbBits);
	int			translateUponReading(int n, int bits);
	int			translateTimeUponReading(int i, bool dur);
	int			translateScale(int n);
	int			updateInterval(CBitReader *pBR, ofstream &xmlfile);
	int			externalUpdateType(CExternalUpdate *pEU, CBitReader *pBR, ofstream &xmlfile);
	void		Replace(CExternalUpdate *pEU, CBitReader *pBR, ofstream &xmlfile);
	void		Insert(CExternalUpdate *pEU, CBitReader *pBR, ofstream &xmlfile);
	void		Delete(CExternalUpdate *pEU, CBitReader *pBR, ofstream &xmlfile);
};


#endif