/*
// This software was originally developed by ETRI and Kyunghee University, in the course
// of development of the ISO/IEC 14496-20 (Lightweight Application Scene Representation 
// -LASeR- and Simple Aggregation Format -SAF-) AMD3: Presentation and Modification of
// Structured Information standard for reference purposes and its performance may not 
// have been optimized.
//
// This software module is an implementation of one or more tools as specified by the
// ISO/IEC 14496-20 AMD3: PMSI standard.
//
// ISO/IEC gives users free license to this software module or modifications thereof
// for use in products claiming conformance to audiovisual and image-coding related
// ITU Recommendations and/or ISO/IEC International Standards.
// 
// ISO/IEC gives users the same free license to this software module or modifications
// thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in products are advised that its use may
// infringe existing patents. 
// 
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
*/
#include "DecTools.h"

#include "Ilocatable.h"
#include "AnimValue.h"
#include "Attribute.h"


class CQName
{
public:
	CQName();

public:
	tstring	m_strName;
};


class CPrivAttr : public CQName
{
public:
	CPrivAttr();

public:
	tstring m_strValue;
};


class CX
{
public:
	CX();

public:
	int					m_nId;
	tstring				m_strClasse;
	tstring				m_strBase;
	tstring				m_strLang;
	int					m_nSpace;
 	vector<CPrivAttr>	*m_apPrivateAttributes;
	bool				m_bExternalResourcesRequired;
	int					m_nNavPrev;
	int					m_nNavNext;
	int					m_nNavUp;
	int					m_nNavRight;
	int					m_nNavLeft;
	int					m_nNavDown;
	int					m_nNavUpRight;
	int					m_nNavUpLeft;
	int					m_nNavDownRight;
	int					m_nNavDownLeft;
	int					m_nFocusable;
	tstring				m_strRequiredFonts; 
	int					m_nX;	// this is for rectClip, v2 extension
	int					m_nY;	// this is for rectClip, v2 extension
	int					m_nActive;
};


class CXpropElem : public CX
{
public:
	CXpropElem();
	~CXpropElem();

public:
	int			m_nAudio_level;
	int			m_nColor;
	int			m_nColor_rendering;
	int			m_nDisplay;
	int			m_nDisplay_align;
	int			m_nFill;
	int			m_nFill_opacity;
	int			m_nFill_rule;
	tstring		m_nstrFont_family;
	int			m_nFont_size;
	int			m_nFont_style;
	int			m_nText_decoration;
	int			m_nFont_variant;
	int			m_nFont_weight;
	int			m_nImage_rendering;
	int			m_nLine_increment;
	int			m_nPointer_events;
	int			m_nShape_rendering;
	int			m_nText_anchor;
	int			m_nText_rendering;
	int			m_nVector_effect;
	int			m_nVisibility;
	int			m_nStop_color;
	int			m_nStop_opacity;
	int			m_nSolid_color;
	int			m_nSolid_opacity;
	int			m_nStroke;
	vector<int>	m_anStroke_dasharray;
	int			m_nStroke_dashoffset;
	int			m_nStroke_linecap;
	int			m_nStroke_linejoin;
	int			m_nStroke_miterlimit;
	int			m_nStroke_opacity;
	int			m_nStroke_width;
	int			m_nViewport_fill;
	int			m_nViewport_fill_opacity;
	int			m_nOverflow;
	int			m_nFocusHighlight;

public:
	int			getDisplay()							{ return m_nDisplay; }
	void		setDisplay(int nDisplay, ofstream &xmlfile);
	int			getPointer_events()						{ return m_nPointer_events; }
	void		setPointer_events(int nPointer_events)	{ m_nPointer_events = nPointer_events; }
	int			getVisibility()							{ return m_nVisibility; }
	void		setVisibility(int nVisibility, ofstream &xmlfile);
	void		setVector_effectType(int nVector_effect, ofstream &xmlfile);
};


class CXset : public CX
{
public:
	CXset();
	~CXset();

public:
	CAttribute			*m_pAttributeName;
	int					m_nAttributeType;
	vector<CSmilTime>*	m_apBegin;
	int					m_nDur;
	int					m_nMin;
	int					m_nMax;
	bool				m_bEnabled;
	vector<CSmilTime>*	m_apEnd;
	int					m_nFill;
	int					m_nRepeatCount;
	int					m_nRepeatDur;
	int					m_nRestart;
	CAnimValue			*m_pTo;
	int					m_nXlink_actuate;
	CURI				*m_pXlink_arcrole;
	CURI				*m_pHref;
	CURI				*m_pXlink_role;
	int					m_nXlink_show;
	tstring				m_strXlink_title;
	int					m_nXlink_type;

public:
	void				setHref(CURI* href)	{ m_pHref = href; }
};


class CA : public CXpropElem, CIlocatable
{
public:
	CA();
	~CA();

public:
	tstring		m_strRequiredExtensions;
	vector<int>	m_anRequiredFeatures;
	tstring		m_strRequiredFormats;
	tstring		m_strSystemLanguage;
	int			m_nXlink_actuate;
	CURI		*m_pXlink_arcrole;
	CURI		*m_pHref;
	CURI		*m_pXlink_role;
	int			m_nXlink_show;
	tstring		m_strXlink_title;
	tstring		m_strTarget;
	int			m_nXlink_type;
	CAttribute	*m_pAttributeName;

public:
	void		setHref(CURI* href)	{ m_pHref = href; }
};


class CAnimate : public CXset
{
public:
	CAnimate();
	~CAnimate();

public:
	bool		m_bAccumulate;
	bool		m_Additive;
// 	AnimValue by;
	int			m_nCalcMode;
//	AnimValue from;
	vector<int>	m_anKeySplines;
	vector<int> m_anKeyTimes;
//	AnimValue[] values;
};


class CAudio : public CXpropElem
{
public:
	CAudio();
	~CAudio();

public:
// 	CSmilTime begin = null;
// 	CSmilTime end = null;
	int			m_nClipBegin;
	int			m_nClipEnd;
	int			m_nDur;
	int			m_nMin;
	int			m_nMax;
	int			m_nRepeatCount;
	int			m_nRepeatDur;
	tstring		m_strRequiredExtensions;
	vector<int>	m_anRequiredFeatures;
	tstring		m_strRequiredFormats;
	int			m_nSyncBehavior;
	CURI		*m_pSyncReference;
	int			m_nSyncTolerance;
	int			m_nSyncMaster;
	tstring		m_strSystemLanguage;
	int			m_nXlink_actuate;
	CURI		*m_pXlink_arcrole;
	CURI		*m_pHref;
	CURI		*m_pXlink_role;
	int			m_nXlink_show;
	tstring		m_strXlink_title;
	int			m_nXlink_type;
	tstring		m_strType;
	int			m_nRestart;

public:
	void		setHref(CURI* href)	{ m_pHref = href; }
};


class CCircle : public CXpropElem, CIlocatable
{
public:
	CCircle();
	~CCircle();

public:
	int			m_nCx;
	int			m_nCy;
	int			m_nR;
	tstring		m_strRequiredExtensions;
	vector<int>	m_anRequiredFeatures;
	tstring		m_strRequiredFormats;
	tstring		m_strSystemLanguage;
	CAttribute	*m_pAttributeName;
};


class CEllipse : public CXpropElem, CIlocatable
{
public:
	CEllipse();
	~CEllipse();

public:
	int			m_nCx;
	int			m_nCy;
	tstring		m_strRequiredExtensions;
	vector<int>	m_anRequiredFeatures;
	tstring		m_strRequiredFormats;
	int			m_nRx;
	int			m_nRy;
	tstring		m_strSystemLanguage;
	CAttribute	*m_pAttributeName;
};


class CG : public CXpropElem
{
public:
	CG();
	~CG();

public:
	tstring		m_strRequiredExtensions;
	vector<int>	m_anRequiredFeatures;
	tstring		m_strRequiredFormats;
	int			m_nRotation;
	CDPoint		*m_pScale;
	tstring		m_strSystemLanguage;
	CMatrix		*m_pTransform;
	CDPoint		*m_pTranslation;
	CAttribute	*m_pAttributeName;
};


class CImage : public CXpropElem
{
public:
	CImage();
	~CImage();

public:
	int			m_nHeight;
	int			m_nOpacity;
	tstring		m_strRequiredExtensions;
	vector<int>	m_anRequiredFeatures;
	tstring		m_strRequiredFormats;
	tstring		m_strRequiredFonts;
	int			m_nRotation;
	CDPoint		*m_pScale;
	tstring		m_strSystemLanguage;
	CMatrix		*m_pTransform;
	int			m_nTransformBehavior;
	CDPoint		*m_pTranslation;
	int			m_nWidth;
	int			m_nX;
	int			m_nY;
	int			m_nXlink_actuate;
	CURI		*m_pXlink_arcrole;
	CURI		*m_pHref;
	CURI		*m_pXlink_role;
	int			m_nXlink_show;
	tstring		m_strXlink_title;
	int			m_nXlink_type;
	tstring		m_strType;
	CAttribute	*m_pAttributeName;
	int			m_nPreserveAspectRatio;

public:
	void		setHref(CURI* href)	{ m_pHref = href; }
};


class CLine : public CXpropElem, CIlocatable
{
public:
	CLine();
	~CLine();

public:
	tstring		m_strRequiredExtensions;
	vector<int>	m_anRequiredFeatures;
	tstring		m_strRequiredFormats;
	tstring		m_strSystemLanguage;
	int			m_nX1;
	int			m_nX2;
	int			m_nY1;
	int			m_nY2;
	CAttribute	*m_pAttributeName;
};


class CPolygon : public CXpropElem, CIlocatable
{
public:
	CPolygon();
	~CPolygon();

public:
	vector<int>	m_anPoints;
	tstring		m_strRequiredExtensions;
	vector<int>	m_anRequiredFeatures;
	tstring		m_strRequiredFormats;
	tstring		m_strSystemLanguage;
	CAttribute	*m_pAttributeName;
};


class CPolyline : public CPolygon
{
public:
	CPolyline();
};


class CDRect : public CXpropElem
{
public:
	CDRect();
	~CDRect();

public:
	int			m_nHeight;
	tstring		m_strRequiredExtensions;
	vector<int>	m_anRequiredFeatures;
	tstring		m_strRequiredFormats;
	int			m_nRotation;
	int			m_nRx;
	int			m_nRy;
	CDPoint		*m_pScale;
	tstring		m_strSystemLanguage;
	CMatrix		*m_pTransform;
	CDPoint		*m_pTranslation;
	int			m_nWidth;
	int			m_nX;
	int			m_nY;
	CAttribute	*m_pAttributeName;
};


class CSet : public CX
{
public:
	CSet();

public:
	CAttribute			*m_pAttributeName;
	int					m_nAttributeType;
	vector<CSmilTime>*	m_apBegin;
	int					m_nDur;
	int					m_nMin;
	int					m_nMax;
	bool				m_bEnabled;
	vector<CSmilTime>*	m_apEnd;
	int					m_nFill; 
	int					m_nRepeatCount;
	int					m_nRepeatDur;
	int					m_nRestart;
	CAnimValue			*m_pTo;
	int					m_nXlink_actuate;
	CURI				*m_pXlink_arcrole;
	CURI				*m_pHref;
	CURI				*m_pXlink_role;
	int					m_nXlink_show;
	tstring				m_strXlink_title;
	int					m_nXlink_type;

public:
	void				setHref(CURI* href)	{ m_pHref = href; }
};


class CSvg : public CXpropElem
{
public:
	CSvg();
	~CSvg();

public:
	vector<int>	m_anHeight;
	int			m_nPreserveAspectRatio;
	int			m_nSyncBehaviorDefault;
	int			m_nSyncToleranceDefault;
	vector<int>	m_anViewBox;
	vector<int>	m_anWidth;
	int			m_nZoomAndPan;
	int			m_nPlaybackOrder;
	int			m_nTimelineBegin;
	int			m_nSnapshotTime;
	tstring		m_strVersion;
	tstring		m_strBaseProfile;
	tstring		m_strContentScriptType;
	int			m_nFullscreen;
};


class CText : public CXpropElem
{
public:
	CText();
	~CText();

public:
	bool		m_bEditable;
	tstring		m_strRequiredExtensions;
	vector<int>	m_anRequiredFeatures;
	tstring		m_strRequiredFormats;
	vector<int>	m_anRotate;
	int			m_nRotation;
	CDPoint		*m_pScale;
	tstring		m_strSystemLanguage;
	CMatrix		*m_pTransform;
 	CDPoint		*m_pTranslation;
	vector<int>	m_anX;
	vector<int>	m_anY;
	CAttribute	*m_pAttributeName;
};


class CVideo : public CAudio
{
public:
	CVideo();

public:
	int			m_nHeight;
	int			m_nOverlay;
	int			m_nRotation;
	CDPoint		*m_pScale;
	CMatrix		*m_pTransform;
	int			m_nTransformBehavior;
	CDPoint		*m_pTranslation;
	int			m_nWidth;
	int			m_nX;
	int			m_nY;
	tstring		m_strContentType;
	int			m_nDisplay;
	int			m_nPointer_events;
	int			m_nVisibility;
	CAttribute	*m_pAttributeName;
	int			m_nPreserveAspectRatio;
	int			m_nFullscreen;
	int			m_nInitialVisibility;
};


class CListener : public CX
{
public:
	CListener();

public:
//	int		m_nDelay;
	bool	m_bEnabled;
	int		m_nPhase;
	CDEvent	*m_pEvent;
	int		m_nKeyCode;
	int		m_nObserver;
//	int		m_nTimeAttribute;
	CURI	*m_pHandler;
	int		m_nPropagate;
	int		m_nDefaultAction;
	int		m_nTarget;
};


class CConditional : public CX
{
public:
	CConditional();

public:
 	vector<CSmilTime>*	m_apBegin;
	bool				m_bEnabled;
	tstring				m_strType;
	tstring				m_strText;
};


class CTextContent : public CXpropElem
{
public:
	CTextContent();
	CTextContent(tstring s);

public:
	tstring		m_strContent;
	bool		m_bInScript;
};


class CTextArea : public CXpropElem
{
public:
	CTextArea();
	~CTextArea();

public:
	tstring		m_strRequiredExtensions;
	vector<int>	m_anRequiredFeatures;
	tstring		m_strRequiredFormats;
	int			m_nRotation;
	CDPoint		*m_pScale;
	tstring		m_strSystemLanguage;
	CMatrix		*m_pTransform;
	CDPoint		*m_pTranslation;
	int			m_nFocusHighlight;
	bool		m_bEditable;
	int			m_nX;
	int			m_nY;
	int			m_nWidth;
	int			m_nHeight;
};


class CTref : public CXpropElem
{
public:
	CTref();

public:
	bool	m_bEditable;
	CURI	*m_pHref;

public:
	void	setHref(CURI* href)	{ m_pHref = href; }
};


class CExternalReference : public CXpropElem
{
public:
	CExternalReference();

public:
	int		m_nUpdateInterval;
	int		m_nSecurity;
	tstring	m_strEntry;
};


class CExternalUpdate : public CXpropElem
{
public:
	CExternalUpdate();

public:
	tstring		m_strAttributeName;
	int			m_nIndex;
	int			m_nOperandElementId;
	CAttribute	*m_pOperandAttributeName;
	int			m_nUpdateInterval;
	int			m_nType;
	tstring		m_strValue;
	CURI		*m_pHref;

public:
	void	setHref(CURI* href)	{ m_pHref = href; }
};